@@ -44,6 +44,7 @@
 #include <kdebug.h>
 #include <kdialog.h>
 #include <kiconloader.h>
+#include <kinputdialog.h>
 #include <klineedit.h>
 #include <klistview.h>
 #include <klocale.h>
@@ -112,7 +113,7 @@ KABC::Address::List AddressEditWidget::addresses()
 {
   KABC::Address::List retList;
 
-  KABC::Address::List::Iterator it;
+  KABC::Address::List::ConstIterator it;
   for ( it = mAddressList.begin(); it != mAddressList.end(); ++it )
     if ( !(*it).isEmpty() )
       retList.append( *it );
@@ -135,7 +136,7 @@ void AddressEditWidget::setAddresses( const KABC::Addressee &addr,
   defaultTypes << KABC::Address::Work;
 
   AddresseeConfig config( mAddressee );
-  QValueList<int> configList = config.noDefaultAddrTypes();
+  const QValueList<int> configList = config.noDefaultAddrTypes();
   QValueList<int>::ConstIterator it;
   for ( it = configList.begin(); it != configList.end(); ++it )
     defaultTypes.remove( *it );
@@ -143,7 +144,7 @@ void AddressEditWidget::setAddresses( const KABC::Addressee &addr,
   // Insert default types.
   // Doing this for mPrefCombo is enough because the list is shared by all
   // combos.
-  for( it = defaultTypes.begin(); it != defaultTypes.end(); ++it ) {
+  for ( it = defaultTypes.begin(); it != defaultTypes.end(); ++it ) {
     if ( !mTypeCombo->hasType( *it ) )
       mTypeCombo->insertType( list, *it, Address( *it ) );
   }
@@ -152,10 +153,10 @@ void AddressEditWidget::setAddresses( const KABC::Addressee &addr,
 
   // find preferred address which will be shown
   int preferred = KABC::Address::Home;  // default if no preferred address set
-  uint i;
-  for (i = 0; i < list.count(); i++)
-    if ( list[i].type() & KABC::Address::Pref ) {
-      preferred = list[i].type();
+  KABC::Address::List::ConstIterator addrIt;
+  for ( addrIt = list.begin(); addrIt != list.end(); ++addrIt )
+    if ( (*addrIt).type() & KABC::Address::Pref ) {
+      preferred = (*addrIt).type();
       break;
     }
 
@@ -172,7 +173,7 @@ void AddressEditWidget::edit()
       mAddressList = dialog.addresses();
 
       bool hasHome = false, hasWork = false;
-      KABC::Address::List::Iterator it;
+      KABC::Address::List::ConstIterator it;
       for ( it = mAddressList.begin(); it != mAddressList.end(); ++it ) {
         if ( (*it).type() == KABC::Address::Home ) {
           if ( !(*it).isEmpty() )
@@ -263,7 +264,7 @@ AddressEditDialog::AddressEditDialog( const KABC::Address::List &list,
   mTypeCombo = new AddressTypeCombo( mAddressList, page );
   topLayout->addMultiCellWidget( mTypeCombo, 0, 0, 0, 1 );
 
-  QLabel *label = new QLabel( KABC::Address::streetLabel() + ":", page );
+  QLabel *label = new QLabel( i18n( "<streetLabel>:", "%1:" ).arg( KABC::Address::streetLabel() ), page );
   label->setAlignment( Qt::AlignTop | Qt::AlignLeft );
   topLayout->addWidget( label, 1, 0 );
   mStreetTextEdit = new QTextEdit( page );
@@ -273,49 +274,55 @@ AddressEditDialog::AddressEditDialog( const KABC::Address::List &list,
   TabPressEater *eater = new TabPressEater( this );
   mStreetTextEdit->installEventFilter( eater );
 
-  label = new QLabel( KABC::Address::postOfficeBoxLabel() + ":", page );
+  label = new QLabel( i18n( "<postOfficeBoxLabel>:", "%1:" ).arg( KABC::Address::postOfficeBoxLabel() ), page );
   topLayout->addWidget( label, 2 , 0 );
   mPOBoxEdit = new KLineEdit( page );
   label->setBuddy( mPOBoxEdit );
   topLayout->addWidget( mPOBoxEdit, 2, 1 );
 
-  label = new QLabel( KABC::Address::localityLabel() + ":", page );
+  label = new QLabel( i18n( "<localityLabel>:", "%1:" ).arg( KABC::Address::localityLabel() ), page );
   topLayout->addWidget( label, 3, 0 );
   mLocalityEdit = new KLineEdit( page );
   label->setBuddy( mLocalityEdit );
   topLayout->addWidget( mLocalityEdit, 3, 1 );
 
-  label = new QLabel( KABC::Address::regionLabel() + ":", page );
+  label = new QLabel( i18n( "<regionLabel>:", "%1:" ).arg( KABC::Address::regionLabel() ), page );
   topLayout->addWidget( label, 4, 0 );
   mRegionEdit = new KLineEdit( page );
   label->setBuddy( mRegionEdit );
   topLayout->addWidget( mRegionEdit, 4, 1 );
 
-  label = new QLabel( KABC::Address::postalCodeLabel() + ":", page );
+  label = new QLabel( i18n( "<postalCodeLabel>:", "%1:" ).arg( KABC::Address::postalCodeLabel() ), page );
   topLayout->addWidget( label, 5, 0 );
   mPostalCodeEdit = new KLineEdit( page );
   label->setBuddy( mPostalCodeEdit );
   topLayout->addWidget( mPostalCodeEdit, 5, 1 );
 
-  label = new QLabel( KABC::Address::countryLabel() + ":", page );
+  label = new QLabel( i18n( "<countryLabel>:", "%1:" ).arg( KABC::Address::countryLabel() ), page );
   topLayout->addWidget( label, 6, 0 );
   mCountryCombo = new KComboBox( page );
   mCountryCombo->setEditable( true );
   mCountryCombo->setDuplicatesEnabled( false );
 
+#if KDE_IS_VERSION(3,3,0)
+  QPushButton *labelButton = new QPushButton( i18n( "Edit Label..." ), page );
+  topLayout->addMultiCellWidget( labelButton, 7, 7, 0, 1 );
+  connect( labelButton, SIGNAL( clicked() ), SLOT( editLabel() ) );
+#endif
+
   fillCountryCombo();
   label->setBuddy( mCountryCombo );
   topLayout->addWidget( mCountryCombo, 6, 1 );
 
   mPreferredCheckBox = new QCheckBox( i18n( "This is the preferred address" ), page );
-  topLayout->addMultiCellWidget( mPreferredCheckBox, 7, 7, 0, 1 );
+  topLayout->addMultiCellWidget( mPreferredCheckBox, 8, 8, 0, 1 );
 
   KSeparator *sep = new KSeparator( KSeparator::HLine, page );
-  topLayout->addMultiCellWidget( sep, 8, 8, 0, 1 );
+  topLayout->addMultiCellWidget( sep, 9, 9, 0, 1 );
 
   QHBox *buttonBox = new QHBox( page );
   buttonBox->setSpacing( spacingHint() );
-  topLayout->addMultiCellWidget( buttonBox, 9, 9, 0, 1 );
+  topLayout->addMultiCellWidget( buttonBox, 10, 10, 0, 1 );
 
   QPushButton *addButton = new QPushButton( i18n( "New..." ), buttonBox );
   connect( addButton, SIGNAL( clicked() ), SLOT( addAddress() ) );
@@ -416,6 +423,20 @@ void AddressEditDialog::changeType()
   }
 }
 
+void AddressEditDialog::editLabel()
+{
+#if KDE_IS_VERSION(3,3,0)
+  bool ok = false;
+  QString result = KInputDialog::getMultiLineText( KABC::Address::labelLabel(),
+                                                   KABC::Address::labelLabel(),
+                                                   mLabel, &ok, this );
+  if ( ok ) {
+    mLabel = result;
+    modified();
+  }
+#endif
+}
+
 void AddressEditDialog::updateAddressEdits()
 {
   if ( mPreviousAddress )
@@ -433,6 +454,7 @@ void AddressEditDialog::updateAddressEdits()
   mPostalCodeEdit->setText( a.postalCode() );
   mPOBoxEdit->setText( a.postOfficeBox() );
   mCountryCombo->setCurrentText( a.country() );
+  mLabel = a.label();
 
   mPreferredCheckBox->setChecked( a.type() & KABC::Address::Pref );
 
@@ -457,6 +479,7 @@ void AddressEditDialog::saveAddress( KABC::Address &addr )
   addr.setCountry( mCountryCombo->currentText() );
   addr.setPostOfficeBox( mPOBoxEdit->text() );
   addr.setStreet( mStreetTextEdit->text() );
+  addr.setLabel( mLabel );
 
 
   if ( mPreferredCheckBox->isChecked() ) {
@@ -579,7 +602,7 @@ AddressTypeDialog::AddressTypeDialog( int type, QWidget *parent )
   mTypeList = KABC::Address::typeList();
   mTypeList.remove( KABC::Address::Pref );
 
-  KABC::Address::TypeList::Iterator it;
+  KABC::Address::TypeList::ConstIterator it;
   for ( it = mTypeList.begin(); it != mTypeList.end(); ++it )
     new QCheckBox( KABC::Address::typeLabel( *it ), mGroup );
 
