@@ -42,13 +42,14 @@
 
 class KAddressBookApp : public KUniqueApplication {
   public:
-    KAddressBookApp() : mMainWin( 0 ) {}
+    KAddressBookApp() : mMainWin( 0 ), mDefaultIsOpen( false ) {}
     ~KAddressBookApp() {}
 
     int newInstance();
 
   private:
     KAddressBookMain *mMainWin;
+    bool mDefaultIsOpen;
 };
 
 int KAddressBookApp::newInstance()
@@ -73,11 +74,21 @@ int KAddressBookApp::newInstance()
         // otherwise, leave the window like it is (hidden or shown)
         KStartupInfo::appStarted();
     } else {
-      if ( !mMainWin ) {
-        mMainWin = new KAddressBookMain;
-        setMainWidget( mMainWin );
+      QString file;
+      if ( args->isSet( "document" ) ) {
+         file = args->getOption( "document" );
+      }
+      if ( !( file.isEmpty() && mDefaultIsOpen ) ) {
+        if ( !mMainWin ) {
+          mMainWin = new KAddressBookMain( file );
+          setMainWidget( mMainWin );
+          mMainWin->show();
+        } else {
+          KAddressBookMain *m = new KAddressBookMain( file );
+          m->show();
+        }
+        if ( file.isEmpty() ) mDefaultIsOpen = true;
       }
-      mMainWin->show();
     }
 
     mMainWin->handleCommandLine();
@@ -102,7 +113,7 @@ int main( int argc, char *argv[] )
   KUniqueApplication::addCmdLineOptions();
 
   if ( !KAddressBookApp::start() )
-    exit( 0 );
+    return 0;
 
   KAddressBookApp app;
   KGlobal::locale()->insertCatalogue( "libkdepim" );
