@@ -139,7 +139,7 @@ void PhoneEditWidget::setPhoneNumbers( const KABC::PhoneNumber::List &list )
   // Doing this for mPrefCombo is enough because the list is shared by all
   // combos.
   QValueList<int>::ConstIterator it;
-  for( it = defaultTypes.begin(); it != defaultTypes.end(); ++it ) {
+  for ( it = defaultTypes.begin(); it != defaultTypes.end(); ++it ) {
     if ( !mPrefCombo->hasType( *it ) )
       mPrefCombo->insertType( list, *it, PhoneNumber( "", *it ) );
   }
@@ -174,7 +174,7 @@ KABC::PhoneNumber::List PhoneEditWidget::phoneNumbers()
 {
   KABC::PhoneNumber::List retList;
 
-  KABC::PhoneNumber::List::Iterator it;
+  KABC::PhoneNumber::List::ConstIterator it;
   for ( it = mPhoneList.begin(); it != mPhoneList.end(); ++it )
     if ( !(*it).number().isEmpty() )
       retList.append( *it );
@@ -197,25 +197,21 @@ void PhoneEditWidget::edit()
 
 void PhoneEditWidget::updatePrefEdit()
 {
-  mPrefEdit->setFocus();
   updateEdit( mPrefCombo );
 }
 
 void PhoneEditWidget::updateSecondEdit()
 {
-  mSecondEdit->setFocus();
   updateEdit( mSecondCombo );
 }
 
 void PhoneEditWidget::updateThirdEdit()
 {
-  mThirdEdit->setFocus();
   updateEdit( mThirdCombo );
 }
 
 void PhoneEditWidget::updateFourthEdit()
 {
-  mFourthEdit->setFocus();
   updateEdit( mFourthCombo );
 }
 
@@ -354,9 +350,10 @@ PhoneEditDialog::PhoneEditDialog( const KABC::PhoneNumber::List &list, QWidget *
   layout->setSpacing( spacingHint() );
 
   mListView = new KListView( page );
-  mListView->setAllColumnsShowFocus( true );
   mListView->addColumn( i18n( "Number" ) );
   mListView->addColumn( i18n( "Type" ) );
+  mListView->setAllColumnsShowFocus( true );
+  mListView->setFullWidth( true );
 
   KButtonBox *buttonBox = new KButtonBox( page, Vertical );
 
@@ -373,11 +370,13 @@ PhoneEditDialog::PhoneEditDialog( const KABC::PhoneNumber::List &list, QWidget *
   connect( mListView, SIGNAL(selectionChanged()), SLOT(slotSelectionChanged()) );
   connect( mListView, SIGNAL(doubleClicked( QListViewItem *, const QPoint &, int  )), this, SLOT( slotEditPhoneNumber()));
 
-  KABC::PhoneNumber::List::Iterator it;
+  KABC::PhoneNumber::List::ConstIterator it;
   for ( it = mPhoneNumberList.begin(); it != mPhoneNumberList.end(); ++it )
     new PhoneViewItem( mListView, *it );
 
   mChanged = false;
+
+  setInitialSize( QSize(500, 250) );
 }
 
 PhoneEditDialog::~PhoneEditDialog()
@@ -458,7 +457,7 @@ PhoneTypeDialog::PhoneTypeDialog( const KABC::PhoneNumber &phoneNumber,
 {
   QWidget *page = plainPage();
   QLabel *label = 0;
-  QGridLayout *layout = new QGridLayout( page, 3, 2, marginHint(), spacingHint() );
+  QGridLayout *layout = new QGridLayout( page, 3, 2, 0, spacingHint() );
 
   label = new QLabel( i18n( "Number:" ), page );
   layout->addWidget( label, 0, 0 );
@@ -477,7 +476,7 @@ PhoneTypeDialog::PhoneTypeDialog( const KABC::PhoneNumber &phoneNumber,
   mTypeList = KABC::PhoneNumber::typeList();
   mTypeList.remove( KABC::PhoneNumber::Pref );
 
-  KABC::PhoneNumber::TypeList::Iterator it;
+  KABC::PhoneNumber::TypeList::ConstIterator it;
   for ( it = mTypeList.begin(); it != mTypeList.end(); ++it )
     new QCheckBox( KABC::PhoneNumber::typeLabel( *it ), mGroup );
 
