@@ -35,11 +35,12 @@
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qpushbutton.h>
+#include <qwhatsthis.h>
 
 #include "soundwidget.h"
 
-SoundWidget::SoundWidget( QWidget *parent, const char *name )
-  : QWidget( parent, name ), mReadOnly( false )
+SoundWidget::SoundWidget( KABC::AddressBook *ab, QWidget *parent, const char *name )
+  : KAB::ContactEditorWidget( ab, parent, name ), mReadOnly( false )
 {
   QGridLayout *topLayout = new QGridLayout( this, 2, 3, KDialog::marginHint(),
                                             KDialog::spacingHint() );
@@ -50,7 +51,7 @@ SoundWidget::SoundWidget( QWidget *parent, const char *name )
   label->setAlignment( Qt::AlignTop );
   topLayout->addMultiCellWidget( label, 0, 1, 0, 0 );
 
-  mPlayButton = new QPushButton( i18n( "Play..." ), this );
+  mPlayButton = new QPushButton( i18n( "Play" ), this );
   mPlayButton->setEnabled( false );
   topLayout->addWidget( mPlayButton, 0, 1 );
 
@@ -62,11 +63,11 @@ SoundWidget::SoundWidget( QWidget *parent, const char *name )
   topLayout->addWidget( mUseSoundUrl, 1, 2 );
 
   connect( mSoundUrl, SIGNAL( textChanged( const QString& ) ),
-           SIGNAL( changed() ) );
+           SLOT( setModified() ) );
   connect( mSoundUrl, SIGNAL( textChanged( const QString& ) ),
            SLOT( urlChanged( const QString& ) ) );
   connect( mUseSoundUrl, SIGNAL( toggled( bool ) ),
-           SIGNAL( changed() ) );
+           SLOT( setModified() ) );
   connect( mUseSoundUrl, SIGNAL( toggled( bool ) ),
            mPlayButton, SLOT( setDisabled( bool ) ) );
   connect( mSoundUrl, SIGNAL( urlSelected( const QString& ) ),
@@ -75,23 +76,21 @@ SoundWidget::SoundWidget( QWidget *parent, const char *name )
            SLOT( updateGUI() ) );
   connect( mPlayButton, SIGNAL( clicked() ),
            SLOT( playSound() ) );
+
+  QWhatsThis::add( this, i18n( "This field stores a sound file which contains the name of the contact to clarify the pronunciation." ) );
+  QWhatsThis::add( mUseSoundUrl, i18n( "Save only the URL to the sound file, not the whole object." ) );
 }
 
 SoundWidget::~SoundWidget()
 {
 }
 
-void SoundWidget::setReadOnly( bool readOnly )
-{
-  mReadOnly = readOnly;
-  mSoundUrl->setEnabled( !mReadOnly );
-}
-
-void SoundWidget::setSound( const KABC::Sound &sound )
+void SoundWidget::loadContact( KABC::Addressee *addr )
 {
   bool blocked = signalsBlocked();
   blockSignals( true );
 
+  KABC::Sound sound = addr->sound();
   if ( sound.isIntern() ) {
     mSound.setData( sound.data() );
     mPlayButton->setEnabled( true );
@@ -106,7 +105,7 @@ void SoundWidget::setSound( const KABC::Sound &sound )
   blockSignals( blocked );
 }
 
-KABC::Sound SoundWidget::sound() const
+void SoundWidget::storeContact( KABC::Addressee *addr )
 {
   KABC::Sound sound;
 
@@ -115,7 +114,13 @@ KABC::Sound SoundWidget::sound() const
   else
     sound.setData( mSound.data() );
 
-  return sound;
+  addr->setSound( sound );
+}
+
+void SoundWidget::setReadOnly( bool readOnly )
+{
+  mReadOnly = readOnly;
+  mSoundUrl->setEnabled( !mReadOnly );
 }
 
 void SoundWidget::playSound()
