@@ -69,15 +69,15 @@ ViewManager::~ViewManager()
 
 void ViewManager::restoreSettings()
 {
-  mViewNameList = KABPrefs::instance()->mViewNames;
-  QString activeViewName = KABPrefs::instance()->mCurrentView;
+  mViewNameList = KABPrefs::instance()->viewNames();
+  QString activeViewName = KABPrefs::instance()->currentView();
 
   mActionSelectView->setItems( mViewNameList );
 
   // Filter
   mFilterList = Filter::restore( mCore->config(), "Filter" );
   mFilterSelectionWidget->setItems( filterNames() );
-  mFilterSelectionWidget->setCurrentItem( KABPrefs::instance()->mCurrentFilter );
+  mFilterSelectionWidget->setCurrentItem( KABPrefs::instance()->currentFilter() );
 
   // Tell the views to reread their config, since they may have
   // been modified by global settings
@@ -101,13 +101,13 @@ void ViewManager::saveSettings()
   }
 
   Filter::save( mCore->config(), "Filter", mFilterList );
-  KABPrefs::instance()->mCurrentFilter = mFilterSelectionWidget->currentItem();
+  KABPrefs::instance()->setCurrentFilter( mFilterSelectionWidget->currentItem() );
 
   // write the view name list
-  KABPrefs::instance()->mViewNames = mViewNameList;
+  KABPrefs::instance()->setViewNames( mViewNameList );
 
   if ( mActiveView )
-    KABPrefs::instance()->mCurrentView = mActiveView->caption();
+    KABPrefs::instance()->setCurrentView( mActiveView->caption() );
 }
 
 QStringList ViewManager::selectedUids() const
@@ -129,8 +129,9 @@ QStringList ViewManager::selectedEmails() const
 KABC::Addressee::List ViewManager::selectedAddressees() const
 {
   KABC::Addressee::List list;
-  QStringList uids = selectedUids();
-  QStringList::Iterator it;
+
+  const QStringList uids = selectedUids();
+  QStringList::ConstIterator it;
   for ( it = uids.begin(); it != uids.end(); ++it ) {
     KABC::Addressee addr = mCore->addressBook()->findByUid( *it );
     if ( !addr.isEmpty() )
@@ -153,12 +154,28 @@ KABC::Field *ViewManager::currentSortField() const
     return 0;
 }
 
+KABC::Field::List ViewManager::viewFields() const
+{
+/*
+  if ( mActiveView )
+    return mActiveView->fields();
+  else
+*/
+    return KABC::Field::List();
+}
+
 void ViewManager::setSelected( const QString &uid, bool selected )
 {
   if ( mActiveView )
     mActiveView->setSelected( uid, selected );
 }
 
+void ViewManager::setFirstSelected( bool selected )
+{
+  if ( mActiveView )
+    mActiveView->setFirstSelected( selected );
+}
+
 void ViewManager::unloadViews()
 {
   mViewDict.clear();
@@ -282,6 +299,7 @@ void ViewManager::editView()
       }
 
       mActiveView->refresh();
+      emit viewFieldsChanged();
     }
   }
 }
@@ -354,9 +372,22 @@ void ViewManager::addView()
   }
 }
 
+void ViewManager::scrollUp()
+{
+  if ( mActiveView )
+    mActiveView->scrollUp();
+}
+
+void ViewManager::scrollDown()
+{
+  if ( mActiveView )
+    mActiveView->scrollDown();
+}
+
 void ViewManager::createViewFactories()
 {
-  KTrader::OfferList plugins = KTrader::self()->query( "KAddressBook/View" );
+  const KTrader::OfferList plugins = KTrader::self()->query( "KAddressBook/View", 
+    QString( "[X-KDE-KAddressBook-ViewPluginVersion] == %1" ).arg(  KAB_VIEW_PLUGIN_VERSION ) );
   KTrader::OfferList::ConstIterator it;
   for ( it = plugins.begin(); it != plugins.end(); ++it ) {
     if ( !(*it)->hasServiceType( "KAddressBook/View" ) )
@@ -392,7 +423,7 @@ void ViewManager::dropped( QDropEvent *e )
   KURL::List urls;
 
   if ( KURLDrag::decode( e, urls) ) {
-    KURL::List::Iterator it = urls.begin();
+    KURL::List::ConstIterator it = urls.begin();
     int c = urls.count();
     if ( c > 1 ) {
       QString questionString = i18n( "Import one contact into your addressbook?", "Import %n contacts into your addressbook?", c );
@@ -405,8 +436,8 @@ void ViewManager::dropped( QDropEvent *e )
   } else if ( KVCardDrag::decode( e, vcards ) ) {
     KABC::VCardConverter converter;
 
-    KABC::Addressee::List list = converter.parseVCards( vcards );
-    KABC::Addressee::List::Iterator it;
+    const KABC::Addressee::List list = converter.parseVCards( vcards );
+    KABC::Addressee::List::ConstIterator it;
     for ( it = list.begin(); it != list.end(); ++it ) {
       KABC::Addressee a = mCore->addressBook()->findByUid( (*it).uid() );
       if ( a.isEmpty() ) { // not yet in address book
@@ -423,12 +454,13 @@ void ViewManager::startDrag()
 {
   // Get the list of all the selected addressees
   KABC::Addressee::List addrList;
-  QStringList uidList = selectedUids();
+  const QStringList uidList = selectedUids();
   if (  uidList.isEmpty() )
-      return;
+    return;
+
   kdDebug(5720) << "ViewManager::startDrag: starting to drag" << endl;
 
-  QStringList::Iterator it;
+  QStringList::ConstIterator it;
   for ( it = uidList.begin(); it != uidList.end(); ++it )
     addrList.append( mCore->addressBook()->findByUid( *it ) );
 
@@ -442,6 +474,7 @@ void ViewManager::startDrag()
   drag->addDragObject( new KVCardDrag( vcards, this ) );
 
   KTempDir tempDir;
+  // can't set tempDir to autoDelete, in case of dropping on the desktop, the copy is async...
   if ( tempDir.status() == 0 ) {
     QString fileName;
     if ( addrList.count() == 1 )
@@ -469,8 +502,12 @@ void ViewManager::setActiveFilter( int index )
 
   if ( ( index - 1 ) < 0 )
     currentFilter = Filter();
+  else if ( ( index - 1 ) < 1 ) {
+    currentFilter = Filter();
+    currentFilter.setMatchRule(Filter::NotMatching);
+  }
   else
-    currentFilter = mFilterList[ index - 1 ];
+    currentFilter = mFilterList[ index - 2 ];
 
   // Check if we have a view. Since the filter combo is created before
   // the view, this slot could be called before there is a valid view.
@@ -499,6 +536,7 @@ void ViewManager::configureFilters()
 QStringList ViewManager::filterNames() const
 {
   QStringList names( i18n( "None" ) );
+  names.append( i18n( "Unfiled" ) );
 
   Filter::List::ConstIterator it;
   for ( it = mFilterList.begin(); it != mFilterList.end(); ++it )
