@@ -3,15 +3,10 @@
 * Distributed under the GPL.
 */
 
-
-
 #include <numeric>
 
-#include <qkeycode.h>
-#include <qpopupmenu.h>
-#include <qptrlist.h>
-#include <qstring.h>
-
+#include "kaccelmenuwatch.h"
+#include <dcopclient.h>
 #include <kaccel.h>
 #include <kaction.h>
 #include <kapplication.h>       // kapp
@@ -23,25 +18,32 @@
 #include <kmessagebox.h>
 #include <kstatusbar.h>         // statusBar()
 #include <kstdaction.h>
+#include <qkeycode.h>
+#include <qpopupmenu.h>
+#include <qptrlist.h>
+#include <qstring.h>
 
-#include "kaccelmenuwatch.h"
 #include "karmutility.h"
 #include "mainwindow.h"
 #include "preferences.h"
 #include "print.h"
-#include "timekard.h"
 #include "task.h"
 #include "taskview.h"
+#include "timekard.h"
 #include "tray.h"
+#include "version.h"
 
-MainWindow::MainWindow()
-  : KMainWindow(0),
-    _accel( new KAccel( this ) ),
-    _watcher( new KAccelMenuWatch( _accel, this ) ),
-    _taskView( new TaskView( this ) ),
-    _totalSum( 0 ),
+MainWindow::MainWindow( const QString &icsfile )
+  : DCOPObject ( "KarmDCOPIface" ),
+    KMainWindow(0), 
+    _accel     ( new KAccel( this ) ),
+    _watcher   ( new KAccelMenuWatch( _accel, this ) ),
+    _totalSum  ( 0 ),
     _sessionSum( 0 )
 {
+
+  _taskView  = new TaskView( this, 0, icsfile );
+
   setCentralWidget( _taskView );
   // status bar
   startStatusBar();
@@ -82,21 +84,22 @@ MainWindow::MainWindow()
 
   _taskView->load();
 
-  if ( _taskView->isReadOnly() )
-      stateChanged( QString::fromLatin1( "readonly" ) ); // disable many actions
-
   // Everything that uses Preferences has been created now, we can let it
   // emit its signals
   _preferences->emitSignals();
   slotSelectionChanged();
 
+  // Register with DCOP
+  if ( !kapp->dcopClient()->isRegistered() ) 
+  {
+    kapp->dcopClient()->registerAs( "karm" );
+    kapp->dcopClient()->setDefaultObject( objId() );
+  }
 }
 
 void MainWindow::slotSelectionChanged()
 {
   Task* item= _taskView->current_item();
-  if ( _taskView->isReadOnly() )
-      item = 0; // don't enable any of those actions if the storage is readonly
   actionDelete->setEnabled(item);
   actionEdit->setEnabled(item);
   actionStart->setEnabled(item && !item->isRunning());
@@ -113,10 +116,22 @@ void MainWindow::slotSelectionChanged()
 void MainWindow::save()
 {
   kdDebug(5970) << i18n("Saving time data to disk.") << endl;
-  _taskView->save();
+  QString err=_taskView->save();  // untranslated error msg.
+  if (err.isEmpty()) statusBar()->message(i18n("Successfully saved tasks and history"),1807);
+  else statusBar()->message(i18n(err.ascii()),7707); // no msgbox since save is called when exiting
   saveGeometry();
 }
 
+void MainWindow::exportcsvHistory()
+{
+  kdDebug(5970) << i18n("Exporting History to disk.") << endl;
+  QString err=_taskView->exportcsvHistory();
+  if (err.isEmpty()) statusBar()->message(i18n("Successfully exported History to CSV-file"),1807);
+  else KMessageBox::error(this, i18n(err.ascii()));
+  saveGeometry();
+  
+}
+
 void MainWindow::quit()
 {
   kapp->quit();
@@ -302,9 +317,13 @@ void MainWindow::makeMenus()
       _taskView, SLOT(exportcsvFile()), actionCollection(),
       "export_csvfile");
   new KAction( i18n("Export &History to CSV File..."), 0,
-      _taskView, SLOT(exportcsvHistory()), actionCollection(),
+      this, SLOT(exportcsvHistory()), actionCollection(),
       "export_csvhistory");
-  /*
+  new KAction( i18n("Import Tasks From &Planner..."), 0,
+      _taskView, SLOT(importPlanner()), actionCollection(),
+      "import_planner");  
+
+/*
   new KAction( i18n("Import E&vents"), 0,
                             _taskView,
                             SLOT( loadFromKOrgEvents() ), actionCollection(),
@@ -412,4 +431,54 @@ void MainWindow::contextMenuRequest( QListViewItem*, const QPoint& point, int )
       pop->popup( point );
 }
 
+//----------------------------------------------------------------------------
+//
+//                       D C O P   I N T E R F A C E
+//
+//----------------------------------------------------------------------------
+
+QString MainWindow::version() const
+{
+  return KARM_VERSION;
+}
+
+QString MainWindow::hastodo( const QString &taskname ) const
+{
+  QString rval = "";
+
+  Task* task = _taskView->first_child();
+  while ( rval.isEmpty() && task )
+  {
+    rval = _hastodo( task, taskname );
+    task = task->nextSibling();
+  }
+  
+  return rval;
+}
+
+QString MainWindow::_hastodo( Task* task, const QString &taskname ) const
+{
+  QString rval = "";
+  if ( task->name() == taskname ) 
+  {
+    rval = task->uid();
+  }
+  else
+  {
+    Task* nexttask = task->firstChild();
+    while ( rval.isEmpty() && nexttask )
+    {
+      rval = _hastodo( nexttask, taskname );
+      nexttask = nexttask->nextSibling();
+    }
+  }
+  return rval;
+}
+
+QString MainWindow::addtodo( const QString& taskname ) 
+{
+  DesktopList desktopList;
+  return _taskView->addTask( taskname, 0, 0, desktopList );
+}
+
 #include "mainwindow.moc"
