@@ -1,5 +1,5 @@
 /* -*- Mode: C++ -*-
-   $Id: KDGanttViewSummaryItem.cpp,v 1.4 2004/03/21 13:34:17 rogowski Exp $
+   $Id: KDGanttViewSummaryItem.cpp,v 1.5 2004/11/03 13:56:18 danders Exp $
    KDGantt - a multi-platform charting engine
 */
 /****************************************************************************
@@ -314,7 +314,7 @@ void KDGanttViewSummaryItem::showItem( bool show, int coordY )
 
   endShape->move(endX,allY);
   endShapeBack->move(endX,allY);
-  textCanvas->move(endX+2*myItemSize,allY-myItemSize/2 );
+  moveTextCanvas(endX,allY);
   startShape->show();
   startShapeBack->show();
   endShape->show();
