@@ -29,6 +29,9 @@
 #include <qptrlist.h>
 #include <qasciidict.h>
 
+#include <kio/authinfo.h>
+#include <kio/slavebase.h>
+
 #include "imaplist.h"
 #include "imapcommand.h"
 #include "imapinfo.h"
@@ -231,8 +234,8 @@ public:
    * @param resultInfo The resultinfo from the command
    * @return success or failure
    */
-  bool clientAuthenticate (const QString & aUser, const QString & aPass,
-               const QString & aAuth, bool isSSL, QString & resultInfo);
+  bool clientAuthenticate (KIO::SlaveBase *slave, KIO::AuthInfo &ai, const QString & aFQDN, 
+    const QString & aAuth, bool isSSL, QString & resultInfo);
 
   /**
    * main loop for the parser
@@ -273,6 +276,10 @@ public:
   void parseExpunge (ulong value, parseString & result);
   /** @brief parse a ACL line */
   void parseAcl (parseString & result);
+  /** @brief parse a ANNOTATION line */
+  void parseAnnotation (parseString & result);
+  /** @brief parse a NAMESPACE line */
+  void parseNamespace (parseString & result);
 
   /**
    * parses the results of a fetch command
@@ -363,7 +370,7 @@ public:
   /** parse one number using parseOneWord */
   static bool parseOneNumber (parseString & inWords, ulong & num);
 
-  /** extract the box,section,list type, uid, uidvalidity from an url */
+  /** extract the box,section,list type, uid, uidvalidity,info from an url */
   static void parseURL (const KURL & _url, QString & _box, QString & _section,
                         QString & _type, QString & _uid, QString & _validity, 
                         QString & _info);
@@ -402,6 +409,8 @@ public:
   /** @brief see if server has a capability */
   bool hasCapability (const QString &);
 
+  void removeCapability (const QString & cap);
+
   static inline void skipWS (parseString & inWords)
   {
     char c;
@@ -412,6 +421,9 @@ public:
     }
   }
 
+  /** @brief find the namespace for the given box */
+  QString namespaceForBox( const QString & box );
+
 
 protected:
 
@@ -454,6 +466,12 @@ protected:
   /** @brief the results from search/acl commands */
   QStringList lastResults;
 
+  /** @brief namespace prefix - delimiter association */
+  QMap<QString, QString> imapNamespaceDelimiter;
+
+  /** @brief list of namespace - prefix */
+  QStringList imapNamespaces;
+
 private:
 
   /** we don't want to be able to copy this object */
