@@ -75,9 +75,7 @@ class Debugger : public ActionPart
     void connectDevice();
     void disconnectDevice();
 
-    void slotReceiveData( Konnector * );
-    void slotProgress( Konnector *, const Progress & );
-    void slotError( Konnector *, const Error & );
+    void slotReceiveData( KSync::Konnector * );
 
   private:
     QPixmap m_pixmap;
