@@ -32,6 +32,7 @@
 #include <qpushbutton.h>
 #include <qwhatsthis.h>
 #include <qhbox.h>
+#include <qcombobox.h>
 
 #include <kfiledialog.h>
 #include <klocale.h>
@@ -41,6 +42,7 @@
 #include <kseparator.h>
 #include <kapplication.h>
 #include <kmessagebox.h>
+#include <kprotocolinfo.h>
 
 #include <netdb.h>
 #include <netinet/in.h>
@@ -57,6 +59,10 @@ using KMail::SieveConfigEditor;
 #include "kmfoldermgr.h"
 #include "kmservertest.h"
 #include "protocols.h"
+#include "folderrequester.h"
+using KMail::FolderRequester;
+#include "kmmainwidget.h"
+#include "kmfolder.h"
 
 #include <cassert>
 #include <stdlib.h>
@@ -325,13 +331,13 @@ void AccountDialog::makeLocalAccountPage()
   KSeparator *hline = new KSeparator( KSeparator::HLine, page);
   topLayout->addMultiCellWidget( hline, 1, 1, 0, 2 );
 
-  QLabel *label = new QLabel( i18n("&Name:"), page );
+  QLabel *label = new QLabel( i18n("Account &name:"), page );
   topLayout->addWidget( label, 2, 0 );
   mLocal.nameEdit = new KLineEdit( page );
   label->setBuddy( mLocal.nameEdit );
   topLayout->addWidget( mLocal.nameEdit, 2, 1 );
 
-  label = new QLabel( i18n("&Location:"), page );
+  label = new QLabel( i18n("File &location:"), page );
   topLayout->addWidget( label, 3, 0 );
   mLocal.locationEdit = new QComboBox( true, page );
   label->setBuddy( mLocal.locationEdit );
@@ -407,9 +413,10 @@ void AccountDialog::makeLocalAccountPage()
   topLayout->addMultiCellWidget( resourceHB, 5, 5, 0, 2 );
 #endif
 
-  mLocal.excludeCheck =
-    new QCheckBox( i18n("E&xclude from \"Check Mail\""), page );
-  topLayout->addMultiCellWidget( mLocal.excludeCheck, 5, 5, 0, 2 );
+  mLocal.includeInCheck =
+    new QCheckBox( i18n("Include in m&anual mail check"),
+                   page );
+  topLayout->addMultiCellWidget( mLocal.includeInCheck, 5, 5, 0, 2 );
 
   mLocal.intervalCheck =
     new QCheckBox( i18n("Enable &interval mail checking"), page );
@@ -431,18 +438,6 @@ void AccountDialog::makeLocalAccountPage()
   label->setBuddy( mLocal.folderCombo );
   topLayout->addWidget( mLocal.folderCombo, 8, 1 );
 
-  /* -sanders Probably won't support this way, use filters insteada
-  label = new QLabel( i18n("Default identity:"), page );
-  topLayout->addWidget( label, 9, 0 );
-  mLocal.identityCombo = new QComboBox( false, page );
-  topLayout->addWidget( mLocal.identityCombo, 9, 1 );
-  // GS - this was moved inside the commented block 9/30/2000
-  //      (I think Don missed it?)
-  label->setEnabled(false);
-  */
-
-  //mLocal.identityCombo->setEnabled(false);
-
   label = new QLabel( i18n("&Pre-command:"), page );
   topLayout->addWidget( label, 9, 0 );
   mLocal.precommand = new KLineEdit( page );
@@ -473,13 +468,13 @@ void AccountDialog::makeMaildirAccountPage()
 
   mMaildir.nameEdit = new KLineEdit( page );
   topLayout->addWidget( mMaildir.nameEdit, 2, 1 );
-  QLabel *label = new QLabel( mMaildir.nameEdit, i18n("&Name:"), page );
+  QLabel *label = new QLabel( mMaildir.nameEdit, i18n("Account &name:"), page );
   topLayout->addWidget( label, 2, 0 );
 
   mMaildir.locationEdit = new QComboBox( true, page );
   topLayout->addWidget( mMaildir.locationEdit, 3, 1 );
   mMaildir.locationEdit->insertStringList(procmailrcParser.getSpoolFilesList());
-  label = new QLabel( mMaildir.locationEdit, i18n("&Location:"), page );
+  label = new QLabel( mMaildir.locationEdit, i18n("Folder &location:"), page );
   topLayout->addWidget( label, 3, 0 );
 
   QPushButton *choose = new QPushButton( i18n("Choo&se..."), page );
@@ -513,9 +508,9 @@ void AccountDialog::makeMaildirAccountPage()
   topLayout->addMultiCellWidget( resourceHB, 4, 4, 0, 2 );
 #endif
 
-  mMaildir.excludeCheck =
-    new QCheckBox( i18n("E&xclude from \"Check Mail\""), page );
-  topLayout->addMultiCellWidget( mMaildir.excludeCheck, 4, 4, 0, 2 );
+  mMaildir.includeInCheck =
+    new QCheckBox( i18n("Include in &manual mail check"), page );
+  topLayout->addMultiCellWidget( mMaildir.includeInCheck, 4, 4, 0, 2 );
 
   mMaildir.intervalCheck =
     new QCheckBox( i18n("Enable &interval mail checking"), page );
@@ -571,7 +566,7 @@ void AccountDialog::makePopAccountPage()
   grid->setRowStretch( 15, 10 );
   grid->setColStretch( 1, 10 );
 
-  QLabel *label = new QLabel( i18n("&Name:"), page1 );
+  QLabel *label = new QLabel( i18n("Account &name:"), page1 );
   grid->addWidget( label, 0, 0 );
   mPop.nameEdit = new KLineEdit( page1 );
   label->setBuddy( mPop.nameEdit );
@@ -608,7 +603,17 @@ void AccountDialog::makePopAccountPage()
   grid->addWidget( mPop.portEdit, 4, 1 );
 
   mPop.storePasswordCheck =
-    new QCheckBox( i18n("Sto&re POP password in configuration file"), page1 );
+    new QCheckBox( i18n("Sto&re POP password"), page1 );
+  QWhatsThis::add( mPop.storePasswordCheck,
+                   i18n("Check this option to have KMail store "
+                   "the password.\nIf KWallet is available "
+                   "the password will be stored there which is considered "
+                   "safe.\nHowever, if KWallet is not available, "
+                   "the password will be stored in KMail's configuration "
+                   "file. The password is stored in an "
+                   "obfuscated format, but should not be "
+                   "considered secure from decryption efforts "
+                   "if access to the configuration file is obtained.") );
   grid->addMultiCellWidget( mPop.storePasswordCheck, 5, 5, 0, 1 );
 
   mPop.leaveOnServerCheck =
@@ -643,9 +648,9 @@ void AccountDialog::makePopAccountPage()
   grid->addMultiCellWidget( resourceHB, 7, 7, 0, 2 );
 #endif
 
-  mPop.excludeCheck =
-    new QCheckBox( i18n("E&xclude from \"Check Mail\""), page1 );
-  grid->addMultiCellWidget( mPop.excludeCheck, 7, 7, 0, 1 );
+  mPop.includeInCheck =
+    new QCheckBox( i18n("Include in man&ual mail check"), page1 );
+  grid->addMultiCellWidget( mPop.includeInCheck, 7, 7, 0, 1 );
 
   QHBox * hbox = new QHBox( page1 );
   hbox->setSpacing( KDialog::spacingHint() );
@@ -688,7 +693,7 @@ void AccountDialog::makePopAccountPage()
   label->setBuddy( mPop.folderCombo );
   grid->addWidget( mPop.folderCombo, 11, 1 );
 
-  label = new QLabel( i18n("Precom&mand:"), page1 );
+  label = new QLabel( i18n("Pre-com&mand:"), page1 );
   grid->addWidget( label, 12, 0 );
   mPop.precommand = new KLineEdit( page1 );
   label->setBuddy(mPop.precommand);
@@ -698,11 +703,18 @@ void AccountDialog::makePopAccountPage()
   tabWidget->addTab( page2, i18n("&Extras") );
   QVBoxLayout *vlay = new QVBoxLayout( page2, marginHint(), spacingHint() );
 
-  mPop.usePipeliningCheck =
-    new QCheckBox( i18n("&Use pipelining for faster mail download"), page2 );
-  connect(mPop.usePipeliningCheck, SIGNAL(clicked()),
-    SLOT(slotPipeliningClicked()));
-  vlay->addWidget( mPop.usePipeliningCheck );
+  vlay->addSpacing( KDialog::spacingHint() );
+
+  QHBoxLayout *buttonLay = new QHBoxLayout( vlay );
+  mPop.checkCapabilities =
+    new QPushButton( i18n("Check &What the Server Supports"), page2 );
+  connect(mPop.checkCapabilities, SIGNAL(clicked()),
+    SLOT(slotCheckPopCapabilities()));
+  buttonLay->addStretch();
+  buttonLay->addWidget( mPop.checkCapabilities );
+  buttonLay->addStretch();
+
+  vlay->addSpacing( KDialog::spacingHint() );
 
   mPop.encryptionGroup = new QButtonGroup( 1, Qt::Horizontal,
     i18n("Encryption"), page2 );
@@ -730,18 +742,24 @@ void AccountDialog::makePopAccountPage()
     mPop.authGroup, "auth plain"  );
   mPop.authCRAM_MD5 = new QRadioButton( i18n("CRAM-MD&5"), mPop.authGroup, "auth cram-md5" );
   mPop.authDigestMd5 = new QRadioButton( i18n("&DIGEST-MD5"), mPop.authGroup, "auth digest-md5" );
+  mPop.authNTLM = new QRadioButton( i18n("&NTLM"), mPop.authGroup, "auth ntlm" );
+  mPop.authGSSAPI = new QRadioButton( i18n("&GSSAPI"), mPop.authGroup, "auth gssapi" );
+  if ( KProtocolInfo::capabilities("pop3").contains("SASL") == 0 )
+  {
+    mPop.authNTLM->hide();
+    mPop.authGSSAPI->hide();
+  }
   mPop.authAPOP = new QRadioButton( i18n("&APOP"), mPop.authGroup, "auth apop" );
+
   vlay->addWidget( mPop.authGroup );
 
-  vlay->addStretch();
+  mPop.usePipeliningCheck =
+    new QCheckBox( i18n("&Use pipelining for faster mail download"), page2 );
+  connect(mPop.usePipeliningCheck, SIGNAL(clicked()),
+    SLOT(slotPipeliningClicked()));
+  vlay->addWidget( mPop.usePipeliningCheck );
 
-  QHBoxLayout *buttonLay = new QHBoxLayout( vlay );
-  mPop.checkCapabilities =
-    new QPushButton( i18n("Check &What the Server Supports"), page2 );
-  connect(mPop.checkCapabilities, SIGNAL(clicked()),
-    SLOT(slotCheckPopCapabilities()));
-  buttonLay->addStretch();
-  buttonLay->addWidget( mPop.checkCapabilities );
+  vlay->addStretch();
 
   connect(kapp,SIGNAL(kdisplayFontChanged()),SLOT(slotFontChanged()));
 }
@@ -771,13 +789,13 @@ void AccountDialog::makeImapAccountPage( bool connected )
   tabWidget->addTab( page1, i18n("&General") );
 
   int row = -1;
-  QGridLayout *grid = new QGridLayout( page1, 15, 2, marginHint(), spacingHint() );
-  grid->addColSpacing( 1, fontMetrics().maxWidth()*15 );
-  grid->setRowStretch( 15, 10 );
-  grid->setColStretch( 1, 10 );
+  QGridLayout *grid = new QGridLayout( page1, 16, 2, marginHint(), spacingHint() );
+  grid->addColSpacing( 1, fontMetrics().maxWidth()*16 );
+//  grid->setRowStretch( 15, 10 );
+//  grid->setColStretch( 1, 10 );
 
   ++row;
-  QLabel *label = new QLabel( i18n("&Name:"), page1 );
+  QLabel *label = new QLabel( i18n("Account &name:"), page1 );
   grid->addWidget( label, row, 0 );
   mImap.nameEdit = new KLineEdit( page1 );
   label->setBuddy( mImap.nameEdit );
@@ -826,7 +844,17 @@ void AccountDialog::makeImapAccountPage( bool connected )
 
   ++row;
   mImap.storePasswordCheck =
-    new QCheckBox( i18n("Sto&re IMAP password in configuration file"), page1 );
+    new QCheckBox( i18n("Sto&re IMAP password"), page1 );
+  QWhatsThis::add( mImap.storePasswordCheck,
+                   i18n("Check this option to have KMail store "
+                   "the password.\nIf KWallet is available "
+                   "the password will be stored there which is considered "
+                   "safe.\nHowever, if KWallet is not available, "
+                   "the password will be stored in KMail's configuration "
+                   "file. The password is stored in an "
+                   "obfuscated format, but should not be "
+                   "considered secure from decryption efforts "
+                   "if access to the configuration file is obtained.") );
   grid->addMultiCellWidget( mImap.storePasswordCheck, row, row, 0, 1 );
 
   if( !connected ) {
@@ -840,11 +868,6 @@ void AccountDialog::makeImapAccountPage( bool connected )
   mImap.hiddenFoldersCheck = new QCheckBox( i18n("Sho&w hidden folders"), page1);
   grid->addMultiCellWidget( mImap.hiddenFoldersCheck, row, row, 0, 1 );
 
-  if( connected ) {
-    ++row;
-    mImap.progressDialogCheck = new QCheckBox( i18n("Show &progress window"), page1);
-    grid->addMultiCellWidget( mImap.progressDialogCheck, row, row, 0, 1 );
-  }
 
   ++row;
   mImap.subscribedFoldersCheck = new QCheckBox(
@@ -871,8 +894,8 @@ void AccountDialog::makeImapAccountPage( bool connected )
     grid->addMultiCellWidget( mImap.listOnlyOpenCheck, row, row, 0, 1 );
   }
 
-  ++row;
 #if 0
+  ++row;
   QHBox* resourceHB = new QHBox( page1 );
   resourceHB->setSpacing( 11 );
   mImap.resourceCheck =
@@ -899,9 +922,9 @@ void AccountDialog::makeImapAccountPage( bool connected )
 #endif
 
   ++row;
-  mImap.excludeCheck =
-    new QCheckBox( i18n("E&xclude from \"Check Mail\""), page1 );
-  grid->addMultiCellWidget( mImap.excludeCheck, row, row, 0, 1 );
+  mImap.includeInCheck =
+    new QCheckBox( i18n("Include in manual mail chec&k"), page1 );
+  grid->addMultiCellWidget( mImap.includeInCheck, row, row, 0, 1 );
 
   ++row;
   mImap.intervalCheck =
@@ -920,15 +943,31 @@ void AccountDialog::makeImapAccountPage( bool connected )
   grid->addWidget( mImap.intervalSpin, row, 1 );
 
   ++row;
-  mImap.trashCombo = new KMFolderComboBox( page1 );
-  mImap.trashCombo->showOutboxFolder( FALSE );
+  label = new QLabel( i18n("&Trash folder:"), page1 );
+  grid->addWidget( label, row, 0 );
+  mImap.trashCombo = new FolderRequester( page1,
+      kmkernel->getKMMainWidget()->folderTree() );
+  mImap.trashCombo->setShowOutbox( false );
+  label->setBuddy( mImap.trashCombo );
   grid->addWidget( mImap.trashCombo, row, 1 );
-  grid->addWidget( new QLabel( mImap.trashCombo, i18n("&Trash folder:"), page1 ), row, 0 );
 
   QWidget *page2 = new QWidget( tabWidget );
   tabWidget->addTab( page2, i18n("S&ecurity") );
   QVBoxLayout *vlay = new QVBoxLayout( page2, marginHint(), spacingHint() );
 
+  vlay->addSpacing( KDialog::spacingHint() );
+
+  QHBoxLayout *buttonLay = new QHBoxLayout( vlay );
+  mImap.checkCapabilities =
+    new QPushButton( i18n("Check &What the Server Supports"), page2 );
+  connect(mImap.checkCapabilities, SIGNAL(clicked()),
+    SLOT(slotCheckImapCapabilities()));
+  buttonLay->addStretch();
+  buttonLay->addWidget( mImap.checkCapabilities );
+  buttonLay->addStretch();
+
+  vlay->addSpacing( KDialog::spacingHint() );
+
   mImap.encryptionGroup = new QButtonGroup( 1, Qt::Horizontal,
     i18n("Encryption"), page2 );
   mImap.encryptionNone =
@@ -954,19 +993,13 @@ void AccountDialog::makeImapAccountPage( bool connected )
      mImap.authGroup );
   mImap.authCramMd5 = new QRadioButton( i18n("CRAM-MD&5"), mImap.authGroup );
   mImap.authDigestMd5 = new QRadioButton( i18n("&DIGEST-MD5"), mImap.authGroup );
+  mImap.authNTLM = new QRadioButton( i18n("&NTLM"), mImap.authGroup );
+  mImap.authGSSAPI = new QRadioButton( i18n("&GSSAPI"), mImap.authGroup );
   mImap.authAnonymous = new QRadioButton( i18n("&Anonymous"), mImap.authGroup );
   vlay->addWidget( mImap.authGroup );
 
   vlay->addStretch();
 
-  QHBoxLayout *buttonLay = new QHBoxLayout( vlay );
-  mImap.checkCapabilities =
-    new QPushButton( i18n("Check &What the Server Supports"), page2 );
-  connect(mImap.checkCapabilities, SIGNAL(clicked()),
-    SLOT(slotCheckImapCapabilities()));
-  buttonLay->addStretch();
-  buttonLay->addWidget( mImap.checkCapabilities );
-
   // TODO (marc/bo): Test this
   mSieveConfigEditor = new SieveConfigEditor( tabWidget );
   mSieveConfigEditor->layout()->setMargin( KDialog::marginHint() );
@@ -1017,7 +1050,7 @@ void AccountDialog::setupSettings()
 #if 0
     mLocal.resourceCheck->setChecked( mAccount->resource() );
 #endif
-    mLocal.excludeCheck->setChecked( mAccount->checkExclude() );
+    mLocal.includeInCheck->setChecked( !mAccount->checkExclude() );
     mLocal.precommand->setText( mAccount->precommand() );
 
     slotEnableLocalInterval( interval >= 1 );
@@ -1042,7 +1075,7 @@ void AccountDialog::setupSettings()
 #if 0
     mPop.resourceCheck->setChecked( mAccount->resource() );
 #endif
-    mPop.excludeCheck->setChecked( mAccount->checkExclude() );
+    mPop.includeInCheck->setChecked( !mAccount->checkExclude() );
     mPop.precommand->setText( ap.precommand() );
     if (ap.useSSL())
       mPop.encryptionSSL->setChecked( TRUE );
@@ -1057,6 +1090,10 @@ void AccountDialog::setupSettings()
       mPop.authCRAM_MD5->setChecked( TRUE );
     else if (ap.auth() == "DIGEST-MD5")
       mPop.authDigestMd5->setChecked( TRUE );
+    else if (ap.auth() == "NTLM")
+      mPop.authNTLM->setChecked( TRUE );
+    else if (ap.auth() == "GSSAPI")
+      mPop.authGSSAPI->setChecked( TRUE );
     else if (ap.auth() == "APOP")
       mPop.authAPOP->setChecked( TRUE );
     else mPop.authUser->setChecked( TRUE );
@@ -1089,7 +1126,7 @@ void AccountDialog::setupSettings()
 #if 0
     mImap.resourceCheck->setChecked( ai.resource() );
 #endif
-    mImap.excludeCheck->setChecked( ai.checkExclude() );
+    mImap.includeInCheck->setChecked( !ai.checkExclude() );
     mImap.intervalCheck->setChecked( interval >= 1 );
     mImap.intervalSpin->setValue( QMAX(1, interval) );
     QString trashfolder = ai.trash();
@@ -1106,6 +1143,10 @@ void AccountDialog::setupSettings()
       mImap.authCramMd5->setChecked( TRUE );
     else if (ai.auth() == "DIGEST-MD5")
       mImap.authDigestMd5->setChecked( TRUE );
+    else if (ai.auth() == "NTLM")
+      mImap.authNTLM->setChecked( TRUE );
+    else if (ai.auth() == "GSSAPI")
+      mImap.authGSSAPI->setChecked( TRUE );
     else if (ai.auth() == "ANONYMOUS")
       mImap.authAnonymous->setChecked( TRUE );
     else if (ai.auth() == "PLAIN")
@@ -1130,7 +1171,6 @@ void AccountDialog::setupSettings()
     if (!prefix.isEmpty() && prefix[prefix.length() - 1] == '/')
       prefix = prefix.left(prefix.length() - 1);
     mImap.prefixEdit->setText( prefix );
-    mImap.progressDialogCheck->setChecked( ai.isProgressDialogEnabled() );
 #if 0
     mImap.resourceCheck->setChecked( ai.resource() );
 #endif
@@ -1139,7 +1179,7 @@ void AccountDialog::setupSettings()
     mImap.storePasswordCheck->setChecked( ai.storePasswd() );
     mImap.intervalCheck->setChecked( interval >= 1 );
     mImap.intervalSpin->setValue( QMAX(1, interval) );
-    mImap.excludeCheck->setChecked( ai.checkExclude() );
+    mImap.includeInCheck->setChecked( !ai.checkExclude() );
     mImap.intervalCheck->setChecked( interval >= 1 );
     mImap.intervalSpin->setValue( QMAX(1, interval) );
     QString trashfolder = ai.trash();
@@ -1156,6 +1196,10 @@ void AccountDialog::setupSettings()
       mImap.authCramMd5->setChecked( TRUE );
     else if (ai.auth() == "DIGEST-MD5")
       mImap.authDigestMd5->setChecked( TRUE );
+    else if (ai.auth() == "GSSAPI")
+      mImap.authGSSAPI->setChecked( TRUE );
+    else if (ai.auth() == "NTLM")
+      mImap.authNTLM->setChecked( TRUE );
     else if (ai.auth() == "ANONYMOUS")
       mImap.authAnonymous->setChecked( TRUE );
     else if (ai.auth() == "PLAIN")
@@ -1179,7 +1223,7 @@ void AccountDialog::setupSettings()
 #if 0
     mMaildir.resourceCheck->setChecked( mAccount->resource() );
 #endif
-    mMaildir.excludeCheck->setChecked( mAccount->checkExclude() );
+    mMaildir.includeInCheck->setChecked( !mAccount->checkExclude() );
     mMaildir.precommand->setText( mAccount->precommand() );
 
     slotEnableMaildirInterval( interval >= 1 );
@@ -1354,6 +1398,7 @@ void AccountDialog::slotCheckImapCapabilities()
 unsigned int AccountDialog::popCapabilitiesFromStringList( const QStringList & l )
 {
   unsigned int capa = 0;
+  kdDebug( 5006 ) << k_funcinfo << l << endl;
   for ( QStringList::const_iterator it = l.begin() ; it != l.end() ; ++it ) {
     QString cur = (*it).upper();
     if ( cur == "PLAIN" )
@@ -1364,6 +1409,10 @@ unsigned int AccountDialog::popCapabilitiesFromStringList( const QStringList & l
       capa |= CRAM_MD5;
     else if ( cur == "DIGEST-MD5" )
       capa |= Digest_MD5;
+    else if ( cur == "NTLM" )
+      capa |= NTLM;
+    else if ( cur == "GSSAPI" )
+      capa |= GSSAPI;
     else if ( cur == "APOP" )
       capa |= APOP;
     else if ( cur == "PIPELINING" )
@@ -1408,6 +1457,8 @@ void AccountDialog::enablePopFeatures( unsigned int capa )
   mPop.authLogin->setEnabled( capa & Login );
   mPop.authCRAM_MD5->setEnabled( capa & CRAM_MD5 );
   mPop.authDigestMd5->setEnabled( capa & Digest_MD5 );
+  mPop.authNTLM->setEnabled( capa & NTLM );
+  mPop.authGSSAPI->setEnabled( capa & GSSAPI );
   mPop.authAPOP->setEnabled( capa & APOP );
   if ( !( capa & Pipelining ) && mPop.usePipeliningCheck->isChecked() ) {
     mPop.usePipeliningCheck->setChecked( false );
@@ -1470,6 +1521,10 @@ unsigned int AccountDialog::imapCapabilitiesFromStringList( const QStringList &
       capa |= CRAM_MD5;
     else if ( cur == "AUTH=DIGEST-MD5" )
       capa |= Digest_MD5;
+    else if ( cur == "AUTH=NTLM" )
+      capa |= NTLM;
+    else if ( cur == "AUTH=GSSAPI" )
+      capa |= GSSAPI;
     else if ( cur == "AUTH=ANONYMOUS" )
       capa |= Anonymous;
     else if ( cur == "STARTTLS" )
@@ -1508,6 +1563,8 @@ void AccountDialog::enableImapAuthMethods( unsigned int capa )
   mImap.authLogin->setEnabled( capa & Login );
   mImap.authCramMd5->setEnabled( capa & CRAM_MD5 );
   mImap.authDigestMd5->setEnabled( capa & Digest_MD5 );
+  mImap.authNTLM->setEnabled( capa & NTLM );
+  mImap.authGSSAPI->setEnabled( capa & GSSAPI );
   mImap.authAnonymous->setEnabled( capa & Anonymous );
 }
 
@@ -1560,7 +1617,7 @@ void AccountDialog::saveSettings()
 #if 0
     mAccount->setResource( mLocal.resourceCheck->isChecked() );
 #endif
-    mAccount->setCheckExclude( mLocal.excludeCheck->isChecked() );
+    mAccount->setCheckExclude( !mLocal.includeInCheck->isChecked() );
 
     mAccount->setPrecommand( mLocal.precommand->text() );
 
@@ -1575,7 +1632,7 @@ void AccountDialog::saveSettings()
 #if 0
     mAccount->setResource( mPop.resourceCheck->isChecked() );
 #endif
-    mAccount->setCheckExclude( mPop.excludeCheck->isChecked() );
+    mAccount->setCheckExclude( !mPop.includeInCheck->isChecked() );
 
     mAccount->setFolder( *mFolderList.at(mPop.folderCombo->currentItem()) );
 
@@ -1603,6 +1660,10 @@ void AccountDialog::saveSettings()
       epa.setAuth("CRAM-MD5");
     else if (mPop.authDigestMd5->isChecked())
       epa.setAuth("DIGEST-MD5");
+    else if (mPop.authNTLM->isChecked())
+      epa.setAuth("NTLM");
+    else if (mPop.authGSSAPI->isChecked())
+      epa.setAuth("GSSAPI");
     else if (mPop.authAPOP->isChecked())
       epa.setAuth("APOP");
     else epa.setAuth("AUTO");
@@ -1615,7 +1676,7 @@ void AccountDialog::saveSettings()
 #if 0
     mAccount->setResource( mImap.resourceCheck->isChecked() );
 #endif
-    mAccount->setCheckExclude( mImap.excludeCheck->isChecked() );
+    mAccount->setCheckExclude( !mImap.includeInCheck->isChecked() );
     mAccount->setFolder( kmkernel->imapFolderMgr()->findById(mAccount->id()) );
 
     KMAcctImap &epa = *(KMAcctImap*)mAccount;
@@ -1632,21 +1693,25 @@ void AccountDialog::saveSettings()
     epa.setListOnlyOpenFolders( mImap.listOnlyOpenCheck->isChecked() );
     epa.setStorePasswd( mImap.storePasswordCheck->isChecked() );
     epa.setPasswd( mImap.passwordEdit->text(), epa.storePasswd() );
-    KMFolder *t = mImap.trashCombo->getFolder();
+    KMFolder *t = mImap.trashCombo->folder();
     if ( t )
-      epa.setTrash( mImap.trashCombo->getFolder()->idString() );
+      epa.setTrash( mImap.trashCombo->folder()->idString() );
     else
       epa.setTrash( kmkernel->trashFolder()->idString() );
 #if 0
     epa.setResource( mImap.resourceCheck->isChecked() );
 #endif
-    epa.setCheckExclude( mImap.excludeCheck->isChecked() );
+    epa.setCheckExclude( !mImap.includeInCheck->isChecked() );
     epa.setUseSSL( mImap.encryptionSSL->isChecked() );
     epa.setUseTLS( mImap.encryptionTLS->isChecked() );
     if (mImap.authCramMd5->isChecked())
       epa.setAuth("CRAM-MD5");
     else if (mImap.authDigestMd5->isChecked())
       epa.setAuth("DIGEST-MD5");
+    else if (mImap.authNTLM->isChecked())
+      epa.setAuth("NTLM");
+    else if (mImap.authGSSAPI->isChecked())
+      epa.setAuth("GSSAPI");
     else if (mImap.authAnonymous->isChecked())
       epa.setAuth("ANONYMOUS");
     else if (mImap.authLogin->isChecked())
@@ -1665,7 +1730,7 @@ void AccountDialog::saveSettings()
 #if 0
     mAccount->setResource( mImap.resourceCheck->isChecked() );
 #endif
-    mAccount->setCheckExclude( mImap.excludeCheck->isChecked() );
+    mAccount->setCheckExclude( !mImap.includeInCheck->isChecked() );
     //mAccount->setFolder( NULL );
     mAccount->setFolder( kmkernel->dimapFolderMgr()->findById(mAccount->id()) );
     kdDebug(5006) << mAccount->name() << endl;
@@ -1678,26 +1743,29 @@ void AccountDialog::saveSettings()
     if (prefix[prefix.length() - 1] != '/') prefix += "/";
     epa.setPrefix( prefix );
     epa.setLogin( mImap.loginEdit->text().stripWhiteSpace() );
-    epa.setProgressDialogEnabled( mImap.progressDialogCheck->isChecked() );
     epa.setHiddenFolders( mImap.hiddenFoldersCheck->isChecked() );
     epa.setOnlySubscribedFolders( mImap.subscribedFoldersCheck->isChecked() );
     epa.setStorePasswd( mImap.storePasswordCheck->isChecked() );
     epa.setPasswd( mImap.passwordEdit->text(), epa.storePasswd() );
-    KMFolder *t = mImap.trashCombo->getFolder();
+    KMFolder *t = mImap.trashCombo->folder();
     if ( t )
-      epa.setTrash( mImap.trashCombo->getFolder()->idString() );
+      epa.setTrash( mImap.trashCombo->folder()->idString() );
     else
       epa.setTrash( kmkernel->trashFolder()->idString() );
 #if 0
     epa.setResource( mImap.resourceCheck->isChecked() );
 #endif
-    epa.setCheckExclude( mImap.excludeCheck->isChecked() );
+    epa.setCheckExclude( !mImap.includeInCheck->isChecked() );
     epa.setUseSSL( mImap.encryptionSSL->isChecked() );
     epa.setUseTLS( mImap.encryptionTLS->isChecked() );
     if (mImap.authCramMd5->isChecked())
       epa.setAuth("CRAM-MD5");
     else if (mImap.authDigestMd5->isChecked())
       epa.setAuth("DIGEST-MD5");
+    else if (mImap.authNTLM->isChecked())
+      epa.setAuth("NTLM");
+    else if (mImap.authGSSAPI->isChecked())
+      epa.setAuth("GSSAPI");
     else if (mImap.authAnonymous->isChecked())
       epa.setAuth("ANONYMOUS");
     else if (mImap.authLogin->isChecked())
@@ -1732,7 +1800,7 @@ void AccountDialog::saveSettings()
 #if 0
     mAccount->setResource( mMaildir.resourceCheck->isChecked() );
 #endif
-    mAccount->setCheckExclude( mMaildir.excludeCheck->isChecked() );
+    mAccount->setCheckExclude( !mMaildir.includeInCheck->isChecked() );
 
     mAccount->setPrecommand( mMaildir.precommand->text() );
   }
