@@ -24,7 +24,7 @@
 #define _ACCOUNT_DIALOG_H_
 
 #include <kdialogbase.h>
-#include "kmfoldercombobox.h"
+#include <qguardedptr.h>
 
 class QRegExpValidator;
 class QCheckBox;
@@ -40,6 +40,7 @@ class KMServerTest;
 class QButtonGroup;
 namespace KMail {
   class SieveConfigEditor;
+  class FolderRequester;
 }
 
 class AccountDialog : public KDialogBase
@@ -68,7 +69,7 @@ class AccountDialog : public KDialogBase
       QPushButton  *resourceClearButton;
       QPushButton  *resourceClearPastButton;
 #endif
-      QCheckBox    *excludeCheck;
+      QCheckBox    *includeInCheck;
       QCheckBox    *intervalCheck;
       QLabel       *intervalLabel;
       KIntNumInput *intervalSpin;
@@ -87,7 +88,7 @@ class AccountDialog : public KDialogBase
       QPushButton  *resourceClearButton;
       QPushButton  *resourceClearPastButton;
 #endif
-      QCheckBox    *excludeCheck;
+      QCheckBox    *includeInCheck;
       QCheckBox    *intervalCheck;
       QLabel       *intervalLabel;
       KIntNumInput *intervalSpin;
@@ -114,7 +115,10 @@ class AccountDialog : public KDialogBase
       QRadioButton *authLogin;
       QRadioButton *authCRAM_MD5;
       QRadioButton *authDigestMd5;
+      QRadioButton *authNTLM;
+      QRadioButton *authGSSAPI;
       QRadioButton *authAPOP;
+
       QPushButton  *checkCapabilities;
       QCheckBox    *usePipeliningCheck;
       QCheckBox    *storePasswordCheck;
@@ -124,7 +128,7 @@ class AccountDialog : public KDialogBase
       QPushButton  *resourceClearButton;
       QPushButton  *resourceClearPastButton;
 #endif
-      QCheckBox    *excludeCheck;
+      QCheckBox    *includeInCheck;
       QCheckBox    *intervalCheck;
       QCheckBox    *filterOnServerCheck;
       QLabel       *intervalLabel;
@@ -154,7 +158,7 @@ class AccountDialog : public KDialogBase
       QCheckBox    *loadOnDemandCheck;
       QCheckBox    *storePasswordCheck;
       QCheckBox    *progressDialogCheck;  // only used by Disconnected IMAP
-      QCheckBox    *excludeCheck;
+      QCheckBox    *includeInCheck;
       QCheckBox    *intervalCheck;
       QCheckBox    *listOnlyOpenCheck;
       QLabel       *intervalLabel;
@@ -169,9 +173,11 @@ class AccountDialog : public KDialogBase
       QRadioButton *authLogin;
       QRadioButton *authCramMd5;
       QRadioButton *authDigestMd5;
+      QRadioButton *authGSSAPI;
+      QRadioButton *authNTLM;
       QRadioButton *authAnonymous;
       QPushButton  *checkCapabilities;
-      KMFolderComboBox  *trashCombo;
+      KMail::FolderRequester *trashCombo;
     };
 
   private slots:
@@ -237,6 +243,8 @@ class AccountDialog : public KDialogBase
       UIDL       = 256,
       STLS       = 512, // TLS for POP
       STARTTLS   = 512, // TLS for IMAP
+      GSSAPI     = 1024,
+      NTLM       = 2048,
       AllCapa    = 0xffffffff
     };
     unsigned int mCurCapa;
