@@ -40,6 +40,7 @@
 #include "kmfoldermgr.h"
 #include "kmfolder.h"
 #include "kmfoldercachedimap.h"
+#include "kmailicalifaceimpl.h"
 #include "kmacctcachedimap.h"
 #include "kmmsgdict.h"
 #include "maildirjob.h"
@@ -422,7 +423,7 @@ void CachedImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
-  if (data.find("UID") != -1)
+  if ( data.find("UID") != -1 && mMsg )
   {
     int uid = (data.right(data.length()-4)).toInt();
     kdDebug( 5006 ) << k_funcinfo << "Server told us uid is: " << uid << endl;
@@ -467,9 +468,15 @@ void CachedImapJob::slotPutMessageResult(KIO::Job *job)
      if ( mMsg->UID() == 0 ) {
         mFolder->removeMsg(i);
      } else {
+        // When removing+readding, no point in telling the imap resources about it
+        bool b = kmkernel->iCalIface().isResourceQuiet();
+        kmkernel->iCalIface().setResourceQuiet( true );
+
         mFolder->take( i );
         mFolder->addMsgKeepUID( mMsg );
         mMsg->setTransferInProgress( false );
+
+        kmkernel->iCalIface().setResourceQuiet( b );
      }
   }
   mMsg = NULL;
@@ -514,7 +521,7 @@ void CachedImapJob::slotAddNextSubfolder( KIO::Job * job )
   KMFolderCachedImap *folder = mFolderList.front();
   mFolderList.pop_front();
   KURL url = mAccount->getUrl();
-  url.setPath(mFolder->imapPath() + folder->name());
+  url.setPath(mFolder->imapPath() + folder->folder()->name());
 
   // Associate the jobData with the parent folder, not with the child
   // This is necessary in case of an error while creating the subfolder,
