@@ -12,9 +12,13 @@
 #include <qstring.h>
 #include <qvaluelist.h>
 #include <qstringlist.h>
+#include <dcopobject.h>
 
 #include <kdialogbase.h>
 #include <klistview.h>
+#include <kcmodule.h>
+#include <klocale.h>
+#include <kdepimmacros.h>
 
 class QPushButton;
 class QLabel;
@@ -30,7 +34,6 @@ class QListBox;
 class QButtonGroup;
 class QRegExpValidator;
 class QVBox;
-class KMFolderComboBox;
 class KMAccount;
 class KMTransportInfo;
 class ListView;
@@ -39,13 +42,23 @@ class KIntSpinBox;
 class SimpleStringListEditor;
 class KConfig;
 class QPoint;
-class CryptPlugWrapperList;
+class ComposerCryptoConfiguration;
+class WarningConfiguration;
+class SMimeConfiguration;
+
 namespace Kpgp {
   class Config;
 }
 namespace KMail {
   class IdentityDialog;
   class IdentityListView;
+  class AccountComboBox;
+  class FolderRequester;
+}
+namespace Kleo {
+  class BackendConfigWidget;
+  class CryptoConfig;
+  class CryptoConfigEntry;
 }
 
 class NewIdentityDialog : public KDialogBase
@@ -56,7 +69,7 @@ public:
   enum DuplicateMode { Empty, ControlCenter, ExistingEntry };
 
   NewIdentityDialog( const QStringList & identities,
-		     QWidget *parent=0, const char *name=0, bool modal=true );
+                     QWidget *parent=0, const char *name=0, bool modal=true );
 
   QString identityName() const { return mLineEdit->text(); }
   QString duplicateIdentity() const { return mComboBox->currentText(); }
@@ -80,11 +93,11 @@ private:
 
 struct LanguageItem
 {
-  LanguageItem() {};
+  LanguageItem() {}
   LanguageItem( const QString & language, const QString & reply=QString::null,
-		const QString & replyAll=QString::null,
-		const QString & forward=QString::null,
-		const QString & indentPrefix=QString::null ) :
+                const QString & replyAll=QString::null,
+                const QString & forward=QString::null,
+                const QString & indentPrefix=QString::null ) :
     mLanguage( language ), mReply( reply ), mReplyAll( replyAll ),
     mForward( forward ), mIndentPrefix( indentPrefix ) {}
 
@@ -99,7 +112,7 @@ class NewLanguageDialog : public KDialogBase
 
   public:
     NewLanguageDialog( LanguageItemList & suppressedLangs, QWidget *parent=0,
-		       const char *name=0, bool modal=true );
+                       const char *name=0, bool modal=true );
     QString language() const;
 
   private:
@@ -133,7 +146,7 @@ signals:
   void profileSelected( KConfig * profile );
 
 private slots:
-  void slotSelectionChanged( QListViewItem * );
+  void slotSelectionChanged();
   void slotOk();
 
 private:
@@ -144,101 +157,110 @@ private:
   QStringList mProfileList;
 };
 
-
-//
-//
-// basic ConfigurationPage (inherit pages from this)
-//
-//
-
-class ConfigurationPage : public QWidget {
+#include <kdialog.h>
+class ConfigModule : public KCModule {
   Q_OBJECT
 public:
-  ConfigurationPage( QWidget * parent=0, const char * name=0 )
-    : QWidget( parent, name ) {}
-  ~ConfigurationPage() {};
+  ConfigModule( QWidget * parent=0, const char * name=0 )
+     : KCModule ( parent, name )
+     {}
+  ~ConfigModule() {}
+
+  virtual void load() = 0;
+  virtual void save() = 0;
+  virtual void defaults() {}
 
   /** Should return the help anchor for this page or tab */
   virtual QString helpAnchor() const = 0;
 
-  /** Should set the page up (ie. read the setting from the @ref
-      KConfig object into the widgets) after creating it in the
-      constructor. Called from @ref ConfigureDialog. */
-  virtual void setup() = 0;
-  /** Called when the installation of a profile is
-      requested. Reimplemenations of this method should do the
-      equivalent of a @ref setup(), but with the given @ref KConfig
-      object instead of KMKernel::config() and only for those entries that
+signals:
+  /** Emitted when the installation of a profile is
+      requested. All connected kcms should load the values
+      from the profile only for those entries that
       really have keys defined in the profile.
+   */
+   void installProfile( KConfig * profile );
 
-      The default implementation does nothing.
-  */
-  virtual void installProfile( KConfig * /*profile*/ ) {};
-  /** Should apply the changed settings (ie. read the settings from
-      the widgets into the @ref KConfig object). Called from @ref
-      ConfigureDialog. */
-  virtual void apply() = 0;
-  /** Should cleanup any temporaries after cancel. The default
-      implementation does nothing. Called from @ref ConfigureDialog. */
-  virtual void dismiss() {}
-
-  void setPageIndex( int aPageIndex ) { mPageIndex = aPageIndex; }
-  int pageIndex() const { return mPageIndex; }
-protected:
-  int mPageIndex;
 };
 
-//
-//
-// TabbedConfigurationPage
-//
-//
 
-class TabbedConfigurationPage : public ConfigurationPage {
+// Individual tab of a ConfigModuleWithTabs
+class ConfigModuleTab : public QWidget {
   Q_OBJECT
 public:
-  TabbedConfigurationPage( QWidget * parent=0, const char * name=0 );
+   ConfigModuleTab( QWidget *parent=0, const char* name=0 )
+      :QWidget( parent, name )
+      {}
+  ~ConfigModuleTab() {}
+  void load();
+  virtual void save() = 0;
+  void defaults();
+  // the below are optional
+  virtual void installProfile(){}
+signals:
+   // forwarded to the ConfigModule
+  void changed(bool);
+public slots:
+  void slotEmitChanged();
+private:
+  // reimplement this for loading values of settings which are available
+  // via GlobalSettings
+  virtual void doLoadFromGlobalSettings() {}
+  // reimplement this for loading values of settings which are not available
+  // via GlobalSettings
+  virtual void doLoadOther() {}
+  // reimplement this for loading default values of settings which are
+  // not available via GlobalSettings (KConfigXT).
+  virtual void doResetToDefaultsOther() {}
+};
 
-  void setup();
-  void dismiss();
-  void installProfile( KConfig * profile );
-  void apply();
+
+/*
+ * ConfigModuleWithTabs represents a kcm with several tabs.
+ * It simply forwards load and save operations to all tabs.
+ */
+class ConfigModuleWithTabs : public ConfigModule {
+  Q_OBJECT
+public:
+  ConfigModuleWithTabs( QWidget * parent=0, const char * name=0 );
+   ~ConfigModuleWithTabs() {}
+
+  // don't reimplement any of those methods
+  virtual void load();
+  virtual void save();
+  virtual void defaults();
+  virtual void installProfile( KConfig * profile );
 
 protected:
-  void addTab( QWidget * tab, const QString & title );
+  void addTab( ConfigModuleTab* tab, const QString & title );
 
 private:
-  ConfigurationPage * configTab( int index, const char * debugMsg ) const;
-
   QTabWidget *mTabWidget;
 
 };
 
+
 //
 //
 // IdentityPage
 //
 //
 
-class IdentityPage : public ConfigurationPage {
+class KDE_EXPORT IdentityPage : public ConfigModule {
   Q_OBJECT
 public:
   IdentityPage( QWidget * parent=0, const char * name=0 );
-  ~IdentityPage() {};
+  ~IdentityPage() {}
 
-  static QString iconLabel();
-  static QString title();
-  static const char * iconName();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
-  void dismiss();
+  void load();
+  void save();
 
 public slots:
   void slotUpdateTransportCombo( const QStringList & );
 
-protected slots:
+private slots:
   void slotNewIdentity();
   void slotModifyIdentity();
   void slotRemoveIdentity();
@@ -246,16 +268,16 @@ protected slots:
       KListView::rename on the selected item */
   void slotRenameIdentity();
   /** connected to @p mIdentityList's renamed() signal. Validates the
-      new name and sets it in the @ref IdentityManager */
+      new name and sets it in the @see KPIM::IdentityManager */
   void slotRenameIdentity( QListViewItem *, const QString &, int );
   void slotContextMenu( KListView*, QListViewItem *, const QPoint & );
   void slotSetAsDefault();
-  void slotIdentitySelectionChanged( QListViewItem * );
+  void slotIdentitySelectionChanged();
 
-protected: // methods
+private: // methods
   void refreshList();
 
-protected: // data members
+private: // data members
   KMail::IdentityDialog   * mIdentityDialog;
   int            mOldNumberOfIdentities;
 
@@ -264,36 +286,27 @@ protected: // data members
   QPushButton             * mRenameButton;
   QPushButton             * mRemoveButton;
   QPushButton             * mSetAsDefaultButton;
-
 };
 
 
 //
 //
-// NetworkPage
+// AccountsPage
 //
 //
 
 // subclasses: one class per tab:
-class NetworkPageSendingTab : public ConfigurationPage {
+class AccountsPageSendingTab : public ConfigModuleTab {
   Q_OBJECT
 public:
-  NetworkPageSendingTab( QWidget * parent=0, const char * name=0 );
-
-  // no icon:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
+  AccountsPageSendingTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
-
-  void setup();
-  void apply();
+  void save();
 
 signals:
   void transportListChanged( const QStringList & );
 
-protected slots:
+private slots:
   void slotTransportSelected();
   void slotAddTransport();
   void slotModifySelectedTransport();
@@ -301,60 +314,57 @@ protected slots:
   void slotTransportUp();
   void slotTransportDown();
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   ListView    *mTransportList;
   QPushButton *mModifyTransportButton;
   QPushButton *mRemoveTransportButton;
   QPushButton *mTransportUpButton;
   QPushButton *mTransportDownButton;
   QCheckBox   *mConfirmSendCheck;
-  QCheckBox   *mSendOutboxCheck;
+  QComboBox   *mSendOnCheckCombo;
   QComboBox   *mSendMethodCombo;
   QComboBox   *mMessagePropertyCombo;
   QLineEdit   *mDefaultDomainEdit;
 
   QPtrList< KMTransportInfo > mTransportInfoList;
-
 };
 
 
-class NetworkPageReceivingTab : public ConfigurationPage {
+class AccountsPageReceivingTab : public ConfigModuleTab {
   Q_OBJECT
 public:
-  NetworkPageReceivingTab( QWidget * parent=0, const char * name=0 );
-
-  // no icon:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
+  AccountsPageReceivingTab( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
-
-  void setup();
-  void apply();
-  void dismiss(); // needed for account list cleanup
+  void save();
 
 signals:
   void accountListChanged( const QStringList & );
 
-protected slots:
+private slots:
   void slotAccountSelected();
   void slotAddAccount();
   void slotModifySelectedAccount();
   void slotRemoveSelectedAccount();
   void slotEditNotifications();
+  void slotTweakAccountList();
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
   QStringList occupiedNames();
 
-protected:
+private:
   ListView      *mAccountList;
   QPushButton   *mModifyAccountButton;
   QPushButton   *mRemoveAccountButton;
   QCheckBox     *mBeepNewMailCheck;
-  QCheckBox     *mSystrayCheck;
-  QRadioButton  *mBlinkingSystray;
-  QRadioButton  *mSystrayOnNew;
+  QCheckBox     *mVerboseNotificationCheck;
   QCheckBox     *mCheckmailStartupCheck;
   QPushButton   *mOtherNewMailActionsButton;
 
@@ -368,25 +378,22 @@ protected:
   QValueList< ModifiedAccountsType* >  mModifiedAccounts;
 };
 
-class NetworkPage : public TabbedConfigurationPage {
+class KDE_EXPORT AccountsPage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
-  NetworkPage( QWidget * parent=0, const char * name=0 );
-
-  static QString iconLabel();
-  static QString title();
-  static const char * iconName();
+  AccountsPage( QWidget * parent=0, const char * name=0 );
   QString helpAnchor() const;
 
+
   // hrmpf. moc doesn't like nested classes with slots/signals...:
-  typedef NetworkPageSendingTab SendingTab;
-  typedef NetworkPageReceivingTab ReceivingTab;
+  typedef AccountsPageSendingTab SendingTab;
+  typedef AccountsPageReceivingTab ReceivingTab;
 
 signals:
   void transportListChanged( const QStringList & );
   void accountListChanged( const QStringList & );
 
-protected:
+private:
   SendingTab   *mSendingTab;
   ReceivingTab *mReceivingTab;
 };
@@ -398,104 +405,93 @@ protected:
 //
 //
 
-class AppearancePageFontsTab : public ConfigurationPage {
+class AppearancePageFontsTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   AppearancePageFontsTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
+  void save();
 
-  void setup();
-  void apply();
   void installProfile( KConfig * profile );
 
-protected slots:
+private slots:
   void slotFontSelectorChanged( int );
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
   void updateFontSelector();
 
-protected:
+private:
   QCheckBox    *mCustomFontCheck;
   QComboBox    *mFontLocationCombo;
   KFontChooser *mFontChooser;
 
   int          mActiveFontIndex;
-  QFont        mFont[10];
+  QFont        mFont[13];
 };
 
-class AppearancePageColorsTab : public ConfigurationPage {
+class AppearancePageColorsTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   AppearancePageColorsTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
+  void save();
 
-  void setup();
-  void apply();
   void installProfile( KConfig * profile );
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   QCheckBox    *mCustomColorCheck;
   ColorListBox *mColorList;
   QCheckBox    *mRecycleColorCheck;
 };
 
-class AppearancePageLayoutTab : public ConfigurationPage {
+class AppearancePageLayoutTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   AppearancePageLayoutTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
   void installProfile( KConfig * profile );
 
-protected: // data
-  QCheckBox    *mShowColorbarCheck;
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private: // data
   QButtonGroup *mFolderListGroup;
   QButtonGroup *mMIMETreeLocationGroup;
   QButtonGroup *mMIMETreeModeGroup;
   QButtonGroup *mReaderWindowModeGroup;
 };
 
-class AppearancePageHeadersTab : public ConfigurationPage {
+class AppearancePageHeadersTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   AppearancePageHeadersTab( QWidget * parent=0, const char * name=0 );
 
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
   void installProfile( KConfig * profile );
 
-protected: // methods
+private: // methods
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
   void setDateDisplay( int id, const QString & format );
 
-protected: // data
+private: // data
   QCheckBox    *mMessageSizeCheck;
+  QCheckBox    *mAttachmentCheck;
   QCheckBox    *mNestedMessagesCheck;
   QCheckBox    *mCryptoIconsCheck;
   QButtonGroup *mNestingPolicy;
@@ -503,14 +499,54 @@ protected: // data
   QLineEdit    *mCustomDateFormatEdit;
 };
 
-class AppearancePage : public TabbedConfigurationPage {
+class AppearancePageReaderTab : public ConfigModuleTab {
+  Q_OBJECT
+public:
+  AppearancePageReaderTab( QWidget * parent=0, const char * name=0 );
+
+  QString helpAnchor() const;
+
+  void save();
+  void installProfile( KConfig * profile );
+
+private:
+  virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+  void readCurrentOverrideCodec();
+
+private: // data
+  QCheckBox *mShowColorbarCheck;
+  QCheckBox *mShowSpamStatusCheck;
+  QCheckBox *mShowEmoticonsCheck;
+  QComboBox *mCharsetCombo;
+  QComboBox *mOverrideCharsetCombo;
+};
+
+
+class AppearancePageSystemTrayTab : public ConfigModuleTab {
+  Q_OBJECT
+public:
+  AppearancePageSystemTrayTab( QWidget * parent=0, const char * name=0 );
+
+  QString helpAnchor() const;
+
+  void save();
+  void installProfile( KConfig * profile );
+
+private:
+  virtual void doLoadFromGlobalSettings();
+
+private: // data
+  QCheckBox    *mSystemTrayCheck;
+  QButtonGroup *mSystemTrayGroup;
+};
+
+class KDE_EXPORT AppearancePage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
   AppearancePage( QWidget * parent=0, const char * name=0 );
 
-  static QString iconLabel();
-  static QString title();
-  static const char * iconName();
   QString helpAnchor() const;
 
   // hrmpf. moc doesn't like nested classes with slots/signals...:
@@ -518,12 +554,16 @@ public:
   typedef AppearancePageColorsTab ColorsTab;
   typedef AppearancePageLayoutTab LayoutTab;
   typedef AppearancePageHeadersTab HeadersTab;
+  typedef AppearancePageReaderTab ReaderTab;
+  typedef AppearancePageSystemTrayTab SystemTrayTab;
 
-protected:
-  FontsTab   *mFontsTab;
-  ColorsTab  *mColorsTab;
-  LayoutTab  *mLayoutTab;
-  HeadersTab *mHeadersTab;
+private:
+  FontsTab      *mFontsTab;
+  ColorsTab     *mColorsTab;
+  LayoutTab     *mLayoutTab;
+  HeadersTab    *mHeadersTab;
+  ReaderTab     *mReaderTab;
+  SystemTrayTab *mSystemTrayTab;
 };
 
 //
@@ -532,58 +572,49 @@ protected:
 //
 //
 
-class ComposerPageGeneralTab : public ConfigurationPage {
+class ComposerPageGeneralTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   ComposerPageGeneralTab( QWidget * parent=0, const char * name=0 );
-
-  // no icon
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
   void installProfile( KConfig * profile );
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
+
+private:
   QCheckBox     *mAutoAppSignFileCheck;
   QCheckBox     *mSmartQuoteCheck;
   QCheckBox     *mAutoRequestMDNCheck;
   QCheckBox     *mWordWrapCheck;
   KIntSpinBox   *mWrapColumnSpin;
+  KIntSpinBox   *mAutoSave;
   QCheckBox     *mExternalEditorCheck;
   KURLRequester *mEditorRequester;
 };
 
-class ComposerPagePhrasesTab : public ConfigurationPage {
+class ComposerPagePhrasesTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   ComposerPagePhrasesTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
 
-protected slots:
+private slots:
   void slotNewLanguage();
   void slotRemoveLanguage();
   void slotLanguageChanged( const QString& );
   void slotAddNewLanguage( const QString& );
 
-protected:
+private:
+  virtual void doLoadFromGlobalSettings();
   void setLanguageItemInformation( int index );
   void saveActiveLanguageItem();
 
-protected:
+private:
   LanguageComboBox *mPhraseLanguageCombo;
   QPushButton      *mRemoveButton;
   QLineEdit        *mPhraseReplyEdit;
@@ -595,75 +626,66 @@ protected:
   LanguageItemList mLanguageList;
 };
 
-class ComposerPageSubjectTab : public ConfigurationPage {
+class ComposerPageSubjectTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   ComposerPageSubjectTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
 
+private:
+  virtual void doLoadFromGlobalSettings();
 
-protected:
+private:
   SimpleStringListEditor *mReplyListEditor;
   QCheckBox              *mReplaceReplyPrefixCheck;
   SimpleStringListEditor *mForwardListEditor;
   QCheckBox              *mReplaceForwardPrefixCheck;
 };
 
-class ComposerPageCharsetTab : public ConfigurationPage {
+class ComposerPageCharsetTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   ComposerPageCharsetTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
 
-protected slots:
+private slots:
   void slotVerifyCharset(QString&);
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   SimpleStringListEditor *mCharsetListEditor;
   QCheckBox              *mKeepReplyCharsetCheck;
 };
 
-class ComposerPageHeadersTab : public ConfigurationPage {
+class ComposerPageHeadersTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   ComposerPageHeadersTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
 
-protected slots:
+private slots:
   void slotMimeHeaderSelectionChanged();
   void slotMimeHeaderNameChanged( const QString & );
   void slotMimeHeaderValueChanged( const QString & );
   void slotNewMimeHeader();
   void slotRemoveMimeHeader();
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   QCheckBox   *mCreateOwnMessageIdCheck;
   QLineEdit   *mMessageIdSuffixEdit;
   QRegExpValidator *mMessageIdSuffixValidator;
@@ -675,38 +697,33 @@ protected:
   QLabel      *mTagValueLabel;
 };
 
-class ComposerPageAttachmentsTab : public ConfigurationPage {
+class ComposerPageAttachmentsTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   ComposerPageAttachmentsTab( QWidget * parent=0, const char * name=0 );
-
-  // no icon
-  static QString iconLabel() {
-    return QString::null;
-  }
-  static const char * iconName() {
-    return 0;
-  }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
 
-protected:
+private slots:
+  void slotOutlookCompatibleClicked();
+
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
+  QCheckBox   *mOutlookCompatibleCheck;
   QCheckBox   *mMissingAttachmentDetectionCheck;
   SimpleStringListEditor *mAttachWordsListEditor;
 };
 
-class ComposerPage : public TabbedConfigurationPage {
+class KDE_EXPORT ComposerPage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
   ComposerPage( QWidget * parent=0, const char * name=0 );
 
-  static QString iconLabel();
-  static QString title();
-  static const char * iconName();
   QString helpAnchor() const;
 
   // hrmpf. moc doesn't like nested classes with slots/signals...:
@@ -717,7 +734,7 @@ public:
   typedef ComposerPageHeadersTab HeadersTab;
   typedef ComposerPageAttachmentsTab AttachmentsTab;
 
-protected:
+private:
   GeneralTab  *mGeneralTab;
   PhrasesTab  *mPhrasesTab;
   SubjectTab  *mSubjectTab;
@@ -732,124 +749,138 @@ protected:
 //
 //
 
-class SecurityPageGeneralTab : public ConfigurationPage {
+class SecurityPageGeneralTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   SecurityPageGeneralTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
-
+  void save();
   void installProfile( KConfig * profile );
 
-protected:
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
   QCheckBox    *mExternalReferences;
   QCheckBox    *mHtmlMailCheck;
-  QCheckBox    *mSendReceivedReceiptCheck;
+  QCheckBox    *mNoMDNsWhenEncryptedCheck;
   QButtonGroup *mMDNGroup;
   QButtonGroup *mOrigQuoteGroup;
+  QCheckBox    *mAutomaticallyImportAttachedKeysCheck;
 };
 
 
-class SecurityPageOpenPgpTab : public ConfigurationPage {
+class SecurityPageComposerCryptoTab : public ConfigModuleTab {
   Q_OBJECT
 public:
-  SecurityPageOpenPgpTab( QWidget * parent=0, const char * name=0 );
+  SecurityPageComposerCryptoTab( QWidget * parent=0, const char * name=0 );
 
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+  void save();
   void installProfile( KConfig * profile );
 
-protected:
-  Kpgp::Config *mPgpConfig;
-  QCheckBox    *mPgpAutoSignatureCheck;
-  QCheckBox    *mPgpAutoEncryptCheck;
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
+  ComposerCryptoConfiguration* mWidget;
 };
 
+class SecurityPageWarningTab : public ConfigModuleTab {
+  Q_OBJECT
+public:
+  SecurityPageWarningTab( QWidget * parent=0, const char * name=0 );
 
-class SecurityPageCryptPlugTab : public ConfigurationPage
+  QString helpAnchor() const;
+
+  void save();
+  void installProfile( KConfig * profile );
+
+private slots:
+  void slotReenableAllWarningsClicked();
+
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
+  WarningConfiguration* mWidget;
+};
+
+class SecurityPageSMimeTab : public ConfigModuleTab, public DCOPObject {
+  Q_OBJECT
+  K_DCOP
+public:
+  SecurityPageSMimeTab( QWidget * parent=0, const char * name=0 );
+  ~SecurityPageSMimeTab();
+
+  QString helpAnchor() const;
+
+  // Can't use k_dcop here. dcopidl can't parse this file, dcopidlng has a namespace bug.
+  void save();
+  void installProfile( KConfig * profile );
+
+private slots:
+  void slotUpdateHTTPActions();
+
+private:
+  //virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
+
+private:
+  SMimeConfiguration* mWidget;
+  Kleo::CryptoConfig* mConfig;
+};
+
+class SecurityPageCryptPlugTab : public ConfigModuleTab
 {
   Q_OBJECT
 public:
   SecurityPageCryptPlugTab( QWidget * parent = 0, const char* name = 0 );
   ~SecurityPageCryptPlugTab();
 
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
-  void dismiss();
-  //void savePluginsConfig( bool silent );
-
-public slots:
-  void slotNewPlugIn();
-  void slotDeletePlugIn();
-  void slotActivatePlugIn();
-  void slotConfigurePlugIn();
-  void slotPlugNameChanged( const QString& );
-  void slotPlugLocationChanged( const QString& );
-  void slotPlugUpdateURLChanged( const QString& );
-
-protected slots:
-  void slotPlugSelectionChanged();
+  void save();
 
 private:
-  KListView     *mPlugList;
+  virtual void doLoadOther();
+  //virtual void doResetToDefaultsOther();
 
-  QPushButton   *mNewButton;
-  QPushButton   *mRemoveButton;
-  QPushButton   *mActivateButton;
-  QPushButton   *mConfigureButton;
-
-  QLineEdit     *mNameEdit;
-  KURLRequester *mLocationRequester;
-  QLineEdit     *mUpdateURLEdit;
-  CryptPlugWrapperList *mlistCryptoAdd;
+private:
+  Kleo::BackendConfigWidget * mBackendConfig;
 };
 
-class SecurityPage : public TabbedConfigurationPage {
+class KDE_EXPORT SecurityPage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
-  SecurityPage(	QWidget * parent=0, const char * name=0 );
+  SecurityPage( QWidget * parent=0, const char * name=0 );
 
-  static QString iconLabel();
-  static const char * iconName();
-  static QString title();
   QString helpAnchor() const;
 
   // OpenPGP tab is special:
-  void setup();
-  void apply();
-  void dismiss();
   void installProfile( KConfig * profile );
 
   typedef SecurityPageGeneralTab GeneralTab;
-  typedef SecurityPageOpenPgpTab OpenPgpTab;
+  typedef SecurityPageComposerCryptoTab ComposerCryptoTab;
+  typedef SecurityPageWarningTab WarningTab;
+  typedef SecurityPageSMimeTab SMimeTab;
   typedef SecurityPageCryptPlugTab CryptPlugTab;
 
-protected:
-  GeneralTab   *mGeneralTab;
-  OpenPgpTab   *mOpenPgpTab;
-  CryptPlugTab *mCryptPlugTab;
+private:
+  GeneralTab    *mGeneralTab;
+  ComposerCryptoTab *mComposerCryptoTab;
+  WarningTab    *mWarningTab;
+  SMimeTab      *mSMimeTab;
+  CryptPlugTab  *mCryptPlugTab;
 };
 
 
@@ -859,75 +890,75 @@ protected:
 //
 //
 
-class MiscPageFolderTab : public ConfigurationPage {
+class MiscPageFolderTab : public ConfigModuleTab {
   Q_OBJECT
 public:
   MiscPageFolderTab( QWidget * parent=0, const char * name=0 );
 
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
+  void save();
+ QString helpAnchor() const;
 
-  static QString title();
-  QString helpAnchor() const;
+private:
+  virtual void doLoadFromGlobalSettings();
+  virtual void doLoadOther();
+  //FIXME virtual void doResetToDefaultsOther();
 
-  void setup();
-  void apply();
-
-protected:
+private:
   QCheckBox    *mEmptyFolderConfirmCheck;
-  QCheckBox    *mWarnBeforeExpire;
+  QCheckBox    *mExcludeImportantFromExpiry;
   QComboBox    *mLoopOnGotoUnread;
-  QCheckBox    *mJumpToUnread;
   QComboBox    *mMailboxPrefCombo;
-  QCheckBox    *mCompactOnExitCheck;
+  QComboBox    *mActionEnterFolder;
   QCheckBox    *mEmptyTrashCheck;
-  QCheckBox    *mExpireAtExit;
   QCheckBox    *mDelayedMarkAsRead;
   KIntSpinBox  *mDelayedMarkTime;
   QCheckBox    *mShowPopupAfterDnD;
-  KMFolderComboBox *mOnStartupOpenFolder;
+  KMail::FolderRequester *mOnStartupOpenFolder;
 };
 
-class MiscPageGroupwareTab : public ConfigurationPage  {
+class MiscPageGroupwareTab : public ConfigModuleTab  {
   Q_OBJECT
 public:
   MiscPageGroupwareTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
+  void save();
   QString helpAnchor() const;
 
-  void setup();
-  void apply();
+private slots:
+  void slotStorageFormatChanged( int );
+  void slotLegacyBodyInvitesToggled( bool on );
+
+private:
+  virtual void doLoadFromGlobalSettings();
+
 private:
   QCheckBox* mEnableGwCB;
   QCheckBox* mEnableImapResCB;
 
-  QVBox* mBox;
+  QWidget* mBox;
   QVBox* gBox;
 
+  QComboBox* mStorageFormatCombo;
   QComboBox* mLanguageCombo;
-  KMFolderComboBox* mFolderCombo;
 
+  QLabel* mFolderComboLabel;
+  QWidgetStack* mFolderComboStack;
+  KMail::FolderRequester* mFolderCombo; // in the widgetstack
+  KMail::AccountComboBox* mAccountCombo; // in the widgetstack
+
+  QCheckBox* mHideGroupwareFolders;
   QCheckBox* mAutoResCB;
   QCheckBox* mAutoDeclConflCB;
   QCheckBox* mAutoDeclRecurCB;
 
   QCheckBox* mLegacyMangleFromTo;
+  QCheckBox* mLegacyBodyInvites;
+  QCheckBox* mAutomaticSending;
 };
 
-class MiscPage : public TabbedConfigurationPage {
+class KDE_EXPORT MiscPage : public ConfigModuleWithTabs {
   Q_OBJECT
 public:
   MiscPage( QWidget * parent=0, const char * name=0 );
-
-  static QString iconLabel();
-  static QString title();
-  static const char * iconName();
   QString helpAnchor() const;
 
   typedef MiscPageFolderTab FolderTab;
