@@ -63,7 +63,7 @@ namespace KMail {
     void readColorConfig();
 
     // returns CSS rules specific to the print media type
-    QString printCssDefinitions() const;
+    QString printCssDefinitions( bool fixed ) const;
 
     // returns CSS rules specific to the screen media type
     QString screenCssDefinitions( const CSSHelper * helper, bool fixed ) const;
@@ -72,7 +72,8 @@ namespace KMail {
     QString commonCssDefinitions() const;
 
     QFont bodyFont( bool fixed, bool print=false ) const {
-      return fixed ? mFixedFont : print ? mPrintFont : mBodyFont ;
+      return fixed ? ( print ? mFixedPrintFont : mFixedFont )
+                   : ( print ? mPrintFont : mBodyFont );
     }
     int fontSize( bool fixed, bool print=false ) const {
       return bodyFont( fixed, print ).pointSize();
@@ -81,7 +82,7 @@ namespace KMail {
     QString quoteFontTag( int level ) const;
 
   private:
-    QFont mBodyFont, mPrintFont, mFixedFont;
+    QFont mBodyFont, mPrintFont, mFixedFont, mFixedPrintFont;
     QFont mQuoteFont[3];
     QColor mQuoteColor[3];
     bool mRecycleQuoteColors;
@@ -101,12 +102,13 @@ namespace KMail {
   bool CSSHelper::Private::operator==( const Private & other ) const {
     for ( int i = 0 ; i < 3 ; ++i )
       if ( mQuoteFont[i] != other.mQuoteFont[i] ||
-	   mQuoteColor[i] != other.mQuoteColor[i] )
-	return false;
+           mQuoteColor[i] != other.mQuoteColor[i] )
+        return false;
     return // eeek!
       mBodyFont == other.mBodyFont &&
       mPrintFont == other.mPrintFont &&
       mFixedFont == other.mFixedFont &&
+      mFixedPrintFont == other.mFixedPrintFont &&
       mRecycleQuoteColors == other.mRecycleQuoteColors &&
       mBackingPixmapOn == other.mBackingPixmapOn &&
       mBackingPixmapStr == other.mBackingPixmapStr &&
@@ -182,8 +184,8 @@ namespace KMail {
       cPgpErrH  = reader.readColorEntry( "PGPMessageErr", &cPgpErrH );
       cHtmlWarning = reader.readColorEntry( "HTMLWarningColor", &cHtmlWarning );
       for ( int i = 0 ; i < 3 ; ++i ) {
-	const QString key = "QuotedText" + QString::number( i+1 );
-	mQuoteColor[i] = reader.readColorEntry( key, &mQuoteColor[i] );
+        const QString key = "QuotedText" + QString::number( i+1 );
+        mQuoteColor[i] = reader.readColorEntry( key, &mQuoteColor[i] );
       }
     }
 
@@ -237,20 +239,22 @@ namespace KMail {
     }
 
     QFont defaultFont = KGlobalSettings::generalFont();
+    QFont defaultFixedFont = KGlobalSettings::fixedFont();
     if ( fonts.readBoolEntry( "defaultFonts", true ) ) {
       mBodyFont = mPrintFont = defaultFont;
-      mFixedFont = KGlobalSettings::fixedFont();
+      mFixedFont = mFixedPrintFont = defaultFixedFont;
       defaultFont.setItalic( true );
       for ( int i = 0 ; i < 3 ; ++i )
-	mQuoteFont[i] = defaultFont;
+        mQuoteFont[i] = defaultFont;
     } else {
       mBodyFont = fonts.readFontEntry(  "body-font",  &defaultFont);
       mPrintFont = fonts.readFontEntry( "print-font", &defaultFont);
-      mFixedFont = fonts.readFontEntry( "fixed-font", &defaultFont);
+      mFixedFont = fonts.readFontEntry( "fixed-font", &defaultFixedFont);
+      mFixedPrintFont = mFixedFont; // FIXME when we have a separate fixed print font
       defaultFont.setItalic( true );
       for ( int i = 0 ; i < 3 ; ++i ) {
-	const QString key = QString( "quote%1-font" ).arg( i+1 );
-	mQuoteFont[i] = fonts.readFontEntry( key, &defaultFont );
+        const QString key = QString( "quote%1-font" ).arg( i+1 );
+        mQuoteFont[i] = fonts.readFontEntry( key, &defaultFont );
       }
     }
 
@@ -298,7 +302,7 @@ namespace KMail {
       "}\n"
       "@media print {\n\n"
       +
-      d->printCssDefinitions()
+      d->printCssDefinitions( fixed )
       +
       "}\n";
   }
@@ -307,8 +311,7 @@ namespace KMail {
     return
       "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
       "<html><head><title></title></head>\n"
-      +
-      QString( fixed ? "<body class\"fixedfont\">\n" : "<body>\n" );
+      "<body>\n";
   }
 
   QString CSSHelper::Private::quoteFontTag( int level ) const {
@@ -341,78 +344,81 @@ namespace KMail {
     }
   }
 
-  QString CSSHelper::Private::printCssDefinitions() const {
+  QString CSSHelper::Private::printCssDefinitions( bool fixed ) const {
     const QString headerFont = QString( "  font-family: \"%1\" ! important;\n"
-					"  font-size: %2pt ! important;\n" )
+                                        "  font-size: %2pt ! important;\n" )
                            .arg( mPrintFont.family() )
                            .arg( mPrintFont.pointSize() );
     const QColorGroup & cg = QApplication::palette().active();
 
+    const QFont printFont = bodyFont( fixed, true /* print */ );
     QString quoteCSS;
-    if ( mPrintFont.italic() )
+    if ( printFont.italic() )
       quoteCSS += "  font-style: italic ! important;\n";
-    if ( mPrintFont.bold() )
+    if ( printFont.bold() )
       quoteCSS += "  font-weight: bold ! important;\n";
     if ( !quoteCSS.isEmpty() )
       quoteCSS = "div.noquote {\n" + quoteCSS + "}\n\n";
 
     return
       QString( "body {\n"
-	       "  font-family: \"%1\" ! important;\n"
-	       "  font-size: %2pt ! important;\n"
-	       "  color: #000000 ! important;\n"
-	       "  background-color: #ffffff ! important\n"
-	       "}\n\n" )
-      .arg( mPrintFont.family(),
-	    QString::number( mPrintFont.pointSize() ) )
+               "  font-family: \"%1\" ! important;\n"
+               "  font-size: %2pt ! important;\n"
+               "  color: #000000 ! important;\n"
+               "  background-color: #ffffff ! important\n"
+               "}\n\n" )
+      .arg( printFont.family(),
+            QString::number( printFont.pointSize() ) )
       +
       QString( "tr.textAtmH,\n"
-	       "tr.rfc822H,\n"
-	       "tr.encrH,\n"
-	       "tr.signOkKeyOkH,\n"
-	       "tr.signOkKeyBadH,\n"
-	       "tr.signWarnH,\n"
-	       "tr.signErrH,\n"
-	       "div.header {\n"
-	       "%1"
-	       "}\n\n"
+               "tr.rfc822H,\n"
+               "tr.encrH,\n"
+               "tr.signOkKeyOkH,\n"
+               "tr.signOkKeyBadH,\n"
+               "tr.signWarnH,\n"
+               "tr.signErrH,\n"
+               "div.header {\n"
+               "%1"
+               "}\n\n"
 
-	       "div.fancy.header > div {\n"
-	       "  background-color: %2 ! important;\n"
-	       "  color: %3 ! important;\n"
-	       "  padding: 4px ! important;\n"
-	       "  border: solid %3 1px ! important;\n"
-	       "}\n\n"
+               "div.fancy.header > div {\n"
+               "  background-color: %2 ! important;\n"
+               "  color: %3 ! important;\n"
+               "  padding: 4px ! important;\n"
+               "  border: solid %3 1px ! important;\n"
+               "}\n\n"
 
-	       "div.fancy.header > div a[href] { color: %3 ! important; }\n\n"
+               "div.fancy.header > div a[href] { color: %3 ! important; }\n\n"
 
-	       "div.fancy.header > table.outer{\n"
-	       "  background-color: %2 ! important;\n"
-	       "  color: %3 ! important;\n"
-	       "  border-bottom: solid %3 1px ! important;\n"
-	       "  border-left: solid %3 1px ! important;\n"
-	       "  border-right: solid %3 1px ! important;\n"
-	       "}\n\n"
+               "div.fancy.header > table.outer{\n"
+               "  background-color: %2 ! important;\n"
+               "  color: %3 ! important;\n"
+               "  border-bottom: solid %3 1px ! important;\n"
+               "  border-left: solid %3 1px ! important;\n"
+               "  border-right: solid %3 1px ! important;\n"
+               "}\n\n"
 
-		   "div.fancy.header > div.sender-pic{\n"
-		   "  font-size:0.8em;\n"
-		   "  border:1px solid black;\n"
-		   "  background-color:InfoBackground;\n"
-		   "}\n\n"
+               "div.spamheader {\n"
+               "  display:none ! important;\n"
+               "}\n\n"
 
-		   "div.fancy.header > div.sender-status{\n"
-		   "  text-align:center;\n"
-		   "}\n\n"
+               "div.htmlWarn {\n"
+               "  border: 2px solid #ffffff ! important;\n"
+               "}\n\n"
 
-	       "div.htmlWarn {\n"
-	       "  border: 2px solid #ffffff ! important;\n"
-	       "}\n\n"
-           "div.senderStatus {\n"
-            
+               "div.senderpic{\n"
+               "  font-size:0.8em ! important;\n"
+               "  border:1px solid black ! important;\n"
+               "  background-color:%2 ! important;\n"
+               "}\n\n"
+
+               "div.senderstatus{\n"
+               "  text-align:center ! important;\n"
+               "}\n\n"
             )
       .arg( headerFont,
-	    cg.background().name(),
-	    cg.foreground().name() )
+            cg.background().name(),
+            cg.foreground().name() )
       + quoteCSS;
   }
 
@@ -421,7 +427,7 @@ namespace KMail {
     const QString bgColor = c4.name();
     const QString linkColor = c2.name();
     const QString headerFont = QString("  font-family: \"%1\" ! important;\n"
-				       "  font-size: %2px ! important;\n")
+                                       "  font-size: %2px ! important;\n")
       .arg( mBodyFont.family() )
       .arg( pointsToPixel( helper->mMetrics, mBodyFont.pointSize() ) );
     const QString background = ( mBackingPixmapOn
@@ -442,171 +448,183 @@ namespace KMail {
 
     for ( int i = 0 ; i < 3 ; ++i ) {
       quoteCSS += QString( "div.quotelevel%1 {\n"
-			   "  color: %2 ! important;\n" )
-	.arg( QString::number(i+1), mQuoteColor[i].name() );
+                           "  color: %2 ! important;\n" )
+        .arg( QString::number(i+1), mQuoteColor[i].name() );
       if ( mQuoteFont[i].italic() )
-	quoteCSS += "  font-style: italic ! important;\n";
+        quoteCSS += "  font-style: italic ! important;\n";
       if ( mQuoteFont[i].bold() )
-	quoteCSS += "  font-weight: bold ! important;\n";
+        quoteCSS += "  font-weight: bold ! important;\n";
       quoteCSS += "}\n\n";
     }
 
     return
       QString( "body {\n"
-	       "  font-family: \"%1\" ! important;\n"
-	       "  font-size: %2 ! important;\n"
-	       "  color: %3 ! important;\n"
-	       "%4"
-	       "}\n\n" )
+               "  font-family: \"%1\" ! important;\n"
+               "  font-size: %2 ! important;\n"
+               "  color: %3 ! important;\n"
+               "%4"
+               "}\n\n" )
       .arg( bodyFont( fixed ).family(),
-	    bodyFontSize,
-	    fgColor,
-	    background )
+            bodyFontSize,
+            fgColor,
+            background )
       +
       QString( "a {\n"
-	       "  color: %1 ! important;\n"
-	       "  text-decoration: none ! important;\n"
-	       "}\n\n"
+               "  color: %1 ! important;\n"
+               "  text-decoration: none ! important;\n"
+               "}\n\n"
 
-	       "table.textAtm { background-color: %2 ! important; }\n\n"
+               "table.textAtm { background-color: %2 ! important; }\n\n"
 
-	       "tr.textAtmH {\n"
-	       "  background-color: %3 ! important;\n"
-	       "%4"
-	       "}\n\n"
+               "tr.textAtmH {\n"
+               "  background-color: %3 ! important;\n"
+               "%4"
+               "}\n\n"
 
-	       "tr.textAtmB {\n"
-	       "  background-color: %3 ! important;\n"
-	       "}\n\n"
+               "tr.textAtmB {\n"
+               "  background-color: %3 ! important;\n"
+               "}\n\n"
 
-	       "table.rfc822 {\n"
-	       "  background-color: %3 ! important;\n"
-	       "}\n\n"
+               "table.rfc822 {\n"
+               "  background-color: %3 ! important;\n"
+               "}\n\n"
 
-	       "tr.rfc822H {\n"
-	       "%4"
-	       "}\n\n" )
+               "tr.rfc822H {\n"
+               "%4"
+               "}\n\n" )
       .arg( linkColor, fgColor, bgColor, headerFont )
       +
       QString( "table.encr {\n"
-	       "  background-color: %1 ! important;\n"
-	       "}\n\n"
+               "  background-color: %1 ! important;\n"
+               "}\n\n"
 
-	       "tr.encrH {\n"
-	       "  background-color: %2 ! important;\n"
-	       "%3"
-	       "}\n\n"
+               "tr.encrH {\n"
+               "  background-color: %2 ! important;\n"
+               "%3"
+               "}\n\n"
 
-	       "tr.encrB { background-color: %4 ! important; }\n\n" )
+               "tr.encrB { background-color: %4 ! important; }\n\n" )
       .arg( cPgpEncrF.name(),
-	    cPgpEncrH.name(),
-	    headerFont,
-	    cPgpEncrB.name() )
+            cPgpEncrH.name(),
+            headerFont,
+            cPgpEncrB.name() )
       +
       QString( "table.signOkKeyOk {\n"
-	       "  background-color: %1 ! important;\n"
-	       "}\n\n"
+               "  background-color: %1 ! important;\n"
+               "}\n\n"
 
-	       "tr.signOkKeyOkH {\n"
-	       "  background-color: %2 ! important;\n"
-	       "%3"
-	       "}\n\n"
+               "tr.signOkKeyOkH {\n"
+               "  background-color: %2 ! important;\n"
+               "%3"
+               "}\n\n"
 
-	       "tr.signOkKeyOkB { background-color: %4 ! important; }\n\n" )
+               "tr.signOkKeyOkB { background-color: %4 ! important; }\n\n" )
       .arg( cPgpOk1F.name(),
-	    cPgpOk1H.name(),
-	    headerFont,
-	    cPgpOk1B.name() )
+            cPgpOk1H.name(),
+            headerFont,
+            cPgpOk1B.name() )
       +
       QString( "table.signOkKeyBad {\n"
-	       "  background-color: %1 ! important;\n"
-	       "}\n\n"
+               "  background-color: %1 ! important;\n"
+               "}\n\n"
 
-	       "tr.signOkKeyBadH {\n"
-	       "  background-color: %2 ! important;\n"
-	       "%3"
-	       "}\n\n"
+               "tr.signOkKeyBadH {\n"
+               "  background-color: %2 ! important;\n"
+               "%3"
+               "}\n\n"
 
-	       "tr.signOkKeyBadB { background-color: %4 ! important; }\n\n" )
+               "tr.signOkKeyBadB { background-color: %4 ! important; }\n\n" )
       .arg( cPgpOk0F.name(),
-	    cPgpOk0H.name(),
-	    headerFont,
-	    cPgpOk0B.name() )
+            cPgpOk0H.name(),
+            headerFont,
+            cPgpOk0B.name() )
       +
       QString( "table.signWarn {\n"
-	       "  background-color: %1 ! important;\n"
-	       "}\n\n"
+               "  background-color: %1 ! important;\n"
+               "}\n\n"
 
-	       "tr.signWarnH {\n"
-	       "  background-color: %2 ! important;\n"
-	       "%3"
-	       "}\n\n"
+               "tr.signWarnH {\n"
+               "  background-color: %2 ! important;\n"
+               "%3"
+               "}\n\n"
 
-	       "tr.signWarnB { background-color: %4 ! important; }\n\n" )
+               "tr.signWarnB { background-color: %4 ! important; }\n\n" )
       .arg( cPgpWarnF.name(),
-	    cPgpWarnH.name(),
-	    headerFont,
-	    cPgpWarnB.name() )
+            cPgpWarnH.name(),
+            headerFont,
+            cPgpWarnB.name() )
       +
       QString( "table.signErr {\n"
-	       "  background-color: %1 ! important;\n"
-	       "}\n\n"
+               "  background-color: %1 ! important;\n"
+               "}\n\n"
 
-	       "tr.signErrH {\n"
-	       "  background-color: %2 ! important;\n"
-	       "%3"
-	       "}\n\n"
+               "tr.signErrH {\n"
+               "  background-color: %2 ! important;\n"
+               "%3"
+               "}\n\n"
 
-	       "tr.signErrB { background-color: %4 ! important; }\n\n" )
+               "tr.signErrB { background-color: %4 ! important; }\n\n" )
       .arg( cPgpErrF.name(),
-	    cPgpErrH.name(),
-	    headerFont,
-	    cPgpErrB.name() )
+            cPgpErrH.name(),
+            headerFont,
+            cPgpErrB.name() )
       +
       QString( "div.htmlWarn {\n"
-	       "  border: 2px solid %1 ! important;\n"
-	       "}\n\n" )
+               "  border: 2px solid %1 ! important;\n"
+               "}\n\n" )
       .arg( cHtmlWarning.name() )
       +
       QString( "div.header {\n"
-	       "%1"
-	       "}\n\n"
+               "%1"
+               "}\n\n"
 
-	       "div.fancy.header > div {\n"
-	       "  background-color: %2 ! important;\n"
-	       "  color: %3 ! important;\n"
-	       "  border: solid %4 1px ! important;\n"
-	       "}\n\n"
+               "div.fancy.header > div {\n"
+               "  background-color: %2 ! important;\n"
+               "  color: %3 ! important;\n"
+               "  border: solid %4 1px ! important;\n"
+               "}\n\n"
 
-	       "div.fancy.header > div a[href] { color: %3 ! important; }\n\n"
+               "div.fancy.header > div a[href] { color: %3 ! important; }\n\n"
 
-	       "div.fancy.header > div a[href]:hover { text-decoration: underline ! important; }\n\n"
+               "div.fancy.header > div a[href]:hover { text-decoration: underline ! important; }\n\n"
 
-	       "div.fancy.header > table.outer {\n"
-	       "  background-color: %5 ! important;\n"
-	       "  color: %4 ! important;\n"
-	       "  border-bottom: solid %4 1px ! important;\n"
-	       "  border-left: solid %4 1px ! important;\n"
-	       "  border-right: solid %4 1px ! important;\n"
-	       "}\n\n"
+               "div.fancy.header > div.spamheader {\n"
+               "  background-color: #cdcdcd ! important;\n"
+               "  border-top: 0px ! important;\n"
+               "  padding: 3px ! important;\n"
+               "  color: black ! important;\n"
+               "  font-weight: bold ! important;\n"
+               "  font-size: smaller ! important;\n"
+               "}\n\n"
 
-		   "div.senderpic{\n"
-		   "  font-size:0.8em;\n"
-		   "  border:1px solid black;\n"
-		   // FIXME: InfoBackground crashes KHTML
-		   //"  background-color:InfoBackground;\n"
-		   "  background-color:yellow;\n"
-		   "}\n\n"
+               "div.fancy.header > table.outer {\n"
+               "  background-color: %5 ! important;\n"
+               "  color: %4 ! important;\n"
+               "  border-bottom: solid %4 1px ! important;\n"
+               "  border-left: solid %4 1px ! important;\n"
+               "  border-right: solid %4 1px ! important;\n"
+               "}\n\n"
 
-		   "div.senderstatus{\n"
-		   "  text-align:center;\n"
-		   "}\n\n" )
+               "div.senderpic{\n"
+               "  padding: 0px ! important;\n"
+               "  font-size:0.8em ! important;\n"
+               "  border:1px solid %6 ! important;\n"
+               // FIXME: InfoBackground crashes KHTML
+               //"  background-color:InfoBackground ! important;\n"
+               "  background-color:%5 ! important;\n"
+               "}\n\n"
+
+               "div.senderstatus{\n"
+               "  text-align:center ! important;\n"
+               "}\n\n"
+               )
 
       .arg( headerFont )
       .arg( cg.highlight().name(),
-	    cg.highlightedText().name(),
-	    cg.foreground().name(),
-	    cg.background().name() )
+            cg.highlightedText().name(),
+            cg.foreground().name(),
+            cg.background().name() )
+      .arg( cg.mid().name() )
       + quoteCSS;
   }
 
@@ -690,6 +708,16 @@ namespace KMail {
       "  vertical-align: top ! important;\n"
       "  width: 100% ! important;\n"
       "}\n\n"
+
+      "span.pimsmileytext {\n"
+      "  position: absolute;\n"
+      "  top: 0px;\n"
+      "  left: 0px;\n"
+      "  visibility: hidden;\n"
+      "}\n\n"
+
+      "img.pimsmileyimg {\n"
+      "}\n\n"
       ;
   }
 
