@@ -92,8 +92,10 @@ namespace KMail {
       mErrorDialogIsActive( false ),
       mPasswordDialogIsActive( false ),
       mACLSupport( true ),
+      mAnnotationSupport( true ),
       mSlaveConnected( false ),
       mSlaveConnectionError( false ),
+      mCheckingSingleFolder( false ),
       mListDirProgressItem( 0 )
   {
     mPort = imapDefaultPort;
@@ -232,7 +234,9 @@ namespace KMail {
     if ( mSlave && mSlaveConnected ) return Connected;
     if ( mPasswordDialogIsActive ) return Connecting;
 
-    if( mAskAgain || passwd().isEmpty() || login().isEmpty() ) {
+    if( mAskAgain || ( ( passwd().isEmpty() || login().isEmpty() ) &&
+                         auth() != "GSSAPI" ) ) {
+
       Q_ASSERT( !mSlave ); // disconnected on 'wrong login' error already, or first try
       QString log = login();
       QString pass = passwd();
@@ -273,7 +277,7 @@ namespace KMail {
       return Error;
     }
     if ( mSlave->isConnected() ) {
-      mSlaveConnected = true;
+      slotSchedulerSlaveConnected( mSlave );
       return Connected;
     }
 
@@ -282,11 +286,17 @@ namespace KMail {
 
   bool ImapAccountBase::handleJobError( KIO::Job *job, const QString& context, bool abortSync )
   {
+    JobIterator it = findJob( job );
+    if ( it != jobsEnd() && (*it).progressItem )
+    {
+      (*it).progressItem->setComplete();
+      (*it).progressItem = 0;
+    }
     return handleError( job->error(), job->errorText(), job, context, abortSync );
   }
 
   // Called when we're really all done.
-  void ImapAccountBase::postProcessNewMail() {
+  void ImapAccountBase::postProcessNewMail( bool showStatusMsg ) {
     setCheckingMail(false);
     int newMails = 0;
     if ( mCountUnread > 0 && mCountUnread > mCountLastUnread ) {
@@ -298,12 +308,13 @@ namespace KMail {
       mCountUnread = 0;
       checkDone( false, CheckOK );
     }
-    BroadcastStatus::instance()->setStatusMsgTransmissionCompleted(
-        name(), newMails);
+    if ( showStatusMsg )
+      BroadcastStatus::instance()->setStatusMsgTransmissionCompleted(
+          name(), newMails);
   }
 
   //-----------------------------------------------------------------------------
-  void ImapAccountBase::changeSubscription( bool subscribe, QString imapPath )
+  void ImapAccountBase::changeSubscription( bool subscribe, const QString& imapPath )
   {
     // change the subscription of the folder
     KURL url = getUrl();
@@ -478,22 +489,27 @@ namespace KMail {
   void ImapAccountBase::slotSchedulerSlaveError(KIO::Slave *aSlave, int errorCode,
       const QString &errorMsg)
   {
+    kdDebug() << k_funcinfo << endl;
       if (aSlave != mSlave) return;
       handleError( errorCode, errorMsg, 0, QString::null, true );
       if ( mAskAgain )
         makeConnection();
       else {
-        if ( !mSlaveConnected )
+        if ( !mSlaveConnected ) {
           mSlaveConnectionError = true;
+          mOwner->resetConnectionList( this );
+        }
         emit connectionResult( errorCode, errorMsg );
-  }
+      }
   }
 
   //-----------------------------------------------------------------------------
   void ImapAccountBase::slotSchedulerSlaveConnected(KIO::Slave *aSlave)
   {
+    kdDebug() << k_funcinfo << endl;
       if (aSlave != mSlave) return;
       mSlaveConnected = true;
+      mNoopTimer.start( 60000 ); // make sure we start sending noops
       emit connectionResult( 0, QString::null ); // success
   }
 
@@ -576,20 +592,21 @@ namespace KMail {
     if ( !mErrorDialogIsActive && errorCode != KIO::ERR_USER_CANCELED )
     {
       mErrorDialogIsActive = true;
-      QString msg;
-      QString caption;
-      if ( errors.count() >= 3 ) {
-        msg = QString( "<qt>") + context + errors[1] + '\n' + errors[2];
-        caption = errors[0];
-      } else {
-        msg = context + '\n' + KIO::buildErrorString( errorCode, errorMsg );
-        caption = i18n("Error");
-      }
+      QString msg = context + '\n' + KIO::buildErrorString( errorCode, errorMsg );
+      QString caption = i18n("Error");
 
-      if ( jobsKilled || errorCode == KIO::ERR_COULD_NOT_LOGIN )
-        KMessageBox::error( kapp->activeWindow(), msg, caption );
+      if ( jobsKilled || errorCode == KIO::ERR_COULD_NOT_LOGIN ) {
+        if ( !errors.isEmpty() )
+            KMessageBox::detailedError( kapp->activeWindow(), msg, errors.join("\n").prepend("<qt>"), caption );
+        else
+            KMessageBox::error( kapp->activeWindow(), msg, caption );
+      }
       else // i.e. we have a chance to continue, ask the user about it
       {
+        if ( errors.count() >= 3 ) { // there is no detailedWarningContinueCancel... (#86517)
+          msg = QString( "<qt>") + context + errors[1] + '\n' + errors[2];
+          caption = errors[0];
+        }
         int ret = KMessageBox::warningContinueCancel( kapp->activeWindow(), msg, caption );
         if ( ret == KMessageBox::Cancel ) {
           jobsKilled = true;
@@ -597,7 +614,7 @@ namespace KMail {
         }
       }
       mErrorDialogIsActive = false;
-    } else
+    } else if ( errorCode != KIO::ERR_USER_CANCELED )
       kdDebug(5006) << "suppressing error:" << errorMsg << endl;
 
     if ( job && !jobsKilled )
@@ -645,6 +662,7 @@ namespace KMail {
   void ImapAccountBase::processNewMailSingleFolder(KMFolder* folder)
   {
     mFoldersQueuedForChecking.append(folder);
+    mCheckingSingleFolder = true;
     if ( checkingMail() )
     {
       disconnect( this, SIGNAL( finishedCheck( bool, CheckStatus ) ),
@@ -683,6 +701,8 @@ namespace KMail {
   {
     mBodyPartList.clear();
     mCurrentMsg = msg;
+    // first delete old parts as we construct our own
+    msg->deleteBodyParts();
     // make the parts and fill the mBodyPartList
     constructParts( stream, 1, 0, 0, msg->asDwMessage() );
     if ( mBodyPartList.count() == 1 ) // we directly set the body later
@@ -690,16 +710,38 @@ namespace KMail {
 
     if ( !as )
     {
-      kdWarning(5006) << "ImapAccountBase::handleBodyStructure - found no attachment strategy!" << endl;
+      kdWarning(5006) << k_funcinfo << " - found no attachment strategy!" << endl;
       return;
     }
 
-    // download parts according to attachmentstrategy
+    // see what parts have to loaded according to attachmentstrategy
     BodyVisitor *visitor = BodyVisitorFactory::getVisitor( as );
     visitor->visit( mBodyPartList );
     QPtrList<KMMessagePart> parts = visitor->partsToLoad();
+    delete visitor;
     QPtrListIterator<KMMessagePart> it( parts );
     KMMessagePart *part;
+    int partsToLoad = 0;
+    // check how many parts we have to load
+    while ( (part = it.current()) != 0 )
+    {
+      ++it;
+      if ( part->loadPart() )
+      {
+        ++partsToLoad;
+      }
+    }
+    if ( (mBodyPartList.count() * 0.5) < partsToLoad )
+    {
+      // more than 50% of the parts have to be loaded anyway so it is faster
+      // to load the message completely
+      kdDebug(5006) << "Falling back to normal mode" << endl;
+      FolderJob *job = msg->parent()->createJob(
+          msg, FolderJob::tGetMessage, 0, "TEXT" );
+      job->start();
+      return;
+    }
+    it.toFirst();
     while ( (part = it.current()) != 0 )
     {
       ++it;
@@ -720,7 +762,6 @@ namespace KMail {
         job->start();
       }
     }
-    delete visitor;
   }
 
   //-----------------------------------------------------------------------------
@@ -737,20 +778,22 @@ namespace KMail {
       kdDebug(5006) << "ImapAccountBase::constructParts - created id " << part->partSpecifier()
         << " of type " << part->originalContentTypeStr() << endl;
       DwBodyPart *dwpart = mCurrentMsg->createDWBodyPart( part );
-      dwpart->Parse(); // also creates an encapsulated DwMessage if necessary
-
-//      kdDebug(5006) << "constructed dwpart " << dwpart << ",dwmsg " << dwmsg << ",parent " << parent
-//       << ",dwparts msg " << dwpart->Body().Message() << endl;
 
       if ( parent )
       {
         // add to parent body
         parent->Body().AddBodyPart( dwpart );
+        dwpart->Parse();
+//        kdDebug(5006) << "constructed dwpart " << dwpart << ",dwmsg " << dwmsg << ",parent " << parent
+//          << ",dwparts msg " << dwpart->Body().Message() <<",id "<<dwpart->ObjectId() << endl;
       } else if ( part->partSpecifier() != "0" &&
                   !part->partSpecifier().endsWith(".HEADER") )
       {
         // add to message
         dwmsg->Body().AddBodyPart( dwpart );
+        dwpart->Parse();
+//        kdDebug(5006) << "constructed dwpart " << dwpart << ",dwmsg " << dwmsg << ",parent " << parent
+//          << ",dwparts msg " << dwpart->Body().Message() <<",id "<<dwpart->ObjectId() << endl;
       } else
         dwpart = 0;
 
@@ -840,6 +883,12 @@ namespace KMail {
     mapJobData.remove( it );
   }
 
+  //-----------------------------------------------------------------------------
+  void KMail::ImapAccountBase::removeJob( KIO::Job* job )
+  {
+    mapJobData.remove( job );
+  }
+
   //-----------------------------------------------------------------------------
   KPIM::ProgressItem* ImapAccountBase::listDirProgressItem()
   {
@@ -852,9 +901,9 @@ namespace KMail {
           true,
           useSSL() || useTLS() );
       connect ( mListDirProgressItem,
-          SIGNAL( progressItemCanceled( ProgressItem* ) ),
-          this,
-          SLOT( slotAbortRequested( ProgressItem* ) ) );
+                SIGNAL( progressItemCanceled( KPIM::ProgressItem* ) ),
+                this,
+                SLOT( slotAbortRequested( KPIM::ProgressItem* ) ) );
       // Start with a guessed value of the old folder count plus 5%. As long
       // as the list of folders doesn't constantly change, that should be good
       // enough.
