@@ -54,6 +54,7 @@
 #include <kabc/stdaddressbook.h>
 #include <klocale.h>
 #include <kdebug.h>
+#include <kinputdialog.h>
 #include <kmessagebox.h>
 
 #include <qstringlist.h>
@@ -1036,8 +1037,6 @@ Kpgp::Result Kleo::KeyResolver::resolveSigningKeysForSigningOnly() {
   CryptoMessageFormat commonFormat = AutoFormat;
 
   for ( unsigned int i = 0 ; i < numConcreteCryptoMessageFormats ; ++i ) {
-    if ( !( concreteCryptoMessageFormats[i] & mCryptoMessageFormats ) )
-      continue;
     if ( signingKeysFor( concreteCryptoMessageFormats[i] ).empty() )
       continue; // skip;
     if ( count.numOf( concreteCryptoMessageFormats[i] ) == count.numTotal() ) {
@@ -1423,17 +1422,18 @@ void Kleo::KeyResolver::addKeys( const std::vector<Item> & items ) {
   for ( std::vector<Item>::const_iterator it = items.begin() ; it != items.end() ; ++it ) {
     SplitInfo si( it->address );
     CryptoMessageFormat f = AutoFormat;
-    for ( unsigned int i = 0 ; i < numConcreteCryptoMessageFormats ; ++i )
-      if ( mCryptoMessageFormats & concreteCryptoMessageFormats[i] & it->format ) {
-	f = concreteCryptoMessageFormats[i];
-	break;
+    for ( unsigned int i = 0 ; i < numConcreteCryptoMessageFormats ; ++i ) {
+      if ( concreteCryptoMessageFormats[i] & it->format ) {
+        f = concreteCryptoMessageFormats[i];
+        break;
       }
+    }
     if ( f == AutoFormat )
       kdWarning() << "Kleo::KeyResolver::addKeys(): Something went wrong. Didn't find a format for \""
-		  << it->address << "\"" << endl;
+                  << it->address << "\"" << endl;
     else
       std::remove_copy_if( it->keys.begin(), it->keys.end(),
-			   std::back_inserter( si.keys ), IsNotForFormat( f ) );
+                           std::back_inserter( si.keys ), IsNotForFormat( f ) );
     d->mFormatInfoMap[ f ].splitInfos.push_back( si );
   }
   dump();
@@ -1461,24 +1461,40 @@ Kleo::KeyResolver::ContactPreferences& Kleo::KeyResolver::lookupContactPreferenc
     // insert into map and grab resulting iterator
     pos = d->mContactPreferencesMap.insert(
       Private::ContactPreferencesMap::value_type( address, pref ) ).first;
+    
   }
   return (*pos).second;
 }
 
 void Kleo::KeyResolver::saveContactPreference( const QString& email, const ContactPreferences& pref ) const
 {
-  KABC::AddressBook *ab = KABC::StdAddressBook::self();
+  KABC::AddressBook *ab = KABC::StdAddressBook::self( true );
   KABC::Addressee::List res = ab->findByEmail( email );
-  if ( !res.isEmpty() ) {
-    KABC::Addressee addr = res.first();
-    addr.insertCustom( "KADDRESSBOOK", "CRYPTOENCRYPTPREF", Kleo::encryptionPreferenceToString( pref.encryptionPreference ) );
-    addr.insertCustom( "KADDRESSBOOK", "CRYPTOSIGNPREF", Kleo::signingPreferenceToString( pref.signingPreference ) );
-    addr.insertCustom( "KADDRESSBOOK", "CRYPTOPROTOPREF", cryptoMessageFormatToString( pref.cryptoMessageFormat ) );
-    addr.insertCustom( "KADDRESSBOOK", "OPENPGPFP", pref.pgpKeyFingerprints.join( "," ) );
-    addr.insertCustom( "KADDRESSBOOK", "SMIMEFP", pref.smimeCertFingerprints.join( "," ) );
-    ab->insertAddressee( addr );
-    // Assumption: 'pref' comes from d->mContactPreferencesMap already, no need to update that
-  }
+
+  KABC::Addressee addr;
+  if ( res.isEmpty() ) {
+     bool ok = true;
+     QString fullName = KInputDialog::getText( i18n( "Name Selection" ), i18n( "Which name shall the contact '%1' have in your addressbook?" ).arg( email ), QString::null, &ok );
+    if ( ok ) {
+      addr.setNameFromString( fullName );
+      addr.insertEmail( email, true );
+    } else
+      return;
+  } else
+    addr = res.first();
+
+  addr.insertCustom( "KADDRESSBOOK", "CRYPTOENCRYPTPREF", Kleo::encryptionPreferenceToString( pref.encryptionPreference ) );
+  addr.insertCustom( "KADDRESSBOOK", "CRYPTOSIGNPREF", Kleo::signingPreferenceToString( pref.signingPreference ) );
+  addr.insertCustom( "KADDRESSBOOK", "CRYPTOPROTOPREF", cryptoMessageFormatToString( pref.cryptoMessageFormat ) );
+  addr.insertCustom( "KADDRESSBOOK", "OPENPGPFP", pref.pgpKeyFingerprints.join( "," ) );
+  addr.insertCustom( "KADDRESSBOOK", "SMIMEFP", pref.smimeCertFingerprints.join( "," ) );
+
+  ab->insertAddressee( addr );
+  KABC::Ticket *ticket = ab->requestSaveTicket( addr.resource() );
+  if ( ticket )
+    ab->save( ticket );
+
+  // Assumption: 'pref' comes from d->mContactPreferencesMap already, no need to update that
 }
 
 Kleo::KeyResolver::ContactPreferences::ContactPreferences()
