@@ -8,12 +8,13 @@
 
 #include <qobject.h>
 #include "kmaccount.h"
+#include <kdepimmacros.h>
 
 class QString;
 class QStringList;
 
 
-class KMAcctMgr: public QObject
+class KDE_EXPORT KMAcctMgr: public QObject
 {
   Q_OBJECT
   friend class KMAccount;
@@ -32,7 +33,8 @@ public:
 
   /** Create a new account of given type with given name. Currently
    the types "local" for local mail folders and "pop" are supported. */
-  virtual KMAccount* create(const QString& type, const QString& name, uint id = 0);
+  virtual KMAccount* create( const QString& type,
+                             const QString& name = QString::null, uint id = 0);
 
   /** Adds an account to the list of accounts */
   virtual void add(KMAccount *account);
@@ -69,6 +71,13 @@ public:
   /// Called on exit (KMMainWin::queryExit)
   void cancelMailCheck();
 
+  /** Read passwords of all accounts from the wallet */
+  void readPasswords();
+
+  /** Reset connection list for the account */
+  void resetConnectionList( KMAccount* acct ) { 
+    mServerConnections[ hostForAccount( acct ) ] = 0; }
+
 public slots:
   virtual void singleCheckMail(KMAccount *, bool _interactive = true);
   virtual void singleInvalidateIMAPFolders(KMAccount *);
