@@ -32,7 +32,10 @@
 #include <kparts/statusbarextension.h>
 #include <kparts/factory.h>
 #include <kparts/event.h>
+#include <kparts/part.h>
+
 #include <qwidget.h>
+#include <kdepimmacros.h>
 
 class KInstance;
 class KAboutData;
@@ -47,7 +50,7 @@ class KMFolderTreeItem;
 
 class ActionManager;
 
-class KMailPart: public KParts::ReadOnlyPart, virtual public KMailPartIface
+class KDE_EXPORT KMailPart: public KParts::ReadOnlyPart, virtual public KMailPartIface
 {
     Q_OBJECT
   public:
@@ -62,7 +65,7 @@ class KMailPart: public KParts::ReadOnlyPart, virtual public KMailPartIface
   public slots:
     virtual void save() { /*TODO*/ }
     virtual void exit();
-    virtual void updateEditMenu() {};
+    virtual void updateEditMenu() {}
     void exportFolder( KMFolder* folder );
     void slotIconChanged( KMFolderTreeItem *fti );
     void slotNameChanged( KMFolderTreeItem *fti );
