@@ -6,13 +6,17 @@
 #include <qguardedptr.h>
 #include <qptrlist.h>
 #include <qvaluelist.h>
+#include <qvaluevector.h>
 #include <kio/job.h>
 #include "kmmsgbase.h" // for KMMsgStatus
 #include <mimelib/string.h>
+#include <kdepimmacros.h>
+#include <kservice.h>
 
 class QPopupMenu;
 class QTextCodec;
 class KMainWindow;
+class KAction;
 class KProgressDialog;
 class KMComposeWin;
 class KMFilter;
@@ -30,7 +34,7 @@ namespace KIO { class Job; }
 typedef QMap<int,KMFolder*> KMMenuToFolder;
 typedef QMap<partNode*, KMMessage*> PartNodeMessageMap;
 
-class KMCommand : public QObject
+class KDE_EXPORT KMCommand : public QObject
 {
   Q_OBJECT
 
@@ -140,7 +144,7 @@ private:
   QValueList<QGuardedPtr<KMFolder> > mFolders;
 };
 
-class KMMailtoComposeCommand : public KMCommand
+class KDE_EXPORT KMMailtoComposeCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -154,7 +158,7 @@ private:
   KMMessage *mMessage;
 };
 
-class KMMailtoReplyCommand : public KMCommand
+class KDE_EXPORT KMMailtoReplyCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -169,7 +173,7 @@ private:
   QString mSelection;
 };
 
-class KMMailtoForwardCommand : public KMCommand
+class KDE_EXPORT KMMailtoForwardCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -183,7 +187,7 @@ private:
   KURL mUrl;
 };
 
-class KMMailtoAddAddrBookCommand : public KMCommand
+class KDE_EXPORT KMMailtoAddAddrBookCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -196,7 +200,7 @@ private:
   KURL mUrl;
 };
 
-class KMAddBookmarksCommand : public KMCommand
+class KDE_EXPORT KMAddBookmarksCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -210,7 +214,7 @@ private:
 };
 
 
-class KMMailtoOpenAddrBookCommand : public KMCommand
+class KDE_EXPORT KMMailtoOpenAddrBookCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -223,7 +227,7 @@ private:
   KURL mUrl;
 };
 
-class KMUrlCopyCommand : public KMCommand
+class KDE_EXPORT KMUrlCopyCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -237,7 +241,7 @@ private:
   KMMainWidget *mMainWidget;
 };
 
-class KMUrlOpenCommand : public KMCommand
+class KDE_EXPORT KMUrlOpenCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -251,7 +255,7 @@ private:
   KMReaderWin *mReaderWin;
 };
 
-class KMUrlSaveCommand : public KMCommand
+class KDE_EXPORT KMUrlSaveCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -267,7 +271,7 @@ private:
   KURL mUrl;
 };
 
-class KMEditMsgCommand : public KMCommand
+class KDE_EXPORT KMEditMsgCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -278,19 +282,21 @@ private:
   virtual Result execute();
 };
 
-class KMShowMsgSrcCommand
+class KDE_EXPORT KMShowMsgSrcCommand : public KMCommand
 {
+  Q_OBJECT
+
 public:
-  KMShowMsgSrcCommand( KMMessage *msg, bool fixedFont );
-		       
-  void start();
+  KMShowMsgSrcCommand( QWidget *parent, KMMessage *msg,
+		       bool fixedFont );
+  virtual Result execute();
 
 private:
   bool mFixedFont;
-  KMMessage *mMsg;
+  bool mMsgWasComplete;
 };
 
-class KMSaveMsgCommand : public KMCommand
+class KDE_EXPORT KMSaveMsgCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -313,13 +319,14 @@ private:
   KURL mUrl;
   QValueList<unsigned long> mMsgList;
   unsigned int mMsgListIndex;
+  KMMessage *mStandAloneMessage;
   QByteArray mData;
   int mOffset;
   size_t mTotalSize;
   KIO::TransferJob *mJob;
 };
 
-class KMOpenMsgCommand : public KMCommand
+class KDE_EXPORT KMOpenMsgCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -340,7 +347,7 @@ private:
   KIO::TransferJob *mJob;
 };
 
-class KMSaveAttachmentsCommand : public KMCommand
+class KDE_EXPORT KMSaveAttachmentsCommand : public KMCommand
 {
   Q_OBJECT
 public:
@@ -378,7 +385,7 @@ private:
   bool mEncoded;
 };
 
-class KMReplyToCommand : public KMCommand
+class KDE_EXPORT KMReplyToCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -393,7 +400,7 @@ private:
   QString mSelection;
 };
 
-class KMNoQuoteReplyToCommand : public KMCommand
+class KDE_EXPORT KMNoQuoteReplyToCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -419,7 +426,7 @@ private:
   QString mSelection;
 };
 
-class KMReplyToAllCommand : public KMCommand
+class KDE_EXPORT KMReplyToAllCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -434,7 +441,7 @@ private:
   QString mSelection;
 };
 
-class KMReplyAuthorCommand : public KMCommand
+class KDE_EXPORT KMReplyAuthorCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -449,7 +456,7 @@ private:
   QString mSelection;
 };
 
-class KMForwardCommand : public KMCommand
+class KDE_EXPORT KMForwardCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -466,7 +473,7 @@ private:
   uint mIdentity;
 };
 
-class KMForwardAttachedCommand : public KMCommand
+class KDE_EXPORT KMForwardAttachedCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -483,7 +490,7 @@ private:
   QGuardedPtr<KMComposeWin> mWin;
 };
 
-class KMRedirectCommand : public KMCommand
+class KDE_EXPORT KMRedirectCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -494,33 +501,27 @@ private:
   virtual Result execute();
 };
 
-class KMBounceCommand : public KMCommand
+class KDE_EXPORT KMPrintCommand : public KMCommand
 {
   Q_OBJECT
 
 public:
-  KMBounceCommand( QWidget *parent, KMMessage *msg );
-
-private:
-  virtual Result execute();
-};
-
-class KMPrintCommand : public KMCommand
-{
-  Q_OBJECT
-
-public:
-  KMPrintCommand( QWidget *parent, KMMessage *msg, 
-                  bool htmlOverride=false, const QTextCodec *codec = 0 );
+  KMPrintCommand( QWidget *parent, KMMessage *msg,
+                  bool htmlOverride=false,
+                  bool htmlLoadExtOverride=false,
+                  bool useFixedFont = false,
+                  const QTextCodec *codec = 0 );
 
 private:
   virtual Result execute();
 
   bool mHtmlOverride;
+  bool mHtmlLoadExtOverride;
+  bool mUseFixedFont;
   const QTextCodec *mCodec;
 };
 
-class KMSetStatusCommand : public KMCommand
+class KDE_EXPORT KMSetStatusCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -538,7 +539,7 @@ private:
   bool mToggle;
 };
 
-class KMFilterCommand : public KMCommand
+class KDE_EXPORT KMFilterCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -553,7 +554,7 @@ private:
 };
 
 
-class KMFilterActionCommand : public KMCommand
+class KDE_EXPORT KMFilterActionCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -569,7 +570,7 @@ private:
 };
 
 
-class KMMetaFilterActionCommand : public QObject
+class KDE_EXPORT KMMetaFilterActionCommand : public QObject
 {
   Q_OBJECT
 
@@ -586,8 +587,29 @@ private:
   KMMainWidget *mMainWidget;
 };
 
+class KDE_EXPORT FolderShortcutCommand : public QObject
+{
+  Q_OBJECT
 
-class KMMailingListFilterCommand : public KMCommand
+public:
+  FolderShortcutCommand( KMMainWidget* mainwidget, KMFolder *folder );
+  ~FolderShortcutCommand();
+
+public slots:
+  void start();
+  /** Assign a KActio to the command which is used to trigger it. This
+   * action will be deleted along with the command, so you don't need to
+   * keep track of it separately. */
+  void setAction( KAction* );
+
+private:
+  KMMainWidget *mMainWidget;
+  KMFolder *mFolder;
+  KAction *mAction;
+};
+
+
+class KDE_EXPORT KMMailingListFilterCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -604,9 +626,9 @@ private:
       move is TRUE this slot will cause all selected messages to
       be moved into the given folder, otherwise messages will be
       copied.
-      Am empty @ref KMMenuToFolder must be passed in. */
+      Am empty @see KMMenuToFolder must be passed in. */
 
-class KMMenuCommand : public KMCommand
+class KDE_EXPORT KMMenuCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -618,7 +640,7 @@ public:
     QObject *receiver, KMMenuToFolder *aMenuToFolder, QPopupMenu *menu );
 };
 
-class KMCopyCommand : public KMMenuCommand
+class KDE_EXPORT KMCopyCommand : public KMMenuCommand
 {
   Q_OBJECT
 
@@ -627,17 +649,23 @@ public:
 		 const QPtrList<KMMsgBase> &msgList );
   KMCopyCommand( KMFolder* destFolder, KMMessage *msg );
 
+protected slots:
+  void slotMsgAdded( KMFolder*, Q_UINT32 );
+
 private:
   virtual Result execute();
 
   KMFolder *mDestFolder;
   QPtrList<KMMsgBase> mMsgList;
+  // List of serial numbers that need to be loaded
+  // Ticked off as they come in via msgAdded signals.
+  QValueList<Q_UINT32> mWaitingForMsgs;
 };
 
 namespace KPIM {
   class ProgressItem;
 }
-class KMMoveCommand : public KMMenuCommand
+class KDE_EXPORT KMMoveCommand : public KMMenuCommand
 {
   Q_OBJECT
 
@@ -657,6 +685,7 @@ protected:
   KMMoveCommand( Q_UINT32 sernum );
   void setDestFolder( KMFolder* folder ) { mDestFolder = folder; }
   void addMsg( KMMsgBase *msg ) { mMsgList.append( msg ); }
+  QValueVector<KMFolder*> mOpenedFolders;
 
 private:
   virtual Result execute();
@@ -670,7 +699,7 @@ private:
   KPIM::ProgressItem *mProgressItem;
 };
 
-class KMDeleteMsgCommand : public KMMoveCommand
+class KDE_EXPORT KMDeleteMsgCommand : public KMMoveCommand
 {
   Q_OBJECT
 
@@ -684,7 +713,7 @@ private:
 
 };
 
-class KMUrlClickedCommand : public KMCommand
+class KDE_EXPORT KMUrlClickedCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -702,7 +731,7 @@ private:
   KMMainWidget *mMainWidget;
 };
 
-class KMLoadPartsCommand : public KMCommand
+class KDE_EXPORT KMLoadPartsCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -727,7 +756,7 @@ private:
   PartNodeMessageMap mPartMap;
 };
 
-class KMResendMessageCommand : public KMCommand
+class KDE_EXPORT KMResendMessageCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -738,7 +767,7 @@ private:
   virtual Result execute();
 };
 
-class KMMailingListCommand : public KMCommand
+class KDE_EXPORT KMMailingListCommand : public KMCommand
 {
   Q_OBJECT
 public:
@@ -753,7 +782,7 @@ protected:
   KMFolder *mFolder;
 };
 
-class KMMailingListPostCommand : public KMMailingListCommand
+class KDE_EXPORT KMMailingListPostCommand : public KMMailingListCommand
 {
   Q_OBJECT
 public:
@@ -762,7 +791,7 @@ protected:
   virtual KURL::List urls() const;
 };
 
-class KMMailingListSubscribeCommand : public KMMailingListCommand
+class KDE_EXPORT KMMailingListSubscribeCommand : public KMMailingListCommand
 {
   Q_OBJECT
 public:
@@ -771,7 +800,7 @@ protected:
   virtual KURL::List urls() const;
 };
 
-class KMMailingListUnsubscribeCommand : public KMMailingListCommand
+class KDE_EXPORT KMMailingListUnsubscribeCommand : public KMMailingListCommand
 {
   Q_OBJECT
 public:
@@ -780,7 +809,7 @@ protected:
   virtual KURL::List urls() const;
 };
 
-class KMMailingListArchivesCommand : public KMMailingListCommand
+class KDE_EXPORT KMMailingListArchivesCommand : public KMMailingListCommand
 {
   Q_OBJECT
 public:
@@ -789,7 +818,7 @@ protected:
   virtual KURL::List urls() const;
 };
 
-class KMMailingListHelpCommand : public KMMailingListCommand
+class KDE_EXPORT KMMailingListHelpCommand : public KMMailingListCommand
 {
   Q_OBJECT
 public:
@@ -798,7 +827,7 @@ protected:
   virtual KURL::List urls() const;
 };
 
-class KMIMChatCommand : public KMCommand
+class KDE_EXPORT KMIMChatCommand : public KMCommand
 {
   Q_OBJECT
 
@@ -818,5 +847,79 @@ private:
   KMMessage *mMessage;
 };
 
+class KDE_EXPORT KMHandleAttachmentCommand : public KMCommand
+{
+  Q_OBJECT
+
+public:
+  /** Supported types of attachment handling */
+  enum AttachmentAction
+  {
+    Open = 1,
+    OpenWith = 2,
+    View = 3,
+    Save = 4,
+    Properties = 5
+  };
+  /**
+   * Construct a new command
+   * @p node the partNode
+   * @p msg the KMMessage
+   * @p atmId the ID of the attachment, the partNode must know this
+   * @p atmName the name of the attachment
+   * @p action what to do with the attachment
+   * @p offer specify a KService that should handle the "open" action
+   */
+  KMHandleAttachmentCommand( partNode* node, KMMessage* msg, int atmId,
+      const QString& atmName, AttachmentAction action, KService::Ptr offer = 0 );
+
+private slots:
+  /** Called from start() via a single shot timer. */
+  virtual void slotStart();
+
+  /**
+   * Called when the part was downloaded
+   * Calls execute
+   */
+  void slotPartComplete();
+
+signals:
+  void showAttachment( int id, const QString& name );
+
+private:
+  virtual Result execute();
+
+  QString createAtmFileLink() const;
+
+  /** Get a KService if it was not specified */
+  KService::Ptr getServiceOffer();
+
+  /** Open needs a valid KService */
+  void atmOpen();
+
+  /** Display an open-with dialog */
+  void atmOpenWith();
+
+  /**
+   * Viewing is not supported by this command
+   * so it just emits showAttachment
+   */
+  void atmView();
+
+  /** Save the attachment */
+  void atmSave();
+
+  /** Display the properties */
+  void atmProperties();
+
+private:
+  partNode* mNode;
+  KMMessage* mMsg;
+  int mAtmId;
+  const QString& mAtmName;
+  AttachmentAction mAction;
+  KService::Ptr mOffer;
+
+};
 
 #endif /*KMCommands_h*/
