@@ -10,7 +10,6 @@
 #include "kmmainwidget.h"
 #include "kmmsgdict.h"
 #include "kmmsgpart.h"
-#include "kmfoldercombobox.h"
 #include "kmfolderdia.h"
 #include "kmfolderimap.h"
 #include "kmfoldermgr.h"
@@ -18,6 +17,8 @@
 #include "kmfoldertree.h"
 #include "kmsearchpatternedit.h"
 #include "kmsearchpattern.h"
+#include "folderrequester.h"
+using KMail::FolderRequester;
 
 #include <kapplication.h>
 #include <kdebug.h>
@@ -49,22 +50,25 @@ const int KMFldSearch::MSGID_COLUMN = 4;
 //-----------------------------------------------------------------------------
 KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
                          KMFolder *curFolder, bool modal):
-  KDialogBase(0, name, modal, i18n("Search in Folders"),
+  KDialogBase(0, name, modal, i18n("Find Messages"),
               User1 | User2 | Close, User1, false,
               KGuiItem( i18n("&Search"), "find" ),
               KGuiItem( i18n("S&top"), "cancel" )),
   mStopped(false),
   mCloseRequested(false),
   mSortColumn(0),
-#if QT_VERSION >= 0x030200
   mSortOrder(Ascending),
-#endif
   mFolder(0),
   mTimer(new QTimer(this)),
   mLastFocus(0),
   mKMMainWidget(w)
 {
+#if !KDE_IS_VERSION( 3, 2, 91 )
+  // HACK - KWin keeps all dialogs on top of their mainwindows, but that's probably
+  // wrong (#76026), and should be done only for modals. CVS HEAD should get
+  // proper fix in KWin (l.lunak@kde.org)
   XDeleteProperty( qt_xdisplay(), winId(), XA_WM_TRANSIENT_FOR );
+#endif
   KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
 
   KConfig* config = KMKernel::config();
@@ -86,13 +90,12 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   mChkbxSpecificFolders->setChecked(true);
   radioGroup->insert( mChkbxSpecificFolders );
 
-  mCbxFolders = new KMFolderComboBox(false, searchWidget);
+  mCbxFolders = new FolderRequester( searchWidget, 
+      kmkernel->getKMMainWidget()->folderTree() );
+  mCbxFolders->setMustBeReadWrite( false );
   mCbxFolders->setFolder(curFolder);
   hbl->addWidget(mCbxFolders);
 
-  connect(mCbxFolders, SIGNAL(activated(int)),
-          this, SLOT(slotFolderActivated(int)));
-
   mChkSubFolders = new QCheckBox(i18n("I&nclude sub-folders"), searchWidget);
   mChkSubFolders->setChecked(true);
   hbl->addWidget(mChkSubFolders);
@@ -105,7 +108,9 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   mPatternEdit->setFrameStyle( QFrame::NoFrame | QFrame::Plain );
   mPatternEdit->setInsideMargin( 0 );
   mSearchPattern = new KMSearchPattern();
-  KMFolderSearch *searchFolder = dynamic_cast<KMFolderSearch*>(curFolder);
+  KMFolderSearch *searchFolder = 0;
+  if (curFolder)
+      searchFolder = dynamic_cast<KMFolderSearch*>(curFolder->storage());
   if (searchFolder) {
       KConfig config(curFolder->location());
       KMFolder *root = searchFolder->search()->root();
@@ -126,19 +131,8 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   if ( list )
       object = list->first();
   delete list;
-  if (!searchFolder && object && object->inherits( "QComboBox" )) {
-      QComboBox *combo = (QComboBox*)object;
-      combo->setCurrentText("Subject");
-  }
-  list = mPatternEdit->queryList( 0, "mRuleValue" );
-  object = 0;
-  if ( list )
-      object = list->first();
-  delete list;
-  if (object && object->inherits( "QWidget" )) {
-      QWidget *widget = (QComboBox*)object;
-      widget->setFocus();
-  }
+  if (!searchFolder && object && ::qt_cast<QComboBox*>(object))
+      static_cast<QComboBox*>(object)->setCurrentText("Subject");
 
   vbl->addWidget( mPatternEdit );
 
@@ -148,7 +142,7 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   connect( mChkbxSpecificFolders, SIGNAL(toggled(bool)),
 	   mChkSubFolders, SLOT(setEnabled(bool)) );
 
-  mLbxMatches = new KListView(searchWidget, "Search in Folders");
+  mLbxMatches = new KListView(searchWidget, "Find Messages");
 
   /*
      Default is to sort by date. TODO: Unfortunately this sorts *while*
@@ -192,7 +186,7 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   hbl2->addWidget(mSearchFolderLbl);
   mSearchFolderEdt = new KLineEdit(searchWidget);
   if (searchFolder)
-    mSearchFolderEdt->setText(searchFolder->name());
+    mSearchFolderEdt->setText(searchFolder->folder()->name());
   else
     mSearchFolderEdt->setText(i18n("Last Search"));
 
@@ -235,6 +229,16 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   connect(this, SIGNAL(user2Clicked()), SLOT(slotStop()));
   connect(this, SIGNAL(finished()), this, SLOT(deleteLater()));
 
+  // give focus to the value field of the first search rule
+  object = mPatternEdit->child( "regExpLineEdit" );
+  if ( object && object->isWidgetType() ) {
+      static_cast<QWidget*>(object)->setFocus();
+      //kdDebug(5006) << "KMFldSearch: focus has been given to widget "
+      //              << object->name() << endl;
+  }
+  else
+      kdDebug(5006) << "KMFldSearch: regExpLineEdit not found" << endl;
+
   //set up actions
   KActionCollection *ac = actionCollection();
   mReplyAction = new KAction( i18n("&Reply..."), "mail_reply", 0, this,
@@ -270,6 +274,10 @@ KMFldSearch::KMFldSearch(KMMainWidget* w, const char* name,
   connect(mTimer, SIGNAL(timeout()), this, SLOT(updStatus()));
   connect(kmkernel->searchFolderMgr(), SIGNAL(folderInvalidated(KMFolder*)),
 	  this, SLOT(folderInvalidated(KMFolder*)));
+
+  connect(mCbxFolders, SIGNAL(folderChanged(KMFolder*)),
+          this, SLOT(slotFolderActivated(KMFolder*)));
+
 }
 
 //-----------------------------------------------------------------------------
@@ -346,15 +354,12 @@ void KMFldSearch::keyPressEvent(QKeyEvent *evt)
 
 
 //-----------------------------------------------------------------------------
-void KMFldSearch::slotFolderActivated(int /*nr*/)
+void KMFldSearch::slotFolderActivated( KMFolder* folder )
 {
-    KMFolder* folder = mCbxFolders->getFolder();
-
     mChkbxSpecificFolders->setChecked(true);
     mBtnSearch->setEnabled(folder);
 }
 
-
 //-----------------------------------------------------------------------------
 void KMFldSearch::activateFolder(KMFolder *curFolder)
 {
@@ -378,9 +383,7 @@ void KMFldSearch::slotSearch()
     mLbxMatches->clear();
 
     mSortColumn = mLbxMatches->sortColumn();
-#if QT_VERSION >= 0x030200
     mSortOrder = mLbxMatches->sortOrder();
-#endif
     mLbxMatches->setSorting(-1);
     mLbxMatches->setShowSortIndicator(false);
 
@@ -395,7 +398,7 @@ void KMFldSearch::slotSearch()
       int count = 0;
       KMFolder *folder;
       while ((folder = mgr->find(fullName))) {
-        if (folder->inherits("KMFolderSearch"))
+        if (folder->storage()->inherits("KMFolderSearch"))
           break;
         fullName = QString("%1 %2").arg(baseName).arg(++count);
       }
@@ -404,7 +407,7 @@ void KMFldSearch::slotSearch()
         folder = mgr->createFolder(fullName, FALSE, KMFolderTypeSearch,
             &mgr->dir());
 
-      mFolder = (KMFolderSearch*)folder;
+      mFolder = dynamic_cast<KMFolderSearch*>( folder->storage() );
     }
     mFolder->stopSearch();
     disconnect(mFolder, SIGNAL(msgAdded(int)),
@@ -421,7 +424,7 @@ void KMFldSearch::slotSearch()
     if (mChkbxAllFolders->isChecked()) {
 	search->setRecursive(true);
     } else {
-	search->setRoot(mCbxFolders->getFolder());
+	search->setRoot(mCbxFolders->folder());
 	search->setRecursive(mChkSubFolders->isChecked());
     }
 
@@ -433,9 +436,9 @@ void KMFldSearch::slotSearch()
     mFolder->setSearch(search);
     enableGUI();
 
-    if (mFolder && !mFolders.contains(mFolder.operator->())) {
+    if (mFolder && !mFolders.contains(mFolder.operator->()->folder())) {
 	mFolder->open();
-	mFolders.append(mFolder.operator->());
+	mFolders.append(mFolder.operator->()->folder());
     }
     mTimer->start(200);
 }
@@ -452,9 +455,7 @@ void KMFldSearch::searchDone()
     if (mCloseRequested)
 	close();
 
-#if QT_VERSION >= 0x030200
     mLbxMatches->setSorting(mSortColumn, mSortOrder == Ascending);
-#endif
     mLbxMatches->setShowSortIndicator(true);
 }
 
@@ -470,7 +471,7 @@ void KMFldSearch::slotAddMsg(int idx)
 	mFolders.append(pFolder);
 	pFolder->open();
     }
-    if(pFolder->type() == KFolderTreeItem::SentMail)
+    if(pFolder->whoField() == "To")
         from = msg->to();
     else
         from = msg->from();
@@ -480,7 +481,7 @@ void KMFldSearch::slotAddMsg(int idx)
         fName = pFolder->name();
 
     (void)new KListViewItem(mLbxMatches, mLbxMatches->lastItem(),
-			    msg->subject(), from, msg->dateIsoStr(),
+			    msg->subject(), from, msg->dateStr(),
 			    fName,
 			    QString::number(mFolder->serNum(idx)));
     if (unget)
@@ -526,6 +527,7 @@ void KMFldSearch::closeEvent(QCloseEvent *e)
 	//Cancel search in progress by setting the search folder search to
 	//the null search
 	mFolder->setSearch(new KMSearch());
+	QTimer::singleShot(0, this, SLOT(slotClose()));
     } else {
 	KDialogBase::closeEvent(e);
     }
@@ -540,7 +542,7 @@ void KMFldSearch::updateCreateButton( const QString &s)
 //-----------------------------------------------------------------------------
 void KMFldSearch::renameSearchFolder()
 {
-    if (mFolder && (mFolder->name() !=mSearchFolderEdt->text())) {
+    if (mFolder && (mFolder->folder()->name() != mSearchFolderEdt->text())) {
 	int i = 1;
 	QString name =  mSearchFolderEdt->text();
 	while (i < 100) {
@@ -560,7 +562,7 @@ void KMFldSearch::openSearchFolder()
 {
     renameSearchFolder();
     KMFolderTree *folderTree = mKMMainWidget->folderTree();
-    QListViewItem *index = folderTree->indexOfFolder((KMFolder*)mFolder);
+    QListViewItem *index = folderTree->indexOfFolder(mFolder->folder());
     if (index) {
 	folderTree->ensureItemVisible(index);
 	folderTree->doFolderSelected(index);
@@ -571,7 +573,7 @@ void KMFldSearch::openSearchFolder()
 //-----------------------------------------------------------------------------
 void KMFldSearch::folderInvalidated(KMFolder *folder)
 {
-    if (folder == mFolder) {
+    if (folder->storage() == mFolder) {
 	mLbxMatches->clear();
 	if (mFolder->search())
 	    connect(mFolder->search(), SIGNAL(finished(bool)),
@@ -610,10 +612,8 @@ void KMFldSearch::enableGUI()
     KMSearch const *search = (mFolder) ? (mFolder->search()) : 0;
     bool searching = (search) ? (search->running()) : false;
     actionButton(KDialogBase::Close)->setEnabled(!searching);
-    if (mChkbxSpecificFolders->isChecked()) {
-	mCbxFolders->setEnabled(!searching);
-	mChkSubFolders->setEnabled(!searching);
-    }
+    mCbxFolders->setEnabled(!searching);
+    mChkSubFolders->setEnabled(!searching);
     mChkbxAllFolders->setEnabled(!searching);
     mChkbxSpecificFolders->setEnabled(!searching);
     mPatternEdit->setEnabled(!searching);
@@ -703,9 +703,11 @@ void KMFldSearch::slotContextMenuRequested( QListViewItem *lvi, const QPoint &,
 
     mMenuToFolder.clear();
     QPopupMenu *msgMoveMenu = new QPopupMenu(menu);
-    KMMoveCommand::folderToPopupMenu( TRUE, this, &mMenuToFolder, msgMoveMenu );
+    mKMMainWidget->folderTree()->folderToPopupMenu( KMFolderTree::MoveMessage, 
+        this, &mMenuToFolder, msgMoveMenu );
     QPopupMenu *msgCopyMenu = new QPopupMenu(menu);
-    KMCopyCommand::folderToPopupMenu( FALSE, this, &mMenuToFolder, msgCopyMenu );
+    mKMMainWidget->folderTree()->folderToPopupMenu( KMFolderTree::CopyMessage, 
+        this, &mMenuToFolder, msgCopyMenu );
 
     // show most used actions
     mReplyAction->plug(menu);
