@@ -9,13 +9,17 @@
 #include <qdir.h>
 #include <qregexp.h>
 
-#include "kfileio.h"
+#include <libkdepim/kfileio.h>
 #include "kmfoldermaildir.h"
 #include "kmfoldermgr.h"
+#include "kmfolder.h"
 #include "undostack.h"
 #include "maildirjob.h"
 #include "kcursorsaver.h"
+#include "jobscheduler.h"
 using KMail::MaildirJob;
+#include "compactionjob.h"
+
 #include <kio/netaccess.h>
 #include <kapplication.h>
 #include <kdebug.h>
@@ -43,8 +47,8 @@ using KMail::MaildirJob;
 
 
 //-----------------------------------------------------------------------------
-KMFolderMaildir::KMFolderMaildir(KMFolderDir* aParent, const QString& aName)
-  : KMFolderIndex(aParent, aName)
+KMFolderMaildir::KMFolderMaildir(KMFolder* folder, const char* name)
+  : KMFolderIndex(folder, name)
 {
 }
 
@@ -53,26 +57,38 @@ KMFolderMaildir::KMFolderMaildir(KMFolderDir* aParent, const QString& aName)
 KMFolderMaildir::~KMFolderMaildir()
 {
   if (mOpenCount>0) close(TRUE);
-  if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed(this);
+  if (kmkernel->undoStack()) kmkernel->undoStack()->folderDestroyed( folder() );
 }
 
 //-----------------------------------------------------------------------------
 int KMFolderMaildir::canAccess()
 {
 
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
 
-  if (access(QFile::encodeName(location()), R_OK | W_OK | X_OK) != 0)
-    return 1;
+  QString sBadFolderName;
+  if (access(QFile::encodeName(location()), R_OK | W_OK | X_OK) != 0) {
+    sBadFolderName = location();
+  } else if (access(QFile::encodeName(location() + "/new"), R_OK | W_OK | X_OK) != 0) {
+    sBadFolderName = location() + "/new";
+  } else if (access(QFile::encodeName(location() + "/cur"), R_OK | W_OK | X_OK) != 0) {
+    sBadFolderName = location() + "/cur";
+  } else if (access(QFile::encodeName(location() + "/tmp"), R_OK | W_OK | X_OK) != 0) {
+    sBadFolderName = location() + "/tmp";
+  }
 
-  if (access(QFile::encodeName(location() + "/new"), R_OK | W_OK | X_OK) != 0)
-    return 1;
-
-  if (access(QFile::encodeName(location() + "/cur"), R_OK | W_OK | X_OK) != 0)
-    return 1;
-
-  if (access(QFile::encodeName(location() + "/tmp"), R_OK | W_OK | X_OK) != 0)
-    return 1;
+  if ( !sBadFolderName.isEmpty() ) {
+    int nRetVal = QFile::exists(sBadFolderName) ? EPERM : ENOENT;
+    KCursorSaver idle(KBusyPtr::idle());
+    if ( nRetVal == ENOENT )
+      KMessageBox::sorry(0, i18n("Error opening %1; this folder is missing.")
+                         .arg(sBadFolderName));
+    else
+      KMessageBox::sorry(0, i18n("Error opening %1; either this is not a valid "
+                                 "maildir folder, or you do not have sufficient access permissions.")
+                         .arg(sBadFolderName));
+    return nRetVal;
+  }
 
   return 0;
 }
@@ -83,32 +99,32 @@ int KMFolderMaildir::open()
   int rc = 0;
 
   mOpenCount++;
+  kmkernel->jobScheduler()->notifyOpeningFolder( folder() );
+
   if (mOpenCount > 1) return 0;  // already open
 
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
 
-  if (canAccess() != 0) {
-      KCursorSaver idle(KBusyPtr::idle());
-      KMessageBox::sorry(0, i18n("Error opening %1. Either this is not a valid "
-                                 "maildir folder or you don't have sufficient access permissions.")
-                         .arg(name()));
-      return EPERM;
+  rc = canAccess();
+  if ( rc != 0 ) {
+      return rc;
   }
 
-  if (!path().isEmpty())
+  if (!folder()->path().isEmpty())
   {
     if (KMFolderIndex::IndexOk != indexStatus()) // test if contents file has changed
     {
       QString str;
       mIndexStream = 0;
-      str = i18n("Folder `%1' changed. Recreating index.")
+      str = i18n("Folder `%1' changed; recreating index.")
 		  .arg(name());
       emit statusMsg(str);
     } else {
       mIndexStream = fopen(QFile::encodeName(indexLocation()), "r+"); // index file
-      if ( mIndexStream )
+      if ( mIndexStream ) {
         fcntl(fileno(mIndexStream), F_SETFD, FD_CLOEXEC);
-      updateIndexStreamPtr();
+        updateIndexStreamPtr();
+      }
     }
 
     if (!mIndexStream)
@@ -130,51 +146,57 @@ int KMFolderMaildir::open()
 }
 
 
+//-----------------------------------------------------------------------------
+int KMFolderMaildir::createMaildirFolders( const QString & folderPath )
+{
+  // Make sure that neither a new, cur or tmp subfolder exists already.
+  QFileInfo dirinfo;
+  dirinfo.setFile( folderPath + "/new" );
+  if ( dirinfo.exists() ) return EEXIST;
+  dirinfo.setFile( folderPath + "/cur" );
+  if ( dirinfo.exists() ) return EEXIST;
+  dirinfo.setFile( folderPath + "/tmp" );
+  if ( dirinfo.exists() ) return EEXIST;
+
+  // create the maildir directory structure
+  if ( ::mkdir( QFile::encodeName( folderPath ), S_IRWXU ) > 0 ) {
+    kdDebug(5006) << "Could not create folder " << folderPath << endl;
+    return errno;
+  }
+  if ( ::mkdir( QFile::encodeName( folderPath + "/new" ), S_IRWXU ) > 0 ) {
+    kdDebug(5006) << "Could not create folder " << folderPath << "/new" << endl;
+    return errno;
+  }
+  if ( ::mkdir( QFile::encodeName( folderPath + "/cur" ), S_IRWXU ) > 0 ) {
+    kdDebug(5006) << "Could not create folder " << folderPath << "/cur" << endl;
+    return errno;
+  }
+  if ( ::mkdir( QFile::encodeName( folderPath + "/tmp" ), S_IRWXU ) > 0 ) {
+    kdDebug(5006) << "Could not create folder " << folderPath << "/tmp" << endl;
+    return errno;
+  }
+
+  return 0; // no error
+}
+
 //-----------------------------------------------------------------------------
 int KMFolderMaildir::create(bool imap)
 {
   int rc;
   int old_umask;
 
-  assert(!name().isEmpty());
+  assert(!folder()->name().isEmpty());
   assert(mOpenCount == 0);
 
-  // Make sure that neither a new, cur or tmp subfolder exists already.
-  QFileInfo dirinfo;
-  dirinfo.setFile(location() + "/new");
-  if (dirinfo.exists()) return 1;
-  dirinfo.setFile(location() + "/cur");
-  if (dirinfo.exists()) return 1;
-  dirinfo.setFile(location() + "/tmp");
-  if (dirinfo.exists()) return 1;
+  rc = createMaildirFolders( location() );
+  if ( rc != 0 )
+    return rc;
 
-  // create the maildir directory structure
-  if (::mkdir(QFile::encodeName(location()), S_IRWXU) > 0)
-  {
-    kdDebug(5006) << "Could not create " << location() << " maildir" << endl;
-    return errno;
-  }
-  if (::mkdir(QFile::encodeName(location() + "/new"), S_IRWXU) > 0)
-  {
-    kdDebug(5006) << "Could not create " << location() << "/new" << endl;
-    return errno;
-  }
-  if (::mkdir(QFile::encodeName(location() + "/cur"), S_IRWXU) > 0)
-  {
-    kdDebug(5006) << "Could not create " << location() << "/cur" << endl;
-    return errno;
-  }
-  if (::mkdir(QFile::encodeName(location() + "/tmp"), S_IRWXU) > 0)
-  {
-    kdDebug(5006) << "Could not create " << location() << "/new" << endl;
-    return errno;
-  }
-
-  if (!path().isEmpty())
+  if (!folder()->path().isEmpty())
   {
     old_umask = umask(077);
     mIndexStream = fopen(QFile::encodeName(indexLocation()), "w+"); //sven; open RW
-	updateIndexStreamPtr(TRUE);
+    updateIndexStreamPtr(TRUE);
     umask(old_umask);
 
     if (!mIndexStream) return errno;
@@ -202,12 +224,17 @@ void KMFolderMaildir::close(bool aForced)
 {
   if (mOpenCount <= 0) return;
   if (mOpenCount > 0) mOpenCount--;
+
   if (mOpenCount > 0 && !aForced) return;
-  if ((this != kmkernel->inboxFolder()) && isSystemFolder() && !aForced)
+
+#if 0 // removed hack that prevented closing system folders (see kmail-devel discussion about mail expiring)
+  if ( (folder() != kmkernel->inboxFolder())
+       && folder()->isSystemFolder() && !aForced)
   {
      mOpenCount = 1;
      return;
   }
+#endif
 
   if (mAutoCreateIndex)
   {
@@ -234,7 +261,7 @@ void KMFolderMaildir::sync()
 {
   if (mOpenCount > 0)
     if (!mIndexStream || fsync(fileno(mIndexStream))) {
-    kmkernel->emergencyExit( i18n("Couldn't sync maildir folder.") );
+    kmkernel->emergencyExit( i18n("Could not sync maildir folder.") );
     }
 }
 
@@ -257,23 +284,16 @@ int KMFolderMaildir::expungeContents()
   return 0;
 }
 
-//-----------------------------------------------------------------------------
-int KMFolderMaildir::compact()
+int KMFolderMaildir::compact( unsigned int startIndex, int nbMessages, const QStringList& entryList, bool& done )
 {
-  if (needsCompact == false)
-    return 0;
-
-  open();
-
   QString subdirNew(location() + "/new/");
   QString subdirCur(location() + "/cur/");
 
-  QDir d(subdirNew);
-  QStringList newFiles(d.entryList());
-
-  for (int i = 0; i < count(); i++)
-  {
-    KMMsgInfo *mi = (KMMsgInfo*)mMsgList[i];
+  unsigned int stopIndex = nbMessages == -1 ? mMsgList.count() :
+                           QMIN( mMsgList.count(), startIndex + nbMessages );
+  //kdDebug(5006) << "KMFolderMaildir: compacting from " << startIndex << " to " << stopIndex << endl;
+  for(unsigned int idx = startIndex; idx < stopIndex; ++idx) {
+    KMMsgInfo* mi = (KMMsgInfo*)mMsgList.at(idx);
     if (!mi)
       continue;
 
@@ -282,12 +302,12 @@ int KMFolderMaildir::compact()
       continue;
 
     // first, make sure this isn't in the 'new' subdir
-    if ( newFiles.contains( filename ) )
+    if ( entryList.contains( filename ) )
       moveInternal(subdirNew + filename, subdirCur + filename, mi);
 
     // construct a valid filename.  if it's already valid, then
     // nothing happens
-    constructValidFileName(filename, mi->status());
+    filename = constructValidFileName( filename, mi->status() );
 
     // if the name changed, then we need to update the actual filename
     if (filename != mi->fileName())
@@ -297,20 +317,28 @@ int KMFolderMaildir::compact()
       setDirty( true );
     }
 
+#if 0
     // we can't have any New messages at this point
     if (mi->isNew())
     {
       mi->setStatus(KMMsgStatusUnread);
       setDirty( true );
     }
+#endif
   }
-  close();
-
-  needsCompact = false;
-
+  done = ( stopIndex == mMsgList.count() );
   return 0;
 }
 
+//-----------------------------------------------------------------------------
+int KMFolderMaildir::compact( bool silent )
+{
+  KMail::MaildirCompactionJob* job = new KMail::MaildirCompactionJob( folder(), true /*immediate*/ );
+  int rc = job->executeNow( silent );
+  // Note that job autodeletes itself.
+  return rc;
+}
+
 //-------------------------------------------------------------
 FolderJob*
 KMFolderMaildir::doCreateJob( KMMessage *msg, FolderJob::JobType jt,
@@ -333,6 +361,14 @@ KMFolderMaildir::doCreateJob( QPtrList<KMMessage>& msgList, const QString& sets,
 
 //-------------------------------------------------------------
 int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
+{
+  if (!canAddMsgNow(aMsg, index_return)) return 0;
+  return addMsgInternal( aMsg, index_return );
+}
+
+//-------------------------------------------------------------
+int KMFolderMaildir::addMsgInternal( KMMessage* aMsg, int* index_return,
+                                     bool stripUid )
 {
 /*
 QFile fileD0( "testdat_xx-kmfoldermaildir-0" );
@@ -342,8 +378,6 @@ if( fileD0.open( IO_WriteOnly ) ) {
     fileD0.close();  // If data is 0 we just create a zero length file.
 }
 */
-  if (!canAddMsgNow(aMsg, index_return)) return 0;
-
   long len;
   unsigned long size;
   bool opened = FALSE;
@@ -356,7 +390,7 @@ if( fileD0.open( IO_WriteOnly ) ) {
   msgParent = aMsg->parent();
   if (msgParent)
   {
-    if (msgParent==this && !kmkernel->folderIsDraftOrOutbox(this))
+    if (msgParent==folder() && !kmkernel->folderIsDraftOrOutbox(folder()))
         return 0;
 
     idx = msgParent->find(aMsg);
@@ -366,9 +400,20 @@ if( fileD0.open( IO_WriteOnly ) ) {
   aMsg->setStatusFields();
   if (aMsg->headerField("Content-Type").isEmpty())  // This might be added by
     aMsg->removeHeaderField("Content-Type");        // the line above
+
+
+  const QString uidHeader = aMsg->headerField( "X-UID" );
+  if ( !uidHeader.isEmpty() && stripUid )
+    aMsg->removeHeaderField( "X-UID" );
+
   msgText = aMsg->asString();
   len = msgText.length();
 
+  // Re-add the uid so that the take can make use of it, in case the
+  // message is currently in an imap folder
+  if ( !uidHeader.isEmpty() && stripUid )
+    aMsg->setHeaderField( "X-UID", uidHeader );
+
   if (len <= 0)
   {
     kdDebug(5006) << "Message added to folder `" << name() << "' contains no data. Ignoring it." << endl;
@@ -376,14 +421,13 @@ if( fileD0.open( IO_WriteOnly ) ) {
   }
 
   // make sure the filename has the correct extension
-  QString filename(aMsg->fileName());
-  constructValidFileName(filename, aMsg->status());
+  QString filename = constructValidFileName( aMsg->fileName(), aMsg->status() );
 
   QString tmp_file(location() + "/tmp/");
   tmp_file += filename;
 
-  if (!kCStringToFile(msgText, tmp_file, false, false, false))
-    kmkernel->emergencyExit( "" ); // kCStringToFile already showed an errormessage
+  if (!KPIM::kCStringToFile(msgText, tmp_file, false, false, false))
+    kmkernel->emergencyExit( "" ); // KPIM::kCStringToFile already showed an errormessage
 
   QFile file(tmp_file);
   size = msgText.length();
@@ -392,7 +436,7 @@ if( fileD0.open( IO_WriteOnly ) ) {
   {
     opened = TRUE;
     rc = open();
-    kdDebug(5006) << "addMsg-open: " << rc << endl;
+    kdDebug(5006) << "KMFolderMaildir::addMsg-open: " << rc << " of folder: " << label() << endl;
     if (rc) return rc;
   }
 
@@ -409,21 +453,24 @@ if( fileD0.open( IO_WriteOnly ) ) {
   if (msgParent)
     if (idx >= 0) msgParent->take(idx);
 
+  // just to be sure it does not end up in the index
+  if ( stripUid ) aMsg->setUID( 0 );
+
   if (filename != aMsg->fileName())
     aMsg->setFileName(filename);
 
-  if (aMsg->isUnread() || aMsg->isNew() || this == kmkernel->outboxFolder())
+  if (aMsg->isUnread() || aMsg->isNew() || folder() == kmkernel->outboxFolder())
   {
     if (mUnreadMsgs == -1)
       mUnreadMsgs = 1;
     else
       ++mUnreadMsgs;
-    emit numUnreadMsgsChanged( this );
+    emit numUnreadMsgsChanged( folder() );
   }
   ++mTotalMsgs;
 
   // store information about the position in the folder file in the message
-  aMsg->setParent(this);
+  aMsg->setParent(folder());
   aMsg->setMsgSize(size);
   idx = mMsgList.append(&aMsg->toMsgBase());
   if (aMsg->getMsgSerNum() <= 0)
@@ -496,10 +543,9 @@ if( fileD1.open( IO_WriteOnly ) ) {
 KMMessage* KMFolderMaildir::readMsg(int idx)
 {
   KMMsgInfo* mi = (KMMsgInfo*)mMsgList[idx];
-  KMMessage *msg = new KMMessage(*mi);
-
+  KMMessage *msg = new KMMessage(*mi); // note that mi is deleted by the line below
+  mMsgList.set(idx,&msg->toMsgBase()); // done now so that the serial number can be computed
   msg->fromDwString(getDwString(idx));
-  mMsgList.set(idx,&msg->toMsgBase());
   return msg;
 }
 
@@ -548,7 +594,7 @@ QCString& KMFolderMaildir::getMsgString(int idx, QCString& mDest)
 
   QFileInfo fi( abs_file );
   mDest.resize(fi.size()+2);
-  mDest = kFileToString(abs_file, false, false);
+  mDest = KPIM::kFileToString(abs_file, false, false);
   size_t newMsgSize = crlf2lf( mDest.data(), fi.size() );
   mDest[newMsgSize] = '\0';
   return mDest;
@@ -573,7 +619,13 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
 
   // open the file and get a pointer to it
   QFile f(file);
-  if (f.open(IO_ReadOnly) == false) return;
+  if ( f.open( IO_ReadOnly ) == false ) {
+    kdWarning(5006) << "The file '" << QFile::encodeName(dir) << "/" << file
+                    << "' could not be opened for reading the message. "
+                       "Please check ownership and permissions."
+                    << endl;
+    return;
+  }
 
   char line[MAX_LINE];
   bool atEof    = false;
@@ -582,7 +634,7 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
 
   QCString dateStr, fromStr, toStr, subjStr;
   QCString xmarkStr, replyToIdStr, msgIdStr, referencesStr;
-  QCString statusStr, replyToAuxIdStr;
+  QCString statusStr, replyToAuxIdStr, uidStr;
 
   // iterate through this file until done
   while (!atEof)
@@ -655,7 +707,7 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
           status |= KMMsgStatusFlag;
       }
 
-      KMMsgInfo *mi = new KMMsgInfo(this);
+      KMMsgInfo *mi = new KMMsgInfo(folder());
       mi->init( subjStr.stripWhiteSpace(),
                 fromStr.stripWhiteSpace(),
                 toStr.stripWhiteSpace(),
@@ -669,6 +721,8 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
       dateStr = dateStr.stripWhiteSpace();
       if (!dateStr.isEmpty())
         mi->setDate(dateStr);
+      if ( !uidStr.isEmpty() )
+         mi->setUID( uidStr.toULong() );
       mi->setDirty(false);
       mMsgList.append(mi);
 
@@ -746,10 +800,16 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
       replyToIdStr = QCString(line+12);
       lastStr = &replyToIdStr;
     }
+    else if (strncasecmp(line, "X-UID:", 6) == 0)
+    {
+      uidStr = QCString(line+6);
+      lastStr = &uidStr;
+    }
+
   }
 
   if (status & KMMsgStatusNew || status & KMMsgStatusUnread ||
-      (this == kmkernel->outboxFolder()))
+      (folder() == kmkernel->outboxFolder()))
   {
     mUnreadMsgs++;
    if (mUnreadMsgs == 0) ++mUnreadMsgs;
@@ -819,15 +879,15 @@ int KMFolderMaildir::createIndexFromContents()
 
   correctUnreadMsgsCount();
 
-  if (kmkernel->outboxFolder() == this && count() > 0)
+  if (kmkernel->outboxFolder() == folder() && count() > 0)
     KMessageBox::information(0, i18n("Your outbox contains messages which were "
-    "most likely not created by KMail.\nPlease remove them from there, if you "
-    "don't want KMail to send them."));
+    "most-likely not created by KMail;\nplease remove them from there if you "
+    "do not want KMail to send them."));
 
   needsCompact = true;
 
-  if (parent())
-    parent()->manager()->invalidateFolder(kmkernel->msgDict(), this);
+  if (folder()->parent())
+    folder()->parent()->manager()->invalidateFolder(kmkernel->msgDict(), folder());
   return 0;
 }
 
@@ -874,59 +934,91 @@ KMMessage* KMFolderMaildir::take(int idx)
     return 0;
 }
 
-bool KMFolderMaildir::removeFile(const QString& filename)
+// static
+bool KMFolderMaildir::removeFile( const QString & folderPath,
+                                  const QString & filename )
 {
   // we need to look in both 'new' and 'cur' since it's possible to
-  // delete a message before the folder is compacted.  since the file
+  // delete a message before the folder is compacted. Since the file
   // naming and moving is done in ::compact, we can't assume any
-  // location at this point
-  QCString abs_file(QFile::encodeName(location() + "/cur/"));
-  abs_file += QFile::encodeName(filename);
+  // location at this point.
+  QCString abs_file( QFile::encodeName( folderPath + "/cur/" + filename ) );
+  if ( ::unlink( abs_file ) == 0 )
+    return true;
 
-  if (::unlink( abs_file ) == 0)
+  if ( errno == ENOENT ) { // doesn't exist
+    abs_file = QFile::encodeName( folderPath + "/new/" + filename );
+    if ( ::unlink( abs_file ) == 0 )
       return true;
-
-  if (errno == ENOENT)  {// doesn't exist
-
-    abs_file = QFile::encodeName(location() + "/new/");
-    abs_file += QFile::encodeName(filename);
-
-    if (::unlink( abs_file ) == 0)
-        return true;
-
   }
 
   kdDebug(5006) << "Can't delete " << abs_file << " " << perror << endl;
   return false;
 }
 
+bool KMFolderMaildir::removeFile( const QString & filename )
+{
+  return removeFile( location(), filename );
+}
+
+#include <sys/types.h>
+#include <dirent.h>
+static bool removeDirAndContentsRecursively( const QString & path )
+{
+  bool success = true;
+
+  QDir d;
+  d.setPath( path );
+  d.setFilter( QDir::Files | QDir::Dirs | QDir::Hidden | QDir::NoSymLinks );
+
+  const QFileInfoList *list = d.entryInfoList();
+  QFileInfoListIterator it( *list );
+  QFileInfo *fi;
+
+  while ( (fi = it.current()) != 0 ) {
+    if( fi->isDir() ) {
+      if ( fi->fileName() != "." && fi->fileName() != ".." )
+        success = success && removeDirAndContentsRecursively( fi->absFilePath() );
+    } else {
+      success = success && d.remove( fi->absFilePath() );
+    }
+    ++it;
+  }
+
+  if ( success ) {
+    success = success && d.rmdir( path ); // nuke ourselves, we should be empty now
+  }
+  return success;
+}
+
 //-----------------------------------------------------------------------------
 int KMFolderMaildir::removeContents()
 {
-    if (!KIO::NetAccess::del(KURL::fromPathOrURL(location()+ "/new/"), 0))
-        return 1;
-    if (!KIO::NetAccess::del(KURL::fromPathOrURL(location()+ "/cur/"), 0))
-        return 1;
-    if (!KIO::NetAccess::del(KURL::fromPathOrURL(location()+ "/tmp/"), 0))
-        return 1;
-
-    /* The subdirs are removed now. Check if there is anything else in the dir
-     * and only if not delete the dir itself. The user could have data stored
-     * that would otherwise be deleted. */
-    QDir dir(location());
-    if ( dir.count() == 2 ) { // only . and ..
-        if (!KIO::NetAccess::del(KURL::fromPathOrURL(location()), 0))
-            return 1;
-    }
-    return 0;
+  // NOTE: Don' use KIO::netaccess, it has reentrancy problems and multiple
+  // mailchecks going on trigger them, when removing dirs
+  if ( !removeDirAndContentsRecursively( location() + "/new/" ) ) return 1;
+  if ( !removeDirAndContentsRecursively( location() + "/cur/" ) ) return 1;
+  if ( !removeDirAndContentsRecursively( location() + "/tmp/" ) ) return 1;
+  /* The subdirs are removed now. Check if there is anything else in the dir
+   * and only if not delete the dir itself. The user could have data stored
+   * that would otherwise be deleted. */
+  QDir dir(location());
+  if ( dir.count() == 2 ) { // only . and ..
+    if ( !removeDirAndContentsRecursively( location() ), 0 ) return 1;
+  }
+  return 0;
 }
 
 static QRegExp *suffix_regex = 0;
 static KStaticDeleter<QRegExp> suffix_regex_sd;
 
 //-----------------------------------------------------------------------------
-QString KMFolderMaildir::constructValidFileName(QString& aFileName, KMMsgStatus status)
+// static
+QString KMFolderMaildir::constructValidFileName( const QString & filename,
+                                                 KMMsgStatus status )
 {
+  QString aFileName( filename );
+
   if (aFileName.isEmpty())
   {
     aFileName.sprintf("%ld.%d.", (long)time(0), getpid());
@@ -938,18 +1030,17 @@ QString KMFolderMaildir::constructValidFileName(QString& aFileName, KMMsgStatus
 
   aFileName.truncate(aFileName.findRev(*suffix_regex));
 
-  QString suffix;
+  // only add status suffix if the message is neither new nor unread
   if (! ((status & KMMsgStatusNew) || (status & KMMsgStatusUnread)) )
   {
-    suffix += ":2,";
+    QString suffix( ":2," );
     if (status & KMMsgStatusReplied)
       suffix += "RS";
     else
       suffix += "S";
+    aFileName += suffix;
   }
 
-  aFileName += suffix;
-
   return aFileName;
 }
 
@@ -972,8 +1063,7 @@ QString KMFolderMaildir::moveInternal(const QString& oldLoc, const QString& newL
   // make sure that our destination filename doesn't already exist
   while (QFile::exists(dest))
   {
-    aFileName = "";
-    constructValidFileName(aFileName, status);
+    aFileName = constructValidFileName( QString(), status );
 
     QFileInfo fi(dest);
     dest = fi.dirPath(true) + "/" + aFileName;
