@@ -2,8 +2,10 @@
 /* Author: Ronen Tzur <rtzur@shani.net> */
 
 #include "kmfolderindex.h"
+#include "kmfolder.h"
 #include "kmmsgdict.h"
 #include "kmdict.h"
+#include "globalsettings.h"
 
 #include <qfileinfo.h>
 
@@ -127,11 +129,15 @@ public:
   off_t baseOffset;
 };
 
+
 //-----------------------------------------------------------------------------
 
 KMMsgDict::KMMsgDict()
 {
-  dict = new KMDict(9973);
+  int lastSizeOfDict = GlobalSettings::msgDictSizeHint();
+  lastSizeOfDict = ( lastSizeOfDict * 11 ) / 10;
+  GlobalSettings::setMsgDictSizeHint( 0 );
+  dict = new KMDict( lastSizeOfDict );
   nextMsgSerNum = 1;
 }
 
@@ -172,7 +178,7 @@ unsigned long KMMsgDict::insert(unsigned long msgSerNum,
       nextMsgSerNum = msn + 1;
   }
 
-  KMFolderIndex *folder = msg->parent();
+  KMFolderIndex* folder = static_cast<KMFolderIndex*>( msg->storage() );
   if (folder && index == -1)
     index = folder->find(msg);
 
@@ -182,14 +188,10 @@ unsigned long KMMsgDict::insert(unsigned long msgSerNum,
     folder->setDirty( true ); // rewrite id file
   }
 
-  // Should not happen, indicates id file corruption
-  while (dict->find((long)msn)) {
-    msn = getNextMsgSerNum();
-    folder->setDirty( true ); // rewrite id file
-  }
-
-  KMMsgDictEntry *entry = new KMMsgDictEntry(folder, index);
-  dict->replace((long)msn, entry);
+  // Insert into the dict. Don't use dict->replace() as we _know_
+  // there is no entry with the same msn, we just made sure.
+  KMMsgDictEntry *entry = new KMMsgDictEntry(folder->folder(), index);
+  dict->insert((long)msn, entry);
 
   KMMsgDictREntry *rentry = folder->rDict();
   if (!rentry) {
@@ -209,6 +211,28 @@ unsigned long KMMsgDict::insert(const KMMsgBase *msg, int index)
 
 //-----------------------------------------------------------------------------
 
+void KMMsgDict::replace(unsigned long msgSerNum,
+		       const KMMsgBase *msg, int index)
+{
+  KMFolderIndex* folder = static_cast<KMFolderIndex*>( msg->storage() );
+  if ( folder && index == -1 )
+    index = folder->find( msg );
+
+  remove( msgSerNum );
+  KMMsgDictEntry *entry = new KMMsgDictEntry( folder->folder(), index );
+  dict->insert( (long)msgSerNum, entry );
+
+  KMMsgDictREntry *rentry = folder->rDict();
+  if (!rentry) {
+    rentry = new KMMsgDictREntry();
+    folder->setRDict(rentry);
+  }
+  rentry->set(index, entry);
+
+}
+
+//-----------------------------------------------------------------------------
+
 void KMMsgDict::remove(unsigned long msgSerNum)
 {
   long key = (long)msgSerNum;
@@ -364,13 +388,18 @@ int KMMsgDict::readFolderIds(KMFolder *folder)
     //if (!msn)
       //kdDebug(5006) << "Dict found zero serial number in folder " << folder->label() << endl;
 
+    // Insert into the dict. Don't use dict->replace() as we _know_
+    // there is no entry with the same msn, we just made sure.
     KMMsgDictEntry *entry = new KMMsgDictEntry(folder, index);
-    dict->replace((long)msn, entry);
+    dict->insert((long)msn, entry);
     if (msn >= nextMsgSerNum)
       nextMsgSerNum = msn + 1;
 
     rentry->set(index, entry);
   }
+  // Remember how many items we put into the dict this time so we can create 
+  // it with an appropriate size next time.
+  GlobalSettings::setMsgDictSizeHint( GlobalSettings::msgDictSizeHint() + count );
 
   fclose(fp);
   folder->setRDict(rentry);
