@@ -24,76 +24,90 @@
 #include "kmenubar.h"
 #include "kpopupmenu.h"
 #include "kmreaderwin.h"
-#include "kmfolderindex.h"
+#include "kmfolder.h"
+#include "kmmainwidget.h"
+#include "kmfoldertree.h"
 
 #include "kmreadermainwin.h"
-#include "kmreadermainwin.moc"
 
-KMReaderMainWin::KMReaderMainWin( bool htmlOverride, char *name )
-  : KMTopLevelWidget( name ), mMsg( 0 )
+KMReaderMainWin::KMReaderMainWin( bool htmlOverride, bool htmlLoadExtOverride,
+                                  char *name )
+  : KMail::SecondaryWindow( name ? name : "readerwindow#" ),
+    mMsg( 0 )
 {
-  KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
   mReaderWin = new KMReaderWin( this, this, actionCollection() );
   //mReaderWin->setShowCompleteMessage( true );
   mReaderWin->setAutoDelete( true );
   mReaderWin->setHtmlOverride( htmlOverride );
-  setCentralWidget( mReaderWin );
-  setupAccel();
-
-  connect( kmkernel, SIGNAL( configChanged() ),
-           this, SLOT( slotConfigChanged() ) );
+  mReaderWin->setHtmlLoadExtOverride( htmlLoadExtOverride );
+  initKMReaderMainWin();
 }
 
 
+//-----------------------------------------------------------------------------
 KMReaderMainWin::KMReaderMainWin( char *name )
-  : KMTopLevelWidget( name ), mMsg( 0 )
+  : KMail::SecondaryWindow( name ? name : "readerwindow#" ),
+    mMsg( 0 )
 {
   mReaderWin = new KMReaderWin( this, this, actionCollection() );
   mReaderWin->setAutoDelete( true );
-  setCentralWidget( mReaderWin );
-  setupAccel();
-
-  connect( kmkernel, SIGNAL( configChanged() ),
-           this, SLOT( slotConfigChanged() ) );
+  initKMReaderMainWin();
 }
 
 
+//-----------------------------------------------------------------------------
 KMReaderMainWin::KMReaderMainWin(KMMessagePart* aMsgPart,
     bool aHTML, const QString& aFileName, const QString& pname,
     const QTextCodec *codec, char *name )
-  : KMTopLevelWidget( name ), mMsg( 0 )
+  : KMail::SecondaryWindow( name ? name : "readerwindow#" ),
+    mMsg( 0 )
 {
-  resize( 550, 600 );
-  mReaderWin = new KMReaderWin( this, this, actionCollection() ); //new reader
+  mReaderWin = new KMReaderWin( this, this, actionCollection() );
   mReaderWin->setOverrideCodec( codec );
   mReaderWin->setMsgPart( aMsgPart, aHTML, aFileName, pname );
+  initKMReaderMainWin();
+}
+
+
+//-----------------------------------------------------------------------------
+void KMReaderMainWin::initKMReaderMainWin() {
   setCentralWidget( mReaderWin );
   setupAccel();
+  setupGUI( ToolBar | Keys | StatusBar | Create, "kmreadermainwin.rc" );
+  applyMainWindowSettings( KMKernel::config(), "Separate Reader Window" );
 
   connect( kmkernel, SIGNAL( configChanged() ),
            this, SLOT( slotConfigChanged() ) );
 }
 
-
+//-----------------------------------------------------------------------------
 KMReaderMainWin::~KMReaderMainWin()
 {
-  saveMainWindowSettings(KMKernel::config(), "Separate Reader Window");
+  saveMainWindowSettings( KMKernel::config(), "Separate Reader Window" );
 }
 
+//-----------------------------------------------------------------------------
+void KMReaderMainWin::setUseFixedFont( bool useFixedFont )
+{
+  mReaderWin->setUseFixedFont( useFixedFont );
+}
 
+//-----------------------------------------------------------------------------
 void KMReaderMainWin::showMsg( const QTextCodec *codec, KMMessage *msg )
 {
   mReaderWin->setOverrideCodec( codec );
   mReaderWin->setMsg( msg, true );
   setCaption( msg->subject() );
   mMsg = msg;
+  toolBar( "mainToolBar" )->show();
 }
 
 //-----------------------------------------------------------------------------
 void KMReaderMainWin::slotPrintMsg()
 {
   KMCommand *command = new KMPrintCommand( this, mReaderWin->message(),
-      mReaderWin->htmlOverride() );
+      mReaderWin->htmlOverride(),  mReaderWin->htmlLoadExtOverride(),
+      mReaderWin->overrideCodec() );
   command->start();
 }
 
@@ -163,49 +177,60 @@ void KMReaderMainWin::slotRedirectMsg()
   command->start();
 }
 
-
 //-----------------------------------------------------------------------------
-void KMReaderMainWin::slotBounceMsg()
+void KMReaderMainWin::slotShowMsgSrc()
 {
-  KMCommand *command = new KMBounceCommand( this, mReaderWin->message() );
+  KMMessage *msg = mReaderWin->message();
+  if ( !msg )
+    return;
+  KMCommand *command = new KMShowMsgSrcCommand( this, msg,
+                                                mReaderWin->isFixedFont() );
   command->start();
 }
 
 //-----------------------------------------------------------------------------
 void KMReaderMainWin::slotConfigChanged()
 {
-  readConfig();
+  //readConfig();
 }
 
 void KMReaderMainWin::setupAccel()
 {
-  if (kmkernel->xmlGuiInstance())
+  if ( kmkernel->xmlGuiInstance() )
     setInstance( kmkernel->xmlGuiInstance() );
-  KStdAction::close( this, SLOT( close() ), actionCollection() );
-  applyMainWindowSettings(KMKernel::config(), "Separate Reader Window");
-  QAccel *accel = new QAccel(mReaderWin, "showMsg()");
-  accel->connectItem(accel->insertItem(Key_Up),
-                     mReaderWin, SLOT(slotScrollUp()));
-  accel->connectItem(accel->insertItem(Key_Down),
-                     mReaderWin, SLOT(slotScrollDown()));
-  accel->connectItem(accel->insertItem(Key_Prior),
-                     mReaderWin, SLOT(slotScrollPrior()));
-  accel->connectItem(accel->insertItem(Key_Next),
-                     mReaderWin, SLOT(slotScrollNext()));
-  accel->connectItem(accel->insertItem(KStdAccel::shortcut(KStdAccel::Copy)),
-                     mReaderWin, SLOT(slotCopySelectedText()));
-  connect( mReaderWin, SIGNAL(popupMenu(KMMessage&,const KURL&,const QPoint&)),
-	  this, SLOT(slotMsgPopup(KMMessage&,const KURL&,const QPoint&)));
-  connect(mReaderWin, SIGNAL(urlClicked(const KURL&,int)),
-	  mReaderWin, SLOT(slotUrlClicked()));
+
+  //----- File Menu
+  //mOpenAction = KStdAction::open( this, SLOT( slotOpenMsg() ),
+  //                                actionCollection() );
+
+  //mSaveAsAction = new KAction( i18n("Save &As..."), "filesave",
+  //                             KStdAccel::shortcut( KStdAccel::Save ),
+  //                             this, SLOT( slotSaveMsg() ),
+  //                             actionCollection(), "file_save_as" );
+
+  mPrintAction = KStdAction::print( this, SLOT( slotPrintMsg() ),
+                                    actionCollection() );
+
+  KAction *closeAction = KStdAction::close( this, SLOT( close() ), actionCollection() );
+  KShortcut closeShortcut = closeAction->shortcut();
+  closeShortcut.append( KKey(Key_Escape));
+  closeAction->setShortcut(closeShortcut);
+
+  //----- View Menu
+  mViewSourceAction = new KAction( i18n("&View Source"), Key_V, this,
+                                   SLOT(slotShowMsgSrc()), actionCollection(),
+                                   "view_source" );
+
 
   mForwardActionMenu = new KActionMenu( i18n("Message->","&Forward"),
 					"mail_forward", actionCollection(),
 					"message_forward" );
+  connect( mForwardActionMenu, SIGNAL( activated() ), this,
+           SLOT( slotForwardMsg() ) );
 
   mForwardAction = new KAction( i18n("&Inline..."), "mail_forward",
 				SHIFT+Key_F, this, SLOT(slotForwardMsg()),
-				actionCollection(), "message_forward" );
+				actionCollection(), "message_forward_inline" );
   mForwardActionMenu->insert( mForwardAction );
 
   mForwardAttachedAction = new KAction( i18n("Message->Forward->","As &Attachment..."),
@@ -219,10 +244,6 @@ void KMReaderMainWin::setupAccel()
 				 actionCollection(), "message_forward_redirect" );
   mForwardActionMenu->insert( mRedirectAction );
 
-  mBounceAction = new KAction( i18n("&Bounce..."), 0, this,
-			      SLOT(slotBounceMsg()), actionCollection(), "bounce" );
-
-
   mReplyActionMenu = new KActionMenu( i18n("Message->","&Reply"),
                                       "mail_reply", actionCollection(),
                                       "message_reply_menu" );
@@ -250,9 +271,23 @@ void KMReaderMainWin::setupAccel()
 				  "reply_list" );
   mReplyActionMenu->insert( mReplyListAction );
 
-  mPrintAction = KStdAction::print (this, SLOT(slotPrintMsg()), actionCollection());
-  createGUI( "kmreadermainwin.rc" );
-  menuBar()->hide();
+
+
+  QAccel *accel = new QAccel(mReaderWin, "showMsg()");
+  accel->connectItem(accel->insertItem(Key_Up),
+                     mReaderWin, SLOT(slotScrollUp()));
+  accel->connectItem(accel->insertItem(Key_Down),
+                     mReaderWin, SLOT(slotScrollDown()));
+  accel->connectItem(accel->insertItem(Key_Prior),
+                     mReaderWin, SLOT(slotScrollPrior()));
+  accel->connectItem(accel->insertItem(Key_Next),
+                     mReaderWin, SLOT(slotScrollNext()));
+  accel->connectItem(accel->insertItem(KStdAccel::shortcut(KStdAccel::Copy)),
+                     mReaderWin, SLOT(slotCopySelectedText()));
+  connect( mReaderWin, SIGNAL(popupMenu(KMMessage&,const KURL&,const QPoint&)),
+	  this, SLOT(slotMsgPopup(KMMessage&,const KURL&,const QPoint&)));
+  connect(mReaderWin, SIGNAL(urlClicked(const KURL&,int)),
+	  mReaderWin, SLOT(slotUrlClicked()));
 
 }
 
@@ -263,13 +298,16 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KURL &aUrl, const QPoi
   mUrl = aUrl;
   mMsg = &aMsg;
 
-  if (!aUrl.isEmpty()) {
+  if(mReaderWin && !mReaderWin->copyText().isEmpty()) {
+    mReaderWin->copyAction()->plug( menu );
+    mReaderWin->selectAllAction()->plug( menu );
+  } else if (!aUrl.isEmpty()) {
     if (aUrl.protocol() == "mailto") {
       // popup on a mailto URL
       mReaderWin->mailToComposeAction()->plug( menu );
       if ( mMsg ) {
-	mReaderWin->mailToReplyAction()->plug( menu );
-	mReaderWin->mailToForwardAction()->plug( menu );
+        mReaderWin->mailToReplyAction()->plug( menu );
+        mReaderWin->mailToForwardAction()->plug( menu );
         menu->insertSeparator();
       }
       mReaderWin->addAddrBookAction()->plug( menu );
@@ -291,22 +329,21 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KURL &aUrl, const QPoi
       return;
     }
 
-    mReplyAction->plug( menu );
-    mReplyAllAction->plug( menu );
-    mReplyAuthorAction->plug( menu );
-    mReplyListAction->plug( menu );
+    mReplyActionMenu->plug( menu );
     mForwardActionMenu->plug( menu );
-    mBounceAction->plug( menu );
 
     menu->insertSeparator();
 
     QPopupMenu* copyMenu = new QPopupMenu(menu);
-    KMMenuCommand::folderToPopupMenu( false, this, &mMenuToFolder, copyMenu );
+    KMMainWidget* mainwin = kmkernel->getKMMainWidget();
+    if ( mainwin )
+      mainwin->folderTree()->folderToPopupMenu( KMFolderTree::CopyMessage, this,
+          &mMenuToFolder, copyMenu );
     menu->insertItem( i18n("&Copy To" ), copyMenu );
     menu->insertSeparator();
+    mViewSourceAction->plug( menu );
     mReaderWin->toggleFixFontAction()->plug( menu );
-    mReaderWin->viewSourceAction()->plug( menu );
-
+    menu->insertSeparator();
     mPrintAction->plug( menu );
     menu->insertItem(  SmallIcon("filesaveas"), i18n( "Save &As..." ), mReaderWin, SLOT( slotSaveMsg() ) );
     menu->insertItem( i18n("Save Attachments..."), mReaderWin, SLOT(slotSaveAttachments()) );
@@ -323,3 +360,5 @@ void KMReaderMainWin::copySelectedToFolder( int menuId )
   KMCommand *command = new KMCopyCommand( mMenuToFolder[menuId], mMsg );
   command->start();
 }
+
+#include "kmreadermainwin.moc"
