@@ -22,15 +22,19 @@ class QScrollBar;
 class QString;
 class QTabDialog;
 class QTextCodec;
+
 class DwHeaders;
 class DwMediaType;
+
 class KActionCollection;
 class KAction;
 class KActionMenu;
+class KRadioAction;
 class KToggleAction;
 class KConfigBase;
 class KHTMLPart;
 class KURL;
+
 class KMFolder;
 class KMMessage;
 class KMMessagePart;
@@ -167,9 +171,16 @@ public:
   bool htmlOverride() const { return mHtmlOverride; }
   void setHtmlOverride( bool override );
 
+  /** Override default load external references setting */
+  bool htmlLoadExtOverride() const { return mHtmlLoadExtOverride; }
+  void setHtmlLoadExtOverride( bool override );
+
   /** Is html mail to be supported? Takes into account override */
   bool htmlMail();
 
+  /** Is loading ext. references to be supported? Takes into account override */
+  bool htmlLoadExternal();
+
   /** Returns the MD5 hash for the list of new features */
   static QString newFeaturesMD5();
 
@@ -185,20 +196,21 @@ public:
   bool atBottom() const;
 
   bool isFixedFont() { return mUseFixedFont; }
+  void setUseFixedFont( bool useFixedFont ) { mUseFixedFont = useFixedFont; }
 
-  /** Return the @ref HtmlWriter connected to the @ref KHTMLPart we use */
+  /** Return the @see HtmlWriter connected to the @see KHTMLPart we use */
   KMail::HtmlWriter * htmlWriter() { return mHtmlWriter; }
 
   // Action to reply to a message
   // but action( "some_name" ) some name could be used instead.
   KToggleAction *toggleFixFontAction() { return mToggleFixFontAction; }
-  KAction *viewSourceAction() { return mViewSourceAction; }
   KAction *mailToComposeAction() { return mMailToComposeAction; }
   KAction *mailToReplyAction() { return mMailToReplyAction; }
   KAction *mailToForwardAction() { return mMailToForwardAction; }
   KAction *addAddrBookAction() { return mAddAddrBookAction; }
   KAction *openAddrBookAction() { return mOpenAddrBookAction; }
   KAction *copyAction() { return mCopyAction; }
+  KAction *selectAllAction() { return mSelectAllAction; }
   KAction *copyURLAction() { return mCopyURLAction; }
   KAction *urlOpenAction() { return mUrlOpenAction; }
   KAction *urlSaveAsAction() { return mUrlSaveAsAction; }
@@ -224,7 +236,7 @@ public:
 
   void setUpdateAttachment() { mAtmUpdate = true; }
 
-  /** Access to the @ref KHTMLPart used for the viewer. Use with
+  /** Access to the @see KHTMLPart used for the viewer. Use with
       care! */
   KHTMLPart * htmlPart() const { return mViewer; }
 
@@ -316,7 +328,6 @@ public slots:
   /** Save the page to a file */
   void slotUrlSave();
   void slotAddBookmarks();
-  void slotShowMsgSrc();
   void slotSaveMsg();
   void slotSaveAttachments();
 
@@ -326,17 +337,24 @@ public slots:
   void contactStatusChanged( const QString &uid);
 
 protected slots:
+  void slotCycleHeaderStyles();
+  void slotBriefHeaders();
+  void slotFancyHeaders();
+  void slotStandardHeaders();
+  void slotLongHeaders();
+  void slotAllHeaders();
+
+  void slotCycleAttachmentStrategy();
+  void slotIconicAttachments();
+  void slotSmartAttachments();
+  void slotInlineAttachments();
+  void slotHideAttachments();
+
   /** Some attachment operations. */
-  void slotAtmOpen();
-  void slotDoAtmOpen();
-  void slotAtmOpenWith();
-  void slotAtmView();
-  void slotAtmSave();
-  void slotAtmProperties();
+  void slotAtmView( int id, const QString& name );
   void slotDelayedResize();
   void slotTouchMessage();
-  void slotAtmLoadPart( int );
-  void slotAtmDistributeClick();
+  void slotHandleAttachment( int );
 
 protected:
   /** reimplemented in order to update the frame width in case of a changed
@@ -387,9 +405,15 @@ private:
   void createWidgets();
   void createActions( KActionCollection * ac );
   void saveSplitterSizes( KConfigBase & c ) const;
+  QString createAtmFileLink() const;
+
+  KRadioAction * actionForHeaderStyle( const KMail::HeaderStyle *,
+                                       const KMail::HeaderStrategy * );
+  KRadioAction * actionForAttachmentStrategy( const KMail::AttachmentStrategy * );
+
 
 private:
-  bool mHtmlMail, mHtmlOverride;
+  bool mHtmlMail, mHtmlLoadExternal, mHtmlOverride, mHtmlLoadExtOverride;
   int mAtmCurrent;
   QString mAtmCurrentName;
   KMMessage *mMessage;
@@ -430,9 +454,10 @@ private:
   partNode* mRootNode;
   QString mIdOfLastViewedMessage;
   QWidget *mMainWindow;
-  KAction *mViewSourceAction, *mMailToComposeAction, *mMailToReplyAction, *mMailToForwardAction,
+  KActionCollection *mActionCollection;
+  KAction *mMailToComposeAction, *mMailToReplyAction, *mMailToForwardAction,
       *mAddAddrBookAction, *mOpenAddrBookAction, *mCopyAction, *mCopyURLAction,
-      *mUrlOpenAction, *mUrlSaveAsAction, *mAddBookmarksAction, *mStartIMChatAction;
+      *mUrlOpenAction, *mUrlSaveAsAction, *mAddBookmarksAction, *mStartIMChatAction, *mSelectAllAction;
 
   KToggleAction *mToggleFixFontAction;
   KURL mUrlClicked;
@@ -440,7 +465,6 @@ private:
   // an attachment should be updated
   bool mAtmUpdate;
   int mChoice;
-  KService::Ptr mOffer;
   unsigned long mWaitingForSerNum;
 };
 
