@@ -149,7 +149,6 @@ bool KMSender::send(KMMessage* aMsg, short sendNow)
     aMsg->setTo("Undisclosed.Recipients: ;");
   }
 
-  aMsg->removeHeaderField( "X-KMail-CryptoFormat" );
 
   // Handle redirections
   QString from  = aMsg->headerField("X-KMail-Redirect-From");
@@ -281,6 +280,7 @@ void KMSender::doSendMsg()
     }
     mCurrentMsg->setStatus(KMMsgStatusSent);
     mCurrentMsg->setStatus(KMMsgStatusRead); // otherwise it defaults to new on imap
+    mCurrentMsg->updateAttachmentState();
 
     const KPIM::Identity & id = kmkernel->identityManager()
       ->identityForUoidOrDefault( mCurrentMsg->headerField( "X-KMail-Identity" ).stripWhiteSpace().toUInt() );
@@ -427,7 +427,7 @@ void KMSender::doSendMsg()
       i18n( "Sending messages" ),
       i18n("Initiating sender process..."),
       true );
-    connect( mProgressItem, SIGNAL( progressItemCanceled( ProgressItem* ) ),
+    connect( mProgressItem, SIGNAL(progressItemCanceled(KPIM::ProgressItem*)),
              this, SLOT( slotAbortSend() ) );
     kapp->ref();
     mSendInProgress = TRUE;
@@ -742,11 +742,10 @@ void KMSender::setStatusByLink(const KMMessage *aMsg)
       break;
     n++;
 
-    KMFolder *folder;
-    int index;
+    KMFolder *folder = 0;
+    int index = -1;
     kmkernel->msgDict()->getLocation(msn, &folder, &index);
-
-    if (folder) {
+    if (folder && index != -1) {
       folder->open();
       if ( status == KMMsgStatusDeleted ) {
         // Move the message to the trash folder
@@ -757,6 +756,8 @@ void KMSender::setStatusByLink(const KMMessage *aMsg)
         folder->setStatus(index, status);
       }
       folder->close();
+    } else {
+      kdWarning(5006) << k_funcinfo << "Cannot update linked message, it could not be found!" << endl;
     }
   }
 }
@@ -1053,13 +1054,15 @@ bool KMSendSMTP::send(KMMessage *aMsg)
 
   if (ti->auth)
   {
-    if(ti->user.isEmpty() || ti->pass.isEmpty())
+    if( (ti->user.isEmpty() || ti->passwd().isEmpty()) && 
+      ti->authType != "GSSAPI" )
     {
       bool b = FALSE;
       int result;
 
       KCursorSaver idle(KBusyPtr::idle());
-      result = KIO::PasswordDialog::getNameAndPassword(ti->user, ti->pass,
+      QString passwd = ti->passwd();
+      result = KIO::PasswordDialog::getNameAndPassword(ti->user, passwd,
 	&b, i18n("You need to supply a username and a password to use this "
 	     "SMTP server."), FALSE, QString::null, ti->name, QString::null);
 
@@ -1068,11 +1071,13 @@ bool KMSendSMTP::send(KMMessage *aMsg)
         abort();
         return FALSE;
       }
-      if (int id = KMTransportInfo::findTransport(ti->name))
+      if (int id = KMTransportInfo::findTransport(ti->name)) {
+        ti->setPasswd( passwd );
         ti->writeConfig(id);
+      }
     }
     destination.setUser(ti->user);
-    destination.setPass(ti->pass);
+    destination.setPass(ti->passwd());
   }
 
   if (!mSlave || !mInProcess)
