@@ -38,18 +38,23 @@
 #include <kmessagebox.h>
 #include <kseparator.h>
 #include <kdebug.h>
+#include <kwallet.h>
+using KWallet::Wallet;
+#include <kprotocolinfo.h>
 
 #include "kmservertest.h"
 #include "kmaccount.h"
 #include "kmkernel.h"
 #include "protocols.h"
+#include "transportmanager.h"
+using namespace KMail;
 
-KMTransportInfo::KMTransportInfo()
+KMTransportInfo::KMTransportInfo() : mPasswdDirty( false ),
+  mStorePasswd( false ), mStorePasswdInConfig( false ), mId( 0 )
 {
   name = i18n("Unnamed");
   port = "25";
   auth = FALSE;
-  storePass = FALSE;
   specifyHostname = false;
 }
 
@@ -63,19 +68,38 @@ void KMTransportInfo::readConfig(int id)
 {
   KConfig *config = KMKernel::config();
   KConfigGroupSaver saver(config, "Transport " + QString::number(id));
+  mId = config->readUnsignedNumEntry( "id", 0 );
   type = config->readEntry("type", "smtp");
   name = config->readEntry("name", i18n("Unnamed"));
   host = config->readEntry("host", "localhost");
   port = config->readEntry("port", "25");
   user = config->readEntry("user");
-  pass = KMAccount::decryptStr(config->readEntry("pass"));
+  mPasswd = KMAccount::decryptStr(config->readEntry("pass"));
   precommand = config->readPathEntry("precommand");
   encryption = config->readEntry("encryption");
   authType = config->readEntry("authtype");
   auth = config->readBoolEntry("auth");
-  storePass = config->readBoolEntry("storepass");
+  mStorePasswd = config->readBoolEntry("storepass");
   specifyHostname = config->readBoolEntry("specifyHostname", false);
   localHostname = config->readEntry("localHostname");
+
+  if ( !storePasswd() )
+    return;
+
+  if ( !mPasswd.isEmpty() ) {
+    // migration to kwallet if available
+    if ( Wallet::isEnabled() ) {
+      config->deleteEntry( "pass" );
+      mPasswdDirty = true;
+      mStorePasswdInConfig = false;
+      writeConfig( id );
+    } else
+      mStorePasswdInConfig = true;
+  } else {
+    // read password if wallet is open, defer otherwise
+    if ( Wallet::isOpen( Wallet::NetworkWallet() ) )
+      readPassword();
+  }
 }
 
 
@@ -83,20 +107,64 @@ void KMTransportInfo::writeConfig(int id)
 {
   KConfig *config = KMKernel::config();
   KConfigGroupSaver saver(config, "Transport " + QString::number(id));
+  if (!mId)
+    mId = TransportManager::createId();
+  config->writeEntry("id", mId);
   config->writeEntry("type", type);
   config->writeEntry("name", name);
   config->writeEntry("host", host);
   config->writeEntry("port", port);
   config->writeEntry("user", user);
-  config->writeEntry("pass", (storePass) ? KMAccount::encryptStr(pass) :
-                                           QString("") );
   config->writePathEntry("precommand", precommand);
   config->writeEntry("encryption", encryption);
   config->writeEntry("authtype", authType);
   config->writeEntry("auth", auth);
-  config->writeEntry("storepass", storePass);
+  config->writeEntry("storepass", storePasswd());
   config->writeEntry("specifyHostname", specifyHostname);
   config->writeEntry("localHostname", localHostname);
+
+  if ( storePasswd() ) {
+    // write password into the wallet if possible and necessary
+    bool passwdStored = false;
+    if ( mPasswdDirty ) {
+      Wallet *wallet = kmkernel->wallet();
+      if ( wallet && wallet->writePassword( "transport-" + QString::number(mId), passwd() ) == 0 ) {
+        passwdStored = true;
+        mPasswdDirty = false;
+        mStorePasswdInConfig = false;
+      }
+    } else {
+      passwdStored = !mStorePasswdInConfig; // already in the wallet
+    }
+    // wallet not available, ask the user if we should use the config file instead
+    if ( !passwdStored && ( mStorePasswdInConfig ||  KMessageBox::warningYesNo( 0,
+         i18n("KWallet is not available. It is strongly recommended to use "
+              "KWallet for managing your passwords.\n"
+              "However, KMail can store the password in its configuration "
+              "file instead. The password is stored in an obfuscated format, "
+              "but should not be considered secure from decryption efforts "
+              "if access to the configuration file is obtained.\n"
+              "Do you want to store the password for account '%1' in the "
+              "configuration file?").arg( name ),
+         i18n("KWallet Not Available"),
+         KGuiItem( i18n("Store Password") ),
+         KGuiItem( i18n("Do Not Store Password") ) )
+         == KMessageBox::Yes ) ) {
+      config->writeEntry( "pass", KMAccount::encryptStr( passwd() ) );
+      mStorePasswdInConfig = true;
+    }
+  }
+
+  // delete already stored password if password storage is disabled
+  if ( !storePasswd() ) {
+    if ( !Wallet::keyDoesNotExist( 
+          Wallet::NetworkWallet(), "kmail", "transport-" + QString::number(mId) ) ) {
+      Wallet *wallet = kmkernel->wallet();
+      if ( wallet )
+        wallet->removeEntry( "transport-" + QString::number(mId) );
+    }
+    config->deleteEntry( "pass" );
+  }
 }
 
 
@@ -129,6 +197,48 @@ QStringList KMTransportInfo::availableTransports()
 }
 
 
+QString KMTransportInfo::passwd() const
+{
+  if ( auth && storePasswd() && mPasswd.isEmpty() )
+    readPassword();
+  return mPasswd;
+}
+
+
+void KMTransportInfo::setPasswd( const QString &passwd )
+{
+  if ( passwd != mPasswd ) {
+    mPasswd = passwd;
+    mPasswdDirty = true;
+  }
+}
+
+
+void KMTransportInfo::setStorePasswd( bool store )
+{
+  if ( mStorePasswd != store && store )
+    mPasswdDirty = true;
+  mStorePasswd = store;
+}
+
+
+void KMTransportInfo::readPassword() const
+{
+  if ( !storePasswd() || !auth )
+    return;
+
+  // ### workaround for broken Wallet::keyDoesNotExist() which returns wrong
+  // results for new entries without closing and reopening the wallet
+  if ( Wallet::isOpen( Wallet::NetworkWallet() ) ?
+       !kmkernel->wallet()->hasEntry( "transport-" + QString::number(mId) ) :
+       Wallet::keyDoesNotExist( Wallet::NetworkWallet(), "kmail", "transport-" + QString::number(mId) ) )
+    return;
+
+  if ( kmkernel->wallet() )
+    kmkernel->wallet()->readPassword( "transport-" + QString::number(mId), mPasswd );
+}
+
+
 KMTransportSelDlg::KMTransportSelDlg( QWidget *parent, const char *name,
   bool modal )
   : KDialogBase( parent, name, modal, i18n("Add Transport"), Ok|Cancel, Ok )
@@ -340,10 +450,13 @@ void KMTransportDialog::makeSmtpPage()
   grid->addWidget( mSmtp.passwordEdit, 9, 1 );
 
   mSmtp.storePasswordCheck =
-    new QCheckBox( i18n("&Store SMTP password in configuration file"), page1 );
+    new QCheckBox( i18n("&Store SMTP password"), page1 );
   QWhatsThis::add(mSmtp.storePasswordCheck,
                   i18n("Check this option to have KMail store "
-                  "the SMTP password in its configuration "
+                  "the password.\nIf KWallet is available "
+                  "the password will be stored there which is considered "
+                  "safe.\nHowever, if KWallet is not available, "
+                  "the password will be stored in KMail's configuration "
                   "file. The password is stored in an "
                   "obfuscated format, but should not be "
                   "considered secure from decryption efforts "
@@ -403,6 +516,12 @@ void KMTransportDialog::makeSmtpPage()
     mSmtp.authGroup  );
   mSmtp.authCramMd5 = new QRadioButton( i18n("CRAM-MD&5"), mSmtp.authGroup );
   mSmtp.authDigestMd5 = new QRadioButton( i18n("&DIGEST-MD5"), mSmtp.authGroup );
+  mSmtp.authNTLM = new QRadioButton( i18n("&NTLM"), mSmtp.authGroup );
+  mSmtp.authGSSAPI = new QRadioButton( i18n("&GSSAPI"), mSmtp.authGroup );
+  if ( KProtocolInfo::capabilities("smtp").contains("SASL") == 0 ) {
+    mSmtp.authNTLM->hide();
+    mSmtp.authGSSAPI->hide();
+  }
   vlay->addWidget( mSmtp.authGroup );
 
   vlay->addStretch();
@@ -429,8 +548,8 @@ void KMTransportDialog::setupSettings()
     mSmtp.portEdit->setText(mTransportInfo->port);
     mSmtp.authCheck->setChecked(mTransportInfo->auth);
     mSmtp.loginEdit->setText(mTransportInfo->user);
-    mSmtp.passwordEdit->setText(mTransportInfo->pass);
-    mSmtp.storePasswordCheck->setChecked(mTransportInfo->storePass);
+    mSmtp.passwordEdit->setText(mTransportInfo->passwd());
+    mSmtp.storePasswordCheck->setChecked(mTransportInfo->storePasswd());
     mSmtp.precommand->setText(mTransportInfo->precommand);
     mSmtp.specifyHostnameCheck->setChecked(mTransportInfo->specifyHostname);
     mSmtp.localHostnameEdit->setText(mTransportInfo->localHostname);
@@ -447,6 +566,10 @@ void KMTransportDialog::setupSettings()
       mSmtp.authCramMd5->setChecked(TRUE);
     else if (mTransportInfo->authType == "DIGEST-MD5")
       mSmtp.authDigestMd5->setChecked(TRUE);
+    else if (mTransportInfo->authType == "NTLM")
+      mSmtp.authNTLM->setChecked(TRUE);
+    else if (mTransportInfo->authType == "GSSAPI")
+      mSmtp.authGSSAPI->setChecked(TRUE);
     else mSmtp.authPlain->setChecked(TRUE);
 
     slotRequiresAuthClicked();
@@ -468,8 +591,8 @@ void KMTransportDialog::saveSettings()
     mTransportInfo->port = mSmtp.portEdit->text().stripWhiteSpace();
     mTransportInfo->auth = mSmtp.authCheck->isChecked();
     mTransportInfo->user = mSmtp.loginEdit->text().stripWhiteSpace();
-    mTransportInfo->pass = mSmtp.passwordEdit->text();
-    mTransportInfo->storePass = mSmtp.storePasswordCheck->isChecked();
+    mTransportInfo->setPasswd( mSmtp.passwordEdit->text() );
+    mTransportInfo->setStorePasswd( mSmtp.storePasswordCheck->isChecked() );
     mTransportInfo->precommand = mSmtp.precommand->text().stripWhiteSpace();
     mTransportInfo->specifyHostname = mSmtp.specifyHostnameCheck->isChecked();
     mTransportInfo->localHostname = mSmtp.localHostnameEdit->text().stripWhiteSpace();
@@ -479,7 +602,9 @@ void KMTransportDialog::saveSettings()
 
     mTransportInfo->authType = (mSmtp.authLogin->isChecked()) ? "LOGIN" :
     (mSmtp.authCramMd5->isChecked()) ? "CRAM-MD5" :
-    (mSmtp.authDigestMd5->isChecked()) ? "DIGEST-MD5" : "PLAIN";
+    (mSmtp.authDigestMd5->isChecked()) ? "DIGEST-MD5" :
+    (mSmtp.authNTLM->isChecked()) ? "NTLM" :
+    (mSmtp.authGSSAPI->isChecked()) ? "GSSAPI" : "PLAIN";
   }
 }
 
@@ -544,6 +669,8 @@ void KMTransportDialog::enableAuthMethods( unsigned int auth ) {
   mSmtp.authLogin->setEnabled( auth & LOGIN && !(auth & PLAIN));
   mSmtp.authCramMd5->setEnabled( auth & CRAM_MD5 );
   mSmtp.authDigestMd5->setEnabled( auth & DIGEST_MD5 );
+  mSmtp.authNTLM->setEnabled( auth & NTLM );
+  mSmtp.authGSSAPI->setEnabled( auth & GSSAPI );
 }
 
 unsigned int KMTransportDialog::authMethodsFromString( const QString & s ) {
@@ -558,6 +685,10 @@ unsigned int KMTransportDialog::authMethodsFromString( const QString & s ) {
       result |= CRAM_MD5;
     else if ( *it == "SASL/DIGEST-MD5" )
       result |= DIGEST_MD5;
+    else if ( *it == "SASL/NTLM" )
+      result |= NTLM;
+    else if ( *it == "SASL/GSSAPI" )
+      result |= GSSAPI;
   return result;
 }
 
@@ -572,6 +703,10 @@ unsigned int KMTransportDialog::authMethodsFromStringList( const QStringList & s
       result |= CRAM_MD5;
     else if ( *it == "DIGEST-MD5" )
       result |= DIGEST_MD5;
+    else if ( *it == "NTLM" )
+      result |= NTLM;
+    else if ( *it == "GSSAPI" )
+      result |= GSSAPI;
   return result;
 }
 
