@@ -10,7 +10,8 @@
 #include <dcopclient.h>
 #include "kmkernel.h" //control center
 #include "kmail_options.h"
-#include <qtimer.h>
+
+#include <kdebug.h>
 
 #undef Status // stupid X headers
 
@@ -18,19 +19,6 @@
 
 #include "kmstartup.h"
 
-// OLD about text.  This is horrbly outdated.
-/*const char* aboutText =
-    "KMail [" KMAIL_VERSION "] by\n\n"
-    "Stefan Taferner <taferner@kde.org>,\n"
-    "Markus Wbben <markus.wuebben@kde.org>\n\n"
-    "based on the work of:\n"
-    "Lynx <lynx@topaz.hknet.com>,\n"
-    "Stephan Meyer <Stephan.Meyer@pobox.com>,\n"
-    "and the above authors.\n\n"
-    "This program is covered by the GPL.\n\n"
-    "Please send bugreports to taferner@kde.org";
-*/
-
 //-----------------------------------------------------------------------------
 
 class KMailApplication : public KUniqueApplication
@@ -51,12 +39,9 @@ void KMailApplication::commitData(QSessionManager& sm) {
 
 int KMailApplication::newInstance()
 {
-  if (dcopClient()->isSuspended())
-  {
-    // Try again later.
-    QTimer::singleShot( 100, this, SLOT(newInstance()) );
-    return 0;
-  }
+  kdDebug(5006) << "KMailApplication::newInstance()" << endl;
+  if (!kmkernel)
+     return 0;
 
   if (!kmkernel->firstInstance() || !kapp->isRestored())
     kmkernel->handleCommandLine( true );
@@ -80,22 +65,17 @@ int main(int argc, char *argv[])
 
   KMailApplication app;
 
-  // import i18n data from libraries:
-  KMail::insertLibraryCatalogues();
-
-#if !KDE_IS_VERSION( 3, 1, 92 ) // replacement is now in KMKernel::config()
-  // Check that all updates have been run on the config file:
-  KMail::checkConfigUpdates();
-#endif
+  // import i18n data and icons from libraries:
+  KMail::insertLibraryCataloguesAndIcons();
 
   // Make sure that the KNotify Daemon is running (this is necessary for people
   // using KMail without KDE)
   KNotifyClient::startDaemon();
 
-  KMail::lockOrDie();
-
   kapp->dcopClient()->suspend(); // Don't handle DCOP requests yet
 
+  KMail::lockOrDie();
+
   //local, do the init
   KMKernel kmailKernel;
   kmailKernel.init();
@@ -114,11 +94,8 @@ int main(int argc, char *argv[])
   // Go!
   int ret = kapp->exec();
   // clean up
-  if (kmkernel->shuttingDown())
-      kmailKernel.notClosedByUser();
-  else
-      kmailKernel.cleanup();
+  kmailKernel.cleanup();
 
-  KMail::cleanup();
+  KMail::cleanup(); // pid file (see kmstartup.cpp)
   return ret;
 }
