@@ -40,6 +40,7 @@
 #include "stl_util.h"
 
 #include <kdebug.h>
+#include <kiconloader.h>
 
 #include <qwidgetstack.h>
 #include <qstring.h>
@@ -208,7 +209,8 @@ KMail::RuleWidgetHandlerManager::RuleWidgetHandlerManager()
 
 KMail::RuleWidgetHandlerManager::~RuleWidgetHandlerManager()
 {
-  for_each( mHandlers.begin(), mHandlers.end(), DeleteAndSetToZero<RuleWidgetHandler> );
+  for_each( mHandlers.begin(), mHandlers.end(),
+	    DeleteAndSetToZero<RuleWidgetHandler>() );
 }
 
 void KMail::RuleWidgetHandlerManager::registerHandler( const RuleWidgetHandler * handler )
@@ -1053,7 +1055,7 @@ namespace {
     QComboBox *statusCombo = new QComboBox( valueStack,
                                             "statusRuleValueCombo" );
     for ( int i = 0; i < KMail::StatusValueCountWithoutHidden; ++i ) {
-      statusCombo->insertItem( i18n( KMail::StatusValues[i] ) );
+      statusCombo->insertItem( SmallIcon( KMail::StatusValues[ i ].icon ), i18n( KMail::StatusValues[ i ].text ) );
     }
     statusCombo->adjustSize();
     QObject::connect( statusCombo, SIGNAL( activated( int ) ),
@@ -1121,7 +1123,7 @@ namespace {
 
     const int status = currentStatusValue( valueStack );
     if ( status != -1 )
-      return QString::fromLatin1( KMail::StatusValues[status] );
+      return QString::fromLatin1( KMail::StatusValues[ status ].text );
     else
       return QString::null;
   }
@@ -1137,7 +1139,7 @@ namespace {
 
     const int status = currentStatusValue( valueStack );
     if ( status != -1 )
-      return i18n( KMail::StatusValues[status] );
+      return i18n( KMail::StatusValues[ status ].text );
     else
       return QString::null;
   }
@@ -1214,7 +1216,7 @@ namespace {
     int valueIndex = 0;
     for ( ; valueIndex < KMail::StatusValueCountWithoutHidden; ++valueIndex )
       if ( value == QString::fromLatin1(
-               KMail::StatusValues[valueIndex] ) )
+               KMail::StatusValues[ valueIndex ].text ) )
         break;
     QComboBox *statusCombo =
       dynamic_cast<QComboBox*>( valueStack->child( "statusRuleValueCombo",
