@@ -36,7 +36,7 @@
 
 FilterOE::FilterOE() :
 Filter(	i18n("Import Outlook Express Emails"),
-    "Laurence Anderson",
+    "Laurence Anderson <p>( Filter accelerated by Danny Kukawka )</p>",
     i18n("<p><b>Outlook Express 4/5/6 import filter</b></p>"
       "<p>You will need to locate the folder where the mailbox has been stored by searching for .dbx or .mbx files under "
       "<ul><li><i>C:\\Windows\\Application Data</i> in Windows 9x"
@@ -48,6 +48,7 @@ Filter(	i18n("Import Outlook Express Emails"),
 
 FilterOE::~FilterOE()
 {
+   endImport();
 }
 
 void FilterOE::import(FilterInfo *info)
@@ -67,6 +68,7 @@ void FilterOE::import(FilterInfo *info)
     info->alert(i18n("No Outlook Express mailboxes found in directory %1.").arg(mailDir));
     return;
   }
+  
   totalFiles = files.count();
   currentFile = 0;
 
@@ -152,7 +154,13 @@ void FilterOE::mbxImport(QDataStream& ds)
     } while ( !ds.atEnd() );
 
     tmp.close();
-    addMessage(inf, folderName, tmp.name());
+    /* comment by Danny Kukawka:
+     * addMessage() == old function, need more time and check for duplicates
+     * addMessage_fastImport == new function, faster and no check for duplicates
+     */
+    if(inf->removeDupMsg) addMessage( inf, folderName, tmp.name() );
+    else addMessage_fastImport( inf, folderName, tmp.name() );
+    
     tmp.unlink();
   }
 }
@@ -240,6 +248,7 @@ void FilterOE::dbxReadEmail(QDataStream& ds, int filePos)
   Q_UINT16 blockSize;
   Q_UINT8 intCount, unknown;
   KTempFile tmp;
+  bool _break = false;
   int wasAt = ds.device()->at();
   ds.device()->at(filePos);
 
@@ -248,14 +257,25 @@ void FilterOE::dbxReadEmail(QDataStream& ds, int filePos)
     QByteArray blockBuffer(blockSize);
     ds.readRawBytes(blockBuffer.data(), blockSize);
     tmp.dataStream()->writeRawBytes(blockBuffer.data(), blockSize);
+    // to detect incomplete mails or corrupted archives. See Bug #86119
+    if(ds.atEnd()){
+      _break = true;
+      break;
+    }
     ds.device()->at(nextAddress);
   } while (nextAddress != 0);
   tmp.close();
-  addMessage(inf, folderName, tmp.name());
-  inf->setCurrent( ++currentEmail / totalEmails * 100);
+  
+  if(!_break){
+	if(inf->removeDupMsg) addMessage( inf, folderName, tmp.name() );
+	else addMessage_fastImport( inf, folderName, tmp.name() );
+	
+	currentEmail++;
+	int currentPercentage = (int) ( ( (float) currentEmail / totalEmails ) * 100 );
+	inf->setCurrent(currentPercentage);
+	ds.device()->at(wasAt);
+  }
   tmp.unlink();
-
-  ds.device()->at(wasAt);
 }
 
 // vim: ts=2 sw=2 et
