@@ -137,12 +137,15 @@ void KNArticleFactory::createReply(KNRemoteArticle *a, QString selectedText, boo
   //newsgroups
   KMime::Headers::FollowUpTo *fup2=a->followUpTo(false);
   if(fup2 && !fup2->isEmpty()) {
-    if(fup2->as7BitString(false).upper()=="POSTER") { //Followup-To: poster
-      if (post)         // warn the user
-        KMessageBox::information(knGlobals.topWidget,i18n("The author has requested a reply by email instead\nof a followup to the newsgroup. (Followup-To: poster)"),
-                                 QString::null,"followupToPosterWarning");
-      art->setDoPost(false);
-      art->setDoMail(true);
+    if( ( fup2->as7BitString(false).upper()=="POSTER" ) ) { //Followup-To: poster
+      if( post && // user wanted to reply by public posting?
+          // ask the user if she wants to ignore this F'up-To: poster
+          ( KMessageBox::Yes != KMessageBox::questionYesNo(knGlobals.topWidget,
+                i18n("The author has requested a reply by email instead\nof a followup to the newsgroup. (Followup-To: poster)\nDo you want to reply in public anyway?")) ))
+      {
+        art->setDoPost(false);
+        art->setDoMail(true);
+      }
       art->newsgroups()->from7BitString(a->newsgroups()->as7BitString(false));
     }
     else
@@ -213,6 +216,7 @@ void KNArticleFactory::createReply(KNRemoteArticle *a, QString selectedText, boo
   attribution.replace(QRegExp("%DATE"),KGlobal::locale()->formatDateTime(a->date()->qdt(),false));
   attribution.replace(QRegExp("%MSID"),a->messageID()->asUnicodeString());
   attribution.replace(QRegExp("%GROUP"),g->groupname());
+  attribution.replace(QRegExp("%L"),"\n");
   attribution+="\n\n";
 
   QString quoted=attribution;
@@ -580,9 +584,9 @@ void KNArticleFactory::edit(KNLocalArticle *a)
     if(acc) {
       KMime::Headers::Newsgroups *grps=a->newsgroups();
       KNGroup *grp=knGlobals.groupManager()->group(grps->firstGroup(), acc);
-      if (grp && grp->identity() && grp->identity()->hasSignature())
+      if (grp && grp->identity())
         id=grp->identity();
-      else if (acc->identity() && acc->identity()->hasSignature())
+      else if (acc->identity())
         id=acc->identity();
     }
   }
@@ -593,6 +597,12 @@ void KNArticleFactory::edit(KNLocalArticle *a)
 
   //open composer
   com=new KNComposer(a, QString::null, id->getSignature());
+  if(id->useSigGenerator() && !id->getSigGeneratorStdErr().isEmpty())
+  KMessageBox::information(knGlobals.topWidget,
+                            i18n("<qt>The signature generator program produced the "
+                                "following output:<br><br>%1</qt>")
+                                .arg(id->getSigGeneratorStdErr()));
+
   c_ompList.append(com);
   connect(com, SIGNAL(composerDone(KNComposer*)), this, SLOT(slotComposerDone(KNComposer*)));
   com->show();
@@ -785,22 +795,25 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   }
 
   KNLocalArticle *art=new KNLocalArticle(0);
-  KNConfig::Identity  *grpId=0,
-                      *defId=0,
-                      *accId=0,
-                      *id=0;
+  KNConfig::Identity *tmpId=0, *id=0;
 
   if (col) {
     if (col->type() == KNCollection::CTgroup) {
-      grpId = (static_cast<KNGroup *>(col))->identity();
-      accId = (static_cast<KNGroup *>(col))->account()->identity();
+      id = (static_cast<KNGroup *>(col))->identity();
+      tmpId = (static_cast<KNGroup *>(col))->account()->identity();
     } else
       if (col->type() == KNCollection::CTnntpAccount) {
-        accId = (static_cast<KNNntpAccount *>(col))->identity();
+        id = (static_cast<KNNntpAccount *>(col))->identity();
       }
   }
 
-  defId=knGlobals.configManager()->identity();
+  // determine active innermost non-empty identity
+  if (!id) {
+    if (tmpId)
+      id = tmpId;
+    else
+      id = knGlobals.configManager()->identity();
+  }
 
   //Message-id
   if(pnt->generateMessageID())
@@ -811,31 +824,24 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   from->setRFC2047Charset(pnt->charset());
 
   //name
-  if(grpId && grpId->hasName())
-    id=grpId;
-  else
-    id=((accId) && accId->hasName())? accId:defId;
   if(id->hasName())
     from->setName(id->name());
 
   //email
-  if(grpId && grpId->hasEmail())
-    id=grpId;
-  else
-    id=((accId) && accId->hasEmail())? accId:defId;
   if(id->hasEmail()&&id->emailIsValid())
     from->setEmail(id->email().latin1());
   else {
-    KMessageBox::sorry(knGlobals.topWidget, i18n("Please enter a valid email address."));
+    if ( id->hasEmail() )
+      KMessageBox::sorry(knGlobals.topWidget,
+	i18n("Please enter a valid email address at the identity tab of the account configuration dialog."));
+    else
+      KMessageBox::sorry(knGlobals.topWidget, 
+         i18n("Please enter a valid email address at the identity section of the configuration dialog."));
     delete art;
     return 0;
   }
 
   //Reply-To
-  if(grpId && grpId->hasReplyTo())
-    id=grpId;
-  else
-    id=((accId) && accId->hasReplyTo())? accId:defId;
   if(id->hasReplyTo()) {
     art->replyTo()->fromUnicodeString(id->replyTo(), pnt->charset());
     if (!art->replyTo()->hasEmail())   // the header is invalid => drop it
@@ -843,10 +849,6 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   }
 
   //Mail-Copies-To
-  if(grpId && grpId->hasMailCopiesTo())
-    id=grpId;
-  else
-    id=((accId) && accId->hasMailCopiesTo())? accId:defId;
   if(id->hasMailCopiesTo()) {
     art->mailCopiesTo()->fromUnicodeString(id->mailCopiesTo(), pnt->charset());
     if (!art->mailCopiesTo()->isValid())   // the header is invalid => drop it
@@ -854,10 +856,6 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   }
 
   //Organization
-  if(grpId && grpId->hasOrga())
-    id=grpId;
-  else
-    id=((accId) && accId->hasOrga())? accId:defId;
   if(id->hasOrga())
     art->organization()->fromUnicodeString(id->orga(), pnt->charset());
 
@@ -883,17 +881,27 @@ KNLocalArticle* KNArticleFactory::newArticle(KNCollection *col, QString &sig, QC
   //X-Headers
   if(withXHeaders) {
     KNConfig::XHeaders::Iterator it;
-    for(it=pnt->xHeaders().begin(); it!=pnt->xHeaders().end(); ++it)
-      art->setHeader( new KMime::Headers::Generic( (QCString("X-")+(*it).name()), art, (*it).value(), pnt->charset() ) );
+    for(it=pnt->xHeaders().begin(); it!=pnt->xHeaders().end(); ++it) {
+      QString name(art->from()->name());
+      if (name.isEmpty())
+        name = QString::fromLatin1(art->from()->email());
+      QString value = (*it).value();
+      value.replace(QRegExp("%NAME"), name);
+      value.replace(QRegExp("%EMAIL"), QString::fromLatin1(art->from()->email()));
+      art->setHeader( new KMime::Headers::Generic( (QCString("X-")+(*it).name()), art, value, pnt->charset() ) );
+    }
   }
 
   //Signature
-  if(grpId && grpId->hasSignature())
-    id=grpId;
-  else
-    id=((accId) && accId->hasSignature())? accId:defId;
   if(id->hasSignature())
+  {
     sig=id->getSignature();
+    if(id->useSigGenerator() && !id->getSigGeneratorStdErr().isEmpty())
+      KMessageBox::information(knGlobals.topWidget,
+                               i18n("<qt>The signature generator program produced the "
+                                   "following output:<br><br>%1</qt>")
+                                   .arg(id->getSigGeneratorStdErr()));
+  }
   else
     sig=QString::null;
 
@@ -949,24 +957,21 @@ and cancel (or supersede) it there."));
     KNConfig::Identity  *defId=knGlobals.configManager()->identity(),
                         *gid=g->identity(),
                         *accId=g->account()->identity();
-    bool ownArticle=true;
+    bool ownArticle = false;
 
-    if(gid && gid->hasName())
-      ownArticle=( gid->name()==remArt->from()->name() );
-    else
-      if (accId && accId->hasName())
-        ownArticle=( accId->name()==remArt->from()->name() );
-      else
-        ownArticle=( defId->name()==remArt->from()->name() );
+    if (gid && gid->hasName())
+      ownArticle |= ( gid->name() == remArt->from()->name() );
+    if (accId && accId->hasName())
+      ownArticle |= ( accId->name() == remArt->from()->name() );
+    ownArticle |= ( defId->name() == remArt->from()->name() );
 
     if(ownArticle) {
+      ownArticle = false;
       if(gid && gid->hasEmail())
-        ownArticle=( gid->email().latin1()==remArt->from()->email() );
-      else
-        if (accId && accId->hasEmail())
-          ownArticle=( accId->email().latin1()==remArt->from()->email() );
-        else
-          ownArticle=( defId->email().latin1()==remArt->from()->email() );
+        ownArticle |= ( gid->email().latin1() == remArt->from()->email() );
+      if (accId && accId->hasEmail())
+        ownArticle |= ( accId->email().latin1() == remArt->from()->email() );
+      ownArticle |= ( defId->email().latin1() == remArt->from()->email() );
     }
 
     if(!ownArticle) {
@@ -1062,7 +1067,7 @@ void KNArticleFactory::slotSendErrorDialogDone()
 //======================================================================================================
 
 
-KNSendErrorDialog::KNSendErrorDialog() 
+KNSendErrorDialog::KNSendErrorDialog()
   : KDialogBase(knGlobals.topWidget, 0, true, i18n("Errors While Sending"), Close, Close, true)
 {
   p_ixmap=knGlobals.configManager()->appearance()->icon(KNConfig::Appearance::sendErr);
@@ -1073,9 +1078,9 @@ KNSendErrorDialog::KNSendErrorDialog()
                                        .arg(i18n("The unsent articles are stored in the \"Outbox\" folder.")), page);
   j_obs=new KNDialogListBox(true, page);
   e_rror=new QLabel(QString::null, page);
-  
+
   connect(j_obs, SIGNAL(highlighted(int)), this, SLOT(slotHighlighted(int)));
-  
+
   KNHelper::restoreWindowSize("sendDlg", this, QSize(320,250));
 }
 
@@ -1106,3 +1111,5 @@ void KNSendErrorDialog::slotHighlighted(int idx)
 
 //-------------------------------
 #include "knarticlefactory.moc"
+
+// kate: space-indent on; indent-width 2;
