@@ -2,7 +2,7 @@
     knarticlewidget.cpp
 
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2004 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -18,12 +18,15 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include <qaccel.h>
 #include <qclipboard.h>
 #include <qdir.h>
 #include <qpaintdevicemetrics.h>
 #include <qpainter.h>
 #include <qtimer.h>
 #include <qurl.h>
+#include <qbuffer.h>
+#include <qbitmap.h>
 
 #include <kprinter.h>
 #include <kiconloader.h>
@@ -39,6 +42,8 @@
 #include <kpgpblock.h>
 #include <kstringhandler.h>
 
+#include <libkdepim/kxface.h>
+
 #include "resource.h"
 #include "knarticlewidget.h"
 #include "knglobals.h"
@@ -56,7 +61,8 @@
 #include <kpopupmenu.h>
 #include <kstandarddirs.h>
 #include <kbookmarkmanager.h>
-
+#include <kaddrbook.h>
+#include <ktempfile.h>
 
 #define PUP_OPEN    1000
 #define PUP_SAVE    2000
@@ -65,6 +71,10 @@
 #define PUP_COPY    5000
 #define PUP_ADDBOOKMARKS 6000
 
+#define PUP_ADDRESSBOOK 7000
+#define PUP_COPYTOCLIPBOARD 8000
+#define PUP_OPENADDRESSBOOK 9000
+
 #define HDR_COL   0
 #define QCOL_1    1
 #define QCOL_2    2
@@ -75,7 +85,9 @@ KNSourceViewWindow::KNSourceViewWindow(const QString &htmlCode)
   : KTextBrowser(0)
 {
   setWFlags(WType_TopLevel | WDestructiveClose);
-  KNConfig::Appearance *app=knGlobals.cfgManager->appearance();
+  QAccel *accel = new QAccel( this, "browser close-accel" );
+  accel->connectItem( accel->insertItem( Qt::Key_Escape ), this , SLOT( close() ));
+  KNConfig::Appearance *app=knGlobals.configManager()->appearance();
 
   setCaption(kapp->makeStdCaption(i18n("Article Source")));
   setPaper( QBrush(app->backgroundColor()) );
@@ -130,11 +142,12 @@ QByteArray KNMimeSource::encodedData(const char *) const
 //=============================================================================================================
 
 
-KNArticleWidget::KNArticleWidget(KActionCollection* actColl, QWidget *parent, const char *name )
+KNArticleWidget::KNArticleWidget(KActionCollection* actColl, KXMLGUIClient* guiClient,
+    QWidget *parent, const char *name )
     : KTextBrowser(parent, name), a_rticle(0), a_tt(0), h_tmlDone(false),
-      emuKMail(false), f_inddialog(0), a_ctions(actColl)
+      f_inddialog(0), a_ctions(actColl), mGuiClient(guiClient)
 {
-  i_nstances.append(this);
+  instances()->append(this);
   setNotifyClick( true );
 
   f_actory = new QMimeSourceFactory();
@@ -147,9 +160,13 @@ KNArticleWidget::KNArticleWidget(KActionCollection* actColl, QWidget *parent, co
   //popups
   u_rlPopup=new KPopupMenu();
   u_rlPopup->insertItem(SmallIcon("fileopen"),i18n("&Open Link"), PUP_OPEN);
-  u_rlPopup->insertItem(SmallIcon("editcopy"),i18n("&Copy Link Location"), PUP_COPYURL);
-  u_rlPopup->insertItem(SmallIcon("bookmark_add"),i18n("Add to Bookmarks"), PUP_ADDBOOKMARKS);
+  u_rlPopup->insertItem(SmallIcon("editcopy"),i18n("&Copy Link Address"), PUP_COPYURL);
+  u_rlPopup->insertItem(SmallIcon("bookmark_add"),i18n("Bookmark This Link"), PUP_ADDBOOKMARKS);
 
+  u_mailtoPopup=new KPopupMenu();
+  u_mailtoPopup->insertItem(i18n("&Add to Address Book"), PUP_ADDRESSBOOK);
+  u_mailtoPopup->insertItem(i18n("&Open in Address Book"), PUP_OPENADDRESSBOOK);
+  u_mailtoPopup->insertItem(SmallIcon("editcopy"), i18n("&Copy to Clipboard"), PUP_COPYTOCLIPBOARD);
 
   a_ttPopup=new KPopupMenu();
   a_ttPopup->insertItem(SmallIcon("fileopen"),i18n("&Open Attachment"), PUP_OPEN);
@@ -177,13 +194,14 @@ KNArticleWidget::KNArticleWidget(KActionCollection* actColl, QWidget *parent, co
                           SLOT(slotVerify()), a_ctions, "article_verify");
   a_ctToggleFullHdrs    = new KToggleAction(i18n("Show &All Headers"), "text_block", 0 , this,
                           SLOT(slotToggleFullHdrs()), a_ctions, "view_showAllHdrs");
-  a_ctToggleFullHdrs->setChecked(knGlobals.cfgManager->readNewsViewer()->showFullHdrs());
+  a_ctToggleFullHdrs->setCheckedState(i18n("Hide &All Headers"));
+  a_ctToggleFullHdrs->setChecked(knGlobals.configManager()->readNewsViewer()->showFullHdrs());
   a_ctToggleRot13       = new KToggleAction(i18n("&Unscramble (Rot 13)"), "decrypted", 0 , this,
                           SLOT(slotToggleRot13()), a_ctions, "view_rot13");
   a_ctToggleFixedFont   = new KToggleAction(i18n("U&se Fixed Font"),  Key_X , this,
                           SLOT(slotToggleFixedFont()), a_ctions, "view_useFixedFont");
-  a_ctToggleFixedFont->setChecked(knGlobals.cfgManager->readNewsViewer()->useFixedFont());
-  a_ctViewSource        = new KAction(i18n("&View Source"),  0 , this,
+  a_ctToggleFixedFont->setChecked(knGlobals.configManager()->readNewsViewer()->useFixedFont());
+  a_ctViewSource        = new KAction(i18n("&View Source"),  Key_V , this,
                           SLOT(slotViewSource()), a_ctions, "article_viewSource");
 
   a_ctSetCharset = new KSelectAction(i18n("Chars&et"), 0, a_ctions, "set_charset");
@@ -209,6 +227,10 @@ KNArticleWidget::KNArticleWidget(KActionCollection* actColl, QWidget *parent, co
   r_ot13=false;
   a_ctToggleRot13->setChecked(false);
   applyConfig();
+
+  t_mpFile = new KTempFile(locateLocal("tmp", "knode"), ".png");
+  t_mpFile->close();
+  t_mpFile->setAutoDelete(true);
 }
 
 
@@ -217,13 +239,19 @@ KNArticleWidget::~KNArticleWidget()
   if(a_rticle && a_rticle->isOrphant())
     delete a_rticle; //don't leak orphant articles
 
-  i_nstances.removeRef(this);
+  instances()->removeRef(this);
+  if(instances()->count() == 0) {
+    delete i_nstances;
+    i_nstances = 0;
+  }
   delete a_tt;
   delete a_ttPopup;
   delete u_rlPopup;
   delete f_actory;
   delete t_imer;
   delete f_inddialog;
+  delete u_mailtoPopup;
+  delete t_mpFile;
 }
 
 void KNArticleWidget::setText( const QString& text, const QString& context )
@@ -347,18 +375,6 @@ void KNArticleWidget::keyPressEvent(QKeyEvent *e)
     case Key_Next:
       scrollBy( 0, visibleHeight()-offs);
       break;
-    case Key_Left:
-      if (emuKMail)
-        emit(keyLeftPressed());
-      else
-        QTextBrowser::keyPressEvent(e);
-      break;
-    case Key_Right:
-      if (emuKMail)
-        emit(keyRightPressed());
-      else
-        QTextBrowser::keyPressEvent(e);
-      break;
     default:
       QTextBrowser::keyPressEvent(e);
   }
@@ -384,8 +400,7 @@ void KNArticleWidget::viewportMousePressEvent(QMouseEvent *e)
     anchorClicked(a, e->button(), &e->globalPos());
   else
     if (e->button()==RightButton) {
-      QPopupMenu *popup = static_cast<QPopupMenu *>(knGlobals.guiClient->factory()->container("body_popup", 
-knGlobals.guiClient));
+      QPopupMenu *popup = static_cast<QPopupMenu *>(mGuiClient->factory()->container("body_popup", mGuiClient));
       if ( popup )
         popup->popup(e->globalPos());
     }
@@ -406,8 +421,8 @@ bool KNArticleWidget::canDecode8BitText(const QCString &charset)
 
 void KNArticleWidget::applyConfig()
 {
-  KNConfig::Appearance *app=knGlobals.cfgManager->appearance();
-  KNConfig::ReadNewsViewer *rnv=knGlobals.cfgManager->readNewsViewer();
+  KNConfig::Appearance *app=knGlobals.configManager()->appearance();
+  KNConfig::ReadNewsViewer *rnv=knGlobals.configManager()->readNewsViewer();
 
   QFont f=(a_ctToggleFixedFont->isChecked()? app->articleFixedFont():app->articleFont());
 
@@ -460,10 +475,9 @@ void KNArticleWidget::applyConfig()
   newPalette.setInactive(newColorGroup);
   setPalette(newPalette);
 
-  if(!knGlobals.cfgManager->readNewsGeneral()->autoMark())
+  if(!knGlobals.configManager()->readNewsGeneral()->autoMark())
     t_imer->stop();
 
-  emuKMail = ((this==knGlobals.artWidget) && knGlobals.cfgManager->readNewsNavigation()->emulateKMail());
   updateContents();
 }
 
@@ -480,7 +494,7 @@ QString KNArticleWidget::toHtmlString(const QString &line, bool parseURLs, bool
   else
     text = line;
 
-  if (!knGlobals.cfgManager->readNewsViewer()->interpretFormatTags())
+  if (!knGlobals.configManager()->readNewsViewer()->interpretFormatTags())
     beautification=false;
 
   int lastReplacement=-1;
@@ -722,10 +736,10 @@ QString KNArticleWidget::toHtmlString(const QString &line, bool parseURLs, bool
       case '/' :
       case '*' :
         if(beautification) {
-          regExp=QString("\\%1[^\\s%2]+\\%3").arg(text[idx]).arg(text[idx]).arg(text[idx]);
+          regExp=QRegExp( QString("\\%1[^\\s%2]+\\%3").arg(text[idx]).arg(text[idx]).arg(text[idx]) );
           if (regExp.search(text,idx)==(int)idx) {
             matchLen = regExp.matchedLength();
-            if (( matchLen > 3 ) &&
+            if (( matchLen >= 3 ) &&
                 ((idx==0)||text[idx-1].isSpace()||(text[idx-1] == '(')) &&
                 ((idx+matchLen==len)||text[idx+matchLen].isSpace()||(text[idx+matchLen]==',')||
                  (text[idx+matchLen]=='.')||(text[idx+matchLen]==')'))) {
@@ -782,11 +796,11 @@ void KNArticleWidget::openURL(const QString &url)
 {
     if(url.isEmpty()) return;
 
-    if (knGlobals.cfgManager->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTdefault)
+    if (knGlobals.configManager()->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTdefault)
         (void) new KRun(KURL( url ));
-    if (knGlobals.cfgManager->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTkonq)
+    if (knGlobals.configManager()->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTkonq)
         kapp->invokeBrowser(url);
-    else if (knGlobals.cfgManager->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTnetscape){
+    else if (knGlobals.configManager()->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTnetscape){
         QString exec("netscape");
         if ( findExec( exec))
         {
@@ -802,7 +816,7 @@ void KNArticleWidget::openURL(const QString &url)
             proc.start(KProcess::DontCare);
         }
     }
-    else if (knGlobals.cfgManager->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTmozilla){
+    else if (knGlobals.configManager()->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTmozilla){
         QString exec("mozilla");
         if ( findExec( exec))
         {
@@ -812,7 +826,7 @@ void KNArticleWidget::openURL(const QString &url)
             proc.start(KProcess::DontCare);
         }
     }
-    else if (knGlobals.cfgManager->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTopera){
+    else if (knGlobals.configManager()->readNewsViewer()->browser()==KNConfig::ReadNewsViewer::BTopera){
         QString exec("opera");
         if ( findExec( exec))
         {
@@ -825,7 +839,7 @@ void KNArticleWidget::openURL(const QString &url)
     } else {
         KProcess proc;
 
-        QStringList command = QStringList::split(' ',knGlobals.cfgManager->readNewsViewer()->browserCommand());
+        QStringList command = QStringList::split(' ',knGlobals.configManager()->readNewsViewer()->browserCommand());
         bool urlAdded=false;
         for ( QStringList::Iterator it = command.begin(); it != command.end(); ++it ) {
             if ((*it).contains("%u")) {
@@ -847,8 +861,8 @@ void KNArticleWidget::saveAttachment(int id)
   KMime::Content *a=a_tt->at(id);
 
   if(a)
-    knGlobals.artManager->saveContentToFile(a,this);
-  else KMessageBox::error(this, i18n("Internal error: Malformed identifier!"));
+    knGlobals.articleManager()->saveContentToFile(a,this);
+  else KMessageBox::error(this, i18n("Internal error: Malformed identifier."));
 }
 
 
@@ -857,8 +871,8 @@ void KNArticleWidget::openAttachment(int id)
  KMime::Content *a=a_tt->at(id);
 
  if(a)
-   knGlobals.artManager->openContent(a);
- else KMessageBox::error(this, i18n("Internal error: Malformed identifier!"));
+   knGlobals.articleManager()->openContent(a);
+ else KMessageBox::error(this, i18n("Internal error: Malformed identifier."));
 }
 
 
@@ -878,7 +892,7 @@ void KNArticleWidget::showBlankPage()
   setMimeSourceFactory(f_actory);
 
   // restore background color, might have been changed by html article
-  setPaper(QBrush(knGlobals.cfgManager->appearance()->backgroundColor()));
+  setPaper(QBrush(knGlobals.configManager()->appearance()->backgroundColor()));
 
   setText(QString::null);
 
@@ -915,20 +929,20 @@ void KNArticleWidget::showErrorMessage(const QString &s)
   setMimeSourceFactory(f_actory);
 
   // restore background color, might have been changed by html article
-  setPaper(QBrush(knGlobals.cfgManager->appearance()->backgroundColor()));
+  setPaper(QBrush(knGlobals.configManager()->appearance()->backgroundColor()));
 
   QString errMsg=s;
   errMsg.replace(QRegExp("\n"),QString("<br>"));  // error messages can contain html-links, but are plain text otherwise
-  QString msg="<qt>"+i18n("<bodyblock><b><font size=+1 color=red>An error occurred!</font></b><hr><br>")+errMsg+"</bodyblock></qt>";
+  QString msg="<qt>"+i18n("<bodyblock><b><font size=+1 color=red>An error occurred.</font></b><hr><br>")+errMsg+"</bodyblock></qt>";
   setText(msg);
 
   // mark article as read, typically the article is expired on the server, so its
   // impossible to read it later anyway.
-  if(knGlobals.cfgManager->readNewsGeneral()->autoMark() &&
+  if(knGlobals.configManager()->readNewsGeneral()->autoMark() &&
      a_rticle && a_rticle->type()==KMime::Base::ATremote && !a_rticle->isOrphant()) {
     KNRemoteArticle::List l;
     l.append((static_cast<KNRemoteArticle*>(a_rticle)));
-    knGlobals.artManager->setRead(l, true);
+    knGlobals.articleManager()->setRead(l, true);
   }
 
   a_rticle=0;
@@ -979,8 +993,8 @@ void KNArticleWidget::setArticle(KNArticle *a)
     if(a->hasContent()) { //article is already loaded => just show it
       createHtmlPage();
     } else {
-      if( !knGlobals.artManager->loadArticle(a_rticle) )
-        articleLoadError( a, i18n("Unable to load the article!") );
+      if( !knGlobals.articleManager()->loadArticle(a_rticle) )
+        articleLoadError( a, i18n("Unable to load the article.") );
       else if(a->hasContent() && !(a->type()==KMime::Base::ATremote)) // try again, but not for remote articles
         createHtmlPage();
     }
@@ -1021,7 +1035,7 @@ void KNArticleWidget::processJob(KNJobData *j)
     if(!j->canceled()) {
       QString html;
       if (!j->success())
-        html= i18n("<b><font size=+1 color=red>An error occurred!</font></b><hr><br>")+j->errorString();
+        html= i18n("<b><font size=+1 color=red>An error occurred.</font></b><hr><br>")+j->errorString();
       else
         html= QString("%1<br>%2").arg(toHtmlString(a->head(),false,false)).arg(toHtmlString(a->body(),false,false));
 
@@ -1055,8 +1069,8 @@ void KNArticleWidget::createHtmlPage()
     a_rticle->setForceDefaultCS(f_orceCS);
   }
 
-  KNConfig::Appearance *app=knGlobals.cfgManager->appearance();
-  KNConfig::ReadNewsViewer *rnv=knGlobals.cfgManager->readNewsViewer();
+  KNConfig::Appearance *app=knGlobals.configManager()->appearance();
+  KNConfig::ReadNewsViewer *rnv=knGlobals.configManager()->readNewsViewer();
 
   delete f_actory;                          // purge old image data
   f_actory = new QMimeSourceFactory();
@@ -1069,6 +1083,22 @@ void KNArticleWidget::createHtmlPage()
 
   QString headerHtml;
   int headerLines=0;
+  int headerCols=3;
+
+  QString xfhead;
+  KMime::Headers::Base *temp = a_rticle->getHeaderByType("X-Face");
+  if (temp)
+    xfhead = temp->asUnicodeString();
+  QString xface = "";
+  if ( !xfhead.isEmpty() )
+  {
+    KPIM::KXFace xf;
+    xf.toImage(xfhead).save(t_mpFile->name(), "PNG");
+    xface = QString::fromLatin1(
+        "<td rowspan=\"4\" width=\"48\" height=\"48\"><img border=\"1\" src=\"%1\"></td>")
+        .arg(t_mpFile->name());
+    ++headerCols;
+  }
 
   if(a_ctToggleFullHdrs->isChecked()) {
     QCString head = a_rticle->head();
@@ -1079,9 +1109,11 @@ void KNArticleWidget::createHtmlPage()
       if (header) {
         if (headerLines > 0)
           headerHtml += "<tr>";
-        headerHtml+=QString("<td align=right><articlefont><b>%1</b></articlefont></td><td width=\"100%\"><articlefont>%2</articlefont></td></tr>")
+        headerHtml+=QString("<td align=right><articlefont><b>%1</b></articlefont></td><td width=\"100%\"><articlefont>%2</articlefont></td>%3</tr>")
                       .arg(toHtmlString(header->type())+":")
-                      .arg(toHtmlString(header->asUnicodeString(),true, false, false, true));
+                      .arg(toHtmlString(header->asUnicodeString(),true, false, false, true))
+                      .arg(xface);
+        xface = "";
         delete header;
         headerLines++;
       }
@@ -1090,7 +1122,7 @@ void KNArticleWidget::createHtmlPage()
   else {
     KMime::Headers::Base *hb;
     KNDisplayedHeader *dh;
-    KNConfig::DisplayedHeaders::Iterator it=knGlobals.cfgManager->displayedHeaders()->iterator();
+    KNConfig::DisplayedHeaders::Iterator it=knGlobals.configManager()->displayedHeaders()->iterator();
     for(; it.current(); ++it) {
       dh=it.current();
       hb=a_rticle->getHeaderByType(dh->header().latin1());
@@ -1118,7 +1150,8 @@ void KNArticleWidget::createHtmlPage()
       } else
         headerHtml+=toHtmlString(hb->asUnicodeString(),true, false, false, true);
 
-      headerHtml += dh->headerCloseTag()+"</articlefont></td></tr>";
+      headerHtml += dh->headerCloseTag()+"</articlefont></td>"+xface+"</tr>";
+      xface = "";
       headerLines++;
     }
   }
@@ -1131,8 +1164,8 @@ void KNArticleWidget::createHtmlPage()
     if (headerLines > 0) {
       html+=QString("<tr><td width=\"40\" bgcolor=\"%1\" rowspan=\"%2\"></td>"+headerHtml)
            .arg(app->headerDecoHexcode()).arg(headerLines);
-      html+=QString("<tr><td colspan=\"3\" bgcolor=\"%1\"><articlefont>")
-            .arg(app->headerDecoHexcode());
+      html+=QString("<tr><td colspan=\"%1\" bgcolor=\"%2\"><articlefont>")
+          .arg(headerCols).arg(app->headerDecoHexcode());
     } else {
       html+=QString("<tr><td bgcolor=\"%1\"><articlefont>")
           .arg(app->headerDecoHexcode());
@@ -1166,7 +1199,7 @@ void KNArticleWidget::createHtmlPage()
   KMime::Content *text=a_rticle->textContent();
   if(text && !canDecode8BitText(text->contentType()->charset())) {
     html+=QString("<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td bgcolor=\"red\"><articlefont><font color=\"black\">%1</font></articlefont></td></tr></table>")
-          .arg(i18n("Unknown charset! Default charset is used instead."));
+          .arg(i18n("Unknown charset. Default charset is used instead."));
     kdDebug(5003) << "KNArticleWidget::createHtmlPage() : unknown charset = " << text->contentType()->charset() << " not available!" << endl;
   }
 
@@ -1181,7 +1214,7 @@ void KNArticleWidget::createHtmlPage()
     KNRemoteArticle *ra = static_cast<KNRemoteArticle*>(a_rticle);
     Kpgp::Module *pgp = knGlobals.pgp;
 
-    if (knGlobals.cfgManager->readNewsGeneral()->autoCheckPgpSigs() || ra->isPgpSigned()) {
+    if (knGlobals.configManager()->readNewsGeneral()->autoCheckPgpSigs() || ra->isPgpSigned()) {
       QPtrList<Kpgp::Block> pgpBlocks;
       QStrList nonPgpBlocks;
       Kpgp::Module::prepareMessageForDecryption( ra->body(), pgpBlocks, nonPgpBlocks );
@@ -1193,8 +1226,8 @@ void KNArticleWidget::createHtmlPage()
         pgpBlock = 0;
       html += "<p>";
       if( !pgpBlock || !pgpBlock->isSigned() ) {
-        if (!knGlobals.cfgManager->readNewsGeneral()->autoCheckPgpSigs())
-          html += "<b>" + i18n("Cannot find a signature in this message!") + "</b>";
+        if (!knGlobals.configManager()->readNewsGeneral()->autoCheckPgpSigs())
+          html += "<b>" + i18n("Cannot find a signature in this message.") + "</b>";
       }
       else {
         QString signer = pgpBlock->signatureUserId();
@@ -1205,7 +1238,7 @@ void KNArticleWidget::createHtmlPage()
           html += i18n( "Message was signed with unknown key 0x%1." )
                   .arg( signerKey );
           html += "<br />";
-          html += i18n( "The validity of the signature can't be verified." );
+          html += i18n( "The validity of the signature cannot be verified." );
         }
         else {
           // determine the validity of the key
@@ -1292,9 +1325,9 @@ void KNArticleWidget::createHtmlPage()
     a_ctRemail->setEnabled(a_rticle->type()==KMime::Base::ATremote);
     a_ctForward->setEnabled(true);
     a_ctCancel->setEnabled( (a_rticle->type()==KMime::Base::ATremote) ||
-                            (a_rticle->collection()==knGlobals.folManager->sent()));
+                            (a_rticle->collection()==knGlobals.folderManager()->sent()));
     a_ctSupersede->setEnabled( (a_rticle->type()==KMime::Base::ATremote) ||
-                               (a_rticle->collection()==knGlobals.folManager->sent()));
+                               (a_rticle->collection()==knGlobals.folderManager()->sent()));
     a_ctSave->setEnabled(true);
     a_ctPrint->setEnabled(true);
     a_ctSelAll->setEnabled(true);
@@ -1304,8 +1337,8 @@ void KNArticleWidget::createHtmlPage()
     a_ctViewSource->setEnabled(true);
 
     //start automark-timer
-    if(a_rticle->type()==KMime::Base::ATremote && knGlobals.cfgManager->readNewsGeneral()->autoMark())
-      t_imer->start( (knGlobals.cfgManager->readNewsGeneral()->autoMarkSeconds()*1000), true);
+    if(a_rticle->type()==KMime::Base::ATremote && knGlobals.configManager()->readNewsGeneral()->autoMark())
+      t_imer->start( (knGlobals.configManager()->readNewsGeneral()->autoMarkSeconds()*1000), true);
     return;
   }
 
@@ -1314,7 +1347,7 @@ void KNArticleWidget::createHtmlPage()
     html+="<bodyblock>";
     if(text->contentType()->isHTMLText()) {
       QString htmlTxt;
-      text->decodedText(htmlTxt, true, knGlobals.cfgManager->readNewsViewer()->removeTrailingNewlines());
+      text->decodedText(htmlTxt, true, knGlobals.configManager()->readNewsViewer()->removeTrailingNewlines());
       html+=htmlTxt+"</bodyblock>";
     }
     else {
@@ -1324,7 +1357,7 @@ void KNArticleWidget::createHtmlPage()
       bool isSig=false;
       QStringList lines;
       QString line;
-      text->decodedText(lines, true, knGlobals.cfgManager->readNewsViewer()->removeTrailingNewlines());
+      text->decodedText(lines, true, knGlobals.configManager()->readNewsViewer()->removeTrailingNewlines());
       QString quoteChars = rnv->quoteCharacters().simplifyWhiteSpace();
       if (quoteChars.isEmpty()) quoteChars = ">";
 
@@ -1363,8 +1396,10 @@ void KNArticleWidget::createHtmlPage()
                 html+=QString("<font color=\"%1\">").arg(app->quotedTextHexcode(newLevel-1));
               }
             }
+            html+=toHtmlString(line,true,true,true)+"<br>";
           }
-          html+=toHtmlString(line,true,true,true)+"<br>";
+          else
+            html+=toHtmlString(line,true,false,true)+"<br>";
         }
         else
           html+="<br>";
@@ -1404,7 +1439,7 @@ void KNArticleWidget::createHtmlPage()
         if(rnv->showAttachmentsInline() && inlinePossible(var)) {
           html+="<tr><td colspan=3>";
           if(ct->isImage()) { //image
-            path=QString::number(attCnt)+"_"+QString::number((int)(a_rticle));
+            path=QString::number(attCnt)+"_"+QString::number((unsigned long)(a_rticle));
             f_actory->setData(path,new KNMimeSource(var->decodedContent(),ct->mimeType()));
             html+=QString("<a href=\"internal://att=%1\"><img src=\"%2\"></a>").arg(attCnt).arg(path);
           }
@@ -1441,9 +1476,9 @@ void KNArticleWidget::createHtmlPage()
   a_ctRemail->setEnabled(a_rticle->type()==KMime::Base::ATremote);
   a_ctForward->setEnabled(true);
   a_ctCancel->setEnabled( (a_rticle->type()==KMime::Base::ATremote) ||
-                          (a_rticle->collection()==knGlobals.folManager->sent()));
+                          (a_rticle->collection()==knGlobals.folderManager()->sent()));
   a_ctSupersede->setEnabled( (a_rticle->type()==KMime::Base::ATremote) ||
-                             (a_rticle->collection()==knGlobals.folManager->sent()));
+                             (a_rticle->collection()==knGlobals.folderManager()->sent()));
   a_ctVerify->setEnabled(true);
   a_ctToggleFullHdrs->setEnabled(true);
   a_ctToggleRot13->setEnabled(true);
@@ -1452,8 +1487,8 @@ void KNArticleWidget::createHtmlPage()
   a_ctViewSource->setEnabled(true);
 
   //start automark-timer
-  if(a_rticle->type()==KMime::Base::ATremote && knGlobals.cfgManager->readNewsGeneral()->autoMark())
-    t_imer->start( (knGlobals.cfgManager->readNewsGeneral()->autoMarkSeconds()*1000), true);
+  if(a_rticle->type()==KMime::Base::ATremote && knGlobals.configManager()->readNewsGeneral()->autoMark())
+    t_imer->start( (knGlobals.configManager()->readNewsGeneral()->autoMarkSeconds()*1000), true);
 }
 
 
@@ -1468,12 +1503,15 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
 {
   anchorType type=ATunknown;
   QString target;
-
+  kdDebug(5003)<<" target :"<<a<<endl;
   if(a.left(17)=="internal://author") {
     type=ATauthor;
   }
   else if(a.left(15)=="internal://att=") {
-    target=a.mid(15, a.length()-15);
+    if(a.right(1) == "/")
+      target=a.mid(15, a.length()-16);
+    else
+      target=a.mid(15, a.length()-15);
     type=ATattachment;
   }
   else if(a.left(7).lower()=="http://" || a.left(8).lower() == "https://" ||
@@ -1496,7 +1534,6 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
 
     target=a.mid(11, a.length()-11);
 
-    if ((button==LeftButton)||(button==MidButton)) {
       if (a_rticle->collection()->type()!=KNArticleCollection::CTgroup)
         type=ATmailto;
       else {
@@ -1509,9 +1546,6 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
           default:                type=ATunknown;
         }
       }
-    } else {
-      type=ATunknown;
-    }
   }
 
   if((button==LeftButton)||(button==MidButton)) {
@@ -1519,7 +1553,6 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
     KNRemoteArticle *a;
     KNArticleWindow *awin;
     KMime::Headers::AddressField adr(a_rticle);
-
     switch(type) {
       case ATauthor:
         kdDebug(5003) << "KNArticleWidget::anchorClicked() : mailto author" << endl;
@@ -1556,7 +1589,7 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
       break;
       case ATattachment:
         kdDebug(5003) << "KNArticleWidget::anchorClicked() : attachment " << target << endl;
-        if(knGlobals.cfgManager->readNewsViewer()->openAttachmentsOnClick())
+        if(knGlobals.configManager()->readNewsViewer()->openAttachmentsOnClick())
           openAttachment(target.toInt());
         else
           saveAttachment(target.toInt());
@@ -1582,7 +1615,6 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
     }
   }
   else {
-
     if(type==ATattachment) {
       kdDebug(5003) << "KNArticleWidget::anchorClicked() : popup for attachment " << target << endl;
       switch(a_ttPopup->exec(*p)) {
@@ -1594,7 +1626,22 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
         break;
       }
     }
-
+    else if ( type==ATauthor || type == ATmailto ) {
+        if ( type == ATauthor )
+          target = a_rticle->from()->asUnicodeString();
+        switch( u_mailtoPopup->exec( *p ) ) {
+        case PUP_ADDRESSBOOK:
+            addAddressbook(target);
+            break;
+        case PUP_OPENADDRESSBOOK:
+            openAddressbook(target);
+            break;
+        case PUP_COPYTOCLIPBOARD:
+            QApplication::clipboard()->setText(target, QClipboard::Clipboard);
+            QApplication::clipboard()->setText(target, QClipboard::Selection);
+            break;
+        }
+    }
     else if(type==ATurl) {
       kdDebug(5003) << "KNArticleWidget::anchorClicked() : popup for url " << target << endl;
       switch(u_rlPopup->exec(*p)) {
@@ -1602,7 +1649,8 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
           openURL(target);
         break;
         case PUP_COPYURL:
-          QApplication::clipboard()->setText(target);
+          QApplication::clipboard()->setText(target, QClipboard::Clipboard);
+          QApplication::clipboard()->setText(target, QClipboard::Selection);
         break;
       case PUP_ADDBOOKMARKS:
           addBookmarks(target );
@@ -1613,12 +1661,21 @@ void KNArticleWidget::anchorClicked(const QString &a, ButtonState button, const
   }
 }
 
+void KNArticleWidget::addAddressbook(const QString & target)
+{
+    KAddrBookExternal::addEmail( target, this );
+}
+
+void KNArticleWidget::openAddressbook(const QString & target)
+{
+    KAddrBookExternal::openEmail( target, target, this );
+}
 
 void KNArticleWidget::slotSave()
 {
   kdDebug(5003) << "KNArticleWidget::slotSave()" << endl;
   if(a_rticle)
-    knGlobals.artManager->saveArticleToFile(a_rticle,this);
+    knGlobals.articleManager()->saveArticleToFile(a_rticle,this);
 }
 
 
@@ -1644,7 +1701,7 @@ void KNArticleWidget::slotPrint()
     QFontMetrics fm=p.fontMetrics();
 
     KNDisplayedHeader *dh;
-    KNConfig::DisplayedHeaders::Iterator it=knGlobals.cfgManager->displayedHeaders()->iterator();
+    KNConfig::DisplayedHeaders::Iterator it=knGlobals.configManager()->displayedHeaders()->iterator();
     dh=it.current();
     while(dh) {
       hb=a_rticle->getHeaderByType(dh->header().latin1());
@@ -1683,7 +1740,7 @@ void KNArticleWidget::slotPrint()
     KMime::Content *txt=a_rticle->textContent();
 
     if(txt) {
-      txt->decodedText(lines, true, knGlobals.cfgManager->readNewsViewer()->removeTrailingNewlines());
+      txt->decodedText(lines, true, knGlobals.configManager()->readNewsViewer()->removeTrailingNewlines());
       for(QStringList::Iterator it=lines.begin(); it!=lines.end(); ++it) {
 
         if(yPos+margin > metrics.height()) {
@@ -1757,24 +1814,30 @@ void KNArticleWidget::slotToggleFullHdrs()
 
   // ok, this is a hack
   if (knGlobals.artWidget == this)
-    knGlobals.cfgManager->readNewsViewer()->setShowFullHdrs(!knGlobals.cfgManager->readNewsViewer()->showFullHdrs());
+    knGlobals.configManager()->readNewsViewer()->setShowFullHdrs(!knGlobals.configManager()->readNewsViewer()->showFullHdrs());
   updateContents();
 }
 
 
 void KNArticleWidget::slotToggleRot13()
 {
+  int x = contentsX();
+  int y = contentsY();
   r_ot13=!r_ot13;
   updateContents();
+  setContentsPos(x, y);
 }
 
 
 void KNArticleWidget::slotToggleFixedFont()
 {
+  int x = contentsX();
+  int y = contentsY();
   // ok, this is a hack
   if (knGlobals.artWidget == this)
-    knGlobals.cfgManager->readNewsViewer()->setUseFixedFont(!knGlobals.cfgManager->readNewsViewer()->useFixedFont());
+    knGlobals.configManager()->readNewsViewer()->setUseFixedFont(!knGlobals.configManager()->readNewsViewer()->useFixedFont());
   applyConfig();
+  setContentsPos(x, y);
 }
 
 
@@ -1832,7 +1895,7 @@ void KNArticleWidget::slotTimeout()
   if(a_rticle && a_rticle->type()==KMime::Base::ATremote && !a_rticle->isOrphant()) {
     KNRemoteArticle::List l;
     l.append((static_cast<KNRemoteArticle*>(a_rticle)));
-    knGlobals.artManager->setRead(l, true);
+    knGlobals.articleManager()->setRead(l, true);
   }
 }
 
@@ -1852,18 +1915,26 @@ void KNArticleWidget::slotVerify()
 //--------------------------------------------------------------------------------------
 
 
-QPtrList<KNArticleWidget> KNArticleWidget::i_nstances;
+QPtrList<KNArticleWidget>* KNArticleWidget::i_nstances = 0;
+
+QPtrList<KNArticleWidget>* KNArticleWidget::instances()
+{
+  if(!i_nstances)
+    i_nstances = new QPtrList<KNArticleWidget>();
+  return i_nstances;
+}
+
 
 void KNArticleWidget::configChanged()
 {
-  for(KNArticleWidget *i=i_nstances.first(); i; i=i_nstances.next())
+  for(KNArticleWidget *i=instances()->first(); i; i=instances()->next())
     i->applyConfig();
 }
 
 
 bool KNArticleWidget::articleVisible(KNArticle *a)
 {
-  for(KNArticleWidget *i=i_nstances.first(); i; i=i_nstances.next())
+  for(KNArticleWidget *i=instances()->first(); i; i=instances()->next())
     if(a==i->article())
       return true;
   return false;
@@ -1872,7 +1943,7 @@ bool KNArticleWidget::articleVisible(KNArticle *a)
 
 void KNArticleWidget::articleRemoved(KNArticle *a)
 {
-  for(KNArticleWidget *i=i_nstances.first(); i; i=i_nstances.next())
+  for(KNArticleWidget *i=instances()->first(); i; i=instances()->next())
     if(a==i->article())
       i->showBlankPage();
 }
@@ -1880,7 +1951,7 @@ void KNArticleWidget::articleRemoved(KNArticle *a)
 
 void KNArticleWidget::articleChanged(KNArticle *a)
 {
-  for(KNArticleWidget *i=i_nstances.first(); i; i=i_nstances.next())
+  for(KNArticleWidget *i=instances()->first(); i; i=instances()->next())
     if(a==i->article())
       i->updateContents();
 }
@@ -1888,7 +1959,7 @@ void KNArticleWidget::articleChanged(KNArticle *a)
 
 void KNArticleWidget::articleLoadError(KNArticle *a, const QString &error)
 {
-  for(KNArticleWidget *i=i_nstances.first(); i; i=i_nstances.next())
+  for(KNArticleWidget *i=instances()->first(); i; i=instances()->next())
     if(a==i->article())
       i->showErrorMessage(error);
 }
@@ -1896,7 +1967,7 @@ void KNArticleWidget::articleLoadError(KNArticle *a, const QString &error)
 
 void KNArticleWidget::collectionRemoved(KNArticleCollection *c)
 {
-  for(KNArticleWidget *i=i_nstances.first(); i; i=i_nstances.next())
+  for(KNArticleWidget *i=instances()->first(); i; i=instances()->next())
     if(i->article() && i->article()->collection()==c)
       i->showBlankPage();
 }
@@ -1904,7 +1975,7 @@ void KNArticleWidget::collectionRemoved(KNArticleCollection *c)
 
 void KNArticleWidget::cleanup()
 {
-  for(KNArticleWidget *i=i_nstances.first(); i; i=i_nstances.next())
+  for(KNArticleWidget *i=instances()->first(); i; i=instances()->next())
     i->setArticle(0); //delete orphant articles => avoid crash in destructor
 }
 
