@@ -2,7 +2,7 @@
     knconfig.h
 
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2004 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -18,15 +18,19 @@
 #define KNCONFIG_H
 
 #include <qasciidict.h>
+#include <qdatetime.h>
 
 #include <kdialogbase.h>
+#include <kcmodule.h>
+#include <kmime_util.h>
 
 #include "knwidgets.h"
 
-class QRadioButton;
-class QMultiLineEdit;
-class QCheckBox;
 class QButtonGroup;
+class QCheckBox;
+class QGroupBox;
+class QRadioButton;
+class QTextEdit;
 
 class KScoringRule;
 class KScoringEditorWidget;
@@ -41,6 +45,9 @@ namespace Kpgp {
   class Config;
   class SecretKeyRequester;
 }
+namespace KNConfig {
+  class GroupCleanupWidget;
+}
 
 class KNNntpAccount;
 class KNAccountManager;
@@ -69,12 +76,12 @@ class Base {
 };
 
 
-class BaseWidget : public QWidget {
+class KDE_EXPORT BaseWidget : public KCModule {
 
   Q_OBJECT
 
   public:
-    BaseWidget(QWidget *p=0, const char *n=0) : QWidget(p, n), d_irty(false) {}
+    BaseWidget(QWidget *p=0, const char *n=0) : KCModule(p, n), d_irty(false) {}
     ~BaseWidget() {}
 
     void show()             { d_irty=true; QWidget::show(); }
@@ -82,15 +89,15 @@ class BaseWidget : public QWidget {
     bool dirty()const            { return d_irty; }
     void setDirty(bool b)   { d_irty=b; }
 
-    virtual void apply() {}
-
   protected:
     bool d_irty;
 
+  protected slots:
+    void slotEmitChanged() { emit changed( true ); }
 };
 
 
-class Identity : public QObject, public Base {
+class KDE_EXPORT Identity : public QObject, public Base {
 
 Q_OBJECT
 
@@ -136,6 +143,8 @@ Q_OBJECT
     QString sigPath()const         { return s_igPath; }
     QString sigText()const         { return s_igText; }
     QString getSignature();
+    QString getSigGeneratorStdErr() { return s_igStdErr; }
+
 
   protected slots:
     void slotReceiveStdout(KProcess *proc, char *buffer, int buflen);
@@ -149,6 +158,7 @@ Q_OBJECT
               m_ailCopiesTo,
               s_igText,
               s_igContents,
+              s_igStdErr,
               s_igPath;
     QCString  s_igningKey;
     bool      u_seSigFile,
@@ -157,7 +167,7 @@ Q_OBJECT
 };
 
 
-class IdentityWidget : public BaseWidget {
+class KDE_EXPORT IdentityWidget : public BaseWidget {
 
   Q_OBJECT
 
@@ -165,7 +175,8 @@ class IdentityWidget : public BaseWidget {
     IdentityWidget(Identity *d, QWidget *p=0, const char *n=0);
     ~IdentityWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QLabel          *f_ileName;
@@ -180,7 +191,7 @@ class IdentityWidget : public BaseWidget {
     QCheckBox       *s_igGenerator;
     QPushButton     *c_hooseBtn,
                     *e_ditBtn;
-    QMultiLineEdit  *s_igEditor;
+    QTextEdit       *s_igEditor;
     QButtonGroup    *b_uttonGroup;
     Kpgp::SecretKeyRequester
                     *s_igningKey;
@@ -197,7 +208,7 @@ class IdentityWidget : public BaseWidget {
 };
 
 
-class NntpAccountListWidget : public BaseWidget {
+class KDE_EXPORT NntpAccountListWidget : public BaseWidget {
 
   Q_OBJECT
 
@@ -205,6 +216,7 @@ class NntpAccountListWidget : public BaseWidget {
     NntpAccountListWidget(QWidget *p=0, const char *n=0);
     ~NntpAccountListWidget();
 
+    void load();
 
   protected:
     class LBoxItem : public KNListBoxItem {
@@ -243,7 +255,7 @@ class NntpAccountListWidget : public BaseWidget {
 };
 
 
-class NntpAccountConfDialog : public KDialogBase  {
+class KDE_EXPORT NntpAccountConfDialog : public KDialogBase  {
 
   Q_OBJECT
 
@@ -251,7 +263,7 @@ class NntpAccountConfDialog : public KDialogBase  {
     NntpAccountConfDialog(KNNntpAccount* acc, QWidget *p=0, const char *n=0);
     ~NntpAccountConfDialog();
 
-  protected:    
+  protected:
     KLineEdit   *n_ame,
                 *s_erver,
                 *u_ser,
@@ -276,10 +288,12 @@ class NntpAccountConfDialog : public KDialogBase  {
     void slotAuthChecked(bool b);
     void slotIntervalChecked(bool b);
 
+  private:
+    GroupCleanupWidget *mCleanupWidget;
 };
 
 
-class SmtpAccountWidget : public BaseWidget {
+class KDE_EXPORT SmtpAccountWidget : public BaseWidget {
 
 Q_OBJECT
 
@@ -287,7 +301,8 @@ Q_OBJECT
     SmtpAccountWidget(QWidget *p=0, const char *n=0);
     ~SmtpAccountWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected slots:
     void useExternalMailerToggled(bool b);
@@ -306,19 +321,19 @@ Q_OBJECT
 };
 
 
-class Appearance : public Base {
+class KDE_EXPORT Appearance : public Base {
 
-#define COL_CNT 14
+#define COL_CNT 12
 #define FNT_CNT 5
 #define HEX_CNT 4
-#define ICON_CNT 18
+#define ICON_CNT 14
 
   friend class AppearanceWidget;
 
   public:
     enum ColorIndex   { background=0, alternateBackground=1, header=2, normalText=3, quoted1=4,
                         quoted2=5, quoted3=6, url=7, unreadThread=8, readThread=9,
-                        unreadArticle=10, readArticle=11, activeItem=12, selectedItem=13 };
+                        unreadArticle=10, readArticle=11 };
 
     enum HexIndex     { quoted1Hex=0, quoted2Hex=1, quoted3Hex=2 };
 
@@ -327,9 +342,8 @@ class Appearance : public Base {
     enum IconIndex    { greyBall=0,        redBall=1,      greyBallChkd=2,
                         redBallChkd=3,     newFups=4,      eyes=5,
                         ignore=6,          mail=7,         posting=8,
-                        canceledPosting=9, savedRemote=10, nntp=11,
-                        group=12,          folder=13,      rootFolder=14,
-                        customFolder=15,   sendErr=16,     null=17 };
+                        canceledPosting=9, savedRemote=10, group=11,
+                        sendErr=12,        null=13 };
     Appearance();
     ~Appearance();
 
@@ -347,8 +361,6 @@ class Appearance : public Base {
     QColor readThreadColor();
     QColor unreadArticleColor();
     QColor readArticleColor();
-    QColor activeItemColor();
-    QColor selectedItemColor();
 
     QString headerDecoHexcode()       { return headerDecoColor().name(); }
     QString quotedTextHexcode(int i);
@@ -385,20 +397,22 @@ class Appearance : public Base {
 };
 
 
-class AppearanceWidget : public BaseWidget {
+class KDE_EXPORT AppearanceWidget : public BaseWidget {
 
   Q_OBJECT
 
   public:
-    AppearanceWidget(Appearance *d, QWidget *p=0, const char *n=0);
+    AppearanceWidget(QWidget *p=0, const char *n=0);
     ~AppearanceWidget();
 
-    void apply();
+    void load();
+    void save();
+    void defaults();
 
     //===================================================================================
     // code taken from KMail, Copyright (C) 2000 Espen Sand, espen@kde.org
 
-    class ColorListItem : public QListBoxText {
+    class KDE_EXPORT ColorListItem : public QListBoxText {
 
       public:
         ColorListItem( const QString &text, const QColor &color=Qt::black );
@@ -417,7 +431,7 @@ class AppearanceWidget : public BaseWidget {
 
     //===================================================================================
 
-    class FontListItem : public QListBoxText {
+    class KDE_EXPORT FontListItem : public QListBoxText {
 
       public:
         FontListItem( const QString &name, const QFont & );
@@ -440,9 +454,7 @@ class AppearanceWidget : public BaseWidget {
               *f_List;
     QCheckBox *c_olorCB,
               *f_ontCB;
-    QPushButton *c_olDefBtn,
-                *c_olChngBtn,
-                *f_ntDefBtn,
+    QPushButton *c_olChngBtn,
                 *f_ntChngBtn;
 
     Appearance *d_ata;
@@ -451,21 +463,19 @@ class AppearanceWidget : public BaseWidget {
     //colors
     void slotColCheckBoxToggled(bool b);
     void slotColItemSelected(QListBoxItem *);   // show color dialog for the entry
-    void slotColDefaultBtnClicked();
     void slotColChangeBtnClicked();
     void slotColSelectionChanged();
 
     //fonts
     void slotFontCheckBoxToggled(bool b);
     void slotFontItemSelected(QListBoxItem *);  // show font dialog for the entry
-    void slotFontDefaultBtnClicked();
     void slotFontChangeBtnClicked();
     void slotFontSelectionChanged();
 
 };
 
 
-class ReadNewsGeneral : public Base {
+class KDE_EXPORT ReadNewsGeneral : public Base {
 
   friend class ReadNewsGeneralWidget;
 
@@ -497,6 +507,12 @@ class ReadNewsGeneral : public Base {
     bool autoCheckPgpSigs()const          { return a_utoCheckPgpSigs; }
     void setAutoCheckPgpSigs(bool b) { d_irty=true; a_utoCheckPgpSigs=b;}
 
+    KMime::DateFormatter::FormatType dateFormat() const { return mDateFormat; }
+    QString dateCustomFormat() const { return mDateCustomFormat; }
+
+    void setShowLines( bool show ) { d_irty = true; s_howLines = show; }
+    void setShowScore( bool show ) { d_irty = true; s_howScore = show; }
+
   protected:
     bool  a_utoCheck,
           a_utoMark,
@@ -515,16 +531,20 @@ class ReadNewsGeneral : public Base {
           c_ollCacheSize,
           a_rtCacheSize;
 
+    KMime::DateFormatter::FormatType mDateFormat;
+    QString mDateCustomFormat;
+
 };
 
 
-class ReadNewsGeneralWidget : public BaseWidget {
+class KDE_EXPORT ReadNewsGeneralWidget : public BaseWidget {
 
   public:
     ReadNewsGeneralWidget(ReadNewsGeneral *d, QWidget *p=0, const char *n=0);
     ~ReadNewsGeneralWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QCheckBox   *a_utoCB,
@@ -546,7 +566,7 @@ class ReadNewsGeneralWidget : public BaseWidget {
 };
 
 
-class ReadNewsNavigation : public Base {
+class KDE_EXPORT ReadNewsNavigation : public Base {
 
   friend class ReadNewsNavigationWidget;
 
@@ -556,7 +576,6 @@ class ReadNewsNavigation : public Base {
 
     void save();
 
-    bool emulateKMail()const               { return e_muKMail; }
     bool markAllReadGoNext()const          { return m_arkAllReadGoNext; }
     bool markThreadReadGoNext() const      { return m_arkThreadReadGoNext; }
     bool markThreadReadCloseThread()const  { return m_arkThreadReadCloseThread; }
@@ -564,8 +583,7 @@ class ReadNewsNavigation : public Base {
     bool ignoreThreadCloseThread()const   { return i_gnoreThreadCloseThread; }
 
   protected:
-    bool  e_muKMail,
-          m_arkAllReadGoNext,
+    bool  m_arkAllReadGoNext,
           m_arkThreadReadGoNext,
           m_arkThreadReadCloseThread,
           i_gnoreThreadGoNext,
@@ -574,17 +592,17 @@ class ReadNewsNavigation : public Base {
 };
 
 
-class ReadNewsNavigationWidget : public BaseWidget {
+class KDE_EXPORT ReadNewsNavigationWidget : public BaseWidget {
 
   public:
     ReadNewsNavigationWidget(ReadNewsNavigation *d, QWidget *p=0, const char *n=0);
     ~ReadNewsNavigationWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
-    QCheckBox   *e_muKMailCB,
-                *m_arkAllReadGoNextCB,
+    QCheckBox   *m_arkAllReadGoNextCB,
                 *m_arkThreadReadGoNextCB,
                 *m_arkThreadReadCloseThreadCB,
                 *i_gnoreThreadGoNextCB,
@@ -595,7 +613,7 @@ class ReadNewsNavigationWidget : public BaseWidget {
 };
 
 
-class ReadNewsViewer : public Base {
+class KDE_EXPORT ReadNewsViewer : public Base {
 
   friend class ReadNewsViewerWidget;
 
@@ -645,7 +663,7 @@ class ReadNewsViewer : public Base {
 };
 
 
-class ReadNewsViewerWidget : public BaseWidget {
+class KDE_EXPORT ReadNewsViewerWidget : public BaseWidget {
 
   Q_OBJECT
 
@@ -653,7 +671,8 @@ class ReadNewsViewerWidget : public BaseWidget {
     ReadNewsViewerWidget(ReadNewsViewer *d, QWidget *p=0, const char *n=0);
     ~ReadNewsViewerWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QCheckBox   *d_ecoCB,
@@ -679,7 +698,7 @@ class ReadNewsViewerWidget : public BaseWidget {
 
 
 
-class DisplayedHeaders : public Base {
+class KDE_EXPORT DisplayedHeaders : public Base {
 
   friend class DisplayedHeadersWidget;
 
@@ -705,7 +724,7 @@ class DisplayedHeaders : public Base {
 };
 
 
-class DisplayedHeadersWidget : public BaseWidget {
+class KDE_EXPORT DisplayedHeadersWidget : public BaseWidget {
 
   Q_OBJECT
 
@@ -713,7 +732,8 @@ class DisplayedHeadersWidget : public BaseWidget {
     DisplayedHeadersWidget(DisplayedHeaders *d, QWidget *p=0, const char *n=0);
     ~DisplayedHeadersWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
 
@@ -750,7 +770,7 @@ class DisplayedHeadersWidget : public BaseWidget {
 };
 
 
-class DisplayedHeaderConfDialog : public KDialogBase {
+class KDE_EXPORT DisplayedHeaderConfDialog : public KDialogBase {
 
   Q_OBJECT
 
@@ -774,7 +794,7 @@ class DisplayedHeaderConfDialog : public KDialogBase {
 };
 
 
-class Scoring : public Base {
+class KDE_EXPORT Scoring : public Base {
 
   friend class ScoringWidget;
 
@@ -794,7 +814,7 @@ class Scoring : public Base {
 };
 
 
-class ScoringWidget : public BaseWidget {
+class KDE_EXPORT ScoringWidget : public BaseWidget {
 
   Q_OBJECT
 
@@ -802,7 +822,8 @@ class ScoringWidget : public BaseWidget {
     ScoringWidget(Scoring *d, QWidget *p=0, const char *n=0);
     ~ScoringWidget();
 
-    void apply();
+    void load();
+    void save();
 
   private:
     KScoringEditorWidget *ksc;
@@ -813,7 +834,7 @@ class ScoringWidget : public BaseWidget {
 };
 
 
-class FilterListWidget : public BaseWidget {
+class KDE_EXPORT FilterListWidget : public BaseWidget {
 
   Q_OBJECT
 
@@ -821,7 +842,8 @@ class FilterListWidget : public BaseWidget {
     FilterListWidget(QWidget *p=0, const char *n=0);
     ~FilterListWidget();
 
-    void apply();
+    void load();
+    void save();
 
     void addItem(KNArticleFilter *f);
     void removeItem(KNArticleFilter *f);
@@ -877,7 +899,7 @@ class FilterListWidget : public BaseWidget {
 };
 
 
-class XHeader {
+class KDE_EXPORT XHeader {
 
   public:
     XHeader()                  {}
@@ -899,7 +921,7 @@ class XHeader {
 typedef QValueList<XHeader> XHeaders;
 
 
-class PostNewsTechnical : public Base {
+class KDE_EXPORT PostNewsTechnical : public Base {
 
   friend class PostNewsTechnicalWidget;
   friend class SmtpAccountWidget;
@@ -940,7 +962,7 @@ class PostNewsTechnical : public Base {
 };
 
 
-class PostNewsTechnicalWidget : public BaseWidget {
+class KDE_EXPORT PostNewsTechnicalWidget : public BaseWidget {
 
   Q_OBJECT
 
@@ -948,7 +970,8 @@ class PostNewsTechnicalWidget : public BaseWidget {
     PostNewsTechnicalWidget(PostNewsTechnical *d, QWidget *p=0, const char *n=0);
     ~PostNewsTechnicalWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     QComboBox   *c_harset,
@@ -976,7 +999,7 @@ class PostNewsTechnicalWidget : public BaseWidget {
 };
 
 
-class XHeaderConfDialog : public KDialogBase {
+class KDE_EXPORT XHeaderConfDialog : public KDialogBase {
 
   public:
     XHeaderConfDialog(const QString &h=QString::null, QWidget *p=0, const char *n=0);
@@ -1029,7 +1052,7 @@ class PostNewsComposer : public Base {
 };
 
 
-class PostNewsComposerWidget : public BaseWidget {
+class KDE_EXPORT PostNewsComposerWidget : public BaseWidget {
 
   Q_OBJECT
 
@@ -1037,7 +1060,8 @@ class PostNewsComposerWidget : public BaseWidget {
     PostNewsComposerWidget(PostNewsComposer *d, QWidget *p=0, const char *n=0);
     ~PostNewsComposerWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
     KIntSpinBox *m_axLen;
@@ -1058,13 +1082,13 @@ class PostNewsComposerWidget : public BaseWidget {
 };
 
 
-class PostNewsSpellingWidget : public BaseWidget {
+class KDE_EXPORT PostNewsSpellingWidget : public BaseWidget {
 
   public:
     PostNewsSpellingWidget(QWidget *p=0, const char *n=0);
     ~PostNewsSpellingWidget();
 
-    void apply();
+    void save();
 
   protected:
      KSpellConfig *c_onf;
@@ -1073,14 +1097,16 @@ class PostNewsSpellingWidget : public BaseWidget {
 
 
 
-class PrivacyWidget : public BaseWidget {
+class KDE_EXPORT PrivacyWidget : public BaseWidget {
 
   Q_OBJECT
 
   public:
     PrivacyWidget(QWidget *p=0, const char *n=0);
     ~PrivacyWidget();
-    void apply();   // overrides BaseWidget::apply()
+
+    void load();
+    void save();
 
   protected:
     Kpgp::Config *c_onf;
@@ -1088,24 +1114,34 @@ class PrivacyWidget : public BaseWidget {
 };
 
 
-class Cleanup : public Base {
+
+//BEGIN: Cleanup configuration -----------------------------------------------
+
+class KDE_EXPORT Cleanup : public Base {
 
   friend class CleanupWidget;
+  friend class GroupCleanupWidget;
 
   public:
-    Cleanup();
-    ~Cleanup();
+    Cleanup( bool global = true );
+    ~Cleanup() {}
 
+    void loadConfig( KConfigBase *conf );
+    void saveConfig( KConfigBase *conf );
     void save();
 
     //expire
-    int maxAgeForRead()const         { return r_eadMaxAge; }
-    int maxAgeForUnread()const       { return u_nreadMaxAge; }
-    bool removeUnavailable()const    { return r_emoveUnavailable; }
-    bool preserveThreads()const      { return p_reserveThr; }
+    int maxAgeForRead() const        { return r_eadMaxAge; }
+    int maxAgeForUnread() const      { return u_nreadMaxAge; }
+    bool removeUnavailable() const   { return r_emoveUnavailable; }
+    bool preserveThreads() const     { return p_reserveThr; }
+    bool isGlobal() const            { return mGlobal; }
+    bool useDefault() const          { return mDefault; }
     bool expireToday();
     void setLastExpireDate();
 
+    void setUseDefault( bool def )   { mDefault = def; }
+
     //compact
     bool compactToday();
     void setLastCompactDate();
@@ -1121,42 +1157,73 @@ class Cleanup : public Base {
           u_nreadMaxAge,
           c_ompactInterval;
 
+  private:
+    /** global vs. per account or per group configuration */
+    bool mGlobal;
+    /** use default cleanup configuration */
+    bool mDefault;
+    /** last expiration and last comapction date */
+    QDateTime mLastExpDate, mLastCompDate;
+
 };
 
 
-class CleanupWidget : public BaseWidget {
+/** Configuration widget for group expireration */
+class KDE_EXPORT GroupCleanupWidget : public QWidget {
 
   Q_OBJECT
 
   public:
-    CleanupWidget(Cleanup *d, QWidget *p=0, const char *n=0);
+    GroupCleanupWidget( Cleanup *data, QWidget *parent = 0, const char *name = 0 );
+
+    void load();
+    void save();
+
+  signals:
+    void changed();
+
+  private:
+    QCheckBox *mDefault, *mExpEnabled, *mExpUnavailable, *mPreserveThreads;
+    KIntSpinBox *mExpDays, *mExpReadDays, *mExpUnreadDays;
+    QGroupBox *mExpGroup;
+    Cleanup *mData;
+
+  private slots:
+    void slotDefaultToggled( bool state );
+
+};
+
+
+/** Global cleanup configuration widget */
+class KDE_EXPORT CleanupWidget : public BaseWidget {
+
+  Q_OBJECT
+
+  public:
+    CleanupWidget(QWidget *p=0, const char *n=0);
     ~CleanupWidget();
 
-    void apply();
+    void load();
+    void save();
 
   protected:
-    QCheckBox   *f_olderCB,
-                *g_roupCB,
-                *u_navailableCB,
-                *t_hrCB;
-    KIntSpinBox *f_olderDays,
-                *g_roupDays,
-                *r_eadDays,
-                *u_nreadDays;
-    QLabel      *f_olderDaysL,
-                *g_roupDaysL,
-                *r_eadDaysL,
-                *u_nreadDaysL;
+    QCheckBox   *f_olderCB;
+    KIntSpinBox *f_olderDays;
+    QLabel      *f_olderDaysL;
 
     Cleanup *d_ata;
 
 
   protected slots:
-    void slotGroupCBtoggled(bool b);
     void slotFolderCBtoggled(bool b);
 
+  private:
+    GroupCleanupWidget *mGroupCleanup;
+
 };
 
+//END: Cleanup configuration -------------------------------------------------
+
 
 /*class Cache : public Base {
 
