@@ -31,6 +31,8 @@
 #include "knscoring.h"
 #include "knarticlemanager.h"
 #include "kngroupmanager.h"
+#include "knnntpaccount.h"
+#include "headerview.h"
 
 
 #define SORT_DEPTH 5
@@ -40,12 +42,14 @@ KNGroup::KNGroup(KNCollection *p)
     l_astNr(0), m_axFetch(0), d_ynDataFormat(1), f_irstNew(-1), l_ocked(false),
     u_seCharset(false), s_tatus(unknown), i_dentity(0)
 {
+  mCleanupConf = new KNConfig::Cleanup( false );
 }
 
 
 KNGroup::~KNGroup()
 {
   delete i_dentity;
+  delete mCleanupConf;
 }
 
 
@@ -67,8 +71,9 @@ const QString& KNGroup::name()
 void KNGroup::updateListItem()
 {
   if(!l_istItem) return;
-  l_istItem->setNumber(1,c_ount);
-  l_istItem->setNumber(2,c_ount-r_eadCount-i_gnoreCount);
+  l_istItem->setTotalCount( c_ount );
+  l_istItem->setUnreadCount( c_ount - r_eadCount - i_gnoreCount );
+  l_istItem->repaint();
 }
 
 
@@ -108,6 +113,8 @@ bool KNGroup::readInfo(const QString &confPath)
     i_dentity=0;
   }
 
+  mCleanupConf->loadConfig( &info );
+
   return (!g_roupname.isEmpty());
 }
 
@@ -154,6 +161,8 @@ void KNGroup::saveInfo()
       info.deleteEntry("sigFile", false);
       info.deleteEntry("sigText", false);
     }
+
+    mCleanupConf->saveConfig( &info );
   }
 }
 
@@ -175,7 +184,7 @@ bool KNGroup::loadHdrs()
   }
 
   kdDebug(5003) << "KNGroup::loadHdrs() : loading headers" << endl;
-  QCString buff;
+  QCString buff, hdrValue;
   KNFile f;
   KQCStringSplitter split;
   int cnt=0, id, lines, fileFormatVersion, artNumber;
@@ -240,6 +249,20 @@ bool KNGroup::loadHdrs()
         art->setArticleNumber(artNumber);
       }
 
+      // optional headers
+      if (fileFormatVersion > 1) {
+        // first line is the number of addiotion headers
+        buff = f.readLine();
+        // following lines contain one header per line
+        for (uint i = buff.toUInt(); i > 0; --i) {
+          buff = f.readLine();
+          int pos = buff.find(':');
+          hdrValue = buff.right( buff.length() - (pos + 2) );
+          if (hdrValue.length() > 0)
+            art->setHeader( new KMime::Headers::Generic( buff.left(pos), art, hdrValue ) );
+        }
+      }
+
       if(append(art)) cnt++;
       else {
         f.close();
@@ -356,10 +379,10 @@ bool KNGroup::unloadHdrs(bool force)
 
 
 // Attention: this method is called from the network thread!
-void KNGroup::insortNewHeaders(QStrList *hdrs, KNProtocolClient *client)
+void KNGroup::insortNewHeaders(QStrList *hdrs, QStrList *hdrfmt, KNProtocolClient *client)
 {
   KNRemoteArticle *art=0, *art2=0;
-  QCString tmp;
+  QCString data, hdr, hdrName;
   KQCStringSplitter split;
   split.setIncludeSep(false);
   int new_cnt=0, added_cnt=0, todo=hdrs->count();
@@ -416,11 +439,29 @@ void KNGroup::insortNewHeaders(QStrList *hdrs, KNProtocolClient *client)
     if(!split.string().isEmpty())
       art->references()->from7BitString(split.string()); //use QCString::copy() ?
 
+    // Bytes
+    split.next();
+
     //Lines
     split.next();
-    split.next();
     art->lines()->setNumberOfLines(split.string().toInt());
 
+    // optinal additional headers
+    mOptionalHeaders = *hdrfmt;
+    for (hdr = hdrfmt->first(); hdr; hdr = hdrfmt->next()) {
+      if (!split.next())
+        break;
+      data = split.string();
+      int pos = hdr.find(':');
+      hdrName = hdr.left( pos );
+      // if the header format is 'full' we have to strip the header name
+      if (hdr.findRev("full") == (int)(hdr.length() - 5))
+        data = data.right( data.length() - (hdrName.length() + 2) );
+
+      // add header
+      art->setHeader( new KMime::Headers::Generic( hdrName, art, data ) );
+    }
+
     // check if we have this article already in this group,
     // if so mark it as new (useful with leafnodes delay-body function)
     art2=byMessageId(art->messageID()->as7BitString(false));
@@ -430,7 +471,7 @@ void KNGroup::insortNewHeaders(QStrList *hdrs, KNProtocolClient *client)
       delete art;
       new_cnt++;
     }
-    else if(append(art)) {
+    else if (append(art)) {
       added_cnt++;
       new_cnt++;
     }
@@ -451,13 +492,9 @@ void KNGroup::insortNewHeaders(QStrList *hdrs, KNProtocolClient *client)
   updateThreadInfo();
 
   // save the new headers
-  int count = saveStaticData(added_cnt);
+  saveStaticData(added_cnt);
   saveDynamicData(added_cnt);
 
-#ifndef NDEBUG
-  qDebug("knode: %d headers wrote to file",count);
-#endif
-
   // update group-info
   c_ount=length();
   n_ewCount+=new_cnt;
@@ -509,10 +546,21 @@ int KNGroup::saveStaticData(int cnt,bool ovr)
       ts << art->id() << ' ';
       ts << art->lines()->numberOfLines() << ' ';
       ts << art->date()->unixTime() << ' ';
-      ts << "1\n";       // version number to achieve backward compatibility easily
+      ts << "2\n";       // version number to achieve backward compatibility easily
 
       ts << art->articleNumber() << '\n';
 
+      // optional headers
+      ts << mOptionalHeaders.count() << '\n';
+      for (QCString hdrName = mOptionalHeaders.first(); hdrName; hdrName = mOptionalHeaders.next()) {
+        hdrName = hdrName.left( hdrName.find(':') );
+        KMime::Headers::Base *hdr = art->getHeaderByType( hdrName );
+        if ( hdr )
+          ts << hdrName << ": " << hdr->asUnicodeString() << '\n';
+        else
+          ts << hdrName << ": \n";
+      }
+
       savedCnt++;
     }
 
@@ -759,7 +807,7 @@ void KNGroup::buildThreads(int cnt, KNProtocolClient *client)
   int iterationCount;
   for (int idx=start; idx<end; idx++){
     art=at(idx);
-    startId=art->id();    
+    startId=art->id();
     isLoop=false;
     iterationCount=0;
     while(art->idRef()!=0 && !isLoop && (iterationCount < end)) {
@@ -768,14 +816,14 @@ void KNGroup::buildThreads(int cnt, KNProtocolClient *client)
       iterationCount++;
     }
 
-    if(isLoop) {      
+    if(isLoop) {
       // this method is called from the nntp-thread!!!
       #ifndef NDEBUG
       qDebug("knode: Sorting : loop in %d",startId);
       #endif
       art=at(idx);
       art->setIdRef(0);
-      art->setThreadingLevel(0);      
+      art->setThreadingLevel(0);
     }
   }
 
@@ -1042,3 +1090,11 @@ void KNGroup::dynDataVer1::getData(KNRemoteArticle *a)
   a->setWatched(ignoredWatched==1);
   a->setIgnored(ignoredWatched==2);
 }
+
+
+KNConfig::Cleanup * KNGroup::activeCleanupConfig()
+{
+  if (!cleanupConfig()->useDefault())
+    return cleanupConfig();
+  return account()->activeCleanupConfig();
+}
