@@ -25,30 +25,32 @@
 #include <qdict.h>
 #include <qptrlist.h>
 #include <qlabel.h>
+#include <qdom.h>
 
 #include <kapplication.h>
 #include <kxmlguiclient.h>
-#include <kxmlguibuilder.h>
 
-#include "libkcal/calendarlocal.h"
-
-#include "KNotesIface.h"
+#include "KNotesAppIface.h"
 
+class KFind;
 class KNote;
-class KNoteConfig;
 class KPopupMenu;
 class KAction;
 class KActionMenu;
 class KGlobalAccel;
 class KXMLGUIFactory;
+class KXMLGUIBuilder;
+class KExtendedSocket;
+class KNotesAlarm;
+class KNotesResourceManager;
 
 namespace KCal {
     class Journal;
 }
 
 
-class KNotesApp : public QLabel, virtual public KNotesIface, public KSessionManaged,
-    public KXMLGUIBuilder, virtual public KXMLGUIClient
+class KNotesApp : public QLabel, public KSessionManaged, virtual public KXMLGUIClient,
+    virtual public KNotesAppIface
 {
     Q_OBJECT
 public:
@@ -80,12 +82,18 @@ public slots:
                      const QString& text = QString::null );
     QString newNoteFromClipboard( const QString& name = QString::null );
 
+    void hideAllNotes() const;
+    void showAllNotes() const;
+
 protected:
     void mousePressEvent( QMouseEvent* );
-    bool eventFilter( QObject*, QEvent* );
 
 protected slots:
     void slotShowNote();
+    void slotWalkThroughNotes();
+
+    void slotOpenFindDialog();
+    void slotFindNext();
 
     void slotPreferences();
     void slotConfigureAccels();
@@ -99,11 +107,14 @@ private:
     void saveConfigs();
 
 private slots:
+    void acceptConnection();
     void saveNotes();
     void updateNoteActions();
     void updateGlobalAccels();
+    void updateNetworkListener();
 
     void createNote( KCal::Journal *journal );
+    void killNote( KCal::Journal *journal );
 
 private:
     class KNoteActionList : public QPtrList<KAction>
@@ -112,18 +123,25 @@ private:
         virtual int compareItems( QPtrCollection::Item s1, QPtrCollection::Item s2 );
     };
 
-    KCal::CalendarLocal m_calendar;
+    KNotesResourceManager *m_manager;
 
-    KNoteConfig     *m_defaultConfig;
+    KNotesAlarm     *m_alarm;
+    KExtendedSocket *m_listener;
 
     QDict<KNote>    m_noteList;
     KNoteActionList m_noteActions;
 
+    KFind           *m_find;
+    QDictIterator<KNote> *m_findPos;
+
     KPopupMenu      *m_note_menu;
     KPopupMenu      *m_context_menu;
 
     KGlobalAccel    *m_globalAccel;
     KXMLGUIFactory  *m_guiFactory;
+    KXMLGUIBuilder  *m_guiBuilder;
+
+    QDomDocument    m_noteGUI;
 };
 
 #endif
