@@ -68,9 +68,9 @@ KAddressbookPlugin::~KAddressbookPlugin()
 {
 }
 
-KParts::Part* KAddressbookPlugin::createPart()
+KParts::ReadOnlyPart* KAddressbookPlugin::createPart()
 {
-  KParts::Part * part = loadPart();
+  KParts::ReadOnlyPart * part = loadPart();
   if ( !part ) return 0;
 
   // Create the stub that allows us to talk to the part
@@ -118,6 +118,10 @@ bool KAddressbookPlugin::createDCOPInterface( const QString& serviceType )
   return false;
 }
 
+void KAddressbookPlugin::configUpdated()
+{
+}
+
 bool KAddressbookPlugin::isRunningStandalone()
 {
   return mUniqueAppWatcher->isRunningStandalone();
