@@ -34,7 +34,7 @@
 #include "plugin.h"
 
 #include <qlabel.h>
-#include "splash.h"
+#include "prefs.h"
 
 #include "mainwindow.h"
 
@@ -43,7 +43,7 @@ using namespace std;
 static const char description[] =
     I18N_NOOP( "KDE personal information manager" );
 
-static const char version[] = "1.0.2";
+static const char version[] = "1.1";
 
 class KontactApp : public KUniqueApplication {
   public:
@@ -62,18 +62,21 @@ static void listPlugins()
   KTrader::OfferList offers = KTrader::self()->query(
     QString::fromLatin1( "Kontact/Plugin" ),
     QString( "[X-KDE-KontactPluginVersion] == %1" ).arg( KONTACT_PLUGIN_VERSION ) );
-  for(KService::List::Iterator it = offers.begin(); it != offers.end(); ++it)
-  {
+  for ( KService::List::Iterator it = offers.begin(); it != offers.end(); ++it ) {
     KService::Ptr service = (*it);
+    // skip summary only plugins
+    QVariant var = service->property( "X-KDE-KontactPluginHasPart" );
+    if ( var.isValid() && var.toBool() == false )
+      continue;
     cout << service->library().remove( "libkontact_" ).latin1() << endl;
   }
 }
 
 static KCmdLineOptions options[] =
 {
-    { "module <module>",   I18N_NOOP("Start with a specific Kontact module"), 0 },
-    { "nosplash",   I18N_NOOP("Disable the splash screen"), 0 },
-    { "list", I18N_NOOP("List all possible modules and exit"), 0 },
+    { "module <module>",   I18N_NOOP( "Start with a specific Kontact module" ), 0 },
+    { "iconify",   I18N_NOOP( "Start in iconified (minimized) mode" ), 0 },
+    { "list", I18N_NOOP( "List all possible modules and exit" ), 0 },
     KCmdLineLastOption
 };
 
@@ -82,34 +85,35 @@ int KontactApp::newInstance()
 {
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
   QString moduleName;
-  if ( args->isSet("module") )
-  {
-    moduleName = QString::fromLocal8Bit(args->getOption("module"));
+  if ( Kontact::Prefs::self()->forceStartupPlugin() ) {
+    moduleName = Kontact::Prefs::self()->forcedStartupPlugin();
+  }
+  if ( args->isSet( "module" ) ) {
+    moduleName = QString::fromLocal8Bit( args->getOption( "module" ) );
   }
-  Kontact::Splash* splash = new Kontact::Splash( 0, "splash" );
-  if ( !mMainWindow && args->isSet("splash") ) // only the first time
-    splash->show();
 
   if ( isRestored() ) {
     // There can only be one main window
     if ( KMainWindow::canBeRestored( 1 ) ) {
-      mMainWindow = new Kontact::MainWindow(splash);
+      mMainWindow = new Kontact::MainWindow();
       setMainWidget( mMainWindow );
       mMainWindow->show();
       mMainWindow->restore( 1 );
     }
   } else {
     if ( !mMainWindow ) {
-      mMainWindow = new Kontact::MainWindow(splash);
+      mMainWindow = new Kontact::MainWindow();
       if ( !moduleName.isEmpty() )
-        mMainWindow->activePluginModule( moduleName );
+        mMainWindow->setActivePluginModule( moduleName );
       mMainWindow->show();
       setMainWidget( mMainWindow );
-    }
-    else
-    {
+      // --iconify is needed in kontact, although kstart can do that too,
+      // because kstart returns immediately so it's too early to talk DCOP to the app.
+      if ( args->isSet( "iconify" ) )
+        KWin::iconifyWindow( mMainWindow->winId(), false /*no animation*/ );
+    } else {
       if ( !moduleName.isEmpty() )
-        mMainWindow->activePluginModule( moduleName );
+        mMainWindow->setActivePluginModule( moduleName );
     }
   }
 
@@ -118,7 +122,7 @@ int KontactApp::newInstance()
   return KUniqueApplication::newInstance();
 }
 
-int main(int argc, char **argv)
+int main( int argc, char **argv )
 {
   KAboutData about( "kontact", I18N_NOOP( "Kontact" ), version, description,
                     KAboutData::License_GPL, I18N_NOOP("(C) 2001-2004 The Kontact developers"), 0, "http://kontact.org" );
@@ -138,8 +142,7 @@ int main(int argc, char **argv)
   KApplication::addCmdLineOptions();
 
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
-  if ( args->isSet( "list" ) )
-  {
+  if ( args->isSet( "list" ) ) {
     listPlugins();
     return 0;
   }
