@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2001,2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -71,8 +72,8 @@ class KDateNavigator: public QFrame
 
   signals:
     void datesSelected( const KCal::DateList & );
-    void incidenceDropped( Incidence * );
-    void incidenceDroppedMove( Incidence *, Incidence * );
+    void incidenceDropped( Incidence *, const QDate & );
+    void incidenceDroppedMove( Incidence *, const QDate & );
     void weekClicked( const QDate &);
 
     void goPrevious();
