@@ -1,6 +1,7 @@
 /*
     This file is part of KOrganizer.
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     Marcus Bains line.
     Copyright (c) 2001 Ali Rahimi
@@ -42,9 +43,12 @@
 #include "koagendaitem.h"
 #include "koprefs.h"
 #include "koglobals.h"
+#include "komessagebox.h"
+#include "incidencechanger.h"
 
 #include "koagenda.h"
 #include "koagenda.moc"
+#include <korganizer/baseview.h>
 
 #include <libkcal/event.h>
 #include <libkcal/todo.h>
@@ -52,6 +56,8 @@
 #include <libkcal/icaldrag.h>
 #include <libkcal/vcaldrag.h>
 #include <libkcal/calendar.h>
+#include <libkcal/calendarresources.h>
+#include <math.h>
 
 ////////////////////////////////////////////////////////////////////////////
 MarcusBains::MarcusBains(KOAgenda *_agenda,const char *name)
@@ -152,7 +158,7 @@ void MarcusBains::updateLocation(bool recalculate)
 */
 KOAgenda::KOAgenda( int columns, int rows, int rowSize, QWidget *parent,
                     const char *name, WFlags f )
-  : QScrollView( parent, name, f )
+  : QScrollView( parent, name, f ), mChanger( 0 )
 {
   mColumns = columns;
   mRows = rows;
@@ -160,6 +166,8 @@ KOAgenda::KOAgenda( int columns, int rows, int rowSize, QWidget *parent,
   mAllDayMode = false;
 
   init();
+
+  viewport()->setMouseTracking(true);
 }
 
 /*
@@ -244,7 +252,6 @@ void KOAgenda::init()
   mItems.setAutoDelete( true );
   mItemsToDelete.setAutoDelete( true );
 
-//  resizeContents( int(mGridSpacingX * mColumns + 1) , int(mGridSpacingY * mRows + 1) );
   resizeContents( int( mGridSpacingX * mColumns ),
                   int( mGridSpacingY * mRows ) );
 
@@ -344,7 +351,10 @@ bool KOAgenda::eventFilter ( QObject *object, QEvent *event )
     case QEvent::MouseButtonRelease:
     case QEvent::MouseMove:
       return eventFilter_mouse( object, static_cast<QMouseEvent *>( event ) );
-
+#ifndef QT_NO_WHEELEVENT
+    case QEvent::Wheel:
+      return eventFilter_wheel( object, static_cast<QWheelEvent *>( event ) );
+#endif
     case QEvent::KeyPress:
     case QEvent::KeyRelease:
       return eventFilter_key( object, static_cast<QKeyEvent *>( event ) );
@@ -352,8 +362,14 @@ bool KOAgenda::eventFilter ( QObject *object, QEvent *event )
     case ( QEvent::Leave ):
       if ( !mActionItem )
         setCursor( arrowCursor );
+      if ( object == viewport() )
+        emit leaveAgenda();
       return true;
 
+    case QEvent::Enter:
+      emit enterAgenda();
+      return QScrollView::eventFilter( object, event );
+
 #ifndef KORG_NODND
     case QEvent::DragEnter:
     case QEvent::DragMove:
@@ -435,7 +451,7 @@ bool KOAgenda::eventFilter_drag( QObject *object, QDropEvent *de )
 
 bool KOAgenda::eventFilter_key( QObject *, QKeyEvent *ke )
 {
-//  kdDebug() << "KOAgenda::eventFilter_key() " << ke->type() << endl;
+  // kdDebug(5850) << "KOAgenda::eventFilter_key() " << ke->type() << endl;
 
   // If Return is pressed bring up an editor for the current selected time span.
   if ( ke->key() == Key_Return ) {
@@ -497,18 +513,52 @@ void KOAgenda::emitNewEventForSelection()
 
 void KOAgenda::finishTypeAhead()
 {
-//  kdDebug() << "KOAgenda::finishTypeAhead()" << endl;
+//  kdDebug(5850) << "KOAgenda::finishTypeAhead()" << endl;
   if ( typeAheadReceiver() ) {
     for( QEvent *e = mTypeAheadEvents.first(); e;
          e = mTypeAheadEvents.next() ) {
-//      kdDebug() << "sendEvent() " << int( typeAheadReceiver() ) << endl;
+//      kdDebug(5850) << "sendEvent() " << int( typeAheadReceiver() ) << endl;
       QApplication::sendEvent( typeAheadReceiver(), e );
     }
   }
   mTypeAheadEvents.clear();
   mTypeAhead = false;
 }
+#ifndef QT_NO_WHEELEVENT
+bool KOAgenda::eventFilter_wheel ( QObject *object, QWheelEvent *e )
+{
+  QPoint viewportPos;
+  bool accepted=false;
+  if  ( ( e->state() & ShiftButton) == ShiftButton ) {
+    if ( object != viewport() ) {
+      viewportPos = ( (QWidget *) object )->mapToParent( e->pos() );
+    } else {
+      viewportPos = e->pos();
+    }
+    //kdDebug(5850)<< "KOAgenda::eventFilter_wheel: type:"<<
+    //  e->type()<<" delta: "<< e->delta()<< endl;
+    emit zoomView( -e->delta() ,
+      contentsToGrid( viewportToContents( viewportPos ) ),
+      Qt::Horizontal );
+    accepted=true;
+  }
 
+  if  ( ( e->state() & ControlButton ) == ControlButton ){
+    if ( object != viewport() ) {
+      viewportPos = ( (QWidget *)object )->mapToParent( e->pos() );
+    } else {
+      viewportPos = e->pos();
+    }
+    emit zoomView( -e->delta() ,
+      contentsToGrid( viewportToContents( viewportPos ) ),
+      Qt::Vertical );
+    emit mousePosSignal(gridToContents(contentsToGrid(viewportToContents( viewportPos ))));
+    accepted=true;
+  }
+  if (accepted ) e->accept();
+  return accepted;
+}
+#endif
 bool KOAgenda::eventFilter_mouse(QObject *object, QMouseEvent *me)
 {
   QPoint viewportPos;
@@ -530,16 +580,20 @@ bool KOAgenda::eventFilter_mouse(QObject *object, QMouseEvent *me)
                                            mClickedItem->itemDate() );
           }
         } else {
-          mActionItem = dynamic_cast<KOAgendaItem *>(object);
-          if (mActionItem) {
-            selectItem(mActionItem);
-            Incidence *incidence = mActionItem->incidence();
-// OLD_RK:            if ( incidence->isReadOnly() || incidence->doesRecur() ) {
+          KOAgendaItem* item = dynamic_cast<KOAgendaItem *>(object);
+          if (item) {
+            Incidence *incidence = item->incidence();
             if ( incidence->isReadOnly() ) {
               mActionItem = 0;
             } else {
+              mActionItem = item;
               startItemAction(viewportPos);
             }
+            // Warning: do selectItem() as late as possible, since all
+            // sorts of things happen during this call. Some can lead to
+            // this filter being run again and mActionItem being set to
+            // null.
+            selectItem(mActionItem);
           }
         }
       } else {
@@ -582,8 +636,6 @@ bool KOAgenda::eventFilter_mouse(QObject *object, QMouseEvent *me)
     case QEvent::MouseMove:
       if (object != viewport()) {
         KOAgendaItem *moveItem = dynamic_cast<KOAgendaItem *>(object);
-// OLD_RK:        if (moveItem && !moveItem->incidence()->isReadOnly() &&
-//            !moveItem->incidence()->doesRecur() )
         if (moveItem && !moveItem->incidence()->isReadOnly() )
           if (!mActionItem)
             setNoActionCursor(moveItem,viewportPos);
@@ -592,8 +644,18 @@ bool KOAgenda::eventFilter_mouse(QObject *object, QMouseEvent *me)
       } else {
           if ( mActionType == SELECT ) {
             performSelectAction( viewportPos );
+
+            // show cursor at end of timespan
+            if ( ((mStartCell.y() < mEndCell.y()) && (mEndCell.x() >= mStartCell.x())) ||
+                 (mEndCell.x() > mStartCell.x()) )
+              emit mousePosSignal(gridToContents(QPoint(mEndCell.x(), mEndCell.y()+1)));
+            else
+              emit mousePosSignal(gridToContents(mEndCell));
+
+            return true;
           }
         }
+      emit mousePosSignal(gridToContents(contentsToGrid(viewportToContents( viewportPos ))));
       break;
 
     case QEvent::MouseButtonDblClick:
@@ -690,6 +752,8 @@ void KOAgenda::endSelectAction( const QPoint &currentPos )
   mScrollUpTimer.stop();
   mScrollDownTimer.stop();
 
+  mActionType = NOP;
+
   emit newTimeSpanSignal( mSelectionStartCell, mSelectionEndCell );
 
   if ( KOPrefs::instance()->mSelectionStartsEditor ) {
@@ -708,8 +772,8 @@ KOAgenda::MouseActionType KOAgenda::isInResizeArea( bool horizontal,
   QPoint contpos = gridToContents( gridpos +
       QPoint( (KOGlobals::self()->reverseLayout())?1:0, 0 ) );
 
-//kdDebug()<<"contpos="<<contpos<<", pos="<<pos<<", gpos="<<gpos<<endl;
-//kdDebug()<<"clXLeft="<<clXLeft<<", clXRight="<<clXRight<<endl;
+//kdDebug(5850)<<"contpos="<<contpos<<", pos="<<pos<<", gpos="<<gpos<<endl;
+//kdDebug(5850)<<"clXLeft="<<clXLeft<<", clXRight="<<clXRight<<endl;
 
   if ( horizontal ) {
     int clXLeft = item->cellXLeft();
@@ -759,6 +823,7 @@ void KOAgenda::startItemAction(const QPoint& viewportPos)
     mActionType = isInResizeArea( mAllDayMode, pos, mActionItem );
   }
 
+
   mActionItem->startMove();
   setActionCursor( mActionType, true );
 }
@@ -791,6 +856,8 @@ void KOAgenda::performItemAction(const QPoint& viewportPos)
       mActionItem = 0;
       mActionType = NOP;
       mItemMoved = false;
+      if ( mItemMoved && mChanger )
+        mChanger->endChange( mActionItem->incidence() );
       return;
     }
   } else {
@@ -810,7 +877,23 @@ void KOAgenda::performItemAction(const QPoint& viewportPos)
 
   // Move or resize item if necessary
   if ( mEndCell != gpos ) {
-    mItemMoved = true;
+    if ( !mItemMoved ) {
+      if ( !mChanger || !mChanger->beginChange( mActionItem->incidence() ) ) {
+        KMessageBox::information( this, i18n("Unable to lock incidence for "
+                             "modification. You cannot change make any changes."),
+                             i18n("Locking Failed"), "AgendaLockingFailed" );
+        mScrollUpTimer.stop();
+        mScrollDownTimer.stop();
+        mActionItem->resetMove();
+        placeSubCells( mActionItem );
+        setCursor( arrowCursor );
+        mActionItem = 0;
+        mActionType = NOP;
+        mItemMoved = false;
+        return;
+      }
+      mItemMoved = true;
+    }
     mActionItem->raise();
     if (mActionType == MOVE) {
       // Move all items belonging to a multi item
@@ -936,71 +1019,76 @@ void KOAgenda::endItemAction()
   mScrollDownTimer.stop();
   setCursor( arrowCursor );
   bool multiModify = false;
+  // FIXME: do the cloning here...
 
   if ( mItemMoved ) {
     bool modify = true;
     if ( mActionItem->incidence()->doesRecur() ) {
-      int res = KMessageBox::questionYesNoCancel( this, 
+      int res = KOMessageBox::fourBtnMsgBox( this, QMessageBox::Question,
           i18n("The item you try to change is a recurring item. Shall the changes "
-               "be applied to all items in the recurrence, "/*"only the future items, "*/
-               "or just to this single occurrence?"), 
-          i18n("Changing a recurring item"), 
-          i18n("&All occurrences"), i18n("Only &this item") );
+               "be applied only to this single occurrence, only to the future items, "
+               "or to all items in the recurrence?"),
+          i18n("Changing Recurring Item"),
+          i18n("Only &This Item"), i18n("Only &Future Items"), i18n("&All Occurrences") );
       switch ( res ) {
-        case KMessageBox::Yes: // All occurences
+        case KMessageBox::Ok: // All occurences
             // Moving the whole sequene of events is handled by the itemModified below.
             modify = true;
             break;
-        case KMessageBox::No: { // Just this occurence
-            // Dissociate this occurence: 
-            // create clone of event, set relation to old event, set cloned event 
-            // for mActionItem, add exception date to old event, emit incidenceChanged 
-            // for the old event, remove the recurrence from the new copy and then just 
-            // go on with the newly adjusted mActionItem and let the usual code take 
+        case KMessageBox::Yes: { // Just this occurence
+            // Dissociate this occurence:
+            // create clone of event, set relation to old event, set cloned event
+            // for mActionItem, add exception date to old event, changeIncidence
+            // for the old event, remove the recurrence from the new copy and then just
+            // go on with the newly adjusted mActionItem and let the usual code take
             // care of the new time!
             modify = true;
             multiModify = true;
             emit startMultiModify( i18n("Dissociate event from recurrence") );
-            Incidence* oldInc = mActionItem->incidence()->clone();
-            Incidence* newInc = mCalendar->dissociateOccurrence( 
-                mActionItem->incidence(), mActionItem->itemDate() );
+            Incidence* oldInc = mActionItem->incidence();
+            Incidence* oldIncSaved = mActionItem->incidence()->clone();
+            Incidence* newInc = mCalendar->dissociateOccurrence(
+                oldInc, mActionItem->itemDate() );
             if ( newInc ) {
               // don't recreate items, they already have the correct position
               emit enableAgendaUpdate( false );
-              emit incidenceChanged( oldInc, mActionItem->incidence() );
+              mChanger->changeIncidence( oldIncSaved, oldInc );
               mActionItem->setIncidence( newInc );
-              emit incidenceAdded( newInc );
+              mActionItem->dissociateFromMultiItem();
+              mChanger->addIncidence( newInc );
               emit enableAgendaUpdate( true );
             } else {
               KMessageBox::sorry( this, i18n("Unable to add the exception item to the "
                   "calendar. No change will be done."), i18n("Error Occurred") );
             }
-            delete oldInc;
+            delete oldIncSaved;
             break; }
-        case KMessageBox::Continue/*Future*/: { // All future occurences
-            // Dissociate this occurence: 
-            // create clone of event, set relation to old event, set cloned event 
-            // for mActionItem, add recurrence end date to old event, emit incidenceChanged 
-            // for the old event, adjust the recurrence for the new copy and then just 
-            // go on with the newly adjusted mActionItem and let the usual code take 
+        case KMessageBox::No/*Future*/: { // All future occurences
+            // Dissociate this occurence:
+            // create clone of event, set relation to old event, set cloned event
+            // for mActionItem, add recurrence end date to old event, changeIncidence
+            // for the old event, adjust the recurrence for the new copy and then just
+            // go on with the newly adjusted mActionItem and let the usual code take
             // care of the new time!
             modify = true;
             multiModify = true;
             emit startMultiModify( i18n("Split future recurrences") );
-            Incidence* oldInc = mActionItem->incidence()->clone();
-            Incidence* newInc = mCalendar->dissociateOccurrence( 
-                mActionItem->incidence(), mActionItem->itemDate(), true );
+            Incidence* oldInc = mActionItem->incidence();
+            Incidence* oldIncSaved = mActionItem->incidence()->clone();
+            Incidence* newInc = mCalendar->dissociateOccurrence(
+                oldInc, mActionItem->itemDate(), false );
             if ( newInc ) {
-              emit incidenceChanged( oldInc, mActionItem->incidence() );
               emit enableAgendaUpdate( false );
+              mActionItem->dissociateFromMultiItem();
               mActionItem->setIncidence( newInc );
-              emit incidenceAdded( newInc );
+              mChanger->addIncidence( newInc );
               emit enableAgendaUpdate( true );
+              mChanger->changeIncidence( oldIncSaved, oldInc );
             } else {
               KMessageBox::sorry( this, i18n("Unable to add the future items to the "
                   "calendar. No change will be done."), i18n("Error Occurred") );
             }
-            delete oldInc;
+            delete oldIncSaved;
             break; }
         default:
           modify = false;
@@ -1012,6 +1100,7 @@ void KOAgenda::endItemAction()
     if ( modify ) {
       mActionItem->endMove();
       KOAgendaItem *placeItem = mActionItem->firstMultiItem();
+      // FIXME: A mChanger->changeIncidence is missing here!
       if  ( !placeItem ) {
         placeItem = mActionItem;
       }
@@ -1032,6 +1121,8 @@ void KOAgenda::endItemAction()
       // Notify about change, so that agenda view can update the event data
       emit itemModified( modif );
     }
+    // FIXME: If the change failed, we need to update the view!
+    mChanger->endChange( mActionItem->incidence() );
   }
 
   mActionItem = 0;
@@ -1114,7 +1205,7 @@ void KOAgenda::adjustItemPosition( KOAgendaItem *item )
 
 void KOAgenda::placeAgendaItem( KOAgendaItem *item, double subCellWidth )
 {
-//  kdDebug() << "KOAgenda::placeAgendaItem(): " << item->incidence()->summary()
+//  kdDebug(5850) << "KOAgenda::placeAgendaItem(): " << item->incidence()->summary()
 //            << " subCellWidth: " << subCellWidth << endl;
 
   // "left" upper corner, no subcells yet, RTL layouts have right/left switched, widths are negative then
@@ -1297,7 +1388,9 @@ void KOAgenda::drawContents(QPainter* p, int cx, int cy, int cw, int ch)
     }
   }
 
-  dbp.setPen( KOPrefs::instance()->mAgendaBgColor.dark(150) );
+  QPen hourPen( KOPrefs::instance()->mAgendaBgColor.dark( 150 ) );
+  QPen halfHourPen( KOPrefs::instance()->mAgendaBgColor.dark( 125 ) );
+  dbp.setPen( hourPen );
 
   // Draw vertical lines of grid, start with the last line not yet visible
   //  kdDebug(5850) << "drawContents cx: " << cx << " cy: " << cy << " cw: " << cw << " ch: " << ch << endl;
@@ -1308,11 +1401,18 @@ void KOAgenda::drawContents(QPainter* p, int cx, int cy, int cw, int ch)
   }
 
   // Draw horizontal lines of grid
-  double y = ( int( cy / lGridSpacingY ) ) * lGridSpacingY;
+  double y = ( int( cy / (2*lGridSpacingY) ) ) * 2 * lGridSpacingY;
   while (y < cy + ch) {
 //    kdDebug(5850) << " y: " << y << endl;
     dbp.drawLine( cx, int( y ), cx + cw, int( y ) );
-    y+=lGridSpacingY;
+    y += 2 * lGridSpacingY;
+  }
+  y = ( 2 * int( cy / (2*lGridSpacingY) ) + 1) * lGridSpacingY;
+  dbp.setPen( halfHourPen );
+  while (y < cy + ch) {
+//    kdDebug(5850) << " y: " << y << endl;
+    dbp.drawLine( cx, int( y ), cx + cw, int( y ) );
+    y+=2*lGridSpacingY;
   }
   p->drawPixmap(cx,cy, db);
 }
@@ -1385,12 +1485,13 @@ QMemArray<int> KOAgenda::minContentsY()
   QMemArray<int> minArray;
   minArray.fill( timeToY( QTime(23, 59) ), mSelectedDates.count() );
   for ( KOAgendaItem *item = mItems.first();
-        item != 0;
-        item = mItems.next() ) {
-    int timeY = timeToY( item->incidence()->dtStart().time() );
-    int index = mSelectedDates.findIndex( item->incidence()->dtStart().date() );
-    if( timeY < minArray[index] && mItemsToDelete.findRef( item ) == -1 )
-      minArray[index] = timeY;
+        item != 0; item = mItems.next() ) {
+    int ymin = item->cellYTop();
+    int index = item->cellXLeft();
+    if ( index>=0 && index<(int)(mSelectedDates.count()) ) {
+      if ( ymin < minArray[index] && mItemsToDelete.findRef( item ) == -1 )
+        minArray[index] = ymin;
+    }
   }
 
   return minArray;
@@ -1401,23 +1502,19 @@ QMemArray<int> KOAgenda::maxContentsY()
   QMemArray<int> maxArray;
   maxArray.fill( timeToY( QTime(0, 0) ), mSelectedDates.count() );
   for ( KOAgendaItem *item = mItems.first();
-        item != 0;
-        item = mItems.next() ) {
-    QDateTime dtEnd;
-    if ( item->incidence()->type() == "Todo" )
-      dtEnd = static_cast<Todo *>( item->incidence() )->dtDue();
-    else
-      dtEnd = item->incidence()->dtEnd();
-    int timeY = timeToY( dtEnd.time() );
-    int index = mSelectedDates.findIndex( dtEnd.date() );
-    if( timeY > maxArray[index] && mItemsToDelete.findRef( item ) == -1 )
-      maxArray[index] = timeY - 1;
+        item != 0; item = mItems.next() ) {
+    int ymax = item->cellYBottom();
+    int index = item->cellXLeft();
+    if ( index>=0 && index<(int)(mSelectedDates.count()) ) {
+      if ( ymax > maxArray[index] && mItemsToDelete.findRef( item ) == -1 )
+        maxArray[index] = ymax;
+    }
   }
 
   return maxArray;
 }
 
-void KOAgenda::setStartTime( QTime startHour )
+void KOAgenda::setStartTime( const QTime &startHour )
 {
   double startPos = ( startHour.hour()/24. + startHour.minute()/1440. +
                       startHour.second()/86400. ) * mRows * gridSpacingY();
@@ -1428,7 +1525,7 @@ void KOAgenda::setStartTime( QTime startHour )
 /*
   Insert KOAgendaItem into agenda.
 */
-KOAgendaItem *KOAgenda::insertItem( Incidence *incidence, QDate qd, int X,
+KOAgendaItem *KOAgenda::insertItem( Incidence *incidence, const QDate &qd, int X,
                                     int YTop, int YBottom )
 {
 #if 0
@@ -1441,6 +1538,19 @@ KOAgendaItem *KOAgenda::insertItem( Incidence *incidence, QDate qd, int X,
     kdDebug(5850) << "KOAgenda: calling insertItem in all-day mode is illegal." << endl;
     return 0;
   }
+
+  //FIXME: dynamic_cast are dirty, Better We implements interface to get the color
+  // from the calendar
+  QColor resourceColor = QColor(); //Default invalid color
+  CalendarResources *calendarResource = dynamic_cast<CalendarResources*>( mCalendar );
+
+  if ( calendarResource ) {
+    ResourceCalendar *resourceCalendar = calendarResource->resource( incidence );
+    resourceColor = *KOPrefs::instance()->resourceColor( resourceCalendar->identifier() );
+  }else{
+    kdDebug(5850) << "KOAgenda:insertItem: Calendar is not a CalendarResources" <<endl;
+  }
+
   mActionType = NOP;
 
   KOAgendaItem *agendaItem = new KOAgendaItem( incidence, qd, viewport() );
@@ -1460,7 +1570,7 @@ KOAgendaItem *KOAgenda::insertItem( Incidence *incidence, QDate qd, int X,
                       int( ( YBottom + 1 ) * mGridSpacingY ) );
   agendaItem->setCellXY( X, YTop, YBottom );
   agendaItem->setCellXRight( X );
-
+  agendaItem->setResourceColor(resourceColor);
   agendaItem->installEventFilter( this );
 
   addChild( agendaItem, int( X * mGridSpacingX ), int( YTop * mGridSpacingY ) );
@@ -1478,15 +1588,29 @@ KOAgendaItem *KOAgenda::insertItem( Incidence *incidence, QDate qd, int X,
 /*
   Insert all-day KOAgendaItem into agenda.
 */
-KOAgendaItem *KOAgenda::insertAllDayItem( Incidence *event, QDate qd,
+KOAgendaItem *KOAgenda::insertAllDayItem( Incidence *event, const QDate &qd,
                                           int XBegin, int XEnd )
 {
   if ( !mAllDayMode ) {
     kdDebug(5850) << "KOAgenda: calling insertAllDayItem in non all-day mode is illegal." << endl;
     return 0;
   }
+
   mActionType = NOP;
 
+  QColor resourceColor = QColor(); //Default invalid color
+  CalendarResources *calendarResource = dynamic_cast<CalendarResources*>(mCalendar);
+  if ( calendarResource ) {
+    ResourceCalendar *resourceCalendar = calendarResource->resource( event );
+    if ( resourceCalendar )
+      resourceColor = *KOPrefs::instance()->resourceColor( resourceCalendar->identifier() );
+    else
+      kdDebug(5850) << "KOAgenda:insertAllDayItem: resource of "<< event->summary()
+        << " is null" << endl;
+  }else{
+    kdDebug(5850) << "KOAgenda:insertAllDayItem: mCalendar is not a CalendarResources" <<endl;
+  }
+
   KOAgendaItem *agendaItem = new KOAgendaItem( event, qd, viewport() );
   connect( agendaItem, SIGNAL( removeAgendaItem( KOAgendaItem* ) ),
            SLOT( removeAgendaItem( KOAgendaItem* ) ) );
@@ -1503,7 +1627,7 @@ KOAgendaItem *KOAgenda::insertAllDayItem( Incidence *event, QDate qd,
   agendaItem->resize( int( endIt ) - int( startIt ), int( mGridSpacingY ) );
 
   agendaItem->installEventFilter( this );
-
+  agendaItem->setResourceColor( resourceColor );
   addChild( agendaItem, int( XBegin * mGridSpacingX ), 0 );
   mItems.append( agendaItem );
 
@@ -1515,7 +1639,7 @@ KOAgendaItem *KOAgenda::insertAllDayItem( Incidence *event, QDate qd,
 }
 
 
-void KOAgenda::insertMultiItem (Event *event,QDate qd,int XBegin,int XEnd,
+void KOAgenda::insertMultiItem (Event *event,const QDate &qd,int XBegin,int XEnd,
                                 int YTop,int YBottom)
 {
   if (mAllDayMode) {
@@ -1529,8 +1653,8 @@ void KOAgenda::insertMultiItem (Event *event,QDate qd,int XBegin,int XEnd,
   int width = XEnd - XBegin + 1;
   int count = 0;
   KOAgendaItem *current = 0;
-  int visibleCount = mSelectedDates.first().daysTo(mSelectedDates.last());
   QPtrList<KOAgendaItem> multiItems;
+  int visibleCount = mSelectedDates.first().daysTo(mSelectedDates.last());
   for ( cellX = XBegin; cellX <= XEnd; ++cellX ) {
     ++count;
     //Only add the items that are visible.
@@ -1541,6 +1665,7 @@ void KOAgenda::insertMultiItem (Event *event,QDate qd,int XBegin,int XEnd,
       else cellYBottom = rows() - 1;
       newtext = QString("(%1/%2): ").arg( count ).arg( width );
       newtext.append( event->summary() );
+
       current = insertItem( event, qd, cellX, cellYTop, cellYBottom );
       current->setText( newtext );
       multiItems.append( current );
@@ -1597,6 +1722,7 @@ void KOAgenda::showAgendaItem( KOAgendaItem *agendaItem )
   if ( !mItems.containsRef( agendaItem ) )
     mItems.append( agendaItem );
   placeSubCells( agendaItem );
+
   agendaItem->show();
 }
 
@@ -1653,23 +1779,21 @@ void KOAgenda::resizeEvent ( QResizeEvent *ev )
 //  kdDebug(5850) << "KOAgenda::resizeEvent" << endl;
   double subCellWidth;
   KOAgendaItem *item;
+  QSize newSize( ev->size() );
   if (mAllDayMode) {
-    mGridSpacingX = double( width() - 2 * frameWidth() ) / (double)mColumns;
-//    kdDebug(5850) << "Frame " << frameWidth() << endl;
-    mGridSpacingY = height() - 2 * frameWidth();
-    resizeContents( int( mGridSpacingX * mColumns ), int( mGridSpacingY ) );
+    mGridSpacingX = double( newSize.width() - 2 * frameWidth() ) / (double)mColumns;
+    mGridSpacingY = newSize.height() - 2 * frameWidth();
 
     for ( item=mItems.first(); item != 0; item=mItems.next() ) {
       subCellWidth = calcSubCellWidth( item );
       placeAgendaItem( item, subCellWidth );
     }
   } else {
-    mGridSpacingX = double(width() - verticalScrollBar()->width() - 2 * frameWidth()) / double(mColumns);
+    mGridSpacingX = double( newSize.width() - verticalScrollBar()->width() - 2 * frameWidth()) / double(mColumns);
     // make sure that there are not more than 24 per day
-    mGridSpacingY = double(height() - 2 * frameWidth()) / double(mRows);
-    if ( mGridSpacingY < mDesiredGridSpacingY ) mGridSpacingY = mDesiredGridSpacingY;
-
-    resizeContents( int( mGridSpacingX * mColumns ), int( mGridSpacingY * mRows ));
+    mGridSpacingY = double(newSize.height() - 2 * frameWidth()) / double(mRows);
+    if ( mGridSpacingY < mDesiredGridSpacingY )
+      mGridSpacingY = mDesiredGridSpacingY;
 
     for ( item=mItems.first(); item != 0; item=mItems.next() ) {
       subCellWidth = calcSubCellWidth( item );
@@ -1683,7 +1807,6 @@ void KOAgenda::resizeEvent ( QResizeEvent *ev )
   marcus_bains();
 
   QScrollView::resizeEvent(ev);
-  viewport()->update();
 }
 
 
@@ -1704,7 +1827,7 @@ void KOAgenda::scrollDown()
 */
 int KOAgenda::minimumWidth() const
 {
-  // TODO:: develop a way to dynamically determine the minimum width
+  // FIXME:: develop a way to dynamically determine the minimum width
   int min = 100;
 
   return min;
@@ -1712,11 +1835,17 @@ int KOAgenda::minimumWidth() const
 
 void KOAgenda::updateConfig()
 {
+  double oldGridSpacingY = mGridSpacingY;
   mDesiredGridSpacingY = KOPrefs::instance()->mHourSize;
  // make sure that there are not more than 24 per day
   mGridSpacingY = (double)height()/(double)mRows;
   if (mGridSpacingY<mDesiredGridSpacingY) mGridSpacingY=mDesiredGridSpacingY;
 
+  //can be two doubles equal?, it's better to compare them with an epsilon
+  if ( fabs( oldGridSpacingY - mGridSpacingY ) > 0.1 )
+    resizeContents( int( mGridSpacingX * mColumns ),
+                  int( mGridSpacingY * mRows ) );
+
   calculateWorkingHours();
 
   marcus_bains();
@@ -1731,23 +1860,35 @@ void KOAgenda::checkScrollBoundaries()
   checkScrollBoundaries(verticalScrollBar()->value());
 }
 
-void KOAgenda::checkScrollBoundaries(int v)
+void KOAgenda::checkScrollBoundaries( int v )
 {
-  int yMin = int( v / mGridSpacingY );
+  int yMin = int( (v) / mGridSpacingY );
   int yMax = int( ( v + visibleHeight() ) / mGridSpacingY );
 
 //  kdDebug(5850) << "--- yMin: " << yMin << "  yMax: " << yMax << endl;
 
-  if (yMin != mOldLowerScrollValue) {
+  if ( yMin != mOldLowerScrollValue ) {
     mOldLowerScrollValue = yMin;
     emit lowerYChanged(yMin);
   }
-  if (yMax != mOldUpperScrollValue) {
+  if ( yMax != mOldUpperScrollValue ) {
     mOldUpperScrollValue = yMax;
     emit upperYChanged(yMax);
   }
 }
 
+int KOAgenda::visibleContentsYMin()
+{
+  int v = verticalScrollBar()->value();
+  return int( v / mGridSpacingY );
+}
+
+int KOAgenda::visibleContentsYMax()
+{
+  int v = verticalScrollBar()->value();
+  return int( ( v + visibleHeight() ) / mGridSpacingY );
+}
+
 void KOAgenda::deselectItem()
 {
   if (mSelectedItem.isNull()) return;
