@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2000,2001,2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,15 +27,15 @@
 
 #include "cellitem.h"
 
-#include <libkcal/incidence.h>
-
-#include <qframe.h>
 #include <qdatetime.h>
 
 class QToolTipGroup;
 class QDragEnterEvent;
 class QDropEvent;
 
+namespace KCal {
+class Incidence;
+}
 using namespace KCal;
 class KOAgendaItem;
 
@@ -74,7 +75,7 @@ class KOAgendaItem : public QWidget, public KOrg::CellItem
 {
     Q_OBJECT
   public:
-    KOAgendaItem(Incidence *incidence, QDate qd, QWidget *parent, const char *name=0,
+    KOAgendaItem(Incidence *incidence, const QDate &qd, QWidget *parent, const char *name=0,
                  WFlags f=0 );
 
     int cellXLeft() const { return mCellXLeft; }
@@ -115,12 +116,15 @@ class KOAgendaItem : public QWidget, public KOrg::CellItem
     KOAgendaItem *prevMultiItem() const { return (mMultiItemInfo)?(mMultiItemInfo->mPrevMultiItem):0; }
     KOAgendaItem *nextMultiItem() const { return (mMultiItemInfo)?(mMultiItemInfo->mNextMultiItem):0; }
     KOAgendaItem *lastMultiItem() const { return (mMultiItemInfo)?(mMultiItemInfo->mLastMultiItem):0; }
+    
+    bool dissociateFromMultiItem();
 
+    bool setIncidence( Incidence * );
     Incidence *incidence() const { return mIncidence; }
     QDate itemDate() { return mDate; }
 
     /** Update the date of this item's occurrence (not in the event) */
-    void setItemDate(QDate qd);
+    void setItemDate( const QDate &qd );
 
     void setText ( const QString & text ) { mLabelText = text; }
     QString text () { return mLabelText; }
@@ -135,6 +139,8 @@ class KOAgendaItem : public QWidget, public KOrg::CellItem
 
     bool overlaps( KOrg::CellItem * ) const;
 
+    void setResourceColor( QColor& color ) { mResourceColor = color; }
+    QColor &resourceColor() {return mResourceColor;}
   signals:
     void removeAgendaItem( KOAgendaItem* );
     void showAgendaItem( KOAgendaItem* );
@@ -142,7 +148,7 @@ class KOAgendaItem : public QWidget, public KOrg::CellItem
   public slots:
     void updateIcons();
     void select(bool=true);
-    void addAttendee(QString);
+    void addAttendee( const QString & );
 
   protected:
     void dragEnterEvent(QDragEnterEvent *e);
@@ -168,14 +174,16 @@ class KOAgendaItem : public QWidget, public KOrg::CellItem
     QDate mDate; //date this events occurs (for recurrence)
     QString mLabelText;
     bool mIconAlarm, mIconRecur, mIconReadonly;
-    bool mIconReply, mIconGroup, mIconOrganizer;
+    bool mIconReply, mIconGroup, mIconGroupTentative;
+    bool mIconOrganizer;
 
     // Multi item pointers
     MultiItemInfo* mMultiItemInfo;
   protected:
     // Variables to remember start position
     MultiItemInfo* mStartMoveInfo;
-
+    //Color of the resource
+    QColor mResourceColor;
   private:
     static QToolTipGroup *mToolTipGroup;
 
