@@ -1,6 +1,7 @@
 /*
     This file is part of KOrganizer.
     Copyright (c) 2000-2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,6 +38,7 @@
 #include <qradiobutton.h>
 #include <qlabel.h>
 #include <qpushbutton.h>
+#include <qwhatsthis.h>
 
 #include <kdialog.h>
 #include <kglobal.h>
@@ -85,6 +87,10 @@ int RecurBase::frequency()
 QComboBox *RecurBase::createWeekCountCombo( QWidget *parent, const char *name )
 {
   QComboBox *combo = new QComboBox( parent, name );
+  QWhatsThis::add( combo,
+                   i18n("The number of the week from the beginning "
+                        "of the month on which this event or to-do "
+                        "should recur.") );
   if ( !combo ) return 0;
   combo->insertItem( i18n("1st") );
   combo->insertItem( i18n("2nd") );
@@ -102,6 +108,9 @@ QComboBox *RecurBase::createWeekCountCombo( QWidget *parent, const char *name )
 QComboBox *RecurBase::createWeekdayCombo( QWidget *parent, const char *name )
 {
   QComboBox *combo = new QComboBox( parent, name );
+  QWhatsThis::add( combo,
+                   i18n("The weekday on which this event or to-do "
+                        "should recur.") );
   if ( !combo ) return 0;
   const KCalendarSystem *calSys = KOGlobals::self()->calendarSystem();
   for( int i = 1; i <= 7; ++i ) {
@@ -113,6 +122,9 @@ QComboBox *RecurBase::createWeekdayCombo( QWidget *parent, const char *name )
 QComboBox *RecurBase::createMonthNameCombo( QWidget *parent, const char *name )
 {
   QComboBox *combo = new QComboBox( parent, name );
+  QWhatsThis::add( combo,
+                   i18n("The month during which this event or to-do "
+                        "should recur.") );
   if ( !combo ) return 0;
   const KCalendarSystem *calSys = KOGlobals::self()->calendarSystem();
   for( int i = 1; i <= 12; ++i ) {
@@ -128,13 +140,17 @@ QBoxLayout *RecurBase::createFrequencySpinBar( QWidget *parent, QLayout *layout,
 {
   QBoxLayout *freqLayout = new QHBoxLayout( layout );
 
+  QString whatsThis = i18n("Sets how often this event or to-do should recur.");
   QLabel *preLabel = new QLabel( everyText, parent );
+  QWhatsThis::add( preLabel, whatsThis );
   freqLayout->addWidget( preLabel );
 
   freqLayout->addWidget( frequencyEdit() );
   preLabel->setBuddy( frequencyEdit() );
+  QWhatsThis::add( preLabel->buddy(), whatsThis );
 
   QLabel *postLabel = new QLabel( unitText, parent );
+  QWhatsThis::add( postLabel, whatsThis );
   freqLayout->addWidget( postLabel );
   freqLayout->addStretch();
   return freqLayout;
@@ -179,6 +195,9 @@ RecurWeekly::RecurWeekly( QWidget *parent, const char *name ) :
       weekDayName = weekDayName.left( 1 );
     }
     mDayBoxes[ (i + weekStart + 6)%7 ] = new QCheckBox( weekDayName, dayBox );
+    QWhatsThis::add( mDayBoxes[ (i + weekStart + 6)%7 ],
+                     i18n("Day of the week on which this event or to-do "
+                          "should recur.") );
   }
 
   topLayout->addStretch( 1 );
@@ -226,9 +245,16 @@ RecurMonthly::RecurMonthly( QWidget *parent, const char *name ) :
   }
 
   mByDayRadio = new QRadioButton( recurOnText, buttonGroup );
+  QWhatsThis::add( mByDayRadio,
+                   i18n("Sets a specific day of the month on which "
+                        "this event or to-do should recur.") );
+  
   buttonLayout->addWidget( mByDayRadio, 0, 0 );
 
+  QString whatsThis = i18n("The day of the month on which this event or to-do "
+                           "should recur.");
   mByDayCombo = new QComboBox( buttonGroup );
+  QWhatsThis::add( mByDayCombo, whatsThis );
   mByDayCombo->setSizeLimit( 7 );
   mByDayCombo->insertItem( i18n("1st") );
   mByDayCombo->insertItem( i18n("2nd") );
@@ -261,13 +287,50 @@ RecurMonthly::RecurMonthly( QWidget *parent, const char *name ) :
   mByDayCombo->insertItem( i18n("29th") );
   mByDayCombo->insertItem( i18n("30th") );
   mByDayCombo->insertItem( i18n("31st") );
+  mByDayCombo->insertItem( i18n("Last") );
+  mByDayCombo->insertItem( i18n("2nd Last") );
+  mByDayCombo->insertItem( i18n("3rd Last") );
+  mByDayCombo->insertItem( i18n("4th Last") );
+  mByDayCombo->insertItem( i18n("5th Last") );
+  // FIXME: After the string freeze is over, insert all possible values, not
+  //        just the ones we already have translated:
+/*  mByDayCombo->insertItem( i18n("6th Last") );
+  mByDayCombo->insertItem( i18n("7th Last") );
+  mByDayCombo->insertItem( i18n("8th Last") );
+  mByDayCombo->insertItem( i18n("9th Last") );
+  mByDayCombo->insertItem( i18n("10th Last") );
+  mByDayCombo->insertItem( i18n("11th Last") );
+  mByDayCombo->insertItem( i18n("12th Last") );
+  mByDayCombo->insertItem( i18n("13th Last") );
+  mByDayCombo->insertItem( i18n("14th Last") );
+  mByDayCombo->insertItem( i18n("15th Last") );
+  mByDayCombo->insertItem( i18n("16th Last") );
+  mByDayCombo->insertItem( i18n("17th Last") );
+  mByDayCombo->insertItem( i18n("18th Last") );
+  mByDayCombo->insertItem( i18n("19th Last") );
+  mByDayCombo->insertItem( i18n("20th Last") );
+  mByDayCombo->insertItem( i18n("21st Last") );
+  mByDayCombo->insertItem( i18n("22nd Last") );
+  mByDayCombo->insertItem( i18n("23rd Last") );
+  mByDayCombo->insertItem( i18n("24th Last") );
+  mByDayCombo->insertItem( i18n("25th Last") );
+  mByDayCombo->insertItem( i18n("26th Last") );
+  mByDayCombo->insertItem( i18n("27th Last") );
+  mByDayCombo->insertItem( i18n("28th Last") );
+  mByDayCombo->insertItem( i18n("29th Last") );
+  mByDayCombo->insertItem( i18n("30th Last") );
+  mByDayCombo->insertItem( i18n("31st Last") );*/
   buttonLayout->addWidget( mByDayCombo, 0, 1 );
 
   QLabel *byDayLabel = new QLabel( i18n("day"), buttonGroup );
+  QWhatsThis::add( byDayLabel, whatsThis );
   buttonLayout->addWidget( byDayLabel, 0, 2 );
 
 
   mByPosRadio = new QRadioButton( recurOnText, buttonGroup);
+  QWhatsThis::add( mByPosRadio,
+                   i18n("Sets a weekday and specific week in the month "
+                        "on which this event or to-do should recur") );
   buttonLayout->addWidget( mByPosRadio, 1, 0 );
 
   mByPosCountCombo = createWeekCountCombo( buttonGroup );
@@ -280,7 +343,12 @@ RecurMonthly::RecurMonthly( QWidget *parent, const char *name ) :
 void RecurMonthly::setByDay( int day )
 {
   mByDayRadio->setChecked( true );
-  mByDayCombo->setCurrentItem( day-1 );
+  // Days from the end are after the ones from the begin, so correct for the
+  // negative sign and add 30 (index starting at 0)
+  if ( day > 0 & day <= 31 )
+    mByDayCombo->setCurrentItem( day-1 );
+  else if ( day < 0 )
+    mByDayCombo->setCurrentItem( 31 - 1 - day );
 }
 
 void RecurMonthly::setByPos( int count, int weekday )
@@ -306,7 +374,10 @@ bool RecurMonthly::byPos()
 
 int RecurMonthly::day()
 {
-  return mByDayCombo->currentItem() + 1;
+  int day = mByDayCombo->currentItem();
+  if ( day >= 31 ) day = 31-day-1;
+  else --day;
+  return day;
 }
 
 int RecurMonthly::count()
@@ -350,12 +421,19 @@ RecurYearly::RecurYearly( QWidget *parent, const char *name ) :
     recurInMonthText = i18n("&Day ");
   }
   mByMonthRadio = new QRadioButton( recurInMonthText, buttonGroup );
+  QWhatsThis::add( mByMonthRadio,
+		   i18n("Sets a specific day in a specific month on which "
+			"this event or to-do should recur.") );
   monthLayout->addWidget( mByMonthRadio );
   mByMonthSpin = new QSpinBox( 1, 31, 1, buttonGroup );
+  QWhatsThis::add( mByMonthSpin,
+		   i18n("The day of the month on which this event or to-do "
+			"should recur.") );
   monthLayout->addWidget( mByMonthSpin );
   QLabel *ofLabel = new QLabel(
       i18n("part between XXX and YYY of 'Recur on day XXX of month YYY'", " &of "),
       buttonGroup );
+  //What do I do here? I'm not sure if this label should have What's This in it... - Antonio
   monthLayout->addWidget( ofLabel );
 
   mByMonthCombo = createMonthNameCombo( buttonGroup );
@@ -372,6 +450,9 @@ RecurYearly::RecurYearly( QWidget *parent, const char *name ) :
     recurOnPosText = i18n("Part before XXX in 'Recur on NNN. WEEKDAY of MONTH'", "&On the" );
   }
   mByPosRadio = new QRadioButton( recurOnPosText, buttonGroup );
+  QWhatsThis::add( mByPosRadio,
+		   i18n("Sets a specific day in a specific week of a specific "
+			"month on which this event or to-do should recur.") );
   posLayout->addWidget( mByPosRadio );
 
   mByPosDayCombo = createWeekCountCombo( buttonGroup );
@@ -400,10 +481,15 @@ RecurYearly::RecurYearly( QWidget *parent, const char *name ) :
   } else {
     recurOnDayText = i18n("Recur on &day #");
   }
+  QString whatsThis = i18n("Sets a specific day within the year on which this "
+			   "event or to-do should recur.");
   mByDayRadio = new QRadioButton( recurOnDayText, buttonGroup );
+  QWhatsThis::add( mByDayRadio, whatsThis );
   dayLayout->addWidget( mByDayRadio );
 
   mByDaySpin = new QSpinBox( 1, 366, 1, buttonGroup );
+  QWhatsThis::add( mByDaySpin, whatsThis );
+  
   dayLayout->addWidget( mByDaySpin );
 
   QString ofTheYear( i18n("part after NNN of 'Recur on day #NNN of the year'", " of the &year"));
@@ -412,6 +498,7 @@ RecurYearly::RecurYearly( QWidget *parent, const char *name ) :
         " of the year");
   }
   ofLabel = new QLabel( ofTheYear, buttonGroup );
+  QWhatsThis::add( ofLabel, whatsThis );
   dayLayout->addWidget( ofLabel );
   ofLabel->setBuddy( mByDaySpin );
 
@@ -500,17 +587,33 @@ ExceptionsWidget::ExceptionsWidget( QWidget *parent, const char *name ) :
   QGridLayout *boxLayout = new QGridLayout( box );
 
   mExceptionDateEdit = new KDateEdit( box );
+  QWhatsThis::add( mExceptionDateEdit,
+		   i18n("A date that should be considered an exception "
+			"to the recurrence rules for this event or to-do.") );
   mExceptionDateEdit->setDate( QDate::currentDate() );
   boxLayout->addWidget( mExceptionDateEdit, 0, 0 );
 
   QPushButton *addExceptionButton = new QPushButton( i18n("&Add"), box );
+  QWhatsThis::add( addExceptionButton,
+		   i18n("Add this date as an exception "
+			"to the recurrence rules for this event or to-do.") );
   boxLayout->addWidget( addExceptionButton, 1, 0 );
   QPushButton *changeExceptionButton = new QPushButton( i18n("&Change"), box );
+  QWhatsThis::add( changeExceptionButton,
+		   i18n("Replace the currently selected date with this date.") );
   boxLayout->addWidget( changeExceptionButton, 2, 0 );
   QPushButton *deleteExceptionButton = new QPushButton( i18n("&Delete"), box );
+  QWhatsThis::add( deleteExceptionButton,
+		   i18n("Delete the currently selected date from the list of dates "
+		        "that should be considered exceptions to the recurrence rules "
+		        "for this event or to-do.") );
   boxLayout->addWidget( deleteExceptionButton, 3, 0 );
 
   mExceptionList = new QListBox( box );
+  QWhatsThis::add( mExceptionList,
+		   i18n("Displays current dates that are being considered "
+			"exceptions to the recurrence rules for this event "
+			"or to-do.") );
   boxLayout->addMultiCellWidget( mExceptionList, 0, 3, 1, 1 );
 
   boxLayout->setRowStretch( 4, 1 );
@@ -598,6 +701,9 @@ RecurrenceRangeWidget::RecurrenceRangeWidget( QWidget *parent,
 
   mRangeGroupBox = new QGroupBox( 1, Horizontal, i18n("Recurrence Range"),
                                   this );
+  QWhatsThis::add( mRangeGroupBox,
+		   i18n("Sets a range for which these recurrence rules will "
+			"apply to this event or to-do.") );
   topLayout->addWidget( mRangeGroupBox );
 
   QWidget *rangeBox = new QWidget( mRangeGroupBox );
@@ -605,11 +711,17 @@ RecurrenceRangeWidget::RecurrenceRangeWidget( QWidget *parent,
   rangeLayout->setSpacing( KDialog::spacingHint() );
 
   mStartDateLabel = new QLabel( i18n("Begin on:"), rangeBox );
+  QWhatsThis::add( mStartDateLabel,
+		   i18n("The date on which the recurrences for this event or to-do "
+			"should begin.") );
   rangeLayout->addWidget( mStartDateLabel );
 
-  QButtonGroup *rangeButtonGroup = new QButtonGroup;
+  QButtonGroup *rangeButtonGroup = new QButtonGroup( this );
+  rangeButtonGroup->hide();
 
   mNoEndDateButton = new QRadioButton( i18n("&No ending date"), rangeBox );
+  QWhatsThis::add( mNoEndDateButton,
+		   i18n("Sets the event or to-do to recur forever.") );
   rangeButtonGroup->insert( mNoEndDateButton );
   rangeLayout->addWidget( mNoEndDateButton );
 
@@ -617,24 +729,37 @@ RecurrenceRangeWidget::RecurrenceRangeWidget( QWidget *parent,
   durationLayout->setSpacing( KDialog::spacingHint() );
 
   mEndDurationButton = new QRadioButton( i18n("End &after"), rangeBox );
+  QWhatsThis::add( mEndDurationButton,
+		   i18n("Sets the event or to-do to stop recurring after a "
+			"certain number of occurrences.") );
   rangeButtonGroup->insert( mEndDurationButton );
   durationLayout->addWidget( mEndDurationButton );
 
+  QString whatsThis = i18n("Number of times the event or to-do should recur "
+		  	   "before stopping.");
   mEndDurationEdit = new QSpinBox( 1, 9999, 1, rangeBox );
+  QWhatsThis::add( mEndDurationEdit, whatsThis );
   durationLayout->addWidget( mEndDurationEdit );
 
   QLabel *endDurationLabel = new QLabel( i18n("&occurrence(s)"), rangeBox );
+  QWhatsThis::add( endDurationLabel, whatsThis );
   durationLayout ->addWidget( endDurationLabel );
   endDurationLabel->setBuddy( mEndDurationEdit );
 
   QBoxLayout *endDateLayout = new QHBoxLayout( rangeLayout );
   endDateLayout->setSpacing( KDialog::spacingHint() );
 
-  mEndDateButton = new QRadioButton( i18n("End &by:"), rangeBox );
+  mEndDateButton = new QRadioButton( i18n("End &on:"), rangeBox );
+  QWhatsThis::add( mEndDateButton,
+                   i18n("Sets the event or to-do to stop recurring on "
+                        "a certain date.") );
   rangeButtonGroup->insert( mEndDateButton );
   endDateLayout->addWidget( mEndDateButton );
 
   mEndDateEdit = new KDateEdit( rangeBox );
+  QWhatsThis::add( mEndDateEdit,
+                   i18n("Date after which the event or to-do should stop "
+                        "recurring") );
   endDateLayout->addWidget( mEndDateEdit );
 
   endDateLayout->addStretch( 1 );
@@ -751,6 +876,9 @@ RecurrenceChooser::RecurrenceChooser( QWidget *parent, const char *name ) :
 
   if ( KOPrefs::instance()->mCompactDialogs ) {
     mTypeCombo = new QComboBox( this );
+    QWhatsThis::add( mTypeCombo,
+                     i18n("Sets the type of recurrence this event or to-do "
+                          "should have.") );
     mTypeCombo->insertItem( i18n("Daily") );
     mTypeCombo->insertItem( i18n("Weekly") );
     mTypeCombo->insertItem( i18n("Monthly") );
@@ -767,9 +895,21 @@ RecurrenceChooser::RecurrenceChooser( QWidget *parent, const char *name ) :
     topLayout->addWidget( ruleButtonGroup );
 
     mDailyButton = new QRadioButton( i18n("&Daily"), ruleButtonGroup );
+    QWhatsThis::add( mDailyButton,
+                     i18n("Sets the event or to-do to recur daily according "
+                          "to the specified rules.") );
     mWeeklyButton = new QRadioButton( i18n("&Weekly"), ruleButtonGroup );
+    QWhatsThis::add( mWeeklyButton,
+                     i18n("Sets the event or to-do to recur weekly according "
+                          "to the specified rules.") );
     mMonthlyButton = new QRadioButton( i18n("&Monthly"), ruleButtonGroup );
+    QWhatsThis::add( mMonthlyButton,
+                     i18n("Sets the event or to-do to recur monthly according "
+                          "to the specified rules.") );
     mYearlyButton = new QRadioButton( i18n("&Yearly"), ruleButtonGroup );
+    QWhatsThis::add( mYearlyButton,
+                     i18n("Sets the event or to-do to recur yearly according "
+                          "to the specified rules.") );
 
     connect( mDailyButton, SIGNAL( toggled( bool ) ),
              SLOT( emitChoice() ) );
@@ -831,6 +971,9 @@ KOEditorRecurrence::KOEditorRecurrence( QWidget* parent, const char *name ) :
   topLayout->setSpacing( KDialog::spacingHint() );
 
   mEnabledCheck = new QCheckBox( i18n("&Enable recurrence"), this );
+  QWhatsThis::add( mEnabledCheck,
+                   i18n("Enables recurrence for this event or to-do according "
+                        "to the specified rules.") );
   connect( mEnabledCheck, SIGNAL( toggled( bool ) ),
            SLOT( setRecurrenceEnabled( bool ) ) );
   topLayout->addMultiCellWidget( mEnabledCheck, 0, 0, 0, 1 );
@@ -838,6 +981,8 @@ KOEditorRecurrence::KOEditorRecurrence( QWidget* parent, const char *name ) :
 
   mTimeGroupBox = new QGroupBox( 1, Horizontal, i18n("Appointment Time "),
                                  this );
+  QWhatsThis::add( mTimeGroupBox,
+                   i18n("Displays appointment time information.") );
   topLayout->addMultiCellWidget( mTimeGroupBox, 1, 1 , 0 , 1 );
 
   if ( KOPrefs::instance()->mCompactDialogs ) {
@@ -857,6 +1002,9 @@ KOEditorRecurrence::KOEditorRecurrence( QWidget* parent, const char *name ) :
   else orientation = Vertical;
 
   mRuleBox = new QGroupBox( 1, orientation, i18n("Recurrence Rule"), this );
+  QWhatsThis::add( mRuleBox,
+                   i18n("Options concerning the type of recurrence this event "
+                        "or to-do should have.") );
   if ( KOPrefs::instance()->mCompactDialogs ) {
     topLayout->addWidget( mRuleBox, 2, 0 );
   } else {
@@ -894,6 +1042,9 @@ KOEditorRecurrence::KOEditorRecurrence( QWidget* parent, const char *name ) :
     mRecurrenceRange = mRecurrenceRangeDialog;
     mRecurrenceRangeButton = new QPushButton( i18n("Recurrence Range..."),
                                               this );
+    QWhatsThis::add( mRecurrenceRangeButton,
+                     i18n("Options concerning the time range during which "
+                          "this event or to-do should recur.") );
     topLayout->addWidget( mRecurrenceRangeButton, 3, 0 );
     connect( mRecurrenceRangeButton, SIGNAL( clicked() ),
              SLOT( showRecurrenceRangeDialog() ) );
@@ -908,6 +1059,9 @@ KOEditorRecurrence::KOEditorRecurrence( QWidget* parent, const char *name ) :
 
   } else {
     mRecurrenceRangeWidget = new RecurrenceRangeWidget( this );
+    QWhatsThis::add( mRecurrenceRangeWidget,
+                     i18n("Options concerning the time range during which "
+                          "this event or to-do should recur.") );
     mRecurrenceRangeDialog = 0;
     mRecurrenceRange = mRecurrenceRangeWidget;
     mRecurrenceRangeButton = 0;
@@ -956,7 +1110,7 @@ void KOEditorRecurrence::showCurrentRule( int current )
   }
 }
 
-void KOEditorRecurrence::setDateTimes( QDateTime start, QDateTime end )
+void KOEditorRecurrence::setDateTimes( const QDateTime &start, const QDateTime &end )
 {
 //  kdDebug(5850) << "KOEditorRecurrence::setDateTimes" << endl;
 
@@ -966,9 +1120,30 @@ void KOEditorRecurrence::setDateTimes( QDateTime start, QDateTime end )
   mWeekly->setDateTimes( start, end );
   mMonthly->setDateTimes( start, end );
   mYearly->setDateTimes( start, end );
+  
+  // Now set the defaults for all unused types, use the start time for it
+  bool enabled = mEnabledCheck->isChecked();
+  int type = mRecurrenceChooser->type();
+  
+  if ( !enabled || type != RecurrenceChooser::Weekly ) {
+    QBitArray days( 7 );
+    days.fill( 0 );
+    days.setBit( (start.date().dayOfWeek()+6) % 7 );
+    mWeekly->setDays( days );
+  } 
+  if ( !enabled || type != RecurrenceChooser::Monthly ) {
+    mMonthly->setByPos( ( start.date().day() - 1 ) / 7 + 1, start.date().dayOfWeek() - 1 );
+    mMonthly->setByDay( start.date().day() );
+  }
+  if ( !enabled || type != RecurrenceChooser::Yearly ) {
+    mYearly->setByDay( start.date().dayOfYear() );
+    mYearly->setByPos( ( start.date().day() - 1 ) / 7 + 1,
+        start.date().dayOfWeek() - 1, start.date().month() );
+    mYearly->setByMonth( start.date().day(), start.date().month() );
+  }
 }
 
-void KOEditorRecurrence::setDefaults( QDateTime from, QDateTime to, bool )
+void KOEditorRecurrence::setDefaults( const QDateTime &from, const QDateTime &to, bool )
 {
   setDateTimes( from, to );
 
