@@ -1,6 +1,7 @@
 /*
     This file is part of KOrganizer.
     Copyright (c) 2000,2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -29,33 +30,49 @@
 
 #include <libkcal/event.h>
 
+#include "koglobals.h"
+
 #include "koeventpopupmenu.h"
 #include "koeventpopupmenu.moc"
 
 KOEventPopupMenu::KOEventPopupMenu()
 {
-  mCurrentEvent = 0;
+  mCurrentIncidence = 0;
+  mCurrentDate = QDate();
   mHasAdditionalItems = false;
 
   insertItem (i18n("&Show"),this,SLOT(popupShow()));
-  mEditOnlyItems.append(insertItem (i18n("edit event","&Edit..."),this,SLOT(popupEdit())));
-  mEditOnlyItems.append(insertItem (SmallIcon("editdelete"),i18n("&Delete"),
+  mEditOnlyItems.append(insertItem (i18n("&Edit..."),this,SLOT(popupEdit())));
+  mEditOnlyItems.append(insertItem (KOGlobals::self()->smallIcon("editdelete"),i18n("&Delete"),
                                    this,SLOT(popupDelete())));
+  mEditOnlyItems.append( insertSeparator() );
+  mEditOnlyItems.append( insertItem( QIconSet( KOGlobals::self()->smallIcon("bell") ),
+                                     i18n("Toggle Alarm"), this,
+                                     SLOT( popupAlarm() ) ) );
+  mRecurrenceItems.append( insertSeparator() );
+  mRecurrenceItems.append( insertItem( i18n("&Dissociate This Occurrence"), 
+                                       this, SLOT( dissociateOccurrence() ) ) );
+  mRecurrenceItems.append( insertItem( i18n("&Dissociate Future Occurrences"), 
+                                       this, SLOT( dissociateFutureOccurrence() ) ) );
 }
 
-void KOEventPopupMenu::showEventPopup(Event *event)
+void KOEventPopupMenu::showIncidencePopup( Incidence *incidence, const QDate &qd )
 {
-  mCurrentEvent = event;
+  mCurrentIncidence = incidence;
+  mCurrentDate = qd;
   
-  if (mCurrentEvent) {
+  if (mCurrentIncidence) {
     // Enable/Disabled menu items only valid for editable events.
     QValueList<int>::Iterator it;
     for( it = mEditOnlyItems.begin(); it != mEditOnlyItems.end(); ++it ) {
-      setItemEnabled(*it,!mCurrentEvent->isReadOnly());
+      setItemEnabled(*it,!mCurrentIncidence->isReadOnly());
+    }
+    for ( it = mRecurrenceItems.begin(); it != mRecurrenceItems.end(); ++it ) {
+      setItemVisible( *it, mCurrentIncidence->doesRecur() );
     }
     popup(QCursor::pos());
   } else {
-    kdDebug() << "KOEventPopupMenu::showEventPopup(): No event selected" << endl;
+    kdDebug(5850) << "KOEventPopupMenu::showEventPopup(): No event selected" << endl;
   }
 }
 
@@ -73,15 +90,32 @@ void KOEventPopupMenu::addAdditionalItem(const QIconSet &icon,const QString &tex
 
 void KOEventPopupMenu::popupShow()
 {
-  if (mCurrentEvent) emit showEventSignal(mCurrentEvent);
+  if (mCurrentIncidence) emit showIncidenceSignal(mCurrentIncidence);
 }
 
 void KOEventPopupMenu::popupEdit()
 {
-  if (mCurrentEvent) emit editEventSignal(mCurrentEvent);
+  if (mCurrentIncidence) emit editIncidenceSignal(mCurrentIncidence);
 }
 
 void KOEventPopupMenu::popupDelete()
 {
-  if (mCurrentEvent) emit deleteEventSignal(mCurrentEvent);
+  if (mCurrentIncidence) emit deleteIncidenceSignal(mCurrentIncidence);
+}
+
+void KOEventPopupMenu::popupAlarm()
+{
+  if (mCurrentIncidence) emit toggleAlarmSignal( mCurrentIncidence );
+}
+
+void KOEventPopupMenu::dissociateOccurrence() 
+{
+  if ( mCurrentIncidence ) 
+    emit dissociateOccurrenceSignal( mCurrentIncidence, mCurrentDate );
+}
+
+void KOEventPopupMenu::dissociateFutureOccurrence() 
+{
+  if ( mCurrentIncidence ) 
+    emit dissociateFutureOccurrenceSignal( mCurrentIncidence, mCurrentDate );
 }
