@@ -1,6 +1,7 @@
 /*
     This file is part of KOrganizer.
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -52,22 +53,30 @@ class KOJournalView : public KOrg::BaseView
 
   public slots:
     // Don't update the view when midnight passed, otherwise we'll have data loss (bug 79145)
-    virtual void dayPassed( QDate ) {}
+    virtual void dayPassed( const QDate & ) {}
     void updateView();
     void flushView();
 
     void showDates( const QDate &start, const QDate &end );
-    void showIncidences( const Incidence::List & );
+    void showIncidences( const Incidence::List &incidenceList );
 
     void changeIncidenceDisplay( Incidence *, int );
+    void setIncidenceChanger( IncidenceChangerBase *changer );
+    void newJournal();
+  signals:
+    void flushEntries();
+    void setIncidenceChangerSignal( IncidenceChangerBase * );
+    void journalEdited( Journal* );
+    void journalDeleted( Journal* );
+
   protected:
     void clearEntries();
 
   private:
     QScrollView *mSV;
     QVBox *mVBox;
-    JournalEntry::List mEntries;
-    DateList mSelectedDates;  // List of dates to be displayed
+    QMap<QDate, JournalDateEntry*> mEntries;
+//    DateList mSelectedDates;  // List of dates to be displayed
 };
 
 #endif
