@@ -36,6 +36,7 @@
 
 #include <libkcal/event.h>
 #include <libkcal/todo.h>
+#include <libkcal/incidenceformatter.h>
 
 #include "version.h"
 #include "koprefs.h"
@@ -55,12 +56,15 @@ bool KOMailClient::mailAttendees(IncidenceBase *incidence,const QString &attachm
   Attendee::List attendees = incidence->attendees();
   if (attendees.count() == 0) return false;
 
-  QString from = KOPrefs::instance()->email();
+  const QString from = incidence->organizer().fullName();
+  const QString organizerEmail = incidence->organizer().email();
   QStringList toList;
   for(uint i=0; i<attendees.count();++i) {
-    QString email = (*attendees.at(i))->email();
-    if( email != from )
-      // Don't send a mail to ourselves
+    const QString email = (*attendees.at(i))->email();
+    // In case we (as one of our identities) are the organizer we are sending this
+    // mail. We could also have added ourselves as an attendee, in which case we 
+    // don't want to send ourselves a notification mail.
+    if( organizerEmail !=  email )
       toList << email;
   }
   if( toList.count() == 0 )
@@ -76,7 +80,7 @@ bool KOMailClient::mailAttendees(IncidenceBase *incidence,const QString &attachm
     subject = "Free Busy Object";
   }
 
-  QString body = createBody(incidence);
+  QString body = IncidenceFormatter::mailBodyString(incidence);
 
   bool bcc = KOPrefs::instance()->mBcc;
 
@@ -85,7 +89,7 @@ bool KOMailClient::mailAttendees(IncidenceBase *incidence,const QString &attachm
 
 bool KOMailClient::mailOrganizer(IncidenceBase *incidence,const QString &attachment)
 {
-  QString to = incidence->organizer();
+  QString to = incidence->organizer().fullName();
 
   QString from = KOPrefs::instance()->email();
 
@@ -97,7 +101,7 @@ bool KOMailClient::mailOrganizer(IncidenceBase *incidence,const QString &attachm
     subject = "Free Busy Message";
   }
 
-  QString body = createBody(incidence);
+  QString body = IncidenceFormatter::mailBodyString(incidence);
 
   bool bcc = KOPrefs::instance()->mBcc;
 
@@ -115,7 +119,7 @@ bool KOMailClient::mailTo(IncidenceBase *incidence,const QString &recipients,
   } else {
     subject = "Free Busy Message";
   }
-  QString body = createBody(incidence);
+  QString body = IncidenceFormatter::mailBodyString(incidence);
   bool bcc = KOPrefs::instance()->mBcc;
   kdDebug () << "KOMailClient::mailTo " << recipients << endl;
   return send(from,recipients,subject,body,bcc,attachment);
@@ -186,7 +190,7 @@ bool KOMailClient::send(const QString &from,const QString &to,
     }
 
     if (attachment.isEmpty()) {
-      if (!kMailOpenComposer(to,"",from,subject,body,0,KURL())) return false;
+      if (!kMailOpenComposer(to,"",bcc ? from : "",subject,body,0,KURL())) return false;
     } else {
       QString meth;
       int idx = attachment.find("METHOD");
@@ -197,9 +201,9 @@ bool KOMailClient::send(const QString &from,const QString &to,
       } else {
         meth = "publish";
       }
-      if (!kMailOpenComposer(to,"",from,subject,body,0,"cal.ics","7bit",
+      if (!kMailOpenComposer(to,"",bcc ? from : "",subject,body,0,"cal.ics","7bit",
                              attachment.utf8(),"text","calendar","method",meth,
-                             "attachment")) return false;
+                             "attachment","utf-8")) return false;
     }
   }
   return true;
@@ -209,6 +213,9 @@ int KOMailClient::kMailOpenComposer(const QString& arg0,const QString& arg1,
   const QString& arg2,const QString& arg3,const QString& arg4,int arg5,
   const KURL& arg6)
 {
+  //kdDebug(5850) << "KOMailClient::kMailOpenComposer( "
+  //  << arg0 << " , " << arg1 << arg2 << " , " << arg3
+  //  << arg4 << " , " << arg5 << " , " << arg6 << " )" << endl;
   int result = 0;
 
   QByteArray data, replyData;
@@ -221,6 +228,9 @@ int KOMailClient::kMailOpenComposer(const QString& arg0,const QString& arg1,
   arg << arg4;
   arg << arg5;
   arg << arg6;
+#if KDE_IS_VERSION( 3, 2, 90 )
+  kapp->updateRemoteUserTimestamp( "kmail" );
+#endif
   if (kapp->dcopClient()->call("kmail","KMailIface","openComposer(QString,QString,QString,QString,QString,int,KURL)", data, replyType, replyData ) ) {
     if ( replyType == "int" ) {
       QDataStream _reply_stream( replyData, IO_ReadOnly );
@@ -240,8 +250,15 @@ int KOMailClient::kMailOpenComposer( const QString& arg0, const QString& arg1,
                                      const QCString& arg7, const QCString& arg8,
                                      const QCString& arg9, const QCString& arg10,
                                      const QCString& arg11, const QString& arg12,
-                                     const QCString& arg13 )
+                                     const QCString& arg13, const QCString& arg14 )
 {
+    //kdDebug(5850) << "KOMailClient::kMailOpenComposer( "
+    //    << arg0 << " , " << arg1 << arg2 << " , " << arg3
+    //   << arg4 << " , " << arg5 << " , " << arg6
+    //    << arg7 << " , " << arg8 << " , " << arg9
+    //    << arg10<< " , " << arg11<< " , " << arg12
+    //    << arg13<< " , " << arg14<< " )" << endl;
+
     int result = 0;
 
     QByteArray data, replyData;
@@ -261,7 +278,12 @@ int KOMailClient::kMailOpenComposer( const QString& arg0, const QString& arg1,
     arg << arg11;
     arg << arg12;
     arg << arg13;
-    if ( kapp->dcopClient()->call("kmail","KMailIface","openComposer(QString,QString,QString,QString,QString,int,QString,QCString,QCString,QCString,QCString,QCString,QString,QCString)", data, replyType, replyData ) ) {
+    arg << arg14;
+#if KDE_IS_VERSION( 3, 2, 90 )
+    kapp->updateRemoteUserTimestamp("kmail");
+#endif
+    if ( kapp->dcopClient()->call("kmail","KMailIface",
+          "openComposer(QString,QString,QString,QString,QString,int,QString,QCString,QCString,QCString,QCString,QCString,QString,QCString,QCString)", data, replyType, replyData ) ) {
         if ( replyType == "int" ) {
             QDataStream _reply_stream( replyData, IO_ReadOnly );
             _reply_stream >> result;
@@ -275,129 +297,3 @@ int KOMailClient::kMailOpenComposer( const QString& arg0, const QString& arg1,
 }
 
 
-QString KOMailClient::createBody(IncidenceBase *incidence)
-{
-  QString CR = ("\n");
-
-  QString body;
-
-  // mailbody for Event
-  if (incidence->type()=="Event") {
-    Event *selectedEvent = static_cast<Event *>(incidence);
-    QString recurrence[]= {i18n("no recurrence", "None"),
-      i18n("Minutely"), i18n("Hourly"), i18n("Daily"),
-      i18n("Weekly"), i18n("Monthly Same Day"), i18n("Monthly Same Position"),
-      i18n("Yearly"), i18n("Yearly"), i18n("Yearly")};
-
-    if (!selectedEvent->organizer().isEmpty()) {
-      body += i18n("Organizer: %1").arg(selectedEvent->organizer());
-      body += CR;
-    }
-    body += i18n("Summary: %1").arg(selectedEvent->summary());
-    body += CR;
-    if (!selectedEvent->location().isEmpty()) {
-      body += i18n("Location: %1").arg(selectedEvent->location());
-      body += CR;
-    }
-    body += i18n("Start Date: %1").arg(selectedEvent->dtStartDateStr());
-    body += CR;
-    if (!selectedEvent->doesFloat()) {
-      body += i18n("Start Time: %1").arg(selectedEvent->dtStartTimeStr());
-      body += CR;
-    }
-    if ( selectedEvent->dtStart()!=selectedEvent->dtEnd() ) {
-      body += i18n("End Date: %1").arg(selectedEvent->dtEndDateStr());
-      body += CR;
-    }
-    if (!selectedEvent->doesFloat()) {
-      body += i18n("End Time: %1").arg(selectedEvent->dtEndTimeStr());
-      body += CR;
-    }
-    if (selectedEvent->doesRecur()) {
-      body += i18n("Recurs: %1")
-               .arg(recurrence[selectedEvent->recurrence()->doesRecur()]);
-      body += CR;
-/* TODO: frequency
-      body += i18n("Frequency: %1")
-               .arg(recurrence[selectedEvent->recurrence()->frequency()]);
-      body += CR;
-*/
-      if (selectedEvent->recurrence()->duration() > 0 ) {
-        body += i18n ("Repeats %1 times")
-                 .arg(QString::number(selectedEvent->recurrence()->duration()));
-        body += CR;
-      } else {
-        if (selectedEvent->recurrence()->duration() != -1) {
-//          body += i18n("Repeat until: %1")
-          body += i18n("End Date: %1")
-                   .arg(selectedEvent->recurrence()->endDateStr());
-          body += CR;
-        } else {
-          body += i18n("Repeats forever");
-          body += CR;
-        }
-      }
-    }
-    QString details = selectedEvent->description();
-    if (!details.isEmpty()) {
-      body += i18n("Details:");
-      body += CR;
-      body += details;
-      body += CR;
-    }
-  }
-
-  // mailbody for Todo
-  if (incidence->type()=="Todo") {
-    Todo *selectedEvent = static_cast<Todo *>(incidence);
-    if (!selectedEvent->organizer().isEmpty()) {
-      body += i18n("Organizer: %1").arg(selectedEvent->organizer());
-      body += CR;
-    }
-    body += i18n("Summary: %1").arg(selectedEvent->summary());
-    body += CR;
-    if (!selectedEvent->location().isEmpty()) {
-      body += i18n("Location: %1").arg(selectedEvent->location());
-      body += CR;
-    }
-    if (selectedEvent->hasStartDate()) {
-      body += i18n("Start Date: %1").arg(selectedEvent->dtStartDateStr());
-      body += CR;
-      if (!selectedEvent->doesFloat()) {
-        body += i18n("Start Time: %1").arg(selectedEvent->dtStartTimeStr());
-        body += CR;
-      }
-    }
-    if (selectedEvent->hasDueDate()) {
-      body += i18n("Due Date: %1").arg(selectedEvent->dtDueDateStr());
-      body += CR;
-      if (!selectedEvent->doesFloat()) {
-        body += i18n("Due Time: %1").arg(selectedEvent->dtDueTimeStr());
-        body += CR;
-      }
-    }
-    QString details = selectedEvent->description();
-    if (!details.isEmpty()) {
-      body += i18n("Details:");
-      body += CR;
-      body += details;
-      body += CR;
-    }
-  }
-
-  // mailbody for FreeBusy
-  if(incidence->type()=="FreeBusy") {
-    body = i18n("This is a Free Busy Object");
-  }
-
-  // mailbody for Journal
-  if(incidence->type()=="Journal") {
-    Incidence *inc = static_cast<Incidence *>(incidence);
-    body = inc->summary();
-    body += CR;
-    body += inc->description();
-    body += CR;
-  }
-
-  return body;
-}
