@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2000,2001,2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,13 +41,12 @@
 #include <libkcal/calendarresources.h>
 #include <libkcal/resourcecalendar.h>
 #include <libkcal/calfilter.h>
+#include <libkcal/incidenceformatter.h>
+#include <libkcal/journal.h>
 
 #include <libkdepim/clicklineedit.h>
 #include <libkdepim/kdatepickerpopup.h>
 
-#ifndef KORG_NOPRINTER
-#include "calprinter.h"
-#endif
 #include "docprefs.h"
 
 #include "koincidencetooltip.h"
@@ -58,7 +58,6 @@ using namespace KOrg;
 #include "kotodoviewitem.h"
 #include "kotodoview.moc"
 
-const int KOTodoView::POPUP_UNSUBTODO=1234;
 
 KOTodoListViewToolTip::KOTodoListViewToolTip (QWidget* parent,
                                               KOTodoListView* lv )
@@ -85,9 +84,8 @@ void KOTodoListViewToolTip::maybeTip( const QPoint & pos)
     r.setRight(headerPos + todolist->header()->sectionSize(col));
 
     /* Show the tip */
-    QString tipText;
-    ToolTipVisitor v;
-    if (v.act(i->todo(), &tipText, true)) {
+    QString tipText( IncidenceFormatter::toolTipString( i->todo() ) );;
+    if ( !tipText.isEmpty() ) {
       tip(r, tipText);
     }
   }
@@ -97,7 +95,7 @@ void KOTodoListViewToolTip::maybeTip( const QPoint & pos)
 
 
 KOTodoListView::KOTodoListView( QWidget *parent, const char *name )
-  : KListView( parent, name ), mCalendar( 0 )
+  : KListView( parent, name ), mCalendar( 0 ), mChanger( 0 )
 {
   mOldCurrent = 0;
   mMousePressed = false;
@@ -186,9 +184,9 @@ void KOTodoListView::contentsDragLeaveEvent( QDragLeaveEvent * )
 void KOTodoListView::contentsDropEvent( QDropEvent *e )
 {
 #ifndef KORG_NODND
-//  kdDebug(5850) << "KOTodoListView::contentsDropEvent" << endl;
+  kdDebug(5850) << "KOTodoListView::contentsDropEvent" << endl;
 
-  if ( !mCalendar ||
+  if ( !mCalendar || !mChanger ||
        ( !ICalDrag::canDecode( e ) && !VCalDrag::canDecode( e ) &&
          !QTextDrag::canDecode( e ) ) ) {
     e->ignore();
@@ -198,7 +196,7 @@ void KOTodoListView::contentsDropEvent( QDropEvent *e )
   DndFactory factory( mCalendar );
   Todo *todo = factory.createDropTodo(e);
 
-  if (todo) {
+  if ( todo ) {
     e->acceptAction();
 
     KOTodoViewItem *destination =
@@ -208,61 +206,71 @@ void KOTodoListView::contentsDropEvent( QDropEvent *e )
 
     Todo *existingTodo = mCalendar->todo(todo->uid());
 
-    if(existingTodo) {
-//       kdDebug(5850) << "Drop existing Todo " << existingTodo << " onto " << destinationEvent << endl;
+    if( existingTodo ) {
+       kdDebug(5850) << "Drop existing Todo " << existingTodo << " onto " << destinationEvent << endl;
       Incidence *to = destinationEvent;
       while(to) {
         if (to->uid() == todo->uid()) {
-          KMessageBox::sorry(this,
-              i18n("Cannot move To-Do to itself or a child of itself."),
-              i18n("Drop To-Do"));
+          KMessageBox::information(this,
+              i18n("Cannot move to-do to itself or a child of itself."),
+              i18n("Drop To-do"), "NoDropTodoOntoItself" );
           delete todo;
           return;
         }
         to = to->relatedTo();
       }
       Todo*oldTodo = existingTodo->clone();
-      existingTodo->setRelatedTo(destinationEvent);
-
-      emit incidenceChanged( oldTodo, existingTodo );
+      if ( mChanger->beginChange( existingTodo ) ) {
+        existingTodo->setRelatedTo( destinationEvent );
+        mChanger->changeIncidence( oldTodo, existingTodo, KOGlobals::RELATION_MODIFIED );
+        mChanger->endChange( existingTodo );
+      } else {
+        KMessageBox::sorry( this, i18n("Unable to change to-do item's parent, "
+                            "because the to-do cannot be locked.") );
+      }
       delete oldTodo;
       delete todo;
     } else {
 //      kdDebug(5850) << "Drop new Todo" << endl;
       todo->setRelatedTo(destinationEvent);
-      if ( !mCalendar->addTodo( todo ) ) {
-        KODialogManager::errorSaveTodo( this );
+      if ( !mChanger->addIncidence( todo ) ) {
+        KODialogManager::errorSaveIncidence( this, todo );
+        delete todo;
         return;
       }
-
-      emit incidenceAdded( todo );
     }
   }
   else {
     QString text;
     KOTodoViewItem *todoi = dynamic_cast<KOTodoViewItem *>(itemAt( contentsToViewport(e->pos()) ));
-    if ( ! todoi ) { 
+    if ( ! todoi ) {
       // Not dropped on a todo item:
       e->ignore();
       kdDebug( 5850 ) << "KOTodoListView::contentsDropEvent(): Not dropped on a todo item" << endl;
       kdDebug( 5850 ) << "TODO: Create a new todo with the given data" << endl;
-      // TODO: Create a new todo with the given text/contact/whatever
+      // FIXME: Create a new todo with the given text/contact/whatever
     } else if ( QTextDrag::decode(e, text) ) {
       //QListViewItem *qlvi = itemAt( contentsToViewport(e->pos()) );
       kdDebug(5850) << "Dropped : " << text << endl;
       QStringList emails = QStringList::split(",",text);
-      Todo*newtodo = todoi->todo();
-      Todo*oldtodo = newtodo->clone();
-      for(QStringList::ConstIterator it = emails.begin();it!=emails.end();++it) {
-        kdDebug(5850) << " Email: " << (*it) << endl;
-        int pos = (*it).find("<");
-        QString name = (*it).left(pos);
-        QString email = (*it).mid(pos);
-        if (!email.isEmpty() && todoi) {
-          newtodo->addAttendee(new Attendee(name,email));
+      Todo*todo = todoi->todo();
+      Todo*oldtodo = todo->clone();
+      if ( mChanger->beginChange( todo ) ) {
+        for(QStringList::ConstIterator it = emails.begin();it!=emails.end();++it) {
+          kdDebug(5850) << " Email: " << (*it) << endl;
+          int pos = (*it).find("<");
+          QString name = (*it).left(pos);
+          QString email = (*it).mid(pos);
+          if (!email.isEmpty() && todoi) {
+            todo->addAttendee( new Attendee( name, email ) );
+          }
         }
+        mChanger->changeIncidence( oldtodo, todo );
+        mChanger->endChange( todo );
+      } else {
+        KMessageBox::sorry( this, i18n("Unable to add attendees to the to-do item, "
+                            "because the to-do cannot be locked.") );
       }
-      emit incidenceChanged( oldtodo, newtodo );
     }
     else {
       kdDebug(5850) << "KOTodoListView::contentsDropEvent(): Todo from drop not decodable" << endl;
@@ -354,7 +362,7 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
   title->setFrameStyle( QFrame::Panel | QFrame::Raised );
   topLayout->addWidget( title );
 
-  mQuickAdd = new KPIM::ClickLineEdit( this, i18n( "Click to add a new Todo" ) );
+  mQuickAdd = new KPIM::ClickLineEdit( this, i18n( "Click to add a new to-do" ) );
   topLayout->addWidget( mQuickAdd );
 
   if ( !KOPrefs::instance()->mEnableQuickTodo ) mQuickAdd->hide();
@@ -370,11 +378,11 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
   mTodoListView->addColumn( i18n("Summary") );
   mTodoListView->addColumn( i18n("Recurs") );
   mTodoListView->addColumn( i18n("Priority") );
-  mTodoListView->setColumnAlignment( 2, AlignHCenter );
+  mTodoListView->setColumnAlignment( ePriorityColumn, AlignHCenter );
   mTodoListView->addColumn( i18n("Complete") );
-  mTodoListView->setColumnAlignment( 3, AlignRight );
+  mTodoListView->setColumnAlignment( ePercentColumn, AlignRight );
   mTodoListView->addColumn( i18n("Due Date/Time") );
-  mTodoListView->setColumnAlignment( 4, AlignLeft );
+  mTodoListView->setColumnAlignment( eDueDateColumn, AlignLeft );
   mTodoListView->addColumn( i18n("Categories") );
 #if 0
   mTodoListView->addColumn( i18n("Sort Id") );
@@ -385,21 +393,27 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
   mTodoListView->setItemsRenameable( true );
   mTodoListView->setRenameable( 0 );
 
-  mTodoListView->setColumnWidthMode( 0, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 1, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 2, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 3, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 4, QListView::Manual );
-  mTodoListView->setColumnWidthMode( 5, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eSummaryColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eRecurColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( ePriorityColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( ePercentColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eDueDateColumn, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eCategoriesColumn, QListView::Manual );
 #if 0
-  mTodoListView->setColumnWidthMode( 6, QListView::Manual );
+  mTodoListView->setColumnWidthMode( eDescriptionColumn, QListView::Manual );
 #endif
 
   mPriorityPopupMenu = new QPopupMenu( this );
-  for ( int i = 1; i <= 5; i++ ) {
-    QString label = QString ("%1").arg( i );
-    mPriority[ mPriorityPopupMenu->insertItem( label ) ] = i;
-  }
+  mPriority[ mPriorityPopupMenu->insertItem( i18n("Unspecified priority", "unspecified") ) ] = 0;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "1 (highest)") ) ] = 1;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "2" ) ) ] = 2;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "3" ) ) ] = 3;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "4" ) ) ] = 4;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "5 (medium)" ) ) ] = 5;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "6" ) ) ] = 6;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "7" ) ) ] = 7;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "8" ) ) ] = 8;
+  mPriority[ mPriorityPopupMenu->insertItem( i18n( "9 (lowest)" ) ) ] = 9;
   connect( mPriorityPopupMenu, SIGNAL( activated( int ) ),
            SLOT( setNewPriority( int ) ));
 
@@ -430,21 +444,23 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
   mItemPopupMenu->insertItem(i18n("Show"), this,
                              SLOT (showTodo()));
   mItemPopupMenu->insertItem(i18n("Edit..."), this,
-                             SLOT (editTodo()));
+                             SLOT (editTodo()), 0, ePopupEdit );
   mItemPopupMenu->insertItem(KOGlobals::self()->smallIconSet("editdelete"), i18n("Delete"), this,
-                             SLOT (deleteTodo()));
+                             SLOT (deleteTodo()), 0, ePopupDelete );
   mItemPopupMenu->insertSeparator();
-  mItemPopupMenu->insertItem(KOGlobals::self()->smallIconSet("todo"), i18n("New To-Do..."), this,
+  mItemPopupMenu->insertItem(KOGlobals::self()->smallIconSet("todo"), i18n("New To-do..."), this,
                              SLOT (newTodo()));
-  mItemPopupMenu->insertItem(i18n("New Sub-To-Do..."), this,
+  mItemPopupMenu->insertItem(i18n("New Sub-to-do..."), this,
                              SLOT (newSubTodo()));
-  mItemPopupMenu->insertItem( i18n("Make Sub-To-Do Independent"), this,
-      SIGNAL( unSubTodoSignal() ), 0, POPUP_UNSUBTODO );
+  mItemPopupMenu->insertItem( i18n("Make Sub-to-do Independent"), this,
+      SIGNAL( unSubTodoSignal() ), 0, ePopupUnSubTodo );
+  mItemPopupMenu->insertItem( i18n("Make Sub-to-dos Independent"), this,
+      SIGNAL( unAllSubTodoSignal() ), 0, ePopupUnAllSubTodo );
   mItemPopupMenu->insertSeparator();
-  mItemPopupMenu->insertItem( i18n("Copy To"), mCopyPopupMenu );
-  mItemPopupMenu->insertItem(i18n("Move To"), mMovePopupMenu);
+  mItemPopupMenu->insertItem( i18n("Copy To"), mCopyPopupMenu, ePopupCopyTo );
+  mItemPopupMenu->insertItem(i18n("Move To"), mMovePopupMenu, ePopupMoveTo );
   mItemPopupMenu->insertSeparator();
-  mItemPopupMenu->insertItem(i18n("delete completed To-Dos","Purge Completed"),
+  mItemPopupMenu->insertItem(i18n("delete completed to-dos","Purge Completed"),
                              this, SLOT( purgeCompleted() ) );
 
   connect( mMovePopupMenu, SIGNAL( dateChanged( QDate ) ),
@@ -453,9 +469,9 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
            mItemPopupMenu, SLOT( hide() ) );
 
   mPopupMenu = new QPopupMenu(this);
-  mPopupMenu->insertItem(KOGlobals::self()->smallIconSet("todo"), i18n("New To-Do..."), this,
+  mPopupMenu->insertItem(KOGlobals::self()->smallIconSet("todo"), i18n("New To-do..."), this,
                          SLOT (newTodo()));
-  mPopupMenu->insertItem(i18n("delete completed To-Dos","Purge Completed"),
+  mPopupMenu->insertItem(i18n("delete completed to-dos","Purge Completed"),
                          this, SLOT(purgeCompleted()));
 
   mDocPrefs = new DocPrefs( name );
@@ -486,15 +502,6 @@ KOTodoView::KOTodoView( Calendar *calendar, QWidget *parent, const char* name)
            SLOT( processSelectionChange() ) );
   connect( mQuickAdd, SIGNAL( returnPressed () ),
            SLOT( addQuickTodo() ) );
-
-  connect( mTodoListView, SIGNAL( incidenceAdded( Incidence* ) ),
-           SIGNAL( incidenceAdded( Incidence* ) ) );
-  connect( mTodoListView, SIGNAL( incidenceChanged( Incidence*, Incidence* ) ),
-           SIGNAL( incidenceChanged( Incidence*, Incidence* ) ) );
-  connect( mTodoListView, SIGNAL( incidenceToBeDeleted( Incidence* ) ),
-           SIGNAL( incidenceToBeDeleted( Incidence* ) ) );
-  connect( mTodoListView, SIGNAL( incidenceDeleted( Incidence* ) ),
-           SIGNAL( incidenceDeleted( Incidence* ) ) );
 }
 
 KOTodoView::~KOTodoView()
@@ -615,7 +622,7 @@ bool KOTodoView::scheduleRemoveTodoItem( KOTodoViewItem *todoItem )
     mItemsToDelete.append( todoItem );
     QTimer::singleShot( 0, this, SLOT( removeTodoItems() ) );
     return true;
-  } else 
+  } else
     return false;
 }
 
@@ -649,9 +656,10 @@ Todo::List KOTodoView::selectedTodos()
 void KOTodoView::changeIncidenceDisplay(Incidence *incidence, int action)
 {
   // The todo view only displays todos, so exit on all other incidences
-  if ( incidence->type() != "Todo" ) 
+  if ( incidence->type() != "Todo" )
     return;
-  bool isFiltered = !calendar()->filter()->filterIncidence( incidence );
+  CalFilter *filter = calendar()->filter();
+  bool isFiltered = filter && !filter->filterIncidence( incidence );
   Todo *todo = static_cast<Todo *>(incidence);
   if ( todo ) {
     KOTodoViewItem *todoItem = 0;
@@ -662,7 +670,7 @@ void KOTodoView::changeIncidenceDisplay(Incidence *incidence, int action)
       case KOGlobals::INCIDENCEADDED:
       case KOGlobals::INCIDENCEEDITED:
         // If it's already there, edit it, otherwise just add
-        if ( todoItem ) { 
+        if ( todoItem ) {
           if ( isFiltered ) {
             scheduleRemoveTodoItem( todoItem );
           } else {
@@ -713,14 +721,6 @@ void KOTodoView::showIncidences( const Incidence::List & )
   kdDebug(5850) << "KOTodoView::showIncidences( const Incidence::List & ): not yet implemented" << endl;
 }
 
-void KOTodoView::printPreview(CalPrinter *calPrinter, const QDate &fd,
-                              const QDate &td)
-{
-#ifndef KORG_NOPRINTER
-  calPrinter->preview(CalPrinter::Todolist, fd, td);
-#endif
-}
-
 CalPrinter::PrintType KOTodoView::printType()
 {
   return CalPrinter::Todolist;
@@ -751,33 +751,48 @@ void KOTodoView::showItem( QListViewItem *item, const QPoint &, int )
 void KOTodoView::popupMenu( QListViewItem *item, const QPoint &, int column )
 {
   mActiveItem = static_cast<KOTodoViewItem *>( item );
-  if ( mActiveItem && !mActiveItem->todo()->isReadOnly() ) {
-    QDate date = mActiveItem->todo()->dtDue().date();
-    if ( mActiveItem->todo()->hasDueDate () ) {
-      mMovePopupMenu->datePicker()->setDate( date );
-    } else {
-      mMovePopupMenu->datePicker()->setDate( QDate::currentDate() );
-    }
-    switch ( column ) {
-      case 2:
-        mPriorityPopupMenu->popup( QCursor::pos() );
-        break;
-      case 3: {
-        mPercentageCompletedPopupMenu->popup( QCursor::pos() );
-        break;
+  if ( mActiveItem && mActiveItem->todo() &&
+       !mActiveItem->todo()->isReadOnly() ) {
+    bool editable = !mActiveItem->todo()->isReadOnly();
+    mItemPopupMenu->setItemEnabled( ePopupEdit, editable );
+    mItemPopupMenu->setItemEnabled( ePopupDelete, editable );
+    mItemPopupMenu->setItemEnabled( ePopupMoveTo, editable );
+    mItemPopupMenu->setItemEnabled( ePopupCopyTo, editable );
+    mItemPopupMenu->setItemEnabled( ePopupUnSubTodo, editable );
+    mItemPopupMenu->setItemEnabled( ePopupUnAllSubTodo, editable );
+
+    if ( editable ) {
+      QDate date = mActiveItem->todo()->dtDue().date();
+      if ( mActiveItem->todo()->hasDueDate () ) {
+        mMovePopupMenu->datePicker()->setDate( date );
+      } else {
+        mMovePopupMenu->datePicker()->setDate( QDate::currentDate() );
       }
-      case 4:
-        mMovePopupMenu->popup( QCursor::pos() );
-        break;
-      case 5:
-        getCategoryPopupMenu( mActiveItem )->popup( QCursor::pos() );
-        break;
-      default:
-        mCopyPopupMenu->datePicker()->setDate( date );
-        mCopyPopupMenu->datePicker()->setDate( QDate::currentDate() );
-        mItemPopupMenu->setItemEnabled( POPUP_UNSUBTODO,
-                                        mActiveItem->todo()->relatedTo() );
-        mItemPopupMenu->popup( QCursor::pos() );
+      switch ( column ) {
+        case ePriorityColumn:
+          mPriorityPopupMenu->popup( QCursor::pos() );
+          break;
+        case ePercentColumn: {
+          mPercentageCompletedPopupMenu->popup( QCursor::pos() );
+          break;
+        }
+        case eDueDateColumn:
+          mMovePopupMenu->popup( QCursor::pos() );
+          break;
+        case eCategoriesColumn:
+          getCategoryPopupMenu( mActiveItem )->popup( QCursor::pos() );
+          break;
+        default:
+          mCopyPopupMenu->datePicker()->setDate( date );
+          mCopyPopupMenu->datePicker()->setDate( QDate::currentDate() );
+          mItemPopupMenu->setItemEnabled( ePopupUnSubTodo,
+                                          mActiveItem->todo()->relatedTo() );
+          mItemPopupMenu->setItemEnabled( ePopupUnAllSubTodo,
+                                          !mActiveItem->todo()->relations().isEmpty() );
+          mItemPopupMenu->popup( QCursor::pos() );
+      }
+    } else {
+      mItemPopupMenu->popup( QCursor::pos() );
     }
   } else mPopupMenu->popup( QCursor::pos() );
 }
@@ -807,49 +822,86 @@ void KOTodoView::showTodo()
 void KOTodoView::deleteTodo()
 {
   if (mActiveItem) {
-    if (mActiveItem->childCount()) {
-      KMessageBox::sorry(this,i18n("Cannot delete To-Do which has children."),
-                         i18n("Delete To-Do"));
-    } else {
-      emit deleteIncidenceSignal(mActiveItem->todo());
-    }
+    emit deleteIncidenceSignal( mActiveItem->todo() );
   }
 }
 
 void KOTodoView::setNewPriority(int index)
 {
-  if (mActiveItem && !mActiveItem->todo()->isReadOnly ()) {
-    Todo *todo = mActiveItem->todo();
+  if ( !mActiveItem || !mChanger ) return;
+  Todo *todo = mActiveItem->todo();
+  if ( !todo->isReadOnly () &&
+       mChanger->beginChange( todo ) ) {
     Todo *oldTodo = todo->clone();
     todo->setPriority(mPriority[index]);
     mActiveItem->construct();
-    emit incidenceChanged( oldTodo, todo, KOGlobals::PRIORITY_MODIFIED );
+
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::PRIORITY_MODIFIED );
+    mChanger->endChange( todo );
     delete oldTodo;
   }
 }
 
-void KOTodoView::setNewPercentage(int index)
+void KOTodoView::setNewPercentage( KOTodoViewItem *item, int percentage )
 {
-  if ( mActiveItem && !mActiveItem->todo()->isReadOnly () ) {
-    Todo *todo = mActiveItem->todo();
+  kdDebug(5850) << "KOTodoView::setNewPercentage( " << percentage << "), item = " << item << endl;
+  if ( !item || !mChanger  ) return;
+  Todo *todo = item->todo();
+  if ( !todo ) return;
+
+  if ( !todo->isReadOnly () && mChanger->beginChange( todo ) ) {
     Todo *oldTodo = todo->clone();
 
-    if (mPercentage[index] == 100) {
-      emit todoCompleted( todo );
+/*  Old code to make sub-items's percentage related to this one's:
+    QListViewItem *myChild = firstChild();
+    KOTodoViewItem *item;
+    while( myChild ) {
+      item = static_cast<KOTodoViewItem*>(myChild);
+      item->stateChange(state);
+      myChild = myChild->nextSibling();
+    }*/
+    if ( percentage == 100 ) {
+      if ( KOPrefs::instance()->recordTodosInJournals() ) {
+        //FIXME: put all completed to-dos in one journal entry
+        QString text = i18n( "Todo completed: %1" ).arg( todo->summary() );
+        Journal *journal = new Journal();
+        mChanger->beginChange( journal );
+        journal->setDtStart( QDateTime::currentDateTime() );
+        journal->setDescription( text );
+        mChanger->addIncidence( journal );
+      }
+      todo->setCompleted( QDateTime::currentDateTime() );
+      // If the todo does recur, it doesn't get set as completed. However, the
+      // item is still checked. Uncheck it again.
+      if ( !todo->isCompleted() ) item->setState( QCheckListItem::Off );
+      else todo->setPercentComplete( percentage );
     } else {
-      todo->setCompleted(false);
+      todo->setCompleted( false );
+      todo->setPercentComplete( percentage );
     }
-    todo->setPercentComplete(mPercentage[index]);
-    mActiveItem->construct();
-    emit incidenceChanged( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+    item->construct();
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+    mChanger->endChange( todo );
     delete oldTodo;
+  } else {
+    item->construct();
+    kdDebug(5850) << "No active item, active item is read-only, or locking failed" << endl;
   }
 }
 
-void KOTodoView::setNewDate(QDate date)
+void KOTodoView::setNewPercentage( int index )
 {
-  if ( mActiveItem && !mActiveItem->todo()->isReadOnly()) {
-    Todo *todo = mActiveItem->todo();
+  setNewPercentage( mActiveItem, mPercentage[index] );
+}
+
+void KOTodoView::setNewDate( QDate date )
+{
+  if ( !mActiveItem || !mChanger ) return;
+  Todo *todo = mActiveItem->todo();
+  if ( !todo ) return;
+
+  if ( !todo->isReadOnly() && mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
 
     QDateTime dt;
     dt.setDate( date );
@@ -857,18 +909,18 @@ void KOTodoView::setNewDate(QDate date)
     if ( !todo->doesFloat() )
       dt.setTime( todo->dtDue().time() );
 
-    Todo *oldTodo = todo->clone();
-
     if ( date.isNull() )
       todo->setHasDueDate( false );
     else if ( !todo->hasDueDate() )
       todo->setHasDueDate( true );
     todo->setDtDue( dt );
-    todo->setRevision( todo->revision() + 1 );
 
     mActiveItem->construct();
-    emit incidenceChanged( oldTodo, todo, KOGlobals::DATE_MODIFIED );
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::COMPLETION_MODIFIED );
+    mChanger->endChange( todo );
     delete oldTodo;
+  } else {
+    kdDebug(5850) << "No active item, active item is read-only, or locking failed" << endl;
   }
 }
 
@@ -877,7 +929,7 @@ void KOTodoView::copyTodoToDate( QDate date )
   QDateTime dt;
   dt.setDate( date );
 
-  if ( mActiveItem ) {
+  if ( mActiveItem && mChanger ) {
     Todo *newTodo = mActiveItem->todo()->clone();
     newTodo->recreate();
 
@@ -890,8 +942,7 @@ void KOTodoView::copyTodoToDate( QDate date )
    if ( newTodo->doesRecur() )
      newTodo->recurrence()->unsetRecurs();
 
-   calendar()->addTodo( newTodo );
-   emit incidenceAdded( newTodo );
+   mChanger->addIncidence( newTodo );
  }
 }
 
@@ -918,19 +969,26 @@ QPopupMenu *KOTodoView::getCategoryPopupMenu( KOTodoViewItem *todoItem )
 
 void KOTodoView::changedCategories(int index)
 {
-  if (mActiveItem && !mActiveItem->todo()->isReadOnly ()) {
-    Todo*todo = mActiveItem->todo();
-    Todo*oldTodo = todo->clone();
+  if ( !mActiveItem || !mChanger ) return;
+  Todo *todo = mActiveItem->todo();
+  if ( !todo ) return;
+
+  if ( !todo->isReadOnly() && mChanger->beginChange( todo ) ) {
+    Todo *oldTodo = todo->clone();
+
     QStringList categories = todo->categories ();
-    if (categories.find (mCategory[index]) != categories.end ())
-      categories.remove (mCategory[index]);
+    if ( categories.find( mCategory[index] ) != categories.end() )
+      categories.remove( mCategory[index] );
     else
-      categories.insert (categories.end(), mCategory[index]);
-    categories.sort ();
-    todo->setCategories (categories);
+      categories.insert( categories.end(), mCategory[index] );
+    categories.sort();
+    todo->setCategories( categories );
     mActiveItem->construct();
-    emit incidenceChanged( oldTodo, todo, KOGlobals::CATEGORY_MODIFIED);
+    mChanger->changeIncidence( oldTodo, todo, KOGlobals::CATEGORY_MODIFIED );
+    mChanger->endChange( todo );
     delete oldTodo;
+  } else {
+    kdDebug(5850) << "No active item, active item is read-only, or locking failed" << endl;
   }
 }
 
@@ -991,19 +1049,19 @@ void KOTodoView::addQuickTodo()
   if ( ! mQuickAdd->text().stripWhiteSpace().isEmpty() ) {
     Todo *todo = new Todo();
     todo->setSummary( mQuickAdd->text() );
-    todo->setOrganizer( Person( KOPrefs::instance()->fullName(), 
+    todo->setOrganizer( Person( KOPrefs::instance()->fullName(),
                         KOPrefs::instance()->email() ) );
-    if ( !calendar()->addTodo( todo ) ) {
-      KODialogManager::errorSaveTodo( this );
+    if ( !mChanger->addIncidence( todo ) ) {
+      KODialogManager::errorSaveIncidence( this, todo );
+      delete todo;
       return;
     }
     mQuickAdd->setText( QString::null );
-    emit incidenceAdded( todo );
-    updateView();
   }
 }
 
-void KOTodoView::emitCompletedSignal( Todo *todo )
+void KOTodoView::setIncidenceChanger( IncidenceChangerBase *changer )
 {
-  emit todoCompleted( todo );
+  mChanger = changer;
+  mTodoListView->setIncidenceChanger( changer );
 }
