@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2000,2001,2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,41 +25,36 @@
 #ifndef KOTODOVIEW_H
 #define KOTODOVIEW_H
 
-#include <qfont.h>
-#include <qfontmetrics.h>
-#include <qlineedit.h>
-#include <qptrlist.h>
-#include <qlistbox.h>
-#include <qpopupmenu.h>
-#include <qlabel.h>
 #include <qmap.h>
-#include <qtimer.h>
-#include <qevent.h>
 #include <qtooltip.h>
-#include <qpoint.h>
-#include <qrect.h>
 
 #include <klistview.h>
 
-#include <libkcal/calendar.h>
 #include <libkcal/todo.h>
-
 #include <korganizer/baseview.h>
-
-#include "kotodoviewitem.h"
-#include "koprefs.h"
-#include "koglobals.h"
 #include "calprinter.h"
 
 class QDragEnterEvent;
 class QDragMoveEvent;
 class QDragLeaveEvent;
 class QDropEvent;
+class QPopupMenu;
 
 class KOTodoListView;
+class KOTodoViewItem;
+class KDatePickerPopup;
 
 class DocPrefs;
 
+namespace KPIM {
+  class ClickLineEdit;
+}
+namespace KCal {
+class Incidence;
+class Calendar;
+}
+using namespace KCal;
+using namespace KOrg;
 
 class KOTodoListViewToolTip : public QToolTip
 {
@@ -82,10 +78,7 @@ class KOTodoListView : public KListView
 
     void setCalendar( Calendar * );
 
-  signals:
-    void todoDropped( Todo * );
-    void todoChanged( Todo*, Todo* );
-    void todoAdded( Todo* );
+    void setIncidenceChanger( IncidenceChangerBase *changer ) { mChanger = changer; }
 
   protected:
     virtual bool event( QEvent * );
@@ -102,6 +95,7 @@ class KOTodoListView : public KListView
 
   private:
     Calendar *mCalendar;
+    KOrg::IncidenceChangerBase *mChanger;
 
     QPoint mPressPos;
     bool mMousePressed;
@@ -110,21 +104,6 @@ class KOTodoListView : public KListView
 };
 
 
-/**
-  This is the line-edit on top of the todoview for fast addition of new todos
-*/
-class KOQuickTodo : public QLineEdit
-{
-    Q_OBJECT
-  public:
-    KOQuickTodo( QWidget *parent = 0 );
-
-  protected:
-    void focusInEvent( QFocusEvent *ev );
-    void focusOutEvent( QFocusEvent *ev );
-};
-
-
 /**
   This class provides a multi-column list view of todo events.
 
@@ -148,9 +127,6 @@ class KOTodoView : public KOrg::BaseView
     /** Return number of shown dates. TodoView does not show dates, */
     int currentDateCount() { return 0; }
 
-    void printPreview( CalPrinter *calPrinter, const QDate &fd,
-                       const QDate &td );
-
     CalPrinter::PrintType printType();
 
     void setDocumentId( const QString & );
@@ -159,21 +135,23 @@ class KOTodoView : public KOrg::BaseView
     void restoreLayout( KConfig *config, const QString &group );
     /** Create a popup menu to set categories */
     QPopupMenu *getCategoryPopupMenu( KOTodoViewItem *todoItem );
+    void setIncidenceChanger( IncidenceChangerBase *changer );
 
   public slots:
     void updateView();
     void updateConfig();
 
-    void changeEventDisplay( Event *, int );
+    void changeIncidenceDisplay( Incidence *, int );
 
     void showDates( const QDate &start, const QDate &end );
-    void showEvents( const Event::List & );
+    void showIncidences( const Incidence::List &incidenceList );
 
     void clearSelection();
 
     void editItem( QListViewItem *item, const QPoint &, int );
     void editItem( QListViewItem *item );
     void showItem( QListViewItem *item, const QPoint &, int );
+    void showItem( QListViewItem *item );
     void popupMenu( QListViewItem *item, const QPoint &, int );
     void newTodo();
     void newSubTodo();
@@ -181,39 +159,30 @@ class KOTodoView : public KOrg::BaseView
     void editTodo();
     void deleteTodo();
 
+    void setNewPercentage( KOTodoViewItem *item, int percentage );
+
     void setNewPriority( int );
     void setNewPercentage( int );
+    void setNewDate( QDate );
+    void copyTodoToDate( QDate );
     void changedCategories( int );
 
     void purgeCompleted();
 
-    void itemClicked( QListViewItem * );
     void itemStateChanged( QListViewItem * );
-    void modified( bool );
-    void setTodoModified( Todo *oldTodo, Todo *todo )
-    {
-      emit todoChanged( oldTodo, todo );
-    }
 
   signals:
-    void newTodoSignal();
-    void newSubTodoSignal( Todo * );
     void unSubTodoSignal();
-    void showTodoSignal( Todo * );
-
-    void editTodoSignal( Todo * );
-    void deleteTodoSignal( Todo * );
-    void todoModifiedSignal( Todo *, Todo *, int );
-    void todoChanged ( Todo*, Todo* );
-    void todoAdded ( Todo* );
-
-    void isModified( bool );
+    void unAllSubTodoSignal();
+    
+    void todoCompleted( Todo * );
 
     void purgeCompletedSignal();
 
   protected slots:
     void processSelectionChange();
     void addQuickTodo();
+    void removeTodoItems();
 
   private:
     /*
@@ -227,8 +196,8 @@ class KOTodoView : public KOrg::BaseView
      */
     friend class KOTodoViewItem;
 
-    void setTodoModified( Todo * );
     QMap<Todo *,KOTodoViewItem *>::ConstIterator insertTodoItem( Todo *todo );
+    bool scheduleRemoveTodoItem( KOTodoViewItem *todoItem );
     void restoreItemState( QListViewItem * );
 
     KOTodoListView *mTodoListView;
@@ -237,6 +206,8 @@ class KOTodoView : public KOrg::BaseView
     QPopupMenu *mPriorityPopupMenu;
     QPopupMenu *mPercentageCompletedPopupMenu;
     QPopupMenu *mCategoryPopupMenu;
+    KDatePickerPopup *mMovePopupMenu;
+    KDatePickerPopup *mCopyPopupMenu;
 
     QMap<int, int> mPercentage;
     QMap<int, int> mPriority;
@@ -245,12 +216,31 @@ class KOTodoView : public KOrg::BaseView
     KOTodoViewItem *mActiveItem;
 
     QMap<Todo *,KOTodoViewItem *> mTodoMap;
+    QPtrList<KOTodoViewItem> mItemsToDelete;
 
     DocPrefs *mDocPrefs;
     QString mCurrentDoc;
-    KOQuickTodo *mQuickAdd;
+    KPIM::ClickLineEdit *mQuickAdd;
 
-    static const int POPUP_UNSUBTODO;
+  public:
+    enum {
+      eSummaryColumn = 0,
+      eRecurColumn = 1,
+      ePriorityColumn = 2,
+      ePercentColumn = 3,
+      eDueDateColumn = 4,
+      eCategoriesColumn = 5,
+      eDescriptionColumn = 6
+    }; 
+    enum {
+      ePopupEdit = 1300,
+      ePopupDelete = 1301,
+      ePopupMoveTo = 1302,
+      ePopupCopyTo = 1303,
+      ePopupUnSubTodo = 1304,
+      ePopupUnAllSubTodo = 1305
+    };
+  
 };
 
 #endif
