@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2000, 2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,7 +39,7 @@ class KOTodoView;
 /**
   This class provides a way of displaying a single Event of Todo-Type in a
   KTodoView.
-  
+
   @author Cornelius Schumacher <schumacher@kde.org>
   @see KOTodoView
 */
@@ -49,7 +50,8 @@ class KOTodoViewItem : public QCheckListItem
       Constructor.
 
       @param parent is the list view to which this item belongs.
-      @param ev is the event to have the item display information for.
+      @param todo is the todo to have the item display information for.
+      @param kotodo is a pointer to the KOTodoView object.
     */
     KOTodoViewItem(QListView *parent, Todo *todo, KOTodoView *kotodo);
     KOTodoViewItem(KOTodoViewItem *parent, Todo *todo, KOTodoView *kotodo);
@@ -62,7 +64,7 @@ class KOTodoViewItem : public QCheckListItem
     QString key(int, bool) const;
 
     void setSortKey(int column,const QString &key);
-    
+
     bool isAlternate();
     int compare( QListViewItem *i, int col, bool ascending ) const;
     virtual void paintCell(QPainter *p, const QColorGroup &cg,
@@ -74,16 +76,7 @@ class KOTodoViewItem : public QCheckListItem
 #else
 #endif
   virtual void stateChange(bool);
-  enum {
-    eSummaryColumn=0,
-    eRecurColumn=1,
-    ePriorityColumn=2,
-    ePercentColumn=3,
-    eDueDateColumn=4,
-    eCategoriesColumn=5,
-    eDescriptionColumn=6
-  };
-    
+
 
   private:
     Todo *mTodo;
