@@ -1,5 +1,6 @@
 /*
     This file is part of KOrganizer.
+
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -15,6 +16,10 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+    As a special exception, permission is given to link this program
+    with any edition of Qt, and distribute the resulting executable,
+    without including the source code for Qt in the source distribution.
 */
 #ifndef KOWHATSNEXTVIEW_H
 #define KOWHATSNEXTVIEW_H
@@ -50,34 +55,30 @@ class KOWhatsNextView : public KOrg::BaseView
                     const char *name = 0);
     ~KOWhatsNextView();
 
-    virtual int maxDatesHint();
     virtual int currentDateCount();
-    virtual Incidence::List selectedIncidences();
+    virtual Incidence::List selectedIncidences() { return Incidence::List(); }
     DateList selectedDates() { return DateList(); }
-    virtual void printPreview(CalPrinter *calPrinter,
-                              const QDate &, const QDate &);
 
   public slots:
     virtual void updateView();
     virtual void showDates(const QDate &start, const QDate &end);
-    virtual void showEvents( const Event::List & );
+    virtual void showIncidences( const Incidence::List &incidenceList );
 
-    void changeEventDisplay(Event *, int);
+    void changeIncidenceDisplay(Incidence *, int);
 
   protected:
-    void appendEvent(Incidence *, bool reply=false);
-    void appendTodo(Incidence *);
+    void appendEvent( Incidence *, const QDateTime &start = QDateTime(), 
+                      const QDateTime &end = QDateTime() );
+    void appendTodo( Incidence * );
 
   private slots:
     void showIncidence(const QString &);
 
   private:
-    void createEventViewer();
-
     QTextBrowser *mView;
     QString mText;
-
-    KOEventViewerDialog *mEventViewer;
+    QDate mStartDate;
+    QDate mEndDate;
 
     Incidence::List mTodos;
 };
