@@ -37,9 +37,6 @@
 
 #include "koglobals.h"
 #include "koprefs.h"
-#ifndef KORG_NOPLUGINS
-#include "kocore.h"
-#endif
 
 #include <kcalendarsystem.h>
 
@@ -56,70 +53,54 @@ void ActiveLabel::mouseReleaseEvent( QMouseEvent * )
 }
 
 
-NavigatorBar::NavigatorBar( const QDate & date, QWidget *parent, const char *name )
-  : QWidget( parent, name ), mDate(date)
+NavigatorBar::NavigatorBar( QWidget *parent, const char *name )
+  : QWidget( parent, name ), mHasMinWidth( false )
 {
-  QBoxLayout *topLayout = new QHBoxLayout( this );
-
-  // Set up the control buttons and date label
-  mCtrlFrame = new QFrame( this );
-  mCtrlFrame->setFrameStyle( QFrame::Panel | QFrame::Raised );
-  mCtrlFrame->setLineWidth( 1 );
-
-  topLayout->addWidget( mCtrlFrame );
-
   QFont tfont = font();
   tfont.setPointSize( 10 );
   tfont.setBold( false );
 
   bool isRTL = KOGlobals::self()->reverseLayout();
 
-  // Create month name button
-  mMonth = new ActiveLabel( mCtrlFrame );
-  mMonth->setFont( tfont );
-  mMonth->setAlignment( AlignCenter );
-  QToolTip::add( mMonth, i18n("Select a Month") );
-
-  // Set minimum width to width of widest month name label
-  int i;
-  int maxwidth = 0;
-
-  const KCalendarSystem *calSys = KOGlobals::self()->calendarSystem();
-  for( i = 1; i <= calSys->monthsInYear(date); ++i ) {
-    int w = QFontMetrics(tfont).width( QString("%1 8888").arg(calSys->monthName( i, calSys->year(date) )) );
-    if ( w > maxwidth ) maxwidth = w;
-  }
-  mMonth->setMinimumWidth( maxwidth );
-
+  QPixmap pix;
   // Create backward navigation buttons
-  mPrevYear = new QPushButton( mCtrlFrame );
-  mPrevYear->setPixmap( KOGlobals::self()->smallIcon( isRTL ? "2rightarrow" : "2leftarrow" ) );
-  mPrevYear->setMinimumHeight(mMonth->height());
+  mPrevYear = new QPushButton( this );
+  pix = KOGlobals::self()->smallIcon( isRTL ? "2rightarrow" : "2leftarrow" );
+  mPrevYear->setPixmap( pix );
+  mPrevYear->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
   QToolTip::add( mPrevYear, i18n("Previous year") );
 
-  mPrevMonth = new QPushButton( mCtrlFrame );
-  mPrevMonth->setPixmap( KOGlobals::self()->smallIcon( isRTL ? "1rightarrow" : "1leftarrow") );
-  mPrevMonth->setMinimumHeight(mMonth->height());
+  pix = KOGlobals::self()->smallIcon( isRTL ? "1rightarrow" : "1leftarrow");
+  mPrevMonth = new QPushButton( this );
+  mPrevMonth->setPixmap( pix );
+  mPrevMonth->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
   QToolTip::add( mPrevMonth, i18n("Previous month") );
 
   // Create forward navigation buttons
-  mNextMonth = new QPushButton( mCtrlFrame );
-  mNextMonth->setPixmap( KOGlobals::self()->smallIcon( isRTL ? "1leftarrow" : "1rightarrow") );
-  mNextMonth->setMinimumHeight(mMonth->height());
+  pix = KOGlobals::self()->smallIcon( isRTL ? "1leftarrow" : "1rightarrow");
+  mNextMonth = new QPushButton( this );
+  mNextMonth->setPixmap( pix );
+  mNextMonth->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
   QToolTip::add( mNextMonth, i18n("Next month") );
 
-  mNextYear = new QPushButton( mCtrlFrame );
-  mNextYear->setPixmap( KOGlobals::self()->smallIcon( isRTL ? "2leftarrow" : "2rightarrow") );
-  mNextYear->setMinimumHeight(mMonth->height());
+  pix = KOGlobals::self()->smallIcon( isRTL ? "2leftarrow" : "2rightarrow");
+  mNextYear = new QPushButton( this );
+  mNextYear->setPixmap( pix );
+  mNextYear->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
   QToolTip::add( mNextYear, i18n("Next year") );
 
+  // Create month name button
+  mMonth = new ActiveLabel( this );
+  mMonth->setFont( tfont );
+  mMonth->setAlignment( AlignCenter );
+  mMonth->setMinimumHeight( mPrevYear->sizeHint().height() );
+  QToolTip::add( mMonth, i18n("Select a month") );
+
   // set up control frame layout
-  QBoxLayout *ctrlLayout = new QHBoxLayout( mCtrlFrame, 1 );
+  QBoxLayout *ctrlLayout = new QHBoxLayout( this, 0, 4 );
   ctrlLayout->addWidget( mPrevYear, 3 );
   ctrlLayout->addWidget( mPrevMonth, 3 );
-  ctrlLayout->addSpacing( 2 );
   ctrlLayout->addWidget( mMonth, 3 );
-  ctrlLayout->addSpacing( 2 );
   ctrlLayout->addWidget( mNextMonth, 3 );
   ctrlLayout->addWidget( mNextYear, 3 );
 
@@ -134,15 +115,52 @@ NavigatorBar::~NavigatorBar()
 {
 }
 
+void NavigatorBar::showButtons( bool left, bool right )
+{
+  if ( left ) {
+    mPrevYear->show();
+    mPrevMonth->show();
+  } else {
+    mPrevYear->hide();
+    mPrevMonth->hide();
+  }
+
+  if ( right ) {
+    mNextYear->show();
+    mNextMonth->show();
+  } else {
+    mNextYear->hide();
+    mNextMonth->hide();
+  }
+
+}
+
 void NavigatorBar::selectDates( const KCal::DateList &dateList )
 {
-  if (dateList.count() > 0) {
+  if ( dateList.count() > 0 ) {
     mDate = dateList.first();
 
     const KCalendarSystem *calSys = KOGlobals::self()->calendarSystem();
 
+    if ( !mHasMinWidth ) {
+      // Set minimum width to width of widest month name label
+      int i;
+      int maxwidth = 0;
+
+      for( i = 1; i <= calSys->monthsInYear( mDate ); ++i ) {
+        int w = QFontMetrics( mMonth->font() ).width( QString("%1 8888")
+            .arg( calSys->monthName( i, calSys->year( mDate ) ) ) );
+        if ( w > maxwidth ) maxwidth = w;
+      }
+      mMonth->setMinimumWidth( maxwidth );
+
+      mHasMinWidth = true;
+    }
+
     // compute the label at the top of the navigator
-    mMonth->setText( QString("%1 %2").arg(calSys->monthName( mDate )).arg(calSys->year(mDate)) );
+    mMonth->setText( i18n( "monthname year", "%1 %2" )
+                     .arg( calSys->monthName( mDate ) )
+                     .arg( calSys->year( mDate ) ) );
   }
 }
 
