@@ -2,6 +2,7 @@
     This file is part of KOrganizer.
 
     Copyright (c) 2003,2004 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -69,11 +70,17 @@ class ResourceItem : public QCheckListItem
     void createSubresourceItems();
 
     void update();
-
+    /*
+    virtual void paintCell(QPainter *p, const QColorGroup &cg,
+      int column, int width, int alignment);
+//     */
+    void setResourceColor(QColor& color); 
+    QColor &resourceColor() {return mResourceColor;}
   protected:
     void stateChange( bool active );
 
     void setGuiState();
+    QColor mResourceColor;
 
   private:
     KCal::ResourceCalendar *mResource;
@@ -124,20 +131,20 @@ class ResourceView : public CalendarViewExtension
     void currentChanged( QListViewItem* );
     void slotSubresourceAdded( ResourceCalendar *, const QString &,
                                const QString &resource,const QString& label );
-    // FIXME proko2: merge once we are back in head by porting imap resource
-    void slotSubresourceAdded( ResourceCalendar *, const QString &,
-                               const QString &resource );
-
     void slotSubresourceRemoved( ResourceCalendar *, const QString &,
-                                 const QString &resource );
+                                 const QString & );
     void closeResource( ResourceCalendar * );
 
     void contextMenuRequested ( QListViewItem *i, const QPoint &pos, int );
 
+    void assignColor();
+    void disableColor();
     void showInfo();
   
     void reloadResource();
     void saveResource();
+    
+    void setStandard();
 
   private:
     KListView *mListView;
@@ -147,4 +154,5 @@ class ResourceView : public CalendarViewExtension
     QPushButton *mEditButton;
     QPtrList<ResourceCalendar> mResourcesToClose;
 };
+
 #endif
