@@ -19,14 +19,13 @@
 #ifndef STATUSDIALOG_H
 #define STATUSDIALOG_H
 
-#include <qcombobox.h>
-
 #include <kdialog.h>
 
 #include <libkcal/attendee.h>
-
 using namespace KCal;
 
+class QComboBox;
+
 class StatusDialog : public KDialog
 {
     Q_OBJECT
