@@ -1,4 +1,4 @@
-/* abbrowser-conduit.cc                           KPilot
+/* KPilot
 **
 ** Copyright (C) 2000,2001 by Dan Pilone
 ** Copyright (C) 2002-2003 by Reinhold Kainhofer
@@ -31,36 +31,35 @@
 
 
 #include "options.h"
-#include "abbrowser-conduit.moc"
-
-#include <unistd.h>
 
 #include <qtimer.h>
-#include <qvbuttongroup.h>
-#include <qcheckbox.h>
 #include <qtextcodec.h>
-#include <time.h>
+#include <qfile.h>
+#include <qregexp.h>
 
-
-#include <kglobal.h>
-#include <kdebug.h>
-#include <kconfig.h>
-#include <kabc/addressbook.h>
 #include <kabc/stdaddressbook.h>
 #include <kabc/resourcefile.h>
+#include <kio/netaccess.h>
 
-#include <pilotUser.h>
 #include <pilotSerialDatabase.h>
+#include <pilotLocalDatabase.h>
 
-#include "abbrowser-factory.h"
 #include "resolutionDialog.h"
 #include "resolutionTable.h"
+#include "abbrowserSettings.h"
+
+#include "abbrowser-conduit.moc"
 
 // Something to allow us to check what revision
 // the modules are that make up a binary distribution.
 //
 //
-const char *abbrowser_conduit_id="$Id: abbrowser-conduit.cc,v 1.88 2003/10/24 13:50:43 kainhofe Exp $";
+extern "C"
+{
+long version_conduit_address = KPILOT_PLUGIN_API;
+const char *id_conduit_address="$Id: abbrowser-conduit.cc,v 1.112 2005/02/10 10:35:11 adridg Exp $";
+}
+
 
 using namespace KABC;
 
@@ -68,27 +67,17 @@ const QString AbbrowserConduit::appString=CSL1("KPILOT");
 const QString AbbrowserConduit::flagString=CSL1("Flag");
 const QString AbbrowserConduit::idString=CSL1("RecordID");
 
-bool AbbrowserConduit::fPilotStreetHome=true;
-bool AbbrowserConduit::fPilotFaxHome=true;
-bool AbbrowserConduit::fArchive=true;
-enum AbbrowserConduit::ePilotOtherEnum AbbrowserConduit::ePilotOther=AbbrowserConduit::eOtherPhone;
 AddressBook*AbbrowserConduit::aBook=0L;
 
-enum AbbrowserConduit::eCustomEnum AbbrowserConduit::eCustom[4] = {
-	AbbrowserConduit::eCustomField,
-	AbbrowserConduit::eCustomField,
-	AbbrowserConduit::eCustomField,
-	AbbrowserConduit::eCustomField
-	} ;
-QString AbbrowserConduit::fCustomFmt=QString::null;
-
 /// This macro just sets the phone number of type "type" to "phone"
 /// Use a macro, because that saves two lines for each call, but does not
 /// have the overhead of a function call
-#define _setPhoneNumber(abEntry, type, nr) \
-	{ PhoneNumber phone = abEntry.phoneNumber(type); \
+static inline void _setPhoneNumber(Addressee &abEntry, int type, const QString &nr)
+{ 
+	PhoneNumber phone = abEntry.phoneNumber(type);
 	phone.setNumber(nr); \
-	abEntry.insertPhoneNumber(phone); }
+	abEntry.insertPhoneNumber(phone); 
+}
 
 
 /*********************************************************************
@@ -108,7 +97,7 @@ AbbrowserConduit::AbbrowserConduit(KPilotDeviceLink * o, const char *n, const QS
 {
 	FUNCTIONSETUP;
 #ifdef DEBUG
-	DEBUGCONDUIT<<abbrowser_conduit_id<<endl;
+	DEBUGCONDUIT<<id_conduit_address<<endl;
 #endif
 	fConduitName=i18n("Addressbook");
 }
@@ -169,49 +158,23 @@ bool AbbrowserConduit::_prepare()
 void AbbrowserConduit::readConfig()
 {
 	FUNCTIONSETUP;
-
-	KConfigGroupSaver g(fConfig, AbbrowserConduitFactory::group());
-
-	// General page
-	fAbookType = (eAbookTypeEnum)fConfig->readNumEntry(
-		AbbrowserConduitFactory::fAbookType, 0);
-	fAbookFile = fConfig->readEntry(
-		AbbrowserConduitFactory::fAbookFile);
-	fArchive=fConfig->readBoolEntry(
-		AbbrowserConduitFactory::fArchive, true);
+	AbbrowserSettings::self()->readConfig();
 
 	// Conflict page
-	SyncAction::eConflictResolution res=(SyncAction::eConflictResolution)fConfig->readNumEntry(
-		AbbrowserConduitFactory::fResolution, SyncAction::eUseGlobalSetting);
-	if (res!=SyncAction::eUseGlobalSetting) fConflictResolution=res;
-
-	// Fields page
-	fPilotStreetHome=!fConfig->readBoolEntry(
-		AbbrowserConduitFactory::fStreetType, true);
-	fPilotFaxHome=!fConfig->readBoolEntry(
-		AbbrowserConduitFactory::fFaxType, true);
-	ePilotOther=(ePilotOtherEnum)(fConfig->readNumEntry(
-		AbbrowserConduitFactory::fOtherField, eOtherPhone));
-
-	// Custom fields page
-	for (int i=0; i<4; i++)
-	{
-		eCustom[i]=(eCustomEnum)(fConfig->readNumEntry(
-			AbbrowserConduitFactory::custom(i), eCustomField) );
-	}
-	fCustomFmt=fConfig->readEntry(AbbrowserConduitFactory::fCustomFmt, QString::null);
+	SyncAction::ConflictResolution res = (SyncAction::ConflictResolution)AbbrowserSettings::conflictResolution();
+	setConflictResolution(res);
 
 #ifdef DEBUG
 	DEBUGCONDUIT << fname
 		<< ": Settings "
-		<< " fConflictResolution=" << fConflictResolution
-		<< " fPilotStreetHome=" << fPilotStreetHome
-		<< " fPilotFaxHome=" << fPilotFaxHome
-		<< " fArchive=" << fArchive
-		<< " eCustom[0]=" << eCustom[0]
-		<< " eCustom[1]=" << eCustom[1]
-		<< " eCustom[2]=" << eCustom[2]
-		<< " eCustom[3]=" << eCustom[3]
+		<< " fConflictResolution=" << getConflictResolution()
+		<< " fPilotStreetHome=" << AbbrowserSettings::pilotStreet()
+		<< " fPilotFaxHome=" << AbbrowserSettings::pilotFax()
+		<< " fArchive=" << AbbrowserSettings::archiveDeleted()
+		<< " eCustom[0]=" << AbbrowserSettings::custom0()
+		<< " eCustom[1]=" << AbbrowserSettings::custom1()
+		<< " eCustom[2]=" << AbbrowserSettings::custom2()
+		<< " eCustom[3]=" << AbbrowserSettings::custom3()
 		<< " fFirstTime=" << isFirstSync()
 		<< endl;
 #endif
@@ -223,12 +186,12 @@ bool AbbrowserConduit::isDeleted(const PilotAddress*addr)
 {
 	if (!addr) return true;
 	if (addr->isDeleted() && !addr->isArchived()) return true;
-	if (addr->isArchived()) return !fArchive;
+	if (addr->isArchived()) return !AbbrowserSettings::archiveDeleted();
 	return false;
 }
 bool AbbrowserConduit::isArchived(const PilotAddress*addr)
 {
-	if (addr && addr->isArchived()) return fArchive;
+	if (addr && addr->isArchived()) return AbbrowserSettings::archiveDeleted();
 	else return false;
 }
 bool AbbrowserConduit::isArchived(const Addressee &addr)
@@ -248,24 +211,48 @@ bool AbbrowserConduit::makeArchived(Addressee &addr)
 bool AbbrowserConduit::_loadAddressBook()
 {
 	FUNCTIONSETUP;
-	KConfigGroupSaver g(fConfig, AbbrowserConduitFactory::group());
-	switch (fAbookType)
+
+	startTickle();
+	switch ( AbbrowserSettings::addressbookType() )
 	{
-		case eAbookResource:
+		case AbbrowserSettings::eAbookResource:
 			DEBUGCONDUIT<<"Loading standard addressbook"<<endl;
 			aBook = StdAddressBook::self();
 			break;
-		case eAbookLocal: { // initialize the abook with the given file
+		case AbbrowserSettings::eAbookFile:
+		{ // initialize the abook with the given file
 			DEBUGCONDUIT<<"Loading custom addressbook"<<endl;
-			aBook = new AddressBook();
-			if (!aBook) return false;
-			KABC::Resource *res = new ResourceFile( fAbookFile, "vcard" );
-			if ( !aBook->addResource( res ) ) {
-				DEBUGCONDUIT << "Unable to open resource for file " << fAbookFile << endl;
-				KPILOT_DELETE( aBook );
+			KURL kurl(AbbrowserSettings::fileName());
+			if(!KIO::NetAccess::download(AbbrowserSettings::fileName(), fABookFile, 0L) &&
+				!kurl.isLocalFile())
+			{
+				emit logError(i18n("You chose to sync with the file \"%1\", which "
+							"cannot be opened. Please make sure to supply a "
+							"valid file name in the conduit's configuration dialog. "
+							"Aborting the conduit.").arg(AbbrowserSettings::fileName()));
+				KIO::NetAccess::removeTempFile(fABookFile);
+				stopTickle();
 				return false;
 			}
-			break;}
+
+			aBook = new AddressBook();
+			if (!aBook)
+			{
+				stopTickle();
+				return false;
+			}
+			KABC::Resource *res = new ResourceFile(fABookFile, CSL1("vcard") );
+
+			bool r = aBook->addResource( res );
+			if ( !r )
+			{
+				DEBUGCONDUIT << "Unable to open resource for file " << fABookFile << endl;
+				KPILOT_DELETE( aBook );
+				stopTickle();
+				return false;
+			}
+			break;
+		}
 		default: break;
 	}
 	// find out if this can fail for reasons other than a non-existent
@@ -277,6 +264,7 @@ bool AbbrowserConduit::_loadAddressBook()
 		emit logError(i18n("Unable to initialize and load the addressbook for the sync.") );
 		kdWarning()<<k_funcinfo<<": Unable to initialize the addressbook for the sync."<<endl;
 		KPILOT_DELETE(aBook);
+		stopTickle();
 		return false;
 	}
 	abChanged = false;
@@ -285,6 +273,7 @@ bool AbbrowserConduit::_loadAddressBook()
 	{
 		kdWarning()<<k_funcinfo<<": Unable to lock addressbook for writing "<<endl;
 		KPILOT_DELETE(aBook);
+		stopTickle();
 		return false;
 	}
 	// get the addresseMap which maps Pilot unique record(address) id's to
@@ -297,6 +286,7 @@ bool AbbrowserConduit::_loadAddressBook()
 	{
 		_mapContactsToPilot(addresseeMap);
 	}
+	stopTickle();
 	return(aBook != 0L);
 }
 bool AbbrowserConduit::_saveAddressBook()
@@ -310,7 +300,7 @@ bool AbbrowserConduit::_saveAddressBook()
 
 	if (ticket)
 	{
-		if (abChanged) 
+		if (abChanged)
 		{
 			res=aBook->save(ticket);
 		}
@@ -332,8 +322,26 @@ bool AbbrowserConduit::_saveAddressBook()
 		kdWarning()<<k_funcinfo<<": No ticket available to save the "
 		<<"addressbook."<<endl;
 	}
-	if (fAbookType!=eAbookResource)
+	if ( AbbrowserSettings::addressbookType()!= AbbrowserSettings::eAbookResource )
 	{
+		KURL kurl(AbbrowserSettings::fileName());
+		if(!kurl.isLocalFile())
+		{
+#ifdef DEBUG
+			DEBUGCONDUIT << "Deleting local addressbook tempfile" << endl;
+#endif
+			if(!KIO::NetAccess::upload(fABookFile, AbbrowserSettings::fileName(), 0L)) {
+				emit logError(i18n("An error occurred while uploading \"%1\". You can try to upload "
+					"the temporary local file \"%2\" manually")
+					.arg(AbbrowserSettings::fileName()).arg(fABookFile));
+			}
+			else {
+				KIO::NetAccess::removeTempFile(fABookFile);
+			}
+			QFile backup(fABookFile + CSL1("~"));
+			backup.remove();
+		}
+
 #ifdef DEBUG
 		DEBUGCONDUIT<<"Deleting addressbook"<<endl;
 #endif
@@ -349,8 +357,8 @@ void AbbrowserConduit::_getAppInfo()
 {
 	FUNCTIONSETUP;
 	// get the address application header information
-	unsigned char *buffer = new unsigned char[PilotAddress::APP_BUFFER_SIZE];
-	int appLen=fDatabase->readAppBlock(buffer, PilotAddress::APP_BUFFER_SIZE);
+	unsigned char *buffer = new unsigned char[PilotRecord::APP_BUFFER_SIZE];
+	int appLen=fDatabase->readAppBlock(buffer, PilotRecord::APP_BUFFER_SIZE);
 
 	unpack_AddressAppInfo(&fAddressAppInfo, buffer, appLen);
 	delete[]buffer;
@@ -381,18 +389,49 @@ void AbbrowserConduit::_setAppInfo()
 	delete[] buffer;
 }
 
+int AbbrowserConduit::getCustom(const int index)
+{
+	FUNCTIONSETUPL(4);
 
+	int customEnum;
+	switch(index) {
+		case 0:
+			customEnum = AbbrowserSettings::custom0();
+			break;
+		case 1:
+			customEnum = AbbrowserSettings::custom1();
+			break;
+		case 2:
+			customEnum = AbbrowserSettings::custom2();
+			break;
+		case 3:
+			customEnum = AbbrowserSettings::custom3();
+			break;
+		default:
+			customEnum = index;
+			break;
+	}
+
+#ifdef DEBUG
+	DEBUGCONDUIT << fname << "Index: " << index << " -> customEnum: "
+		<< customEnum << endl;
+#endif
+
+	return customEnum;
+}
 
 QString AbbrowserConduit::getCustomField(const Addressee &abEntry, const int index)
 {
-	FUNCTIONSETUP;
+	FUNCTIONSETUPL(4);
 
-	switch (eCustom[index]) {
-		case eCustomBirthdate: {
+	switch (getCustom(index)) {
+		case AbbrowserSettings::eCustomBirthdate: {
 			QDateTime bdate(abEntry.birthday().date());
-			if (!bdate.isValid()) return abEntry.custom(appString, CSL1("CUSTOM")+QString::number(index));
+			if (!bdate.isValid())
+				return abEntry.custom(appString, CSL1("CUSTOM")+QString::number(index));
 			QString tmpfmt(KGlobal::locale()->dateFormat());
-			if (!fCustomFmt.isEmpty()) KGlobal::locale()->setDateFormat(fCustomFmt);
+			if (!AbbrowserSettings::customDateFormat().isEmpty())
+				KGlobal::locale()->setDateFormat(AbbrowserSettings::customDateFormat());
 #ifdef DEBUG
 			DEBUGCONDUIT<<"Birthdate: "<<KGlobal::locale()->formatDate(bdate.date())<<" (QDate: "<<bdate.toString()<<endl;
 #endif
@@ -400,27 +439,28 @@ QString AbbrowserConduit::getCustomField(const Addressee &abEntry, const int ind
 			KGlobal::locale()->setDateFormat(tmpfmt);
 			return ret;
 		}
-		case eCustomURL:
+		case AbbrowserSettings::eCustomURL:
 			return abEntry.url().url();
 			break;
-		case eCustomIM:
+		case AbbrowserSettings::eCustomIM:
 			return abEntry.custom(CSL1("KADDRESSBOOK"), CSL1("X-IMAddress"));
 			break;
-		case eCustomField:
+		case AbbrowserSettings::eCustomField:
 		default:
 			return abEntry.custom(appString, CSL1("CUSTOM")+QString::number(index));
 			break;
 	}
 }
+
 void AbbrowserConduit::setCustomField(Addressee &abEntry,  int index, QString cust)
 {
-	FUNCTIONSETUP;
+	FUNCTIONSETUPL(4);
 
-	switch (eCustom[index]) {
-		case eCustomBirthdate: {
+	switch (getCustom(index)) {
+		case AbbrowserSettings::eCustomBirthdate: {
 			QDate bdate;
 			bool ok=false;
-			if (!fCustomFmt.isEmpty())
+			if (AbbrowserSettings::customDateFormat().isEmpty())
 			{
 				// empty format means use locale setting
 				bdate=KGlobal::locale()->readDate(cust, &ok);
@@ -428,7 +468,15 @@ void AbbrowserConduit::setCustomField(Addressee &abEntry,  int index, QString cu
 			else
 			{
 				// use given format
-				bdate=KGlobal::locale()->readDate(cust, fCustomFmt, &ok);
+				bdate=KGlobal::locale()->readDate(cust, AbbrowserSettings::customDateFormat(), &ok);
+			}
+			
+			if (!ok)
+			{
+				QString format = KGlobal::locale()->dateFormatShort();
+				QRegExp re(CSL1("%[yY][^%]*"));
+				format.remove(re); // Remove references to year and following punctuation
+				bdate = KGlobal::locale()->readDate(cust, format, &ok);
 			}
 #ifdef DEBUG
 			DEBUGCONDUIT<<"Birthdate from "<<index<<"-th custom field: "<<bdate.toString()<<endl;
@@ -437,15 +485,15 @@ void AbbrowserConduit::setCustomField(Addressee &abEntry,  int index, QString cu
 			if (bdate.isValid())
 				return abEntry.setBirthday(bdate);
 			else
-				return abEntry.insertCustom(CSL1("KADDRESSBOOK"), CSL1("X-IMAddress"), cust);
+				return abEntry.insertCustom(CSL1("KADDRESSBOOK"), CSL1("X-Birthday"), cust);
 			break; }
-		case eCustomURL: {
+		case AbbrowserSettings::eCustomURL: {
 			return abEntry.setUrl(cust);
 			break;}
-		case eCustomIM: {
+		case AbbrowserSettings::eCustomIM: {
 			return abEntry.insertCustom(CSL1("KADDRESSBOOK"), CSL1("X-IMAddress"), cust);
 			break;}
-		case eCustomField:
+		case AbbrowserSettings::eCustomField:
 		default: {
 			return abEntry.insertCustom(appString, CSL1("CUSTOM")+QString::number(index), cust);
 			break;}
@@ -457,23 +505,23 @@ void AbbrowserConduit::setCustomField(Addressee &abEntry,  int index, QString cu
 
 QString AbbrowserConduit::getOtherField(const Addressee & abEntry)
 {
-	switch(ePilotOther)
+	switch(AbbrowserSettings::pilotOther())
 	{
-		case eOtherPhone:
+		case AbbrowserSettings::eOtherPhone:
 			return abEntry.phoneNumber(0).number();
-		case eAssistant:
+		case AbbrowserSettings::eAssistant:
 			return abEntry.custom(CSL1("KADDRESSBOOK"), CSL1("AssistantsName"));
-		case eBusinessFax:
+		case AbbrowserSettings::eBusinessFax:
 			return abEntry.phoneNumber(PhoneNumber::Fax | PhoneNumber::Work).number();
-		case eCarPhone:
+		case AbbrowserSettings::eCarPhone:
 			return abEntry.phoneNumber(PhoneNumber::Car).number();
-		case eEmail2:
+		case AbbrowserSettings::eEmail2:
 			return abEntry.emails().first();
-		case eHomeFax:
+		case AbbrowserSettings::eHomeFax:
 			return abEntry.phoneNumber(PhoneNumber::Fax | PhoneNumber::Home).number();
-		case eTelex:
+		case AbbrowserSettings::eTelex:
 			return abEntry.phoneNumber(PhoneNumber::Bbs).number();
-		case eTTYTTDPhone:
+		case AbbrowserSettings::eTTYTTDPhone:
 			return abEntry.phoneNumber(PhoneNumber::Pcs).number();
 		default:
 			return QString::null;
@@ -482,30 +530,30 @@ QString AbbrowserConduit::getOtherField(const Addressee & abEntry)
 void AbbrowserConduit::setOtherField(Addressee & abEntry, QString nr)
 {
 //	PhoneNumber phone;
-	switch(ePilotOther)
+	switch (AbbrowserSettings::pilotOther())
 	{
-		case eOtherPhone:
-			_setPhoneNumber(abEntry, 0, nr)
+		case AbbrowserSettings::eOtherPhone:
+			_setPhoneNumber(abEntry, 0, nr);
 			break;
-		case eAssistant:
+		case AbbrowserSettings::eAssistant:
 			abEntry.insertCustom(CSL1("KADDRESSBOOK"), CSL1("AssistantsName"), nr);
 			break;
-		case eBusinessFax:
-			_setPhoneNumber(abEntry, PhoneNumber::Fax | PhoneNumber::Work, nr)
+		case AbbrowserSettings::eBusinessFax:
+			_setPhoneNumber(abEntry, PhoneNumber::Fax | PhoneNumber::Work, nr);
 			break;
-		case eCarPhone:
-			_setPhoneNumber(abEntry, PhoneNumber::Car, nr)
+		case AbbrowserSettings::eCarPhone:
+			_setPhoneNumber(abEntry, PhoneNumber::Car, nr);
 			break;
-		case eEmail2:
+		case AbbrowserSettings::eEmail2:
 			return abEntry.insertEmail(nr);
-		case eHomeFax:
-			_setPhoneNumber(abEntry, PhoneNumber::Fax|PhoneNumber::Home, nr)
+		case AbbrowserSettings::eHomeFax:
+			_setPhoneNumber(abEntry, PhoneNumber::Fax|PhoneNumber::Home, nr);
 			break;
-		case eTelex:
-			_setPhoneNumber(abEntry, PhoneNumber::Bbs, nr)
+		case AbbrowserSettings::eTelex:
+			_setPhoneNumber(abEntry, PhoneNumber::Bbs, nr);
 			break;
-		case eTTYTTDPhone:
-			_setPhoneNumber(abEntry, PhoneNumber::Pcs, nr)
+		case AbbrowserSettings::eTTYTTDPhone:
+			_setPhoneNumber(abEntry, PhoneNumber::Pcs, nr);
 			break;
 	}
 }
@@ -514,12 +562,12 @@ void AbbrowserConduit::setOtherField(Addressee & abEntry, QString nr)
 
 PhoneNumber AbbrowserConduit::getFax(const Addressee & abEntry)
 {
-	return abEntry.phoneNumber(PhoneNumber::Fax |
-		( (fPilotFaxHome) ?(PhoneNumber::Home) :(PhoneNumber::Work)));
+	return abEntry.phoneNumber( PhoneNumber::Fax |
+		( (AbbrowserSettings::pilotFax()==0) ?(PhoneNumber::Home) :(PhoneNumber::Work)));
 }
 void AbbrowserConduit::setFax(Addressee & abEntry, QString fax)
 {
-	_setPhoneNumber(abEntry, PhoneNumber::Fax | (fPilotFaxHome ? PhoneNumber::Home : PhoneNumber::Work ), fax);
+	_setPhoneNumber(abEntry, PhoneNumber::Fax | ( (AbbrowserSettings::pilotFax()==0) ? PhoneNumber::Home : PhoneNumber::Work ), fax);
 }
 
 
@@ -529,12 +577,12 @@ void AbbrowserConduit::setFax(Addressee & abEntry, QString fax)
  *  return an address with preferred + home/work flag (from config dlg). */
 KABC::Address AbbrowserConduit::getAddress(const Addressee & abEntry)
 {
-	int type=(fPilotStreetHome)?(KABC::Address::Home):(KABC::Address::Work);
+	int type=(AbbrowserSettings::pilotStreet==0)?(KABC::Address::Home):(KABC::Address::Work);
 	KABC::Address ad(abEntry.address(KABC::Address::Pref));
 	if (!ad.isEmpty()) return ad;
 	ad=abEntry.address(type);
 	if (!ad.isEmpty()) return ad;
-	ad=abEntry.address((fPilotStreetHome) ?(KABC::Address::Work):(KABC::Address::Home));
+	ad=abEntry.address((AbbrowserSettings::pilotStreet==0) ?(KABC::Address::Work):(KABC::Address::Home));
 	if (!ad.isEmpty()) return ad;
 
 	return abEntry.address(type | KABC::Address::Pref);
@@ -568,7 +616,7 @@ QString AbbrowserConduit::_getCatForHH(const QStringList cats, const QString cur
 		}
 	}
 	// If we have a free label, return the first possible cat
-	QString lastCat(fAddressAppInfo.category.name[15]);
+	QString lastCat = QString::fromLatin1(fAddressAppInfo.category.name[15]);
 	if (lastCat.isEmpty()) return cats.first();
 	return QString::null;
 }
@@ -599,7 +647,7 @@ void AbbrowserConduit::showAddressee(const Addressee & abAddress)
 	DEBUGCONDUIT << "\t\tLast name = " << abAddress.familyName() << endl;
 	DEBUGCONDUIT << "\t\tFirst name = " << abAddress.givenName() << endl;
 	DEBUGCONDUIT << "\t\tCompany = " << abAddress.organization() << endl;
-	DEBUGCONDUIT << "\t\tJob Title = " << abAddress.title() << endl;
+	DEBUGCONDUIT << "\t\tJob Title = " << abAddress.prefix() << endl;
 	DEBUGCONDUIT << "\t\tNote = " << abAddress.note() << endl;
 	DEBUGCONDUIT << "\t\tHome phone = " << abAddress.phoneNumber(PhoneNumber::Home).number() << endl;
 	DEBUGCONDUIT << "\t\tWork phone = " << abAddress.phoneNumber(PhoneNumber::Work).number() << endl;
@@ -614,25 +662,27 @@ void AbbrowserConduit::showAddressee(const Addressee & abAddress)
 
 void AbbrowserConduit::showPilotAddress(PilotAddress *pilotAddress)
 {
-	FUNCTIONSETUP;
-	DEBUGCONDUIT << "\tPilot Address" << endl;
+	FUNCTIONSETUPL(3);
+	if (debug_level >= 3)
+	{
 	if (!pilotAddress) {
-		DEBUGCONDUIT<< "\t\tEMPTY"<<endl;
+		DEBUGCONDUIT<< fname << "| EMPTY"<<endl;
 		return;
 	}
-	DEBUGCONDUIT << "\t\tLast name = " << pilotAddress->getField(entryLastname) << endl;
-	DEBUGCONDUIT << "\t\tFirst name = " << pilotAddress->getField(entryFirstname) << endl;
-	DEBUGCONDUIT << "\t\tCompany = " << pilotAddress->getField(entryCompany) << endl;
-	DEBUGCONDUIT << "\t\tJob Title = " << pilotAddress->getField(entryTitle) << endl;
-	DEBUGCONDUIT << "\t\tNote = " << pilotAddress->getField(entryNote) << endl;
-	DEBUGCONDUIT << "\t\tHome phone = " << pilotAddress->getPhoneField(PilotAddress::eHome, false) << endl;
-	DEBUGCONDUIT << "\t\tWork phone = " << pilotAddress->getPhoneField(PilotAddress::eWork, false) << endl;
-	DEBUGCONDUIT << "\t\tMobile phone = " << pilotAddress->getPhoneField(PilotAddress::eMobile, false) << endl;
-	DEBUGCONDUIT << "\t\tEmail = " << pilotAddress->getPhoneField(PilotAddress::eEmail, false) << endl;
-	DEBUGCONDUIT << "\t\tFax = " << pilotAddress->getPhoneField(PilotAddress::eFax, false) << endl;
-	DEBUGCONDUIT << "\t\tPager = " << pilotAddress->getPhoneField(PilotAddress::ePager, false) << endl;
-	DEBUGCONDUIT << "\t\tOther = " << pilotAddress->getPhoneField(PilotAddress::eOther, false) << endl;
-	DEBUGCONDUIT << "\t\tCategory = " << pilotAddress->getCategoryLabel() << endl;
+	DEBUGCONDUIT << fname << "| Last name = " << pilotAddress->getField(entryLastname) << endl;
+	DEBUGCONDUIT << fname << "| First name = " << pilotAddress->getField(entryFirstname) << endl;
+	DEBUGCONDUIT << fname << "| Company = " << pilotAddress->getField(entryCompany) << endl;
+	DEBUGCONDUIT << fname << "| Job Title = " << pilotAddress->getField(entryTitle) << endl;
+	DEBUGCONDUIT << fname << "| Note = " << pilotAddress->getField(entryNote) << endl;
+	DEBUGCONDUIT << fname << "| Home phone = " << pilotAddress->getPhoneField(PilotAddress::eHome, false) << endl;
+	DEBUGCONDUIT << fname << "| Work phone = " << pilotAddress->getPhoneField(PilotAddress::eWork, false) << endl;
+	DEBUGCONDUIT << fname << "| Mobile phone = " << pilotAddress->getPhoneField(PilotAddress::eMobile, false) << endl;
+	DEBUGCONDUIT << fname << "| Email = " << pilotAddress->getPhoneField(PilotAddress::eEmail, false) << endl;
+	DEBUGCONDUIT << fname << "| Fax = " << pilotAddress->getPhoneField(PilotAddress::eFax, false) << endl;
+	DEBUGCONDUIT << fname << "| Pager = " << pilotAddress->getPhoneField(PilotAddress::ePager, false) << endl;
+	DEBUGCONDUIT << fname << "| Other = " << pilotAddress->getPhoneField(PilotAddress::eOther, false) << endl;
+	DEBUGCONDUIT << fname << "| Category = " << pilotAddress->getCategoryLabel() << endl;
+	}
 }
 #endif
 
@@ -641,13 +691,21 @@ void AbbrowserConduit::showAdresses(Addressee &pcAddr, PilotAddress *backupAddr,
 	PilotAddress *palmAddr)
 {
 #ifdef DEBUG
-	DEBUGCONDUIT << "abEntry:" << endl;
-	showAddressee(pcAddr);
-	DEBUGCONDUIT << "pilotAddress:" << endl;
-	showPilotAddress(palmAddr);
-	DEBUGCONDUIT << "backupAddress:" << endl;
-	showPilotAddress(backupAddr);
-	DEBUGCONDUIT << "------------------------------------------------" << endl;
+	FUNCTIONSETUPL(3);
+	if (debug_level >= 3)
+	{
+		DEBUGCONDUIT << fname << "abEntry:" << endl;
+		showAddressee(pcAddr);
+		DEBUGCONDUIT << fname << "pilotAddress:" << endl;
+		showPilotAddress(palmAddr);
+		DEBUGCONDUIT << fname << "backupAddress:" << endl;
+		showPilotAddress(backupAddr);
+		DEBUGCONDUIT << fname << "------------------------------------------------" << endl;
+	}
+#else
+	Q_UNUSED(pcAddr);
+	Q_UNUSED(backupAddr);
+	Q_UNUSED(palmAddr);
 #endif
 }
 
@@ -662,24 +720,23 @@ void AbbrowserConduit::showAdresses(Addressee &pcAddr, PilotAddress *backupAddr,
 /* virtual */ bool AbbrowserConduit::exec()
 {
 	FUNCTIONSETUP;
-	DEBUGCONDUIT<<abbrowser_conduit_id<<endl;
-
-	if(!fConfig)
-	{
-		kdWarning() << k_funcinfo << ": No config file was set!" << endl;
-		emit logError(i18n("Unable to load configuration of the addressbook conduit."));
-		return false;
-	}
+#ifdef DEBUG
+	DEBUGCONDUIT << fname << id_conduit_address << endl;
+#endif
 
 	_prepare();
 
 	fFirstSync = false;
-	// Database names probably in latin1.
-	if(!openDatabases(QString::fromLatin1("AddressDB"), &fFirstSync))
+	if(!openDatabases(CSL1("AddressDB"), &fFirstSync))
 	{
 		emit logError(i18n("Unable to open the addressbook databases on the handheld."));
 		return false;
 	}
+
+#ifdef DEBUG
+	DEBUGCONDUIT << fname << ": First sync now " << fFirstSync << endl;
+#endif
+
 	_getAppInfo();
 	if(!_loadAddressBook())
 	{
@@ -688,6 +745,13 @@ void AbbrowserConduit::showAdresses(Addressee &pcAddr, PilotAddress *backupAddr,
 	}
 	fFirstSync = fFirstSync || (aBook->begin() == aBook->end());
 
+#ifdef DEBUG
+	DEBUGCONDUIT << fname << ": First sync now " << fFirstSync
+		<< " and addressbook is "
+		<< ((aBook->begin() == aBook->end()) ? "" : "non-")
+		<< "empty." << endl;
+#endif
+
 	// perform syncing from palm to abbrowser
 	// iterate through all records in palm pilot
 	pilotindex = 0;
@@ -695,10 +759,10 @@ void AbbrowserConduit::showAdresses(Addressee &pcAddr, PilotAddress *backupAddr,
 #ifdef DEBUG
 	DEBUGCONDUIT << fname << ": fullsync=" << isFullSync() << ", firstSync=" <<    isFirstSync() << endl;
 	DEBUGCONDUIT << fname << ": "
-		<< "syncDirection=" << fSyncDirection << ", "
-		<< "archive = " << fArchive << endl;
-	DEBUGCONDUIT << fname << ": conflictRes="<< fConflictResolution << endl;
-	DEBUGCONDUIT << fname << ": PilotStreetHome=" << fPilotStreetHome << ", PilotFaxHOme" << fPilotFaxHome << endl;
+		<< "syncDirection=" << getSyncDirection() << ", "
+		<< "archive = " << AbbrowserSettings::archiveDeleted() << endl;
+	DEBUGCONDUIT << fname << ": conflictRes="<< getConflictResolution() << endl;
+	DEBUGCONDUIT << fname << ": PilotStreetHome=" << AbbrowserSettings::pilotStreet() << ", PilotFaxHOme" << AbbrowserSettings::pilotFax() << endl;
 #endif
 
 	if (!isFirstSync())
@@ -723,17 +787,24 @@ void AbbrowserConduit::slotPalmRecToPC()
 	FUNCTIONSETUP;
 	PilotRecord *palmRec = 0L, *backupRec = 0L;
 
-	if (fSyncDirection==SyncAction::eCopyPCToHH)
+	if (getSyncDirection()==SyncAction::eCopyPCToHH)
 	{
+#ifdef DEBUG
+		DEBUGCONDUIT << fname << ": Done; change to PCtoHH phase." << endl;
+#endif
 		abiter = aBook->begin();
 		QTimer::singleShot(0, this, SLOT(slotPCRecToPalm()));
 		return;
 	}
 
 	if(isFullSync())
+	{
 		palmRec = fDatabase->readRecordByIndex(pilotindex++);
+	}
 	else
-		palmRec = dynamic_cast <PilotSerialDatabase * >(fDatabase)->readNextModifiedRec();
+	{
+		palmRec = fDatabase->readNextModifiedRec();
+	}
 
 	if(!palmRec)
 	{
@@ -743,14 +814,14 @@ void AbbrowserConduit::slotPalmRecToPC()
 	}
 
 	// already synced, so skip:
-	if(syncedIds.contains(palmRec->getID()))
+	if(syncedIds.contains(palmRec->id()))
 	{
 		KPILOT_DELETE(palmRec);
 		QTimer::singleShot(0, this, SLOT(slotPalmRecToPC()));
 		return;
 	}
 
-	backupRec = fLocalDatabase->readRecordById(palmRec->getID());
+	backupRec = fLocalDatabase->readRecordById(palmRec->id());
 	PilotRecord*compareRec=(backupRec)?(backupRec):(palmRec);
 	Addressee e = _findMatch(PilotAddress(fAddressAppInfo, compareRec));
 
@@ -761,7 +832,7 @@ void AbbrowserConduit::slotPalmRecToPC()
 
 	syncAddressee(e, backupAddr, palmAddr);
 
-	syncedIds.append(palmRec->getID());
+	syncedIds.append(palmRec->id());
 	KPILOT_DELETE(palmAddr);
 	KPILOT_DELETE(backupAddr);
 	KPILOT_DELETE(palmRec);
@@ -776,9 +847,12 @@ void AbbrowserConduit::slotPCRecToPalm()
 {
 	FUNCTIONSETUP;
 
-	if ( (fSyncDirection==SyncAction::eCopyHHToPC) ||
+	if ( (getSyncDirection()==SyncAction::eCopyHHToPC) ||
 		abiter == aBook->end() || (*abiter).isEmpty() )
 	{
+#ifdef DEBUG
+		DEBUGCONDUIT << fname << ": Done; change to delete records." << endl;
+#endif
 		pilotindex = 0;
 		QTimer::singleShot(0, this, SLOT(slotDeletedRecord()));
 		return;
@@ -806,6 +880,9 @@ void AbbrowserConduit::slotPCRecToPalm()
 	recordid_t rid = recID.toLong(&ok);
 	if (recID.isEmpty() || !ok || !rid)
 	{
+#ifdef DEBUG
+		DEBUGCONDUIT << fname << ": This is a new record." << endl;
+#endif
 		// it's a new item(no record ID and not inserted by the Palm -> PC sync), so add it
 		syncAddressee(ad, 0L, 0L);
 		QTimer::singleShot(0, this, SLOT(slotPCRecToPalm()));
@@ -830,15 +907,33 @@ void AbbrowserConduit::slotPCRecToPalm()
 	if (backupRec) backupAddr=new PilotAddress(fAddressAppInfo, backupRec);
 	if(!backupRec || isFirstSync() || !_equal(backupAddr, ad)  )
 	{
+#ifdef DEBUG
+		DEBUGCONDUIT << fname << ": Updating entry." << endl;
+#endif
 		palmRec = fDatabase->readRecordById(rid);
-		PilotAddress*palmAddr=0L;
-		if (palmRec) palmAddr= new PilotAddress(fAddressAppInfo, palmRec);
+		PilotAddress *palmAddr = 0L;
+		if (palmRec)
+		{
+			palmAddr = new PilotAddress(fAddressAppInfo, palmRec);
+		}
+		else
+		{
+#ifdef DEBUG
+		DEBUGCONDUIT << fname << ": No HH record with id " << rid << endl;
+#endif
+		}
 		syncAddressee(ad, backupAddr, palmAddr);
 		// update the id just in case it changed
-		if (palmRec) rid=palmRec->getID();
+		if (palmRec) rid=palmRec->id();
 		KPILOT_DELETE(palmRec);
 		KPILOT_DELETE(palmAddr);
 	}
+	else
+	{
+#ifdef DEBUG
+		DEBUGCONDUIT << fname << ": Entry not updated." << endl;
+#endif
+	}
 	KPILOT_DELETE(backupAddr);
 	KPILOT_DELETE(backupRec);
 	syncedIds.append(rid);
@@ -861,22 +956,22 @@ void AbbrowserConduit::slotDeletedRecord()
 	}
 
 	// already synced, so skip this record:
-	if(syncedIds.contains(backupRec->getID()))
+	if(syncedIds.contains(backupRec->id()))
 	{
 		KPILOT_DELETE(backupRec);
 		QTimer::singleShot(0, this, SLOT(slotDeletedRecord()));
 		return;
 	}
 
-	QString uid = addresseeMap[backupRec->getID()];
+	QString uid = addresseeMap[backupRec->id()];
 	Addressee e = aBook->findByUid(uid);
-	PilotRecord*palmRec=fDatabase->readRecordById(backupRec->getID());
+	PilotRecord*palmRec=fDatabase->readRecordById(backupRec->id());
 	PilotAddress*backupAddr=0L;
 	if (backupRec) backupAddr=new PilotAddress(fAddressAppInfo, backupRec);
 	PilotAddress*palmAddr=0L;
 	if (palmRec) palmAddr=new PilotAddress(fAddressAppInfo, palmRec);
 
-	syncedIds.append(backupRec->getID());
+	syncedIds.append(backupRec->id());
 	syncAddressee(e, backupAddr, palmAddr);
 
 	KPILOT_DELETE(palmAddr);
@@ -891,7 +986,7 @@ void AbbrowserConduit::slotDeletedRecord()
 void AbbrowserConduit::slotDeleteUnsyncedPCRecords()
 {
 	FUNCTIONSETUP;
-	if (fSyncDirection==SyncAction::eCopyHHToPC)
+	if (getSyncDirection()==SyncAction::eCopyHHToPC)
 	{
 		QStringList uids;
 		RecordIDList::iterator it;
@@ -925,7 +1020,7 @@ void AbbrowserConduit::slotDeleteUnsyncedPCRecords()
 void AbbrowserConduit::slotDeleteUnsyncedHHRecords()
 {
 	FUNCTIONSETUP;
-	if (fSyncDirection==SyncAction::eCopyPCToHH)
+	if (getSyncDirection()==SyncAction::eCopyPCToHH)
 	{
 		RecordIDList ids=fDatabase->idList();
 		RecordIDList::iterator it;
@@ -980,41 +1075,36 @@ bool AbbrowserConduit::syncAddressee(Addressee &pcAddr, PilotAddress*backupAddr,
 		PilotAddress*palmAddr)
 {
 	FUNCTIONSETUP;
-showAdresses(pcAddr, backupAddr, palmAddr);
+	showAdresses(pcAddr, backupAddr, palmAddr);
 
-	if (fSyncDirection==SyncAction::eCopyPCToHH)
+	if (getSyncDirection()==SyncAction::eCopyPCToHH)
 	{
 		if (pcAddr.isEmpty())
 		{
-#ifdef DEBUG
-			DEBUGCONDUIT<<"0a "<<endl;
-#endif
 			return _deleteAddressee(pcAddr, backupAddr, palmAddr);
 		}
 		else
 		{
-#ifdef DEBUG
-			DEBUGCONDUIT<<"0b "<<endl;
-#endif
 			return _copyToHH(pcAddr, backupAddr, palmAddr);
 		}
 	}
 
-	if (fSyncDirection==SyncAction::eCopyHHToPC)
+	if (getSyncDirection()==SyncAction::eCopyHHToPC)
 	{
-#ifdef DEBUG
-			DEBUGCONDUIT<<"0c "<<endl;
-#endif
 		if (!palmAddr)
+		{
 			return _deleteAddressee(pcAddr, backupAddr, palmAddr);
+		}
 		else
+		{
 			return _copyToPC(pcAddr, backupAddr, palmAddr);
+		}
 	}
 
 	if ( !backupAddr || isFirstSync() )
 	{
 #ifdef DEBUG
-			DEBUGCONDUIT<<"1"<<endl;
+			DEBUGCONDUIT<< fname << ": Special case: no backup." << endl;
 #endif
 		/*
 		Resolution matrix (0..does not exist, E..exists, D..deleted flag set, A..archived):
@@ -1185,15 +1275,15 @@ bool AbbrowserConduit::_copyToHH(Addressee &pcAddr, PilotAddress*backupAddr,
 	}
 	_copy(paddr, pcAddr);
 #ifdef DEBUG
-	DEBUGCONDUIT<<"palmAddr->id="<<paddr->getID()<<", pcAddr.ID="<<
+	DEBUGCONDUIT << fname << "palmAddr->id="<<paddr->id()<<", pcAddr.ID="<<
 		pcAddr.custom(appString, idString)<<endl;
 #endif
 
 	if(_savePalmAddr(paddr, pcAddr))
 	{
 #ifdef DEBUG
-		DEBUGCONDUIT<<"Vor _saveAbEntry, palmAddr->id="<<
-		paddr->getID()<<", pcAddr.ID="<<pcAddr.custom(appString, idString)<<endl;
+		DEBUGCONDUIT<< fname << "Vor _saveAbEntry, palmAddr->id="<<
+		paddr->id()<<", pcAddr.ID="<<pcAddr.custom(appString, idString)<<endl;
 #endif
 		_savePCAddr(pcAddr, backupAddr, paddr);
 	}
@@ -1246,25 +1336,25 @@ bool AbbrowserConduit::_deleteAddressee(Addressee &pcAddr, PilotAddress*backupAd
 
 	if (palmAddr)
 	{
-		if (!syncedIds.contains(palmAddr->getID())) syncedIds.append(palmAddr->getID());
+		if (!syncedIds.contains(palmAddr->id())) syncedIds.append(palmAddr->id());
 		palmAddr->makeDeleted();
 		PilotRecord *pilotRec = palmAddr->pack();
-		pilotRec->makeDeleted();
+		pilotRec->setDeleted();
 		pilotindex--;
 		fDatabase->writeRecord(pilotRec);
 		fLocalDatabase->writeRecord(pilotRec);
-		syncedIds.append(pilotRec->getID());
+		syncedIds.append(pilotRec->id());
 		KPILOT_DELETE(pilotRec);
 	}
 	else if (backupAddr)
 	{
-		if (!syncedIds.contains(backupAddr->getID())) syncedIds.append(backupAddr->getID());
+		if (!syncedIds.contains(backupAddr->id())) syncedIds.append(backupAddr->id());
 		backupAddr->makeDeleted();
 		PilotRecord *pilotRec = backupAddr->pack();
-		pilotRec->makeDeleted();
+		pilotRec->setDeleted();
 		pilotindex--;
 		fLocalDatabase->writeRecord(pilotRec);
-		syncedIds.append(pilotRec->getID());
+		syncedIds.append(pilotRec->id());
 		KPILOT_DELETE(pilotRec);
 	}
 	if (!pcAddr.isEmpty())
@@ -1292,15 +1382,19 @@ bool AbbrowserConduit::_savePalmAddr(PilotAddress *palmAddr, Addressee &pcAddr)
 	FUNCTIONSETUP;
 
 #ifdef DEBUG
-	DEBUGCONDUIT << "Saving to pilot " << palmAddr->id()
+	DEBUGCONDUIT << fname << ": Saving to pilot " << palmAddr->id()
 		<< " " << palmAddr->getField(entryFirstname)
 		<< " " << palmAddr->getField(entryLastname)<< endl;
 #endif
 
 	PilotRecord *pilotRec = palmAddr->pack();
+#ifdef DEBUG
+	DEBUGCONDUIT << fname << ": record with id=" << pilotRec->id()
+		<< " len=" << pilotRec->getLen() << endl;
+#endif
 	recordid_t pilotId = fDatabase->writeRecord(pilotRec);
 #ifdef DEBUG
-	DEBUGCONDUIT<<"PilotRec nach writeRecord ("<<pilotId<<": ID="<<pilotRec->getID()<<endl;
+	DEBUGCONDUIT << fname << ": Wrote "<<pilotId<<": ID="<<pilotRec->id()<<endl;
 #endif
 	fLocalDatabase->writeRecord(pilotRec);
 	KPILOT_DELETE(pilotRec);
@@ -1362,7 +1456,7 @@ int AbbrowserConduit::_compare(const QString & str1, const QString & str2) const
 bool AbbrowserConduit::_equal(const PilotAddress *piAddress, const Addressee &abEntry,
 	enum eqFlagsType flags) const
 {
-	FUNCTIONSETUP;
+	FUNCTIONSETUPL(8);
 	// empty records are never equal!
 	if (!piAddress) return false;
 	if (abEntry.isEmpty()) return false;
@@ -1376,7 +1470,7 @@ bool AbbrowserConduit::_equal(const PilotAddress *piAddress, const Addressee &ab
 			return false;
 		if(_compare(abEntry.givenName(), piAddress->getField(entryFirstname)))
 			return false;
-		if(_compare(abEntry.title(), piAddress->getField(entryTitle)))
+		if(_compare(abEntry.prefix(), piAddress->getField(entryTitle)))
 			return false;
 		if(_compare(abEntry.organization(), piAddress->getField(entryCompany)))
 			return false;
@@ -1458,7 +1552,7 @@ void AbbrowserConduit::_copy(PilotAddress *toPilotAddr, Addressee &fromAbEntry)
 	if(!fromAbEntry.additionalName().isEmpty()) firstAndMiddle += CSL1(" ") + fromAbEntry.additionalName();
 	toPilotAddr->setField(entryFirstname, firstAndMiddle);
 	toPilotAddr->setField(entryCompany, fromAbEntry.organization());
-	toPilotAddr->setField(entryTitle, fromAbEntry.title());
+	toPilotAddr->setField(entryTitle, fromAbEntry.prefix());
 	toPilotAddr->setField(entryNote, fromAbEntry.note());
 
 	// do email first, to ensure its gets stored
@@ -1529,7 +1623,7 @@ void AbbrowserConduit::_copy(Addressee &toAbEntry, PilotAddress *fromPiAddr)
 	toAbEntry.setFamilyName(fromPiAddr->getField(entryLastname));
 	toAbEntry.setGivenName(fromPiAddr->getField(entryFirstname));
 	toAbEntry.setOrganization(fromPiAddr->getField(entryCompany));
-	toAbEntry.setTitle(fromPiAddr->getField(entryTitle));
+	toAbEntry.setPrefix(fromPiAddr->getField(entryTitle));
 	toAbEntry.setNote(fromPiAddr->getField(entryNote));
 
 	// copy the phone stuff
@@ -1570,7 +1664,7 @@ void AbbrowserConduit::_copy(Addressee &toAbEntry, PilotAddress *fromPiAddr)
 	// pilot id may be zero(since it could be new) but couldn't hurt
 	// to even assign it to zero; let's us know what state the
 	// toAbEntry is in
-	toAbEntry.insertCustom(appString, idString, QString::number(fromPiAddr->getID()));
+	toAbEntry.insertCustom(appString, idString, QString::number(fromPiAddr->id()));
 
 
 	int cat = fromPiAddr->getCat();
@@ -1597,7 +1691,7 @@ void AbbrowserConduit::_copy(Addressee &toAbEntry, PilotAddress *fromPiAddr)
 	@returns string of the merged entries.
 */
 QString AbbrowserConduit::_smartMergeString(const QString &pc, const QString & backup,
-	const QString & palm, eConflictResolution confRes)
+	const QString & palm, ConflictResolution confRes)
 {
 	FUNCTIONSETUP;
 
@@ -1664,7 +1758,7 @@ bool AbbrowserConduit::_buildResolutionTable(ResolutionTable*tab, const Addresse
 	appendAddr(i18n("Last name"), pcAddr.familyName(), entryLastname);
 	appendAddr(i18n("First name"), pcAddr.givenName(), entryFirstname);
 	appendAddr(i18n("Organization"), pcAddr.organization(), entryCompany);
-	appendAddr(i18n("Title"), pcAddr.title(), entryTitle);
+	appendAddr(i18n("Title"), pcAddr.prefix(), entryTitle);
 	appendAddr(i18n("Note"), pcAddr.note(), entryNote);
 	appendAddr(i18n("Custom 1"), getCustomField(pcAddr, 0), entryCustom1);
 	appendAddr(i18n("Custom 2"), getCustomField(pcAddr, 1), entryCustom2);
@@ -1737,7 +1831,7 @@ bool AbbrowserConduit::_applyResolutionTable(ResolutionTable*tab, Addressee &pcA
 	SETFIELD(FamilyName, entryLastname);
 	SETFIELD(GivenName, entryFirstname);
 	SETFIELD(Organization, entryCompany);
-	SETFIELD(Title, entryTitle);
+	SETFIELD(Prefix, entryTitle);
 	SETFIELD(Note, entryNote);
 	SETCUSTOMFIELD(0, entryCustom1);
 	SETCUSTOMFIELD(1, entryCustom2);
@@ -1797,7 +1891,7 @@ bool AbbrowserConduit::_smartMergeTable(ResolutionTable*tab)
 	{
 		// try to merge the three strings
 		item->fResolved=_smartMergeString(item->fEntries[0],
-			item->fEntries[2], item->fEntries[1], fConflictResolution);
+			item->fEntries[2], item->fEntries[1], getConflictResolution());
 		// if a conflict occurred, set the default to something sensitive:
 		if (item->fResolved.isNull() && !(item->fEntries[0].isEmpty() &&
 			item->fEntries[1].isEmpty() && item->fEntries[2].isEmpty() ) )
