@@ -1,4 +1,4 @@
-/* pilotDOCEntry.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 2002 by Reinhold Kainhofer
 **
@@ -32,7 +32,7 @@
 
 
 static const char *pilotDOCEntry_id =
-	"$Id: pilotDOCEntry.cc,v 1.4 2003/03/08 01:43:05 waba Exp $";
+	"$Id: pilotDOCEntry.cc,v 1.6 2005/01/30 01:25:07 adridg Exp $";
 const int PilotDOCEntry::TEXT_SIZE = 4096;
 
 
@@ -77,7 +77,7 @@ PilotDOCEntry & PilotDOCEntry::operator =(const PilotDOCEntry & e)
 
 
 
-void *PilotDOCEntry::pack(void *buf, int *len)
+void *PilotDOCEntry::pack_(void *buf, int *len)
 {
 //      int len;
 	if (compress)
