@@ -1,4 +1,4 @@
-/* null-conduit.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 2000-2001 by Adriaan de Groot
 **
@@ -38,7 +38,7 @@
 //
 //
 static const char *null_conduit_id=
-	"$Id: null-conduit.cc,v 1.36 2003/07/04 15:31:00 adridg Exp $";
+	"$Id: null-conduit.cc,v 1.40 2004/11/17 09:59:43 adridg Exp $";
 
 // Only include what we really need:
 // First UNIX system stuff, then std C++,
@@ -53,7 +53,7 @@ static const char *null_conduit_id=
 #include "pilotSerialDatabase.h"
 #include "null-factory.h"
 #include "null-conduit.h"
-
+#include "nullSettings.h"
 
 // A conduit that does nothing has a very
 // simple constructor and destructor.
@@ -83,18 +83,7 @@ NullConduit::~NullConduit()
 	FUNCTIONSETUP;
 	DEBUGCONDUIT<<null_conduit_id<<endl;
 
-	if (!fConfig)
-	{
-		kdWarning() << k_funcinfo
-			<< ": No configuration set for NULL conduit."
-			<< endl;
-		return false;
-	}
-
-	fConfig->setGroup(NullConduitFactory::group);
-
-	bool r = fConfig->readBoolEntry(NullConduitFactory::failImmediately);
-	if (r)
+	if ( NullConduitSettings::failImmediately() )
 	{
 #ifdef DEBUG
 		DEBUGCONDUIT << fname << ": Config says to fail now." << endl;
@@ -103,7 +92,7 @@ NullConduit::~NullConduit()
 		return false;
 	}
 
-	QString m=fConfig->readEntry(NullConduitFactory::message);
+	QString m(NullConduitSettings::logMessage());
 	if (!m.isEmpty())
 	{
 		addSyncLogEntry(m);
