@@ -1,4 +1,4 @@
-/* todo-conduit.cc  Todo-Conduit for syncing KPilot and KOrganizer
+/* Todo-Conduit for syncing KPilot and KOrganizer
 **
 ** Copyright (C) 2002-2003 Reinhold Kainhofer
 ** Copyright (C) 1998-2001 Dan Pilone
@@ -32,42 +32,31 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-static const char *TodoConduit_id = "$Id: todo-conduit.cc,v 1.41 2004/01/15 09:58:49 kainhofe Exp $";
-
-#include <options.h>
-#include <unistd.h>
+#include "options.h"
 
 #include <qdatetime.h>
-#include <qtimer.h>
 #include <qtextcodec.h>
 
-#include <pilotUser.h>
-#include <kconfig.h>
-
 #include <libkcal/calendar.h>
 #include <libkcal/todo.h>
 
-
-/*
-** KDE 2.2 uses class KORecurrence in a different header file.
-*/
-#ifdef KDE2
-#define DateList_t QDateList
-#define DateListIterator_t QDateListIterator
-#else
-#define DateList_t KCal::DateList
-#define DateListIterator_t KCal::DateList::ConstIterator
-#endif
-
-#include <pilotSerialDatabase.h>
 #include <pilotLocalDatabase.h>
 
 #include "todo-conduit.moc"
+#include "vcalconduitSettings.h"
+#include "todo-factory.h"
 
 // define conduit versions, one for the version when categories were synced for the first time, and the current version number
 #define CONDUIT_VERSION_CATEGORYSYNC 10
 #define CONDUIT_VERSION 10
 
+extern "C"
+{
+long version_conduit_todo = KPILOT_PLUGIN_API;
+
+const char *id_conduit_todo = "$Id: todo-conduit.cc,v 1.57 2005/01/13 15:52:41 adridg Exp $";
+
+}
 
 
 TodoConduitPrivate::TodoConduitPrivate(KCal::Calendar *b) :
@@ -131,27 +120,29 @@ KCal::Incidence *TodoConduitPrivate::findIncidence(PilotAppCategory*tosearch)
 
 KCal::Incidence *TodoConduitPrivate::getNextIncidence()
 {
+	FUNCTIONSETUP;
 	if (reading) {
-                ++fAllTodosIterator;
-                if ( fAllTodosIterator == fAllTodos.end() ) return 0;
-        } else {
-	        reading=true;
-                fAllTodosIterator = fAllTodos.begin();
-        }
-        return *fAllTodosIterator;
+		++fAllTodosIterator;
+	} 
+	else {
+		reading=true;
+		fAllTodosIterator = fAllTodos.begin();
+	}
+	
+	return(fAllTodosIterator == fAllTodos.end()) ? 0L : *fAllTodosIterator;
 }
 
 
 
 KCal::Incidence *TodoConduitPrivate::getNextModifiedIncidence()
 {
-FUNCTIONSETUP;
+	FUNCTIONSETUP;
 	KCal::Todo*e=0L;
 	if (!reading)
 	{
 		reading=true;
-                fAllTodosIterator = fAllTodos.begin();
-                if ( fAllTodosIterator != fAllTodos.end() ) e=*fAllTodosIterator;
+		fAllTodosIterator = fAllTodos.begin();
+		if ( fAllTodosIterator != fAllTodos.end() ) e=*fAllTodosIterator;
 	}
 	else
 	{
@@ -160,15 +151,16 @@ FUNCTIONSETUP;
 	while (fAllTodosIterator != fAllTodos.end() &&
 		e && e->syncStatus()!=KCal::Incidence::SYNCMOD)
 	{
-		++fAllTodosIterator;
-		e=*fAllTodosIterator;
+		e = (++fAllTodosIterator != fAllTodos.end()) ? *fAllTodosIterator : 0L;
+
 #ifdef DEBUG
-if (e)
-DEBUGCONDUIT<< e->summary()<<" had SyncStatus="<<e->syncStatus()<<endl;
+	if(e)
+		DEBUGCONDUIT<< e->summary()<<" had SyncStatus="<<e->syncStatus()<<endl;
 #endif
+
 	}
-        if ( fAllTodosIterator == fAllTodos.end() ) return 0;
-	else return *fAllTodosIterator;
+
+	return (fAllTodosIterator == fAllTodos.end()) ? 0L : *fAllTodosIterator;
 }
 
 
@@ -183,11 +175,11 @@ TodoConduit::TodoConduit(KPilotDeviceLink *d,
 {
 	FUNCTIONSETUP;
 #ifdef DEBUG
-	DEBUGCONDUIT << TodoConduit_id << endl;
+	DEBUGCONDUIT << id_conduit_todo << endl;
 #endif
 	fConduitName=i18n("To-do");
         
-        (void) TodoConduit_id;
+        (void) id_conduit_todo;
 }
 
 
@@ -215,25 +207,16 @@ void TodoConduit::_getAppInfo()
 	FUNCTIONSETUP;
 	// get the address application header information
 	unsigned char *buffer =
-		new unsigned char[PilotTodoEntry::APP_BUFFER_SIZE];
-	int appLen = fDatabase->readAppBlock(buffer,PilotTodoEntry::APP_BUFFER_SIZE);
+		new unsigned char[PilotRecord::APP_BUFFER_SIZE];
+	int appLen = fDatabase->readAppBlock(buffer,PilotRecord::APP_BUFFER_SIZE);
 
 	unpack_ToDoAppInfo(&fTodoAppInfo, buffer, appLen);
 	delete[]buffer;
 	buffer = NULL;
 
 #ifdef DEBUG
-	DEBUGCONDUIT << fname << " lastUniqueId"
-		<< fTodoAppInfo.category.lastUniqueID << endl;
+	PilotAppCategory::dumpCategories(fTodoAppInfo.category);
 #endif
-	for (int i = 0; i < 16; i++)
-	{
-#ifdef DEBUG
-		DEBUGCONDUIT << fname << " cat " << i << " =" <<
-			fTodoAppInfo.category.name[i] << endl;
-#endif
-	}
-
 }
 
 
@@ -253,8 +236,9 @@ void TodoConduit::readConfig()
 	VCalConduitBase::readConfig();
 	// determine if the categories have ever been synce. Needed to prevent loosing the categories on the desktop.
 	// also use a full sync for the first time to make sure the palm categories are really transferred to the desktop
-	categoriesSynced = fConfig->readNumEntry("ConduitVersion", 0)>=CONDUIT_VERSION_CATEGORYSYNC;
-	if (!categoriesSynced & !isFullSync() ) fSyncDirection=SyncAction::eFullSync;
+	categoriesSynced = config()->conduitVersion()>=CONDUIT_VERSION_CATEGORYSYNC;
+	if (!categoriesSynced & !isFullSync() ) 
+		setSyncDirection(SyncAction::eFullSync);
 #ifdef DEBUG
 	DEBUGCONDUIT<<"categoriesSynced="<<categoriesSynced<<endl;
 #endif
@@ -266,9 +250,9 @@ void TodoConduit::postSync()
 {
 	FUNCTIONSETUP;
 	VCalConduitBase::postSync();
-	fConfig->setGroup(configGroup());
 	// after this successful sync the categories have been synced for sure
-	fConfig->writeEntry("ConduitVersion", CONDUIT_VERSION);
+	config()->setConduitVersion( CONDUIT_VERSION );
+	config()->writeConfig();
 	_setAppInfo();
 }
 
@@ -337,7 +321,7 @@ void TodoConduit::preRecord(PilotRecord*r)
 	if (!categoriesSynced && r)
 	{
 		const PilotAppCategory*de=newPilotEntry(r);
-		KCal::Incidence *e = fP->findIncidence(r->getID());
+		KCal::Incidence *e = fP->findIncidence(r->id());
 		setCategory(dynamic_cast<KCal::Todo*>(e), dynamic_cast<const PilotTodoEntry*>(de));
 	}
 }
@@ -373,7 +357,7 @@ QString TodoConduit::_getCat(const QStringList cats, const QString curr) const
 		}
 	}
 	// If we have a free label, return the first possible cat
-	QString lastName(fTodoAppInfo.category.name[15]);
+	QString lastName( QString::fromLatin1(fTodoAppInfo.category.name[15]) );
 	if (lastName.isEmpty()) return cats.first();
 	return QString::null;
 }
@@ -400,8 +384,9 @@ KCal::Todo *TodoConduit::incidenceFromRecord(KCal::Todo *e, const PilotTodoEntry
 		return NULL;
 	}
 
-	e->setOrganizer(fCalendar->getEmail());
-	e->setPilotId(de->getID());
+   // We don't want this, do we?
+//	e->setOrganizer(fCalendar->getEmail());
+	e->setPilotId(de->id());
 	e->setSyncStatus(KCal::Incidence::SYNCNONE);
 	e->setSecrecy(de->isSecret() ? KCal::Todo::SecrecyPrivate : KCal::Todo::SecrecyPublic);
 
@@ -430,6 +415,9 @@ KCal::Todo *TodoConduit::incidenceFromRecord(KCal::Todo *e, const PilotTodoEntry
 
 	// COMPLETED? //
 	e->setCompleted(de->getComplete());
+	if ( de->getComplete() && !e->hasCompletedDate() ) {
+		e->setCompleted( QDateTime::currentDateTime() );
+	}
 
 	e->setSummary(de->getDescription());
 	e->setDescription(de->getNote());
@@ -456,3 +444,8 @@ void TodoConduit::setCategory(KCal::Todo *e, const PilotTodoEntry *de)
 		}
 	}
 }
+
+VCalConduitSettings *TodoConduit::config()
+{ 
+  return ToDoConduitFactory::config(); 
+}
