@@ -1,4 +1,4 @@
-/* todo-factory.cc                      KPilot
+/* KPilot
 **
 ** Copyright (C) 2002-2003 Reinhold Kainhofer
 ** Copyright (C) 2001 by Dan Pilone
@@ -29,13 +29,12 @@
 
 #include "options.h"
 
-#include <kinstance.h>
 #include <kaboutdata.h>
 
 #include "todo-setup.h"
 #include "todo-conduit.h"
 #include "todo-factory.moc"
-
+#include "vcalconduitSettings.h"
 
 extern "C"
 {
@@ -47,6 +46,7 @@ void *init_conduit_todo()
 
 }
 
+VCalConduitSettings* ToDoConduitFactory::fConfig=0L;
 
 ToDoConduitFactory::ToDoConduitFactory(QObject *p, const char *n) :
 	VCalConduitFactoryBase(p,n)
@@ -55,9 +55,9 @@ ToDoConduitFactory::ToDoConduitFactory(QObject *p, const char *n) :
 
 	fInstance = new KInstance("todoconduit");
 	fAbout = new KAboutData("todoConduit",
-		I18N_NOOP("ToDo Conduit for KPilot"),
+		I18N_NOOP("To-do Conduit for KPilot"),
 		KPILOT_VERSION,
-		I18N_NOOP("Configures the ToDo Conduit for KPilot"),
+		I18N_NOOP("Configures the To-do Conduit for KPilot"),
 		KAboutData::License_GPL,
 		"(C) 2001, Adriaan de Groot\n(C) 2002-2003, Reinhold Kainhofer");
 	fAbout->addAuthor("Dan Pilone",
@@ -74,7 +74,6 @@ ToDoConduitFactory::ToDoConduitFactory(QObject *p, const char *n) :
 		I18N_NOOP("Maintainer"),
 		"reinhold@kainhofer.com",
 		"http://reinhold.kainhofer.com/Linux/");
-	group="todoOptions";
 }
 
 ToDoConduitFactory::~ToDoConduitFactory()
@@ -85,6 +84,15 @@ ToDoConduitFactory::~ToDoConduitFactory()
 	KPILOT_DELETE(fAbout);
 }
 
+VCalConduitSettings* ToDoConduitFactory::config()
+{
+	if (!fConfig) {
+ 		fConfig = new VCalConduitSettings( CSL1("ToDo") );
+ 		if (fConfig) fConfig->readConfig();
+	}
+	return fConfig;
+}
+
 /* virtual */ QObject *ToDoConduitFactory::createObject( QObject *p,
 	const char *n,
 	const char *c,
@@ -117,27 +125,7 @@ ToDoConduitFactory::~ToDoConduitFactory()
 			return 0L;
 		}
 	}
-
-	if (qstrcmp(c,"ConduitConfig")==0)
-	{
-		QWidget *w = dynamic_cast<QWidget *>(p);
-
-		if (w)
-		{
-			return new ConduitConfigImplementation(w,n,a,
-				ToDoWidgetSetup::create);
-		}
-		else
-		{
-#ifdef DEBUG
-			DEBUGCONDUIT << fname
-				<< ": Couldn't cast parent to widget."
-				<< endl;
-#endif
-			return 0L;
-		}
-	}
-
+	else
 	if (qstrcmp(c,"SyncAction")==0)
 	{
 		KPilotDeviceLink *d = dynamic_cast<KPilotDeviceLink *>(p);
