@@ -1,4 +1,4 @@
-/* vcal-conduit.cc                      KPilot
+/* KPilot
 **
 ** Copyright (C) 2002-2003 Reinhold Kainhofer
 ** Copyright (C) 2001 by Dan Pilone
@@ -27,40 +27,23 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-static const char *vcalconduit_id = "$Id: vcal-conduit.cc,v 1.96 2003/12/26 20:55:58 adridg Exp $";
-
-#include <options.h>
-#include <unistd.h>
-
-#include <qdatetime.h>
-#include <qtimer.h>
-
-#include <pilotUser.h>
-#include <kconfig.h>
-
-#include <libkcal/calendarlocal.h>
-
-
-/*
-** KDE 2.2 uses class KORecurrence in a different header file.
-*/
-#ifdef KDE2
-#include <korecurrence.h>
-#define Recurrence_t KCal::KORecurrence
-#define DateList_t QDateList
-#define DateListIterator_t QDateListIterator
-#else
+#include "options.h"
+#include <qtextcodec.h>
+#include <libkcal/calendar.h>
 #include <libkcal/recurrence.h>
 #define Recurrence_t KCal::Recurrence
-#define DateList_t KCal::DateList
-#define DateListIterator_t KCal::DateList::ConstIterator
-#endif
-
-#include <pilotSerialDatabase.h>
-#include <pilotLocalDatabase.h>
-
+#include <pilotDateEntry.h>
+#include <pilotDatabase.h>
 #include "vcal-conduit.moc"
+#include "vcal-factory.h"
 
+extern "C"
+{
+
+long version_conduit_vcal = KPILOT_PLUGIN_API;
+const char *id_conduit_vcal = "$Id: vcal-conduit.cc,v 1.108 2005/01/13 15:52:41 adridg Exp $";
+
+}
 
 
 
@@ -99,9 +82,9 @@ void VCalConduitPrivate::removeIncidence(KCal::Incidence *e)
 KCal::Incidence *VCalConduitPrivate::findIncidence(recordid_t id)
 {
 	KCal::Event::List::ConstIterator it;
-        for( it = fAllEvents.begin(); it != fAllEvents.end(); ++it ) {
+	for( it = fAllEvents.begin(); it != fAllEvents.end(); ++it ) {
 		KCal::Event *event = *it;
-                if ((recordid_t)event->pilotId() == id) return event;
+		if ((recordid_t)event->pilotId() == id) return event;
 	}
 	return 0L;
 }
@@ -115,7 +98,7 @@ KCal::Incidence *VCalConduitPrivate::findIncidence(PilotAppCategory*tosearch)
 	QDateTime dt=readTm( entry->getEventStart() );
 
 	KCal::Event::List::ConstIterator it;
-        for( it = fAllEvents.begin(); it != fAllEvents.end(); ++it ) {
+	for( it = fAllEvents.begin(); it != fAllEvents.end(); ++it ) {
 		KCal::Event *event = *it;
 		if ( (event->dtStart() == dt) && (event->summary() == title) ) return event;
 	}
@@ -127,14 +110,15 @@ KCal::Incidence *VCalConduitPrivate::findIncidence(PilotAppCategory*tosearch)
 KCal::Incidence *VCalConduitPrivate::getNextIncidence()
 {
 	FUNCTIONSETUP;
+
 	if (reading) {
-                ++fAllEventsIterator;
-                if ( fAllEventsIterator == fAllEvents.end() ) return 0;
-        } else {
-	        reading=true;
-                fAllEventsIterator = fAllEvents.begin();
-        }
-        return *fAllEventsIterator;
+		++fAllEventsIterator;
+	} else {
+		reading=true;
+		fAllEventsIterator = fAllEvents.begin();
+	}
+	// At end of list, or empty list.
+	return (fAllEventsIterator == fAllEvents.end()) ? 0L : *fAllEventsIterator;
 }
 
 /** Find the next incidence in the list which ddoes not have the SYNCNONE flag set. The
@@ -156,13 +140,12 @@ KCal::Incidence *VCalConduitPrivate::getNextModifiedIncidence()
 		++fAllEventsIterator;
 	}
 	while ( fAllEventsIterator != fAllEvents.end() &&
-		e && e->syncStatus()==KCal::Incidence::SYNCNONE)
+		e && e->syncStatus()==KCal::Incidence::SYNCNONE && e->pilotId() > 0)
 	{
 		++fAllEventsIterator;
 		e=*fAllEventsIterator;
 	}
-	if ( fAllEventsIterator == fAllEvents.end() ) return 0;
-	else return *fAllEventsIterator;
+	return (fAllEventsIterator == fAllEvents.end()) ? 0L : 	*fAllEventsIterator;
 }
 
 
@@ -177,10 +160,10 @@ VCalConduit::VCalConduit(KPilotDeviceLink *d,
 {
 	FUNCTIONSETUP;
 #ifdef DEBUG
-	DEBUGCONDUIT << vcalconduit_id << endl;
+	DEBUGCONDUIT << id_conduit_vcal << endl;
 #endif
 	fConduitName=i18n("Calendar");
-        (void) vcalconduit_id;
+	(void) id_conduit_vcal;
 }
 
 
@@ -193,6 +176,32 @@ VCalConduitPrivateBase* VCalConduit::newVCalPrivate(KCal::Calendar *fCalendar) {
 	return new VCalConduitPrivate(fCalendar);
 }
 
+void VCalConduit::_getAppInfo()
+{
+	FUNCTIONSETUP;
+	// get the address application header information
+	unsigned char *buffer =
+		new unsigned char[PilotDateEntry::APP_BUFFER_SIZE];
+	int appLen = fDatabase->readAppBlock(buffer,PilotDateEntry::APP_BUFFER_SIZE);
+
+	unpack_AppointmentAppInfo(&fAppointmentAppInfo, buffer, appLen);
+	delete[]buffer;
+	buffer = NULL;
+
+#ifdef DEBUG
+	DEBUGCONDUIT << fname << " lastUniqueId"
+		<< fAppointmentAppInfo.category.lastUniqueID << endl;
+#endif
+	for (int i = 0; i < 16; i++)
+	{
+#ifdef DEBUG
+		DEBUGCONDUIT << fname << " cat " << i << " =" <<
+			fAppointmentAppInfo.category.name[i] << endl;
+#endif
+	}
+
+}
+
 const QString VCalConduit::getTitle(PilotAppCategory*de)
 {
 	PilotDateEntry*d=dynamic_cast<PilotDateEntry*>(de);
@@ -234,6 +243,7 @@ PilotRecord*VCalConduit::recordFromIncidence(PilotDateEntry*de, const KCal::Even
 	setExceptions(de, e);
 	de->setDescription(e->summary());
 	de->setNote(e->description());
+	setCategory(de, e);
 DEBUGCONDUIT<<"-------- "<<e->summary()<<endl;
 	return de->pack();
 }
@@ -255,7 +265,8 @@ KCal::Event *VCalConduit::incidenceFromRecord(KCal::Event *e, const PilotDateEnt
 		return NULL;
 	}
 
-	e->setOrganizer(fCalendar->getEmail());
+   // We don't want this, do we?
+//	e->setOrganizer(fCalendar->getEmail());
 	e->setSyncStatus(KCal::Incidence::SYNCNONE);
 	e->setSecrecy(de->isSecret() ?
 		KCal::Event::SecrecyPrivate :
@@ -275,6 +286,9 @@ KCal::Event *VCalConduit::incidenceFromRecord(KCal::Event *e, const PilotDateEnt
 #endif
 	e->setDescription(de->getNote());
 
+	// used by e.g. Agendus and Datebk
+	setCategory(e, de);
+
 	return e;
 }
 
@@ -383,7 +397,7 @@ void VCalConduit::setAlarms(PilotDateEntry*de, const KCal::Event *e)
 	KCal::Alarm::List alms=e->alarms();
 	KCal::Alarm* alm=0;
 	KCal::Alarm::List::ConstIterator it;
-        for ( it = alms.begin(); it != alms.end(); ++it ) {
+	for ( it = alms.begin(); it != alms.end(); ++it ) {
 		if ((*it)->enabled()) alm=*it;
 	}
 
@@ -735,3 +749,69 @@ void VCalConduit::setExceptions(PilotDateEntry *dateEntry, const KCal::Event *ve
 	dateEntry->setExceptionCount(excount);
 	dateEntry->setExceptions(ex_List);
 }
+
+
+void VCalConduit::setCategory(PilotDateEntry *de, const KCal::Event *e)
+{
+	if (!de || !e) return;
+	de->setCategory(_getCat(e->categories(), de->getCategoryLabel()));
+}
+
+/**
+ * _getCat returns the id of the category from the given categories list. If none of the categories exist
+ * on the palm, the "Nicht abgelegt" (don't know the english name) is used.
+ */
+
+QString VCalConduit::_getCat(const QStringList cats, const QString curr) const
+{
+	int j;
+	if (cats.size()<1) return QString::null;
+	if (cats.contains(curr)) return curr;
+	for ( QStringList::ConstIterator it = cats.begin(); it != cats.end(); ++it ) {
+		for (j=1; j<=15; j++)
+		{
+			QString catName = PilotAppCategory::codec()->
+			  toUnicode(fAppointmentAppInfo.category.name[j]);
+			if (!(*it).isEmpty() && !(*it).compare( catName ) )
+			{
+				return catName;
+			}
+		}
+	}
+	// If we have a free label, return the first possible cat
+	QString lastName(QString::fromLatin1(fAppointmentAppInfo.category.name[15]));
+	if (lastName.isEmpty()) return cats.first();
+	return QString::null;
+}
+
+void VCalConduit::setCategory(KCal::Event *e, const PilotDateEntry *de)
+{
+	if (!e || !de) return;
+	QStringList cats=e->categories();
+	int cat=de->getCat();
+	if (0<cat && cat<=15)
+	{
+		QString newcat=PilotAppCategory::codec()->toUnicode(fAppointmentAppInfo.category.name[cat]);
+		if (!cats.contains(newcat))
+		{
+			cats.append( newcat );
+			e->setCategories(cats);
+		}
+	}
+}
+
+
+PilotAppCategory*VCalConduit::newPilotEntry(PilotRecord*r)
+{
+	if (r) return new PilotDateEntry(fAppointmentAppInfo,r);
+	else return new PilotDateEntry(fAppointmentAppInfo);
+}
+
+KCal::Incidence*VCalConduit::newIncidence()
+{
+  return new KCal::Event;
+}
+
+VCalConduitSettings *VCalConduit::config() {
+  return VCalConduitFactory::config();
+}
