@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* addressEditor.cc		KPilot
+/* KPilot
 **
 ** Copyright (C) 2000 by Dan Pilone
 **
@@ -52,7 +52,7 @@
 #include "addressEditor.moc"
 
 static const char *addressEditor_id =
-	"$Id: addressEditor.cc,v 1.15 2004/01/19 14:18:01 adridg Exp $";
+	"$Id: addressEditor.cc,v 1.16 2004/11/14 13:47:27 kainhofe Exp $";
 
 AddressEditor::AddressEditor(PilotAddress * p,
 	struct AddressAppInfo *appInfo,
