@@ -1,6 +1,7 @@
-/* addressWidget.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003 Reinhold Kainhofer <reinhold@kainhofer.com>
 ** Copyright (C) 2004 by Adriaan de Groot
 **
 ** This file defines the addressWidget, that part of KPilot that
@@ -28,7 +29,7 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 static const char *addresswidget_id =
-	"$Id: addressWidget.cc,v 1.70 2004/07/04 18:19:04 adridg Exp $";
+	"$Id: addressWidget.cc,v 1.74 2005/01/20 01:44:06 adridg Exp $";
 
 
 
@@ -283,6 +284,12 @@ void AddressWidget::setupWidget()
 		i18n("<qt>Delete the selected address from the address book.</qt>") :
 		i18n("<qt><i>Deleting is disabled by the 'internal editors' setting.</i></qt>") ;
 
+	button = new QPushButton(i18n("Export..."), this);
+	grid->addWidget(button, 3,1);
+	connect(button, SIGNAL(clicked()), this, SLOT(slotExport()));
+	QWhatsThis::add(button,
+		i18n("<qt>Export all addresses in the selected category to CSV format.</qt>") );
+
 	QWhatsThis::add(fDeleteButton,wt);
 }
 
@@ -319,7 +326,7 @@ void AddressWidget::updateWidget()
 
 			if (!title.isEmpty())
 			{
-				title.remove(QRegExp("\n.*"));
+				title.remove(QRegExp(CSL1("\n.*")));
 				PilotListItem *p = new PilotListItem(title,
 					listIndex,
 					fAddressList.current());
@@ -651,7 +658,7 @@ void AddressWidget::writeAddress(PilotAddress * which,
 	PilotRecord *pilotRec = which->pack();
 
 	myDB->writeRecord(pilotRec);
-	markDBDirty("AddressDB");
+	markDBDirty(CSL1("AddressDB"));
 	delete pilotRec;
 
 	// Clean up in the case that we allocated our own DB.
@@ -663,3 +670,68 @@ void AddressWidget::writeAddress(PilotAddress * which,
 	}
 }
 
+#define plu_quiet 1
+#include "pilot-addresses.c"
+
+void AddressWidget::slotExport()
+{
+	FUNCTIONSETUP;
+	
+	int currentCatID = findSelectedCategory(fCatList,
+		&(fAddressAppInfo.category));
+
+	QString saveFile = KFileDialog::getSaveFileName(
+		QString::null,
+		CSL1("*.csv|Comma Separated Values"),
+		this,
+		( (currentCatID==-1) ?
+			i18n("Export All Addresses") :
+			i18n("Export Address Category %1").arg(PilotAppCategory::codec()->toUnicode(fAddressAppInfo.category.name[currentCatID]))));
+
+	if (saveFile.isEmpty())
+	{
+#ifdef DEBUG
+		DEBUGKPILOT << fname << ": No save file selected." << endl;
+#endif
+		return;
+	}
+	if (QFile::exists(saveFile) &&
+		KMessageBox::warningContinueCancel(this,
+			i18n("The file <i>%1</i> exists. Overwrite?").arg(saveFile),
+			i18n("Overwrite File?"),
+			i18n("Overwrite"))!=KMessageBox::Continue)
+	{
+#ifdef DEBUG
+		DEBUGKPILOT << fname << ": Overwrite file canceled." << endl;
+#endif
+		return;
+	}
+
+	FILE *f = fopen(QFile::encodeName(saveFile),"w");
+	if (!f)
+	{
+		KMessageBox::sorry(this,
+			i18n("The file <i>%1</i> could not be opened for writing.").arg(saveFile));
+		return;
+	}
+	fAddressList.first();
+
+#ifdef DEBUG
+	DEBUGKPILOT << fname << ": Adding records..." << endl;
+#endif
+
+	while (fAddressList.current())
+	{
+		const PilotAddress *a = fAddressList.current();
+		if ((currentCatID == -1) ||
+			(a->getCat() == currentCatID))
+		{
+			write_record_CSV(f, &fAddressAppInfo, a->address(),
+				a->getAttrib(), a->getCat(), 0);
+		}
+		fAddressList.next();
+	}
+
+	fclose(f);
+}
+
