@@ -1,7 +1,8 @@
-/* conduitConfigDialog.cc                KPilot
+/* KPilot
 **
 ** Copyright (C) 2001 by Dan Pilone
 ** Copyright (C) 2002-2004 by Adriaan de Groot
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** This file defines a .ui-based configuration dialog for conduits.
 */
@@ -28,7 +29,7 @@
 */
 
 static const char *conduitconfigdialog_id =
-	"$Id: conduitConfigDialog.cc,v 1.54.2.1 2004/10/06 14:41:54 thorsen Exp $";
+	"$Id: conduitConfigDialog.cc,v 1.65 2005/02/06 15:46:21 adridg Exp $";
 
 #include "options.h"
 
@@ -43,7 +44,6 @@ static const char *conduitconfigdialog_id =
 #include <qvbox.h>
 #include <qsplitter.h>
 #include <qheader.h>
-#include <qlabel.h>
 #include <qtimer.h>
 
 #include <kservice.h>
@@ -56,7 +56,9 @@ static const char *conduitconfigdialog_id =
 #include <klibloader.h>
 #include <kseparator.h>
 #include <kconfigskeleton.h>
+#include <kdialogbase.h>
 
+#include "uiDialog.h"
 #include "plugin.h"
 #include "kpilotConfig.h"
 #include "kpilotConfigDialog.h"
@@ -71,14 +73,15 @@ static const char *conduitconfigdialog_id =
 #define CONDUIT_LIBRARY (3)
 #define CONDUIT_ORDER	(4)
 
+
 extern "C"
 {
-	KCModule *create_kpilotconfig( QWidget *parent, const char * )
+	KDE_EXPORT KCModule *create_kpilotconfig( QWidget *parent, const char * )
 	{
 		return new ConduitConfigWidget( parent, "kcmkpilotconfig" );
 	}
 
-	ConfigWizard *create_wizard(QWidget *parent, int m)
+	KDE_EXPORT ConfigWizard *create_wizard(QWidget *parent, int m)
 	{
 		return new ConfigWizard(parent,"Wizard", m);
 	}
@@ -230,7 +233,8 @@ ConduitConfigWidgetBase::ConduitConfigWidgetBase(QWidget *parent, const char *n)
 
 	// Create the title
 	QVBoxLayout *vbox = new QVBoxLayout(this, 0, KDialog::spacingHint());
-	fTitleText = new QLabel("Conduit Setup - Addressbook", this);
+	// String below is just to make space; no need to translate.
+	fTitleText = new QLabel(CSL1("Conduit Setup - Addressbook"), this);
 	QFont titleFont(fTitleText->font());
 	titleFont.setBold(true);
 	fTitleText->setFont(titleFont);
@@ -407,8 +411,8 @@ void ConduitConfigWidget::fillLists()
 
 #define IC(a,b,c) p = new KPilotCheckListItem(conduits,i18n(a),QCheckListItem::CheckBox); \
 	p->setText(CONDUIT_COMMENT,i18n(c)); \
-	p->setText(CONDUIT_LIBRARY,"internal_" b); \
-	p->setText(CONDUIT_DESKTOP,"internal_" b); \
+	p->setText(CONDUIT_LIBRARY,CSL1("internal_" b)); \
+	p->setText(CONDUIT_DESKTOP,CSL1("internal_" b)); \
 	if (potentiallyInstalled.findIndex(p->text(CONDUIT_DESKTOP))>=0) \
 		p->setOriginalState(true);
 
@@ -463,8 +467,11 @@ void ConduitConfigWidget::fillLists()
 
 static void dumpConduitInfo(const KLibrary *lib)
 {
+#ifdef DEBUG
+	FUNCTIONSETUP;
 	DEBUGKPILOT << "Plugin version = " << PluginUtility::pluginVersion(lib) << endl;
 	DEBUGKPILOT << "Plugin id      = " << PluginUtility::pluginVersionString(lib) << endl;
+#endif
 }
 
 static ConduitConfigBase *handleGeneralPages(QWidget *w, QListViewItem *p)
@@ -707,7 +714,7 @@ void ConduitConfigWidget::selected(QListViewItem *p)
 	// set the dialog title to the selected item
 	QListViewItem *pParent = p->parent();
 	QString title;
-	title = pParent ? pParent->text(CONDUIT_NAME) + " - " : "";
+	title = pParent ? pParent->text(CONDUIT_NAME) + CSL1(" - ") : QString() ;
 	title += p ? p->text(CONDUIT_NAME) : i18n("KPilot Setup");
 	fTitleText->setText(title);
 }
