@@ -1,4 +1,4 @@
-/* fileInstaller.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
 **
@@ -30,7 +30,7 @@
 */
 
 static const char *fileinstaller_id =
-	"$Id: fileInstaller.cc,v 1.15 2004/07/31 13:55:53 adridg Exp $";
+	"$Id: fileInstaller.cc,v 1.18 2005/01/13 15:52:41 adridg Exp $";
 
 
 #include "options.h"
@@ -92,7 +92,7 @@ void FileInstaller::clearPending()
 {
 	FUNCTIONSETUP;
 
-	if(!(s.endsWith("pdb", false) || s.endsWith("prc", false))) {
+	if(!(s.endsWith(CSL1(".pdb"), false) || s.endsWith(CSL1(".prc"), false))) {
 		KMessageBox::detailedSorry(w, i18n("Cannot install %1").arg(s),
 			i18n("Only PalmOS database files (like *.pdb and *.prc) can be installed by the file installer."));
 		return false;
@@ -103,7 +103,7 @@ void FileInstaller::clearPending()
 #endif
 
 	KURL srcName(s);
-	KURL destDir(fDirName + CSL1("/") + srcName.filename());
+	KURL destDir(fDirName + CSL1("/") + srcName.fileName());
 
 #if KDE_IS_VERSION(3,1,9)
 	return KIO::NetAccess::copy(srcName, destDir, w);
