@@ -1,4 +1,4 @@
-/* listCat.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 2000-2001 by Adriaan de Groot
 **
@@ -29,7 +29,7 @@
 */
 
 static const char *listCat_id =
-	"$Id: listCat.cc,v 1.9.4.2 2003/03/12 23:31:14 adridg Exp $";
+	"$Id: listCat.cc,v 1.12 2004/11/14 13:47:27 kainhofe Exp $";
 
 #include "options.h"
 
