@@ -3,10 +3,11 @@
 /* listCat.h			KPilot
 **
 ** Copyright (C) 2000-2001 by Adriaan de Groot
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** This is a specialization of KListView to allow the user to
 ** DnD a fixed set of objects into a fixed set of categories
-** (categories set at construction time). 
+** (categories set at construction time).
 */
 
 /*
@@ -22,7 +23,7 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 ** MA 02111-1307, USA.
 */
 
@@ -30,10 +31,8 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-#include <qstringlist.h>
-#include <qlistview.h>
-
 #include <klistview.h>
+class QStringList;
 
 /**
  * This Widget extends KListView for a particular purpose:
@@ -52,23 +51,23 @@
  * QListViewItem *stdKDE = lc->addCategory(i18n("Standard KDE"));
  * (void) new QListViewItem(stdKDE,i18n("red"),"#FF0000");
  * </PRE>
- * to set sensible column headers for a list of colors 
+ * to set sensible column headers for a list of colors
  * and their HTML equivalents (although why you would want
  * to categorize colors is beyond me).
  *
- * @version $Id: listCat.h,v 1.7.6.3 2003/03/20 21:08:27 adridg Exp $
+ * @version $Id: listCat.h,v 1.14 2005/02/06 22:48:34 winterz Exp $
  */
 
 class ListCategorizer : public KListView
 {
 	Q_OBJECT
-	
+
 public:
 	/**
 	 * Constructor.
 	 *
 	 * This creates a new empty ListCategorizer with
-	 * @ref startOpen set to false. The parameters
+	 * @see startOpen set to false. The parameters
 	 * @p parent and @p name are the usual Qt ones.
 	 */
 	ListCategorizer(QWidget *parent,
@@ -78,44 +77,44 @@ public:
 	 *
 	 * This creates a ListCategorizer with the given @p categories
 	 * already inserted. In addition, this constructor lets you
-	 * specify whether or not @ref startOpen is set.
+	 * specify whether or not @see startOpen is set.
 	 */
 	ListCategorizer(const QStringList& categories,
 		bool startOpen,
-		QWidget *parent, 
+		QWidget *parent,
 		const char *name = 0);
 
 	/**
 	 * Add a list of categories to the ListCategorizer.
 	 * All the categories are added without descriptions;
-	 * use @addCategory on a per-category basis for that.
+	 * use @see addCategory on a per-category basis for that.
 	 */
 	void addCategories(const QStringList&);
 	/**
 	 * Add a category with name @p name and optional
 	 * @p description. This can be useful if you want
 	 * either a description for the category or want to
-	 * refer to this category in the future without 
-	 * using @ref findCategory().
+	 * refer to this category in the future without
+	 * using @see findCategory().
 	 *
-	 * @ret the QListViewItem created for the category
+	 * @return the QListViewItem created for the category
 	 */
-	QListViewItem *addCategory(const QString& name, 
+	QListViewItem *addCategory(const QString& name,
 		const QString& description = QString::null);
 	/**
 	 * Returns the list of names of the categories in
 	 * the ListCategorizer.
 	 */
-	QStringList categories() const 
-	{ 
-		return listSiblings(firstChild()); 
+	QStringList categories() const
+	{
+		return listSiblings(firstChild());
 	} ;
 
 	/**
 	 * Add a single item to the category named @p category,
 	 * with name @p name and description set to @p description.
 	 * This might be a convenience function, but it's probably
-	 * more convenient to just use @ref QListViewItem's
+	 * more convenient to just use @see QListViewItem's
 	 * constructor. That way you can also hide more data in
 	 * the remaining columns.
 	 */
@@ -128,7 +127,7 @@ public:
 	 * the names of all the items categorized under a given
 	 * category, or, more usefully, set @p column to something
 	 * other that 0 (name) or 1 (description) to return the
-	 * @ref QStringList hidden in the non-visible columns.
+	 * @see QStringList hidden in the non-visible columns.
 	 */
 	QStringList items(const QString& category,int column=0) const
 	{
@@ -136,15 +135,15 @@ public:
 	}
 
 	/**
-	 * Given a category @categoryName return the @ref QListViewItem
+	 * Given a category @see categoryName return the @see QListViewItem
 	 * that represents that category. Probably a useless function,
-	 * since just remembering the pointer @ref addCategory gives
+	 * since just remembering the pointer @see addCategory gives
 	 * you is faster and uses hardly any memory.
 	 */
 	QListViewItem *findCategory(const QString& categoryName) const;
 	/**
 	 * Return the list of strings in column @column of all siblings
-	 * of the given item @p p. If you remembered a pointer to a 
+	 * of the given item @p p. If you remembered a pointer to a
 	 * category, you can use
 	 * <PRE>
 	 * QStringList l = lc->listSiblings(stdKDE->firstChild(),2);
@@ -186,7 +185,7 @@ protected:
 
 private:
 	/**
-	 * Call several @ref KListView functions to set up useful
+	 * Call several @see KListView functions to set up useful
 	 * behavior for this particular class.
 	 */
 	void setupWidget();
