@@ -20,7 +20,7 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 ** MA 02111-1307, USA.
 */
 
@@ -28,45 +28,28 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-#include <time.h>
-
-#ifndef _PILOT_MEMO_H_
 #include <pi-memo.h>
-#endif
-
-#ifndef KDE_VERSION
-#include <kdeversion.h>
-#endif
-
-#if KDE_VERSION >= 0x30100
-#include <ktextedit.h>
-#else
-#define KTextEdit QTextEdit
-#include <qtextedit.h>
-#endif
-
 #include <qptrlist.h>
 
+#include "pilotComponent.h"
+
 class KPilotInstaller;
 class QListBox;
 class QComboBox;
 class QPushButton;
+class KTextEdit;
 
 class PilotMemo;
 class PilotListItem;
 
-#ifndef _KPILOT_PILOTCOMPONENT_H
-#include "pilotComponent.h"
-#endif
-
 class MemoWidget : public PilotComponent
 {
 Q_OBJECT
-  
+
 public:
 	MemoWidget(QWidget* parent, const QString& dbpath);
 	virtual ~MemoWidget();
-  
+
 	// Pilot Component Methods:
 	/* virtual */ void showComponent();
 	/* virtual */ void hideComponent();
@@ -76,18 +59,19 @@ public:
 	// Added by David Bishop, please move to correct location!
 	bool saveAsXML(const QString &fileName,const QPtrList<PilotListItem> &menu_item );
 	bool saveAsText(const QString &fileName,const QPtrList<PilotListItem> &menu_item );
-	
-	typedef enum { 
-		MAX_MEMO_LEN = 8192 
+
+	typedef enum {
+		MAX_MEMO_LEN = 8192
 		} Constants ;
 
 protected:
 	void initializeCategories(PilotDatabase *);
 	void initializeMemos(PilotDatabase *);
 
-	// TODO: Ugh, I have no idea if this fix is in BRANCH as well.
 	void saveChangedMemo();
-  
+
+	bool addMemo(const QString &text, int category);
+
 public slots:
 	/**
 	* Called whenever the selected memo changes in order to:
@@ -98,25 +82,30 @@ public slots:
 	*/
 	void slotShowMemo(int);
 	void slotUpdateButtons();
-	// void slotTextChanged();
+
 	void slotImportMemo();
 	void slotExportMemo();
-	void slotDeleteMemo(); // Delets the currently selected memo(s)
+	void slotDeleteMemo();
+	void slotAddMemo();
 	void slotSetCategory(int);
 
+protected:
+	void showMemo(const PilotMemo *);
+
+
 private:
 	void setupWidget();
 	void updateWidget(); // Called with the lists have changed..
 	void writeMemo(PilotMemo* which);
 	QComboBox* fCatList;
-  
+
 	KTextEdit*		fTextWidget;
 	struct MemoAppInfo	fMemoAppInfo;
 	QPtrList<PilotMemo>	fMemoList;
 	QListBox*		fListBox;
 
 	QPushButton *fExportButton,*fDeleteButton;
-	
+
 	int lastSelectedMemo;
 };
 
