@@ -1,6 +1,7 @@
-/* pilotComponent.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** This file defines a base class for components -- internal conduits --
 ** in KPilot. This includes a number of general utility functions.
@@ -45,7 +46,7 @@
 #include "pilotComponent.moc"
 
 static const char *pilotComponent_id =
-	"$Id: pilotComponent.cc,v 1.30 2003/10/13 13:58:19 kainhofe Exp $";
+	"$Id: pilotComponent.cc,v 1.35 2004/11/14 13:47:27 kainhofe Exp $";
 
 // This is a pilot constant and should probably be defined
 // in a more sensible place but I'm lazy right now.
@@ -201,8 +202,7 @@ int PilotComponent::findSelectedCategory(QComboBox * fCatList,
 			kdWarning() << k_funcinfo
 				<< ": Selected category ("
 				<< selectedCategory
-				<< ") expands to "
-				<< qstringExpansion(selectedCategory) << endl;
+				<< ") " << endl;
 			kdWarning() << k_funcinfo
 				<< ": Categories expand to " << endl;
 #endif
@@ -215,8 +215,7 @@ int PilotComponent::findSelectedCategory(QComboBox * fCatList,
 					<< ": Category ["
 					<< currentCatID
 					<< "] = "
-					<< charExpansion(info->
-					name[currentCatID]) << endl;
+					<< (info->name[currentCatID]) << endl;
 #endif
 				currentCatID++;
 			}
@@ -239,7 +238,7 @@ void PilotComponent::populateCategories(QComboBox * c,
 #ifdef DEBUG
 	DEBUGKPILOT << fname
 		<< ": Combo box @"
-		<< (int) c << " and info @" << (int) info << endl;
+		<< (long) c << " and info @" << (long) info << endl;
 #endif
 
 	c->clear();
@@ -278,7 +277,7 @@ void PilotComponent::slotShowComponent()
 	FUNCTIONSETUP;
 
 #ifdef DEBUG
-	DEBUGKPILOT << fname << ": Showing component @" << (int) this << endl;
+	DEBUGKPILOT << fname << ": Showing component @" << (long) this << endl;
 #endif
 
 	emit showComponent(this);
@@ -294,9 +293,8 @@ void PilotComponent::slotShowComponent()
 void PilotComponent::markDBDirty(const QString db)
 {
 	FUNCTIONSETUP;
-	KPilotConfigSettings&c=KPilotConfig::getConfig();
-	c.setDatabaseGroup().addDirtyDatabase(db);
-	c.sync();
+	KPilotConfig::addDirtyDatabase(db);
+	KPilotConfig::sync();
 }
 
 void PilotComponent::showKPilotComponent( bool toShow )
