@@ -3,6 +3,7 @@
 /* pilotDaemonDCOP.h			KPilotDaemon
 **
 ** Copyright (C) 2000 by Adriaan de Groot
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** This file defines the DCOP interface for
 ** the KPilotDaemon. The daemon has *two* interfaces:
@@ -57,6 +58,7 @@ k_dcop:
 	*/
 	virtual ASYNC quitNow() = 0;
 	virtual ASYNC reloadSettings() = 0; // Indicate changed config file.
+	virtual ASYNC setTempDevice(QString d) =0; // use this device
 	virtual void stopListening() = 0;
 	virtual void startListening() = 0;
 	virtual bool isListening() =0 ;
@@ -85,6 +87,8 @@ k_dcop:
 
 k_dcop_signals:
 	void kpilotDaemonStatusChanged();
+	// used to push status information 
+	void kpilotDaemonStatusDetails( QDateTime, QString, QStringList, QString, QString, QString, bool );
 } ;
 
 #endif
