@@ -1,6 +1,7 @@
-/* syncStack.cc                       KPilot
+/* KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** This defines the "ActionQueue", which is the pile of actions
 ** that will occur during a HotSync.
@@ -28,22 +29,27 @@
 */
 #include "options.h"
 
-static const char *syncStack_id = "$Id: syncStack.cc,v 1.22 2003/12/04 21:10:13 adridg Exp $";
+static const char *syncStack_id = "$Id: syncStack.cc,v 1.37 2004/11/14 13:47:27 kainhofe Exp $";
 
 #include <unistd.h>
 
 #include <qtimer.h>
 #include <qfile.h>
+#include <qdir.h>
+#include <qtextcodec.h>
 
 #include <kservice.h>
 #include <kservicetype.h>
 #include <kuserprofile.h>
 #include <klibloader.h>
+#include <ksavefile.h>
 
 #include "pilotUser.h"
 #include "hotSync.h"
 #include "interactiveSync.h"
 #include "fileInstaller.h"
+#include "kpilotSettings.h"
+#include "pilotAppCategory.h"
 
 #include "syncStack.moc"
 
@@ -63,32 +69,118 @@ WelcomeAction::WelcomeAction(KPilotDeviceLink *p) :
 
 	addSyncLogEntry(i18n("KPilot %1 HotSync starting...\n")
 		.arg(QString::fromLatin1(KPILOT_VERSION)));
+	emit logMessage( i18n("Using encoding %1 on the handheld.").arg(PilotAppCategory::codecName()) );
 	emit syncDone(this);
 	return true;
 }
 
-SorryAction::SorryAction(KPilotDeviceLink *p) :
-	SyncAction(p,"sorryAction")
+SorryAction::SorryAction(KPilotDeviceLink *p, const QString &s) :
+	SyncAction(p,"sorryAction"),
+	fMessage(s)
 {
+	if (fMessage.isEmpty())
+	{
+		fMessage = i18n("KPilot is busy and cannot process the "
+			"HotSync right now.");
+	}
 }
 
 bool SorryAction::exec()
 {
 	FUNCTIONSETUP;
 
-	addSyncLogEntry(i18n("KPilot is busy and cannot process the "
-		"HotSync right now."));
+	addSyncLogEntry(fMessage);
 	return delayDone();
 }
 
-ConduitProxy::ConduitProxy(KPilotDeviceLink *p,
-	const QString &name,
-	int m) :
-	ConduitAction(p,name.latin1()),
-	fDesktopName(name),
-	fMode(m)
+LocalBackupAction::LocalBackupAction(KPilotDeviceLink *p, const QString &d) :
+	SyncAction(p,"LocalBackupAction"),
+	fDir(d)
+{
+}
+
+bool LocalBackupAction::exec()
 {
 	FUNCTIONSETUP;
+
+	startTickle();
+
+	QString dirname = fDir +
+		PilotAppCategory::codec()->toUnicode(fHandle->getPilotUser()->getUserName()) +
+		CSL1("/");
+	QDir dir(dirname,QString::null,QDir::Unsorted,QDir::Files);
+
+	if (!dir.exists())
+	{
+		emit logMessage( i18n("Cannot create local backup.") );
+		return false;
+	}
+
+	logMessage( i18n("Creating local backup of databases in %1.").arg(dirname) );
+	addSyncLogEntry( i18n("Creating local backup ..") );
+	qApp->processEvents();
+
+	QStringList files = dir.entryList();
+
+	for (QStringList::Iterator i = files.begin() ;
+		i != files.end();
+		++i)
+	{
+		KSaveFile::backupFile(dirname + (*i));
+	}
+
+	stopTickle();
+
+	return delayDone();
+}
+
+
+ConduitProxy::ConduitProxy(KPilotDeviceLink *p,
+	const QString &name,
+	SyncAction::SyncMode m,
+	bool local) :
+	ConduitAction(p,name.latin1()),
+	fDesktopName(name),
+	fMode(m),
+	fLocal(local)
+{
+	FUNCTIONSETUP;
+}
+
+/* static */ QStringList ConduitProxy::flagsForMode(SyncAction::SyncMode m)
+{
+	FUNCTIONSETUP;
+	QStringList l;
+	switch(m)
+	{
+	case eBackup :
+		l.append(CSL1("--backup"));
+		break;
+	case eTest:
+		l.append(CSL1("--test"));
+		break;
+	case eFastSync: /* FALLTHRU */
+	case eHotSync:
+		/* Nothing to do for fast or hotsync */
+		break;
+	case eFullSync:
+		l.append(CSL1("--full"));
+		break;
+	case eCopyHHToPC:
+		l.append(CSL1("--copyHHToPC"));
+		break;
+	case eCopyPCToHH:
+		l.append(CSL1("--copyPCToHH"));
+		break;
+	case eRestore:
+		kdWarning() << k_funcinfo << ": Running conduits during restore." << endl;
+		l.append(CSL1("--test"));
+		break;
+	case eDefaultSync:
+		Q_ASSERT(eDefaultSync != m); // Bail
+		break;
+	}
+	return l;
 }
 
 /* virtual */ bool ConduitProxy::exec()
@@ -103,7 +195,7 @@ ConduitProxy::ConduitProxy(KPilotDeviceLink *p,
 			<< ": Can't find desktop file for conduit "
 			<< fDesktopName
 			<< endl;
-		addSyncLogEntry(i18n("Couldn't find conduit %1.").arg(fDesktopName));
+		addSyncLogEntry(i18n("Could not find conduit %1.").arg(fDesktopName));
 		emit syncDone(this);
 		return true;
 	}
@@ -128,33 +220,22 @@ ConduitProxy::ConduitProxy(KPilotDeviceLink *p,
 			<< ": Can't load library "
 			<< o->library()
 			<< endl;
-		addSyncLogEntry(i18n("Couldn't load conduit %1.").arg(fDesktopName));
+		addSyncLogEntry(i18n("Could not load conduit %1.").arg(fDesktopName));
 		emit syncDone(this);
 		return true;
 	}
 
-	QStringList l;
-	switch(fMode & ActionQueue::ActionMask)
-	{
-	case ActionQueue::Backup :
-		l.append(CSL1("--backup"));
-		break;
-	default:
-		;
-	}
-	if (fMode & ActionQueue::FlagTest)
-		l.append(CSL1("--test"));
-	if (fMode & ActionQueue::FlagLocal)
-		l.append(CSL1("--local"));
-	// do a full sync also when changing PCs
-	if ( (fMode & ActionQueue::FlagFull) || 
-	     (fHandle->getPilotUser()->getLastSyncPC()!=(unsigned long)gethostid() ) )
-		l.append(CSL1("--full"));
-	if (fMode & ActionQueue::FlagHHToPC)
-		l.append(CSL1("--copyHHToPC"));
-	if (fMode & ActionQueue::FlagPCToHH)
-		l.append(CSL1("--copyPCToHH"));
+	QStringList l = flagsForMode(fMode);
 
+	if (fLocal)
+	{
+		l.append(CSL1("--local"));
+	}
+
+
+#ifdef DEBUG
+	DEBUGDAEMON << fname << ": Flags: " << fMode << endl;
+#endif
 
 	QObject *object = factory->create(fHandle,name(),"SyncAction",l);
 
@@ -163,7 +244,7 @@ ConduitProxy::ConduitProxy(KPilotDeviceLink *p,
 		kdWarning() << k_funcinfo
 			<< ": Can't create SyncAction."
 			<< endl;
-		addSyncLogEntry(i18n("Couldn't create conduit %1.").arg(fDesktopName));
+		addSyncLogEntry(i18n("Could not create conduit %1.").arg(fDesktopName));
 		emit syncDone(this);
 		return true;
 	}
@@ -175,25 +256,13 @@ ConduitProxy::ConduitProxy(KPilotDeviceLink *p,
 		kdWarning() << k_funcinfo
 			<< ": Can't cast to ConduitAction."
 			<< endl;
-		addSyncLogEntry(i18n("Couldn't create conduit %1.").arg(fDesktopName));
+		addSyncLogEntry(i18n("Could not create conduit %1.").arg(fDesktopName));
 		emit syncDone(this);
 		return true;
 	}
-	fConduit->setConfig(fConfig);
 
 	addSyncLogEntry(i18n("[Conduit %1]").arg(fDesktopName));
 
-	QString conduitFlags = TODO_I18N("Running with flags: ");
-	for (QStringList::ConstIterator i = l.begin() ; i!=l.end(); ++i)
-	{
-		conduitFlags.append(*i);
-		conduitFlags.append(CSL1("  "));
-	}
-
-	logMessage(conduitFlags);
-#ifdef DEBUG
-	DEBUGKPILOT<<conduitFlags<<endl;
-#endif
 	// Handle the syncDone signal properly & unload the conduit.
 	QObject::connect(fConduit,SIGNAL(syncDone(SyncAction *)),
 		this,SLOT(execDone(SyncAction *)));
@@ -217,7 +286,7 @@ void ConduitProxy::execDone(SyncAction *p)
 	{
 		kdError() << k_funcinfo
 			<< ": Unknown conduit @"
-			<< (int) p
+			<< (long) p
 			<< " finished."
 			<< endl;
 		emit syncDone(this);
@@ -225,43 +294,14 @@ void ConduitProxy::execDone(SyncAction *p)
 	}
 
 	delete p;
+	addSyncLogEntry(CSL1("\n"),false); // Put bits of the conduit logs on separate lines
 	emit syncDone(this);
 }
 
-#if 0
-ActionQueue::ActionQueue(KPilotDeviceLink *d,
-	KConfig *config,
-	const QStringList &conduits,
-	const QString &dir,
-	const QStringList &files) :
-	SyncAction(d,"ActionQueue"),
-	fReady(false),
-	fConfig(config),
-	fInstallerDir(dir),
-	fInstallerFiles(files),
-	fConduits(conduits)
-{
-	FUNCTIONSETUP;
-
-#ifdef DEBUG
-	if (!conduits.count())
-	{
-		DEBUGCONDUIT << fname << ": No conduits." << endl;
-	}
-	else
-	{
-		DEBUGCONDUIT << fname << ": Conduits : " << conduits.join(CSL1(" + ")) << endl;
-	}
-#endif
-
-	kdWarning() << "SyncStack usage is deprecated." << endl;
-}
-#endif
 
 ActionQueue::ActionQueue(KPilotDeviceLink *d) :
 	SyncAction(d,"ActionQueue"),
-	fReady(false),
-	fConfig(0L)
+	fReady(false)
 	// The string lists have default constructors
 {
 	FUNCTIONSETUP;
@@ -272,75 +312,20 @@ ActionQueue::~ActionQueue()
 	FUNCTIONSETUP;
 }
 
-#if 0
-void ActionQueue::prepare(int m)
-{
-	FUNCTIONSETUP;
 
-#ifdef DEBUG
-	DEBUGDAEMON << fname
-		<< ": Using sync mode " << m
-		<< endl;
-#endif
-
-	switch ( m & (Test | Backup | Restore | HotSync))
-	{
-	case Test:
-	case Backup:
-	case Restore:
-	case HotSync:
-		fReady=true;
-		break;
-	default:
-		kdWarning() << k_funcinfo
-			<< ": Strange sync mode " << m << " set. Aborting."
-			<< endl;
-		return;
-	}
-
-	queueInit(m);
-	if (m & WithConduits)
-		queueConduits(fConfig,fConduits,m);
-
-	switch ( m & (Test | Backup | Restore | HotSync))
-	{
-	case Test:
-		addAction(new TestLink(fHandle));
-		break;
-	case Backup:
-		addAction(new BackupAction(fHandle));
-		break;
-	case Restore:
-		addAction(new RestoreAction(fHandle));
-		break;
-	case HotSync:
-		break;
-	default:
-		// We already checked for this case!
-		fReady=false;
-		return;
-	}
-
-	if (m & WithInstaller)
-		queueInstaller(fInstallerDir,fInstallerFiles);
-
-	queueCleanup();
-}
-#endif
-
-void ActionQueue::queueInit(int m)
+void ActionQueue::queueInit(bool checkUser)
 {
 	FUNCTIONSETUP;
 
 	addAction(new WelcomeAction(fHandle));
 
-	if (m & WithUserCheck)
+	if (checkUser)
 	{
 		addAction(new CheckUser(fHandle));
 	}
 }
 
-void ActionQueue::queueConduits(KConfig *config,const QStringList &l,int m)
+void ActionQueue::queueConduits(const QStringList &l,SyncAction::SyncMode m, bool /*local*/)
 {
 	FUNCTIONSETUP;
 
@@ -354,20 +339,24 @@ void ActionQueue::queueConduits(KConfig *config,const QStringList &l,int m)
 		if ((*it).startsWith(CSL1("internal_")))
 		{
 #ifdef DEBUG
-			DEBUGDAEMON << k_funcinfo <<
-				"Ignoring conduit " << *it << endl;
+			DEBUGDAEMON << fname <<
+				": Ignoring conduit " << *it << endl;
 #endif
 			continue;
 		}
+
+#ifdef DEBUG
+		DEBUGDAEMON << fname
+			<< ": Creating proxy with mode=" << m << endl;
+#endif
 		ConduitProxy *cp = new ConduitProxy(fHandle,*it,m);
-		cp->setConfig(config);
 		addAction(cp);
 	}
 }
 
-void ActionQueue::queueInstaller(const QString &dir, const QStringList &files)
+void ActionQueue::queueInstaller(const QString &dir)
 {
-	addAction(new FileInstallAction(fHandle,dir,files));
+	addAction(new FileInstallAction(fHandle,dir));
 }
 
 void ActionQueue::queueCleanup()
@@ -398,7 +387,19 @@ void ActionQueue::actionCompleted(SyncAction *b)
 
 	if (isEmpty())
 	{
-		emit syncDone(this);
+		delayDone();
+		return;
+	}
+	if (!fHandle->tickle())
+	{
+		emit logError(i18n("The connection to the handheld "
+			"was lost. Synchronization cannot continue."));
+		SyncAction *del = 0L;
+		while ( (del = nextAction()) )
+		{
+			delete del;
+		}
+		delayDone();
 		return;
 	}
 
