@@ -1,6 +1,7 @@
-/* kpilotlink.cc			KPilot
+/* KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 */
 
@@ -24,22 +25,11 @@
 /*
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
-static const char *kpilotlink_id = "$Id: kpilotlink.cc,v 1.43 2003/08/03 11:15:59 kainhofe Exp $";
+static const char *kpilotlink_id = "$Id: kpilotlink.cc,v 1.84 2005/01/13 15:52:42 adridg Exp $";
 
 #include "options.h"
 
-#include <pi-source.h>
-#include <pi-socket.h>
-#include <pi-dlp.h>
-#include <pi-file.h>
-#include <pi-version.h>
 
-#ifndef PILOT_LINK_VERSION
-#error "You need at least pilot-link version 0.9.5"
-#endif
-
-#define PILOT_LINK_NUMBER	((100*PILOT_LINK_VERSION) + \
-				PILOT_LINK_MAJOR)
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -50,35 +40,99 @@ static const char *kpilotlink_id = "$Id: kpilotlink.cc,v 1.43 2003/08/03 11:15:5
 
 #include <iostream>
 
+#include <pi-source.h>
+#include <pi-socket.h>
+#include <pi-dlp.h>
+#include <pi-file.h>
+#if !(PILOT_LINK_NUMBER < PILOT_LINK_0_12_0)
+#include <pi-buffer.h>
+#endif
+
 #include <qdir.h>
 #include <qtimer.h>
 #include <qdatetime.h>
 #include <qsocketnotifier.h>
+#include <qthread.h>
+#include <qtextcodec.h>
 
 #include <kconfig.h>
 #include <kmessagebox.h>
+#include <kstandarddirs.h>
 
 #include "pilotUser.h"
 #include "pilotSysInfo.h"
 #include "pilotCard.h"
+#include "pilotAppCategory.h"
 
 #include "kpilotlink.moc"
 
 
+// singleton helper class
+class KPilotDeviceLink::KPilotDeviceLinkPrivate
+{
+public:
+	static KPilotDeviceLinkPrivate*self()
+	{
+		if (!mThis) mThis = new KPilotDeviceLinkPrivate();
+		return mThis;
+	}
 
-KPilotDeviceLink *KPilotDeviceLink::fDeviceLink = 0L;
+	bool canBind( QString device )
+	{
+		showList();
+		return !mBoundDevices.contains( device );
+	}
 
-KPilotDeviceLink::KPilotDeviceLink(QObject * parent, const char *name) :
+	void bindDevice( QString device )
+	{
+		mBoundDevices.append( device );
+		showList();
+	}
+
+	void unbindDevice( QString device )
+	{
+		mBoundDevices.remove( device );
+		showList();
+	}
+
+protected:
+	KPilotDeviceLinkPrivate() {}
+	~KPilotDeviceLinkPrivate() {}
+
+	QStringList mBoundDevices;
+	static KPilotDeviceLinkPrivate*mThis;
+
+private:
+	inline void showList() const
+	{
+#ifdef DEBUG
+		FUNCTIONSETUPL(3);
+		DEBUGDAEMON << fname << "Bound devices: "
+			<< ((mBoundDevices.count() > 0) ? mBoundDevices.join(CSL1(", ")) : CSL1("<none>")) << endl;
+#endif
+	}
+} ;
+
+KPilotDeviceLink::KPilotDeviceLinkPrivate *KPilotDeviceLink::KPilotDeviceLinkPrivate::mThis = 0L;
+
+
+KPilotDeviceLink::KPilotDeviceLink(QObject * parent, const char *name, const QString &tempDevice) :
 	QObject(parent, name),
-	fStatus(Init),
+	fLinkStatus(Init),
+	fTickleDone(true),
+	fTickleThread(0L),
+	fWorkaroundUSB(false),
+	fWorkaroundUSBTimer(0L),
 	fPilotPath(QString::null),
-	fDeviceType(None),
 	fRetries(0),
 	fOpenTimer(0L),
 	fSocketNotifier(0L),
 	fSocketNotifierActive(false),
 	fPilotMasterSocket(-1),
-	fCurrentPilotSocket(-1)
+	fCurrentPilotSocket(-1),
+	fTempDevice(tempDevice),
+	fPilotUser(0L),
+	fPilotSysInfo(0L)
 {
 	FUNCTIONSETUP;
 
@@ -88,8 +142,6 @@ KPilotDeviceLink::KPilotDeviceLink(QObject * parent, const char *name) :
 		<< endl;
 #endif
 
-	Q_ASSERT(fDeviceLink == 0L);
-	fDeviceLink = this;
 	messagesMask=0xffffffff;
 
 	(void) kpilotlink_id;
@@ -99,22 +151,16 @@ KPilotDeviceLink::~KPilotDeviceLink()
 {
 	FUNCTIONSETUP;
 	close();
-	fDeviceLink = 0L;
-}
-
-KPilotDeviceLink *KPilotDeviceLink::init(QObject * parent, const char *name)
-{
-	FUNCTIONSETUP;
-
-	Q_ASSERT(!fDeviceLink);
-
-	return new KPilotDeviceLink(parent, name);
+	KPILOT_DELETE(fWorkaroundUSBTimer);
+	KPILOT_DELETE(fPilotSysInfo);
+	KPILOT_DELETE(fPilotUser);
 }
 
 void KPilotDeviceLink::close()
 {
 	FUNCTIONSETUP;
 
+	KPILOT_DELETE(fWorkaroundUSBTimer);
 	KPILOT_DELETE(fOpenTimer);
 	KPILOT_DELETE(fSocketNotifier);
 	fSocketNotifierActive=false;
@@ -138,35 +184,42 @@ void KPilotDeviceLink::close()
 		pi_close(fPilotMasterSocket);
 		::close(fPilotMasterSocket);
 	}
+	KPilotDeviceLinkPrivate::self()->unbindDevice( fRealPilotPath );
 	fPilotMasterSocket = (-1);
 	fCurrentPilotSocket = (-1);
 }
 
-void KPilotDeviceLink::reset(DeviceType t, const QString & dP)
+void KPilotDeviceLink::reset(const QString & dP)
 {
 	FUNCTIONSETUP;
 
-	fStatus = Init;
+	fLinkStatus = Init;
 	fRetries = 0;
 
 	// Release all resources
 	//
-	//
 	close();
 	fPilotPath = QString::null;
 
-	fDeviceType = t;
-	if (t == None)
-		return;
-	fDeviceType=OldStyleUSB;
-
 	fPilotPath = dP;
+	if (fPilotPath.isEmpty())
+	   fPilotPath = fTempDevice;
 	if (fPilotPath.isEmpty())
 		return;
 
 	reset();
 }
 
+static inline void startOpenTimer(KPilotDeviceLink *dev, QTimer *&t)
+{
+	if ( !t ){
+		t = new QTimer(dev);
+		QObject::connect(t, SIGNAL(timeout()),
+			dev, SLOT(openDevice()));
+	}
+	t->start(1000, false);
+}
+
 void KPilotDeviceLink::reset()
 {
 	FUNCTIONSETUP;
@@ -177,12 +230,9 @@ void KPilotDeviceLink::reset()
 	checkDevice();
 
 	// Timer already deleted by close() call.
-	fOpenTimer = new QTimer(this);
-	QObject::connect(fOpenTimer, SIGNAL(timeout()),
-		this, SLOT(openDevice()));
-	fOpenTimer->start(1000, false);
+	startOpenTimer(this,fOpenTimer);
 
-	fStatus = WaitingForDevice;
+	fLinkStatus = WaitingForDevice;
 }
 
 void KPilotDeviceLink::checkDevice()
@@ -206,12 +256,17 @@ void KPilotDeviceLink::checkDevice()
 		// It doesn't exist, mention this in the log
 		// (relevant as long as we use only one device type)
 		//
-		emit logError(i18n("Pilot device %1 doesn't exist. "
-			"Assuming the device uses DevFS.")
+		emit logError(i18n("Pilot device %1 does not exist. "
+			"Probably it is a USB device and will appear during a HotSync.")
 				.arg(fPilotPath));
 	}
 }
 
+void KPilotDeviceLink::setTempDevice( const QString &d )
+{
+	fTempDevice = d;
+	KPilotDeviceLinkPrivate::self()->bindDevice( fTempDevice );
+}
 
 void KPilotDeviceLink::openDevice()
 {
@@ -220,9 +275,9 @@ void KPilotDeviceLink::openDevice()
 	// This transition (from Waiting to Found) can only be
 	// taken once.
 	//
-	if (fStatus == WaitingForDevice)
+	if (fLinkStatus == WaitingForDevice)
 	{
-		fStatus = FoundDevice;
+		fLinkStatus = FoundDevice;
 	}
 
 	shouldPrint(OpenMessage,i18n("Trying to open device %1...")
@@ -232,24 +287,27 @@ void KPilotDeviceLink::openDevice()
 	{
 		emit logMessage(i18n("Device link ready."));
 	}
+	else if (open(fTempDevice))
+	{
+		emit logMessage(i18n("Device link ready."));
+	}
 	else
 	{
 		shouldPrint(OpenFailMessage,i18n("Could not open device: %1 "
 				"(will retry)").
 				arg(fPilotPath));
 
-		if (fStatus != PilotLinkError)
+		if (fLinkStatus != PilotLinkError)
 		{
-			fOpenTimer->start(1000, false);
+			startOpenTimer(this,fOpenTimer);
 		}
 	}
 }
 
-bool KPilotDeviceLink::open()
+bool KPilotDeviceLink::open(QString device)
 {
 	FUNCTIONSETUPL(2);
 
-	struct pi_sockaddr addr;
 	int ret;
 	int e = 0;
 	QString msg;
@@ -262,23 +320,37 @@ bool KPilotDeviceLink::open()
 	}
 	fCurrentPilotSocket = (-1);
 
+	if ( device.isEmpty() )
+	{
+		device = pilotPath();
+	}
+	if (device.isEmpty())
+	{
+		kdWarning() << k_funcinfo
+			<< ": No point in trying empty device."
+			<< endl;
+
+		msg = i18n("The Pilot device is not configured yet.");
+		e = 0;
+		goto errInit;
+	}
+	fRealPilotPath = KStandardDirs::realPath ( device );
+
+	if ( !KPilotDeviceLinkPrivate::self()->canBind( fRealPilotPath ) ) {
+		msg = i18n("Already listening on that device");
+		e=0;
+		kdWarning() << k_funcinfo <<": Pilot Path " << pilotPath().latin1() << " already connected." << endl;
+		goto errInit;
+	}
+
+
 	if (fPilotMasterSocket == -1)
 	{
-		if (fPilotPath.isEmpty())
-		{
-			kdWarning() << k_funcinfo
-				<< ": No point in trying empty device."
-				<< endl;
-
-			msg = i18n("The Pilot device is not configured yet.");
-			e = 0;
-			goto errInit;
-		}
 #ifdef DEBUG
-		DEBUGDAEMON << fname << ": Typing to open " << fPilotPath << endl;
+		DEBUGDAEMON << fname << ": Typing to open " << fRealPilotPath << endl;
 #endif
 
-#if PILOT_LINK_NUMBER < 10
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_10_0
 		fPilotMasterSocket = pi_socket(PI_AF_SLP,
 			PI_SOCK_STREAM, PI_PF_PADP);
 #else
@@ -299,50 +371,76 @@ bool KPilotDeviceLink::open()
 			<< ": Got master " << fPilotMasterSocket << endl;
 #endif
 
-		fStatus = CreatedSocket;
+		fLinkStatus = CreatedSocket;
 	}
 
-	Q_ASSERT(fStatus == CreatedSocket);
+	Q_ASSERT(fLinkStatus == CreatedSocket);
 
 #ifdef DEBUG
 	DEBUGDAEMON << fname << ": Binding to path " << fPilotPath << endl;
 #endif
 
-#if PILOT_LINK_NUMBER < 10
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_12_0
+	struct pi_sockaddr addr;
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_10_0
 	addr.pi_family = PI_AF_SLP;
 #else
 	addr.pi_family = PI_AF_PILOT;
 #endif
-	strncpy(addr.pi_device, QFile::encodeName(fPilotPath),sizeof(addr.pi_device));
-
+	strlcpy(addr.pi_device, QFile::encodeName(device),sizeof(addr.pi_device));
 
 	ret = pi_bind(fPilotMasterSocket,
 		(struct sockaddr *) &addr, sizeof(addr));
+#else
+	ret = pi_bind(fPilotMasterSocket, QFile::encodeName(device));
+#endif
 
 	if (ret >= 0)
 	{
-		fStatus = DeviceOpen;
+		fLinkStatus = DeviceOpen;
 		fOpenTimer->stop();
 
+		KPilotDeviceLinkPrivate::self()->bindDevice( fRealPilotPath );
 		fSocketNotifier = new QSocketNotifier(fPilotMasterSocket,
 			QSocketNotifier::Read, this);
 		QObject::connect(fSocketNotifier, SIGNAL(activated(int)),
 			this, SLOT(acceptDevice()));
 		fSocketNotifierActive=true;
+		
+		if (fWorkaroundUSB)
+		{
+#ifdef DEBUG
+			DEBUGDAEMON << fname << ": Adding Z31 workaround." << endl;
+#endif
+			// Special case for Zire 31, 72, Tungsten T5,
+			// all of which may make a non-HotSync connection
+			// to the PC. Must detect this and bail quickly.
+			//
+			fWorkaroundUSBTimer = new QTimer(this);
+			connect(fWorkaroundUSBTimer,SIGNAL(timeout()),
+				this,SLOT(workaroundUSB()));
+			fWorkaroundUSBTimer->start(5000,true);
+		}
+		
 		return true;
 	}
 	else
 	{
 #ifdef DEBUG
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_12_0
 		DEBUGDAEMON << fname
 			<< ": Tried "
 			<< addr.pi_device
 			<< " and got "
 			<< strerror(errno)
 			<< endl;
+#else
+		DEBUGDAEMON << fname
+			<< ": Tried " << device << " and got " << strerror(errno) << endl;
+#endif
 #endif
 
-		if (isTransient() && (fRetries < 5))
+		if (fRetries < 5)
 		{
 			return false;
 		}
@@ -383,7 +481,7 @@ errInit:
 		msg += i18n(" These is no such device.");
 		break;
 	case EPERM:
-		msg += i18n(" You don't have permission to open the "
+		msg += i18n(" You do not have permission to open the "
 			"Pilot device.");
 		break;
 	default:
@@ -402,7 +500,7 @@ errInit:
 			<< ": (" << strerror(e) << ")" << endl;
 	}
 
-	fStatus = PilotLinkError;
+	fLinkStatus = PilotLinkError;
 	emit logError(msg);
 	return false;
 }
@@ -436,6 +534,8 @@ void KPilotDeviceLink::acceptDevice()
 	}
 
 #ifdef DEBUG
+	DEBUGDAEMON << fname
+		<< ": Found connection on device "<<pilotPath().latin1()<<endl;
 	DEBUGDAEMON << fname
 		<< ": Current status "
 		<< statusString()
@@ -451,13 +551,15 @@ void KPilotDeviceLink::acceptDevice()
 
 		// Presumably, strerror() returns things in
 		// local8Bit and not latin1.
-		emit logError(i18n("Can't listen on Pilot socket (%1)").
+		emit logError(i18n("Cannot listen on Pilot socket (%1)").
 			arg(QString::fromLocal8Bit(s)));
 
 		close();
 		return;
 	}
 
+	KPILOT_DELETE(fWorkaroundUSBTimer);
+
 	emit logProgress(QString::null,10);
 
 	fCurrentPilotSocket = pi_accept(fPilotMasterSocket, 0, 0);
@@ -467,33 +569,34 @@ void KPilotDeviceLink::acceptDevice()
 
 		kdWarning() << "pi_accept: " << s << endl;
 
-		emit logError(i18n("Can't accept Pilot (%1)")
+		emit logError(i18n("Cannot accept Pilot (%1)")
 			.arg(QString::fromLocal8Bit(s)));
 
-		fStatus = PilotLinkError;
+		fLinkStatus = PilotLinkError;
 		close();
 		return;
 	}
 
-	if ((fStatus != DeviceOpen) || (fPilotMasterSocket == -1))
+	if ((fLinkStatus != DeviceOpen) || (fPilotMasterSocket == -1))
 	{
-		fStatus = PilotLinkError;
+		fLinkStatus = PilotLinkError;
 		kdError() << k_funcinfo
 			<< ": Already connected or unable to connect!"
 			<< endl;
-		emit logError(TODO_I18N("Can't accept Pilot (%1)")
-			.arg(TODO_I18N("already connected")));
+		emit logError(i18n("Cannot accept Pilot (%1)")
+			.arg(i18n("already connected")));
 		close();
 		return;
 	}
 
 	emit logProgress(QString::null, 30);
 
+        KPILOT_DELETE(fPilotSysInfo);
 	fPilotSysInfo = new KPilotSysInfo;
 	if (dlp_ReadSysInfo(fCurrentPilotSocket, fPilotSysInfo->sysInfo()) < 0)
 	{
 		emit logError(i18n("Unable to read system information from Pilot"));
-		fStatus=PilotLinkError;
+		fLinkStatus=PilotLinkError;
 		return;
 	}
 #ifdef DEBUG
@@ -502,7 +605,7 @@ void KPilotDeviceLink::acceptDevice()
 		DEBUGDAEMON << fname
 			<< ": RomVersion=" << fPilotSysInfo->getRomVersion()
 			<< " Locale=" << fPilotSysInfo->getLocale()
-#if PILOT_LINK_NUMBER < 10
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_10_0
 			/* No prodID member */
 #else
 			<< " Product=" << fPilotSysInfo->getProductID()
@@ -513,18 +616,18 @@ void KPilotDeviceLink::acceptDevice()
 	fPilotSysInfo->boundsCheck();
 
 	emit logProgress(QString::null, 60);
+        KPILOT_DELETE(fPilotUser);
 	fPilotUser = new KPilotUser;
 
 	/* Ask the pilot who it is.  And see if it's who we think it is. */
 #ifdef DEBUG
 	DEBUGDAEMON << fname << ": Reading user info @"
-		<< (int) fPilotUser << endl;
+		<< (long) fPilotUser << endl;
 	DEBUGDAEMON << fname << ": Buffer @"
-		<< (int) fPilotUser->pilotUser() << endl;
+		<< (long) fPilotUser->pilotUser() << endl;
 #endif
 
 	dlp_ReadUserInfo(fCurrentPilotSocket, fPilotUser->pilotUser());
-	fPilotUser->boundsCheck();
 
 #ifdef DEBUG
 	DEBUGDAEMON << fname
@@ -536,11 +639,13 @@ void KPilotDeviceLink::acceptDevice()
 	/* Tell user (via Pilot) that we are starting things up */
 	if ((ret=dlp_OpenConduit(fCurrentPilotSocket)) < 0)
 	{
+#ifdef DEBUG
 		DEBUGDAEMON << k_funcinfo
 			<< ": dlp_OpenConduit returned " << ret << endl;
+#endif
 
 #if 0
-		fStatus = SyncDone;
+		fLinkStatus = SyncDone;
 		emit logMessage(i18n
 			("Exiting on cancel. All data not restored."));
 		return;
@@ -548,17 +653,168 @@ void KPilotDeviceLink::acceptDevice()
 		emit logError(i18n("Could not read user information from the Pilot. "
 			"Perhaps you have a password set on the device?"));
 	}
-	fStatus = AcceptedDevice;
+	fLinkStatus = AcceptedDevice;
 
 
 	emit logProgress(QString::null, 100);
-	emit deviceReady();
+	emit deviceReady( this );
 }
 
-void KPilotDeviceLink::tickle() const
+void KPilotDeviceLink::workaroundUSB()
 {
 	FUNCTIONSETUP;
-	pi_tickle(pilotSocket());
+	
+	Q_ASSERT((fLinkStatus == DeviceOpen) || (fLinkStatus == WorkaroundUSB));
+	if (fLinkStatus == DeviceOpen)
+	{
+		reset();
+	}
+	fLinkStatus = WorkaroundUSB;
+	
+	if (!QFile::exists(fRealPilotPath))
+	{
+		// Fake connection has vanished again.
+		// Resume polling for regular connection.
+		startOpenTimer(this,fOpenTimer);
+		return;
+	}
+	if (fOpenTimer)
+	{
+		fOpenTimer->stop();
+	}
+	KPILOT_DELETE(fWorkaroundUSBTimer);
+	QTimer::singleShot(1000,this,SLOT(workaroundUSB()));
+}
+
+bool KPilotDeviceLink::tickle() const
+{
+	// No FUNCTIONSETUP here because it may be called from
+	// a separate thread.
+	return pi_tickle(pilotSocket()) >= 0;
+}
+
+class TickleThread : public QThread
+{
+public:
+	TickleThread(KPilotDeviceLink *d, bool *done, int timeout) :
+		QThread(),
+		fHandle(d),
+		fDone(done),
+		fTimeout(timeout)
+	{ };
+	virtual ~TickleThread();
+
+	virtual void run();
+
+	static const int ChecksPerSecond = 5;
+	static const int SecondsPerTickle = 5;
+
+private:
+	KPilotDeviceLink *fHandle;
+	bool *fDone;
+	int fTimeout;
+} ;
+
+TickleThread::~TickleThread()
+{
+}
+
+void TickleThread::run()
+{
+	FUNCTIONSETUP;
+	int subseconds = ChecksPerSecond;
+	int ticktock = SecondsPerTickle;
+	int timeout = fTimeout;
+#ifdef DEBUG_CERR
+	DEBUGDAEMON << fname << ": Running for " << timeout << " seconds." << endl;
+	DEBUGDAEMON << fname << ": Done @" << (void *) fDone << endl;
+#endif
+	while (!(*fDone))
+	{
+		QThread::msleep(1000/ChecksPerSecond);
+		if (!(--subseconds))
+		{
+#ifdef DEBUG_CERR
+// Don't dare use kdDebug() here, we're in a separate thread
+			DEBUGDAEMON << fname << ": One second." << endl;
+#endif
+			if (timeout)
+			{
+				if (!(--timeout))
+				{
+					QApplication::postEvent(fHandle, new QEvent((QEvent::Type)(KPilotDeviceLink::TickleTimeoutEvent)));
+					break;
+				}
+			}
+			subseconds=ChecksPerSecond;
+			if (!(--ticktock))
+			{
+#ifdef DEBUG_CERR
+				DEBUGDAEMON << fname << ": Kietel kietel!." << endl;
+#endif
+				ticktock=SecondsPerTickle;
+				fHandle->tickle();
+			}
+		}
+	}
+#ifdef DEBUG_CERR
+	DEBUGDAEMON << fname << ": Finished." << endl;
+#endif
+}
+
+/*
+** Deal with events, especially the ones used to signal a timeout
+** in a tickle thread.
+*/
+
+/* virtual */ bool KPilotDeviceLink::event(QEvent *e)
+{
+	if (e->type() == TickleTimeoutEvent)
+	{
+		stopTickle();
+		emit timeout();
+		return true;
+	}
+	else	return QObject::event(e);
+}
+
+/*
+Start a tickle thread with the indicated timeout.
+*/
+void KPilotDeviceLink::startTickle(unsigned int timeout)
+{
+	FUNCTIONSETUP;
+
+	Q_ASSERT(fTickleDone);
+
+	/*
+	** We've told the thread to finish up, but it hasn't
+	** done so yet - so wait for it to do so, should be
+	** only 200ms at most.
+	*/
+	if (fTickleDone && fTickleThread)
+	{
+		fTickleThread->wait();
+		KPILOT_DELETE(fTickleThread);
+	}
+
+#ifdef DEBUG
+	DEBUGDAEMON << fname << ": Done @" << (void *) (&fTickleDone) << endl;
+#endif
+	fTickleDone = false;
+	fTickleThread = new TickleThread(this,&fTickleDone,timeout);
+	fTickleThread->start();
+}
+
+void KPilotDeviceLink::stopTickle()
+{
+	FUNCTIONSETUP;
+	fTickleDone = true;
+	if (fTickleThread)
+	{
+		fTickleThread->wait();
+		KPILOT_DELETE(fTickleThread);
+	}
 }
 
 
@@ -595,26 +851,32 @@ bool KPilotDeviceLink::installFile(const QString & f, const bool deleteFile)
 	if (!QFile::exists(f))
 		return false;
 
+	char buffer[PATH_MAX];
+	memset(buffer,0,PATH_MAX);
+	strlcpy(buffer,QFile::encodeName(f),PATH_MAX);
 	struct pi_file *pf =
-		pi_file_open(const_cast < char *>
-			((const char *) QFile::encodeName(f)));
+		pi_file_open(buffer);
 
 	if (!f)
 	{
 		kdWarning() << k_funcinfo
-			<< ": Can't open file " << f << endl;
+			<< ": Cannot open file " << f << endl;
 		emit logError(i18n
-			("<qt>Can't install the file &quot;%1&quot;.</qt>").
+			("<qt>Cannot install the file &quot;%1&quot;.</qt>").
 			arg(f));
 		return false;
 	}
 
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_12_0
 	if (pi_file_install(pf, fCurrentPilotSocket, 0) < 0)
+#else
+	if (pi_file_install(pf, fCurrentPilotSocket, 0, 0L) < 0)
+#endif
 	{
 		kdWarning() << k_funcinfo
-			<< ": Can't pi_file_install " << f << endl;
+			<< ": Cannot pi_file_install " << f << endl;
 		emit logError(i18n
-			("<qt>Can't install the file &quot;%1&quot;.</qt>").
+			("<qt>Cannot install the file &quot;%1&quot;.</qt>").
 			arg(f));
 		return false;
 	}
@@ -633,12 +895,12 @@ void KPilotDeviceLink::addSyncLogEntry(const QString & entry, bool log)
 
 	QString t(entry);
 
-#if (PILOT_LINK_VERSION * 1000 + PILOT_LINK_MAJOR * 10 + PILOT_LINK_MINOR) < 110
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_11_0
 	t.append("X");
 #endif
 
 	dlp_AddSyncLogEntry(fCurrentPilotSocket,
-		const_cast < char *>(t.latin1()));
+		const_cast<char *>((const char *)PilotAppCategory::codec()->fromUnicode(entry)));
 	if (log)
 	{
 		emit logMessage(entry);
@@ -650,61 +912,52 @@ int KPilotDeviceLink::openConduit()
 	return dlp_OpenConduit(fCurrentPilotSocket);
 }
 
-QString KPilotDeviceLink::deviceTypeString(int i) const
-{
-	FUNCTIONSETUP;
-	switch (i)
-	{
-	case None:
-		return QString::fromLatin1("None");
-	case Serial:
-		return QString::fromLatin1("Serial");
-	case OldStyleUSB:
-		return QString::fromLatin1("OldStyleUSB");
-	case DevFSUSB:
-		return QString::fromLatin1("DevFSUSB");
-	default:
-		return QString::fromLatin1("<unknown>");
-	}
-}
-
 QString KPilotDeviceLink::statusString() const
 {
 	FUNCTIONSETUP;
-	QString s = QString::fromLatin1("KPilotDeviceLink=");
+	QString s = CSL1("KPilotDeviceLink=");
 
 
-	switch (fStatus)
+	switch (fLinkStatus)
 	{
 	case Init:
-		s.append(QString::fromLatin1("Init"));
+		s.append(CSL1("Init"));
 		break;
 	case WaitingForDevice:
-		s.append(QString::fromLatin1("WaitingForDevice"));
+		s.append(CSL1("WaitingForDevice"));
 		break;
 	case FoundDevice:
-		s.append(QString::fromLatin1("FoundDevice"));
+		s.append(CSL1("FoundDevice"));
 		break;
 	case CreatedSocket:
-		s.append(QString::fromLatin1("CreatedSocket"));
+		s.append(CSL1("CreatedSocket"));
 		break;
 	case DeviceOpen:
-		s.append(QString::fromLatin1("DeviceOpen"));
+		s.append(CSL1("DeviceOpen"));
 		break;
 	case AcceptedDevice:
-		s.append(QString::fromLatin1("AcceptedDevice"));
+		s.append(CSL1("AcceptedDevice"));
 		break;
 	case SyncDone:
-		s.append(QString::fromLatin1("SyncDone"));
+		s.append(CSL1("SyncDone"));
 		break;
 	case PilotLinkError:
-		s.append(QString::fromLatin1("PilotLinkError"));
+		s.append(CSL1("PilotLinkError"));
+		break;
+	case WorkaroundUSB:
+		s.append(CSL1("WorkaroundUSB"));
 		break;
 	}
 
 	return s;
 }
 
+void KPilotDeviceLink::endOfSync()
+{
+	dlp_EndOfSync(pilotSocket(), 0);
+	KPILOT_DELETE(fPilotSysInfo);
+	KPILOT_DELETE(fPilotUser);
+}
 
 void KPilotDeviceLink::finishSync()
 {
@@ -713,16 +966,30 @@ void KPilotDeviceLink::finishSync()
 	getPilotUser()->setLastSyncPC((unsigned long) gethostid());
 	getPilotUser()->setLastSyncDate(time(0));
 
+	
+#ifdef DEBUG	
+	DEBUGDAEMON << fname << ": Writing username " << getPilotUser()->getUserName() << endl;
+#endif
 	dlp_WriteUserInfo(pilotSocket(),getPilotUser()->pilotUser());
 	addSyncLogEntry(i18n("End of HotSync\n"));
-	dlp_EndOfSync(pilotSocket(), 0);
+	endOfSync();
 }
 
 int KPilotDeviceLink::getNextDatabase(int index,struct DBInfo *dbinfo)
 {
 	FUNCTIONSETUP;
 
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_12_0
 	return dlp_ReadDBList(pilotSocket(),0,dlpDBListRAM,index,dbinfo);
+#else
+	pi_buffer_t buf = { 0,0,0 };
+	int r = dlp_ReadDBList(pilotSocket(),0,dlpDBListRAM,index,&buf);
+	if (r >= 0)
+	{
+		memcpy(dbinfo,buf.data,sizeof(struct DBInfo));
+	}
+	return r;
+#endif
 }
 
 // Find a database with the given name. Info about the DB is stored into dbinfo (e.g. to be used later on with retrieveDatabase).
@@ -739,18 +1006,22 @@ bool KPilotDeviceLink::retrieveDatabase(const QString &fullBackupName,
 {
 	FUNCTIONSETUP;
 
-	// The casts here look funny because:
-	//
-	// fullBackupName is a QString
-	// QFile::encodeName() gives us a QCString
-	// which needs an explicit cast to become a const char *
-	// which needs a const cast to become a char *
-	//
-	//
+#ifdef DEBUG
+	DEBUGDAEMON << fname << ": Writing DB <" << info->name << "> "
+		<< " to " << fullBackupName << endl;
+#endif
+
 	struct pi_file *f;
-	f = pi_file_create(const_cast < char *>
-		((const char *) (QFile::encodeName(fullBackupName))),
-		info);
+	if (fullBackupName.isEmpty())
+	{
+		// Don't even bother trying to convert or retrieve.
+		return false;
+	}
+	QCString encodedName = QFile::encodeName(fullBackupName);
+	char filenameBuf[PATH_MAX];
+	memset(filenameBuf,0,PATH_MAX);
+	strlcpy(filenameBuf,(const char *)encodedName,PATH_MAX);
+	f = pi_file_create(filenameBuf,info);
 
 	if (f == 0)
 	{
@@ -759,7 +1030,11 @@ bool KPilotDeviceLink::retrieveDatabase(const QString &fullBackupName,
 		return false;
 	}
 
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_12_0
 	if (pi_file_retrieve(f, pilotSocket(), 0) < 0)
+#else
+	if (pi_file_retrieve(f, pilotSocket(), 0, 0L) < 0)
+#endif
 	{
 		kdWarning() << k_funcinfo
 			<< ": Failed, unable to back up database" << endl;
@@ -775,21 +1050,53 @@ bool KPilotDeviceLink::retrieveDatabase(const QString &fullBackupName,
 
 QPtrList<DBInfo> KPilotDeviceLink::getDBList(int cardno, int flags)
 {
-  bool cont=true;
-  QPtrList<DBInfo>dbs;
-  int index=0;
-  while (cont)
-  {
-    DBInfo*dbi=new DBInfo();
-    if (dlp_ReadDBList(pilotSocket(), cardno, flags, index, dbi)<0) {
-      KPILOT_DELETE(dbi);
-      cont=false;
-    } else {
-      index=dbi->index+1;
-      dbs.append(dbi);
-    }
-  }
-  return dbs;
+	bool cont=true;
+	QPtrList<DBInfo>dbs;
+	int index=0;
+	while (cont)
+	{
+#if PILOT_LINK_NUMBER < PILOT_LINK_0_12_0
+		DBInfo*dbi=new DBInfo();
+		if (dlp_ReadDBList(pilotSocket(), cardno, flags, index, dbi)<0)
+		{
+			KPILOT_DELETE(dbi);
+			cont=false;
+		}
+		else
+		{
+			index=dbi->index+1;
+			dbs.append(dbi);
+		}
+#else
+		pi_buffer_t buf = { 0,0,0 };
+		pi_buffer_clear(&buf);
+		// DBInfo*dbi=new DBInfo();
+		if (dlp_ReadDBList(pilotSocket(), cardno, flags | dlpDBListMultiple, index, &buf)<0)
+		{
+			cont=false;
+		}
+		else
+		{
+			DBInfo *db_n = 0L;
+			DBInfo *db_it = (DBInfo *)buf.data;
+			int info_count = buf.used / sizeof(struct DBInfo);
+
+			while(info_count>0)
+			{
+				db_n = new DBInfo();
+				memcpy(db_n,db_it,sizeof(struct DBInfo));
+				++db_it;
+				info_count--;
+				dbs.append(db_n);
+			}
+			if (db_n)
+			{
+				index=db_n->index+1;
+			}
+		}
+#endif
+	}
+	return dbs;
 }
 
 KPilotCard *KPilotDeviceLink::getCardInfo(int card)
@@ -797,8 +1104,11 @@ KPilotCard *KPilotDeviceLink::getCardInfo(int card)
 	KPilotCard *cardinfo=new KPilotCard();
 	if (dlp_ReadStorageInfo(pilotSocket(), card, cardinfo->cardInfo())<0)
 	{
-	  KPILOT_DELETE(cardinfo);
-	  return 0L;
+		kdWarning() << k_funcinfo << ": Could not get info for card "
+			<< card << endl;
+
+		KPILOT_DELETE(cardinfo);
+		return 0L;
 	};
 	return cardinfo;
 }
