@@ -3,6 +3,7 @@
 /* kpilotlink.h			KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** Encapsulates all the communication with the pilot. Also
 ** does daemon-like polling of the Pilot. Interesting status
@@ -22,7 +23,7 @@
 **
 ** You should have received a copy of the GNU Lesser General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 ** MA 02111-1307, USA.
 */
 
@@ -33,15 +34,16 @@
 #include <time.h>
 #include <pi-dlp.h>
 
-#ifndef QOBJECT_H
 #include <qobject.h>
-#endif
 
 
 class QTimer;
 class QDateTime;
 class QSocketNotifier;
+class QThread;
 class KPilotUser;
+class KPilotSysInfo;
+class KPilotCard;
 struct DBInfo;
 
 /*
@@ -50,7 +52,7 @@ struct DBInfo;
 ** until it was finally cleaned up again in 2001. In the meantime
 ** it had become something that wrapped a lot more than just
 ** pilot-link. This class currently does:
-** 
+**
 ** * Client (ie. conduit) handling of kpilotlink protocol connections
 ** * Pilot-link handling
 **
@@ -68,7 +70,7 @@ struct DBInfo;
 ** tag. The tag lnch is used by the Pilot's launcher app. Some
 ** parts of KPilot require such a tag.
 */
-struct db 
+struct db
 {
 	char name[256];
 	int flags;
@@ -79,17 +81,9 @@ struct db
 
 #define pi_mktag(c1,c2,c3,c4) (((c1)<<24)|((c2)<<16)|((c3)<<8)|(c4))
 
-/**
- * Convert a struct tm from the pilot-link package to a QDateTime
- */
-QDateTime readTm(const struct tm &t);
-/**
- * Convert a QDateTime to a struct tm for use with the pilot-link package
- */
-struct tm writeTm(const QDateTime &dt);
 
 
-class KPilotDeviceLink : public QObject
+class KDE_EXPORT KPilotDeviceLink : public QObject
 {
 friend class SyncAction;
 Q_OBJECT
@@ -97,20 +91,16 @@ Q_OBJECT
 /*
 ** Constructors and destructors.
 */
-protected:
+public:
 	/**
 	* Creates a pilot link that can sync to the pilot.
 	*
 	* Call reset() on it to start looking for a device.
 	*/
-	KPilotDeviceLink(QObject *parent, const char *name);
-private:
-	static KPilotDeviceLink *fDeviceLink;
-
-public:
+	KPilotDeviceLink( QObject *parent = 0, const char *name = 0, const QString &tempDevice = QString::null );
 	virtual ~KPilotDeviceLink();
-	static KPilotDeviceLink *link() { return fDeviceLink; } ;
-	static KPilotDeviceLink *init(QObject *parent=0L,const char *n=0L);
+//	bool init(QObject *parent=0L,const char *n=0L);
+
 
 /*
 ** Status information
@@ -130,59 +120,146 @@ public:
 		DeviceOpen,
 		AcceptedDevice,
 		SyncDone,
-		PilotLinkError
+		PilotLinkError,
+		WorkaroundUSB
 		} LinkStatus;
 
-	LinkStatus status() const { return fStatus; } ;
+	LinkStatus status() const { return fLinkStatus; } ;
 	virtual QString statusString() const;
 
 	/**
 	* True if HotSync has been started but not finished yet
 	* (ie. the physical Pilot is waiting for sync commands)
 	*/
-	bool getConnected() const { return fStatus == AcceptedDevice; }
-	
-public slots:
-	/**
-	* Tickle the palm to reset the timeout
-	*/
-	void tickle() const;
+	bool getConnected() const { return fLinkStatus == AcceptedDevice; }
 
 private:
-	LinkStatus fStatus;
+	LinkStatus fLinkStatus;
+
+/**
+* Tickle handling.
+*
+* During a HotSync, the Pilot expects to be kept awake by (nearly)
+* continuous communication with the PC. The Pilot doesn't like
+* long periods of inactivity, since they drain the batteries while
+* the communications hardware is kept powered up. If the period of
+* inactivity is too long, the Pilot times out, shuts down the
+* communication, and the HotSync is broken.
+
+* Sometimes, however, periods of inactivity cannot be avoided --
+* for instance, if you _have_ to ask the user something during a
+* sync, or if you are fetching a large amount of data from a slow
+* source (libkabc can do that, if your addressbook is on an LDAP
+* server). During these periods of inactivity (as far as the Pilot
+* can tell), you can "tickle" the Pilot to keep it awake. This
+* prevents the communications from being shut down. It's not
+* a good idea to do this all the time -- battery life and possible
+* corruption of the dlp_ communications streams. Hence, you should
+* start and stop tickling the Pilot around any computation which:
+*   - may take a long time
+*   - does not in itself _ever_ communicate directly with the Pilot
+*
+*
+*
+* You can call slot tickle() whenever you like just to do a
+* dlp_tickle() call on the Pilot. It will return true if the
+* tickle was successful, false otherwise (this can be used to
+* detect if the communication with the Pilot has shut down for
+* some reason).
+*
+* The protected methods startTickle() and stopTickle() are intended
+* to be called only from SyncActions -- I can't think of any other
+* legitimate use, since everything being done during a HotSync is
+* done via subclasses of SyncActions anyway, and SyncAction provides
+* access to these methods though its own start- and stopTickle().
+*
+* Call startTickle with a timeout in seconds, or 0 for no timeout.
+* This timeout is _unrelated_ to the timeout in the Pilot's
+* communications. Instead, it indicates how long to continue
+* tickling the Pilot before emitting the timeout() signal. This
+* can be useful for placing an upper bound on the amount of
+* time to wait for, say, user interaction -- you don't want an
+* inattentive user to drain the batteries during a sync because
+* he doesn't click on "Yes" for some question. If you pass a
+* timeout of 0, the Pilot will continue to be tickled until you
+* call stopTickle().
+*
+* Call stopTickle() to stop tickling the Pilot and continue with
+* normal operation. You _must_ call stopTickle() before calling
+* anything else that might communicate with the Pilot, to avoid
+* corrupting the dlp_ communications stream. (TODO: Mutex the heck
+* out of this to avoid this problem). Note that stopTickle() may
+* hang up the caller for a small amount of time (up to 200ms)
+* before returning.
+*
+* event() and TickleTimeoutEvent are part of the implementation
+* of tickling, and are only accidentally visible.
+*
+* Signal timeout() is emitted if startTickle() has been called
+* with a non-zero timeout and that timeout has elapsed. The
+* tickler is stopped before timeout is emitted.
+*/
+public slots:
+	bool tickle() const;
+protected:
+	void startTickle(unsigned int timeout=0);
+	void stopTickle();
+public:
+	virtual bool event(QEvent *e);
+	static const unsigned int TickleTimeoutEvent = 1066;
+
+signals:
+	void timeout();
+
+private:
+	bool fTickleDone;
+	QThread *fTickleThread;
+
+
+
 
 
 /*
 ** Used for initially attaching to the device.
-** deviceReady() is emitted when the device has been opened
+** deviceReady(KPilotDeviceLink*) is emitted when the device has been opened
 ** and a Sync can start.
 */
 public:
 	/**
 	* Information on what kind of device we're dealing with.
 	*/
-	typedef enum { None,
-		Serial,
-		OldStyleUSB,
-		DevFSUSB
-		} DeviceType;
-
-	DeviceType deviceType() const { return fDeviceType; } ;
-	QString deviceTypeString(int i) const;
-	bool isTransient() const
-	{
-		return (fDeviceType==OldStyleUSB) ||
-			(fDeviceType==DevFSUSB);
-	}
-
 	QString pilotPath() const { return fPilotPath; } ;
 
 	/**
 	* Return the device link to the Init state and try connecting
 	* to the given device path (if it's non-empty).
 	*/
-	void reset(DeviceType t,const QString &pilotPath = QString::null);
+	void reset(const QString &pilotPath);
 
+	
+	/**
+	* Special-cases. Call this after a reset to set device-
+	* specific workarounds; the only one currently known
+	* is the Zire 31/72 T5 quirk of doing a non-HotSync
+	* connect when it's switched on.
+	*/
+	void setWorkarounds(bool usb)
+	{
+		fWorkaroundUSB = usb;
+	} ;
+	
+	/**
+	* sets an additional device, which should be tried as fallback
+	* usefull for hotplug enviroments
+	*/
+	void setTempDevice( const QString &device );
+
+private:
+	bool fWorkaroundUSB;
+	QTimer *fWorkaroundUSBTimer;
+
+private slots:
+	void workaroundUSB();
 
 public slots:
 	/**
@@ -210,14 +287,14 @@ protected slots:
 	* device. This indicates the beginning of a hotsync.
 	*/
 	void acceptDevice();
-
+	
 protected:
 	/**
 	* Does the low-level opening of the device and handles the
 	* pilot-link library initialisation.
 	*/
-	bool open();
-	
+	bool open( QString device = QString::null );
+
 	/**
 	* Check for device permissions and existence, emitting
 	* warnings for weird situations. This is primarily intended
@@ -234,7 +311,7 @@ protected:
 	int messages;
 	int messagesMask;
 	static const int messagesType;
-	
+
 	void shouldPrint(int,const QString &);
 
 signals:
@@ -242,7 +319,7 @@ signals:
 	* Emitted once the user information has been read and
 	* the HotSync is really ready to go.
 	*/
-	void deviceReady();
+	void deviceReady( KPilotDeviceLink* );
 
 protected:
 	int pilotSocket() const { return fCurrentPilotSocket; } ;
@@ -254,11 +331,11 @@ private:
 	* Usually /dev/pilot, /dev/ttySx, or /dev/usb/x.
 	*/
 	QString fPilotPath;
-
 	/**
-	* What kind of device is this?
+	* Path with resolved symlinks, to prevent double binding
+	* to the same device.
 	*/
-	DeviceType fDeviceType;
+	QString fRealPilotPath;
 
 	/**
 	* For transient devices: how often have we tried pi_bind()?
@@ -277,7 +354,13 @@ private:
 	*/
 	int fPilotMasterSocket;
 	int fCurrentPilotSocket;
+	QString fTempDevice;
 
+	/**
+	* Handle cases where we can't accept or open the device,
+	* and data remains available on the pilot socket.
+	*/
+	int fAcceptedCount;
 signals:
 	/**
 	* Whenever a conduit adds a Sync log entry (actually,
@@ -294,28 +377,31 @@ public:
 protected:
 	bool installFile(const QString &, const bool deleteFile=true);
 
- 	/**
- 	* Write a log entry to the pilot. Note that the library
- 	* function takes a char *, not const char * (which is
- 	* highly dubious). Causes signal logEntry(const char *)
- 	* to be emitted if @p log is true.
- 	*/
- 	void addSyncLogEntry(const QString &entry,bool log=true);
+	/**
+	* Write a log entry to the pilot. Note that the library
+	* function takes a char *, not const char * (which is
+	* highly dubious). Causes signal logEntry(const char *)
+	* to be emitted if @p log is true.
+	*/
+	void addSyncLogEntry(const QString &entry,bool log=true);
 
 signals:
- 	/**
- 	* Whenever a conduit adds a Sync log entry (actually,
- 	* KPilotLink itself adds some log entries itself),
+	/**
+	* Whenever a conduit adds a Sync log entry (actually,
+	* KPilotLink itself adds some log entries itself),
 	* this signal is emitted.
 	*/
- 	void logMessage(const QString &);
- 	void logError(const QString &);
- 	void logProgress(const QString &, int);
+	void logMessage(const QString &);
+	void logError(const QString &);
+	void logProgress(const QString &, int);
 
 
 /*
 ** Pilot User Identity functions.
 */
+protected:
+	KPilotUser  *fPilotUser;
+	KPilotSysInfo *fPilotSysInfo;
 public:
 	/**
 	* Returns the user information as set in the KPilot settings dialog.
@@ -325,11 +411,11 @@ public:
 	* local copies of the KPilotUser structure and modify them.
 	*/
 	KPilotUser *getPilotUser() { return fPilotUser; }
+	KPilotSysInfo *getSysInfo() { return fPilotSysInfo; }
+	KPilotCard *getCardInfo(int card=0);
+	void endOfSync();
 	void finishSync();
 
-protected:
-	KPilotUser  *fPilotUser;
-
 /*
 ** Actions intended just to abstract away the pilot-link library interface.
 */
@@ -340,7 +426,7 @@ protected:
 	int openConduit();
 public:
 	int getNextDatabase(int index,struct DBInfo *);
-	int findDatabase(const char *name, struct DBInfo*, 
+	int findDatabase(const char *name, struct DBInfo*,
 		int index=0, long type=0, long creator=0);
 
 	/**
@@ -348,7 +434,8 @@ public:
 	* local file @p path.
 	*/
 	bool retrieveDatabase(const QString &path, struct DBInfo *db);
-	
+	QPtrList<DBInfo> getDBList(int cardno=0, int flags=dlpDBListRAM);
+
 public:
 	/**
 	 * Get the time from the handheld device into a QDateTime
@@ -358,7 +445,7 @@ public:
 	 * Set the time on the handheld to the give QDateTime
 	 */
 	bool setTime(const time_t &pctime);
-	
+
 	/**
 	 * Get the version number from the handheld
 	 */
@@ -371,8 +458,9 @@ public:
 	 * Get the minor PalmOS version number
 	 */
 	unsigned long minorVersion() const;
-	 
-	
+
+private:
+	class KPilotDeviceLinkPrivate;
 } ;
 
 bool operator < ( const struct db &, const struct db &) ;
