@@ -3,6 +3,7 @@
 /* pilotLocalDatabase.h			KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 ** See the .cc file for an explanation of what this file is for.
 */
@@ -28,24 +29,19 @@
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-// Database class for a local (file based) pilot datbase.
+#include "options.h"    /* for PI_SIZE_T */
 
-#include <time.h>	/* for broken pilot-link libraries */
-
-#ifndef _PILOT_MACROS_H_
-#include <pi-macros.h>	/* for recordid_t */
-#endif
-
-#ifndef _KPILOT_PILOTDATABASE_H
 #include "pilotDatabase.h"
-#endif
 
-class PilotLocalDatabase : public PilotDatabase
+// #define SHADOW_LOCAL_DB (1)
+
+class KDE_EXPORT PilotLocalDatabase : public PilotDatabase
 {
 public:
 	/**
 	* Opens the local database. If the database cannot be found at the
-	* given position, a default path is used ($KDEHOME/share/apps/kpilot/DBBackup)
+	* given position, a default path is used
+	*     ($KDEHOME/share/apps/kpilot/DBBackup)
 	* and if the file is found there, it is opened. In some cases this should
 	* not be done, so the parameter useDefaultPath controls this behavior.
 	* If it is set to true, the default path is used if the file cannot be
@@ -54,8 +50,7 @@ public:
 	* created explicitly at the specified location.
 	*/
 	PilotLocalDatabase( const QString& path,
-		const QString& name, bool useDefaultPath=true,
-		QObject *p=0L,const char *n=0L);
+		const QString& name, bool useDefaultPath=true);
 	/**
 	* Opens the local database. A default path is used
 	* ($KDEHOME/share/apps/kpilot/DBBackup)
@@ -66,16 +61,25 @@ public:
 	* the state of the previous conduit run,  useConduitDBs=true
 	* opens the database in $KDEHOME/share/apps/kpilot/conduits
 	*/
-	PilotLocalDatabase(const QString &name, bool useConduitDBs=false,
-		QObject *p=0L,const char *n=0L);
+	PilotLocalDatabase(const QString &name, bool useConduitDBs=false);
 
 
 	virtual ~PilotLocalDatabase();
 
-	/** Creates the database with the given creator, type and flags on the given card (default is RAM). If the database already exists, this function does nothing. */
-	virtual bool createDatabase(long creator=0, long type=0, int cardno=0, int flags=0, int version=0);
-	/** Deletes the database (by name, as given in the constructor and stored in the fDBName field. ) */
+	/** Creates the database with the given creator, type and flags on
+	* the given card (default is RAM). If the database already exists,
+	* this function does nothing.
+	*/
+	virtual bool createDatabase(long creator=0,
+		long type=0, int cardno=0, int flags=0, int version=0);
+
+
+
+	/** Deletes the database (by name, as given in the constructor
+	* and stored in the fDBName field. )
+	*/
 	virtual int deleteDatabase();
+
 	// Reads the application block info
 	virtual int readAppBlock(unsigned char* buffer, int maxLen);
 	// Writes the application block info.
@@ -84,19 +88,30 @@ public:
 	virtual int recordCount();
 	// Returns a QValueList of all record ids in the database.
 	virtual QValueList<recordid_t> idList();
-	// Reads a record from database by id, returns record length
+	// Reads a record from database by id, returns record
 	virtual PilotRecord* readRecordById(recordid_t id);
-	// Reads a record from database, returns the record length
+	// Reads a record from database, returns the record
 	virtual PilotRecord* readRecordByIndex(int index);
 	// Reads the next record from database in category 'category'
 	virtual PilotRecord* readNextRecInCategory(int category);
+	/**
+	* Returns the next "new" record, ie. the next record
+	* that has not been synced yet. These records all have ID=0, so are
+	* not easy to find with the other methods. The record is the one
+	* contained in the database, not a copy like the read*() functions
+	* give you -- so be careful with it. Don't delete it, in any case.
+	* Casting it to non-const and marking it deleted is OK, though,
+	* which is mostly its intended use.
+	*/
+	const PilotRecord *findNextNewRecord();
+
 	/**
 	* Reads the next record from database that has the dirty flag set.
 	* ind (if a valid pointer is given) will receive the index of the
 	* returned record.
 	*/
-	virtual PilotRecord* readNextModifiedRec(int *ind=NULL);
-	// Writes a new record to database (if 'id' == 0, one will be assigned to newRecord)
+	virtual PilotRecord* readNextModifiedRec(int *ind=0L);
+	// Writes a new record to database (if 'id' == 0, none is assigned, either)
 	virtual recordid_t writeRecord(PilotRecord* newRecord);
 	/**
 	* Deletes a record with the given recordid_t from the database,
@@ -112,7 +127,7 @@ public:
 	virtual int cleanup();
 
 
-	// Writes a new ID to the record specified the index.  Not supported on Serial connections
+	// Writes a new ID to the record specified.
 	virtual recordid_t writeID(PilotRecord* rec);
 	QString getDBName() const { return fDBName; }
 
@@ -133,6 +148,9 @@ public:
 	struct DBInfo getDBInfo() const { return fDBInfo; }
 	void setDBInfo(struct DBInfo dbi) {fDBInfo=dbi; }
 
+	virtual DBType dbType() const;
+
+
 protected:
 	// Changes any forward slashes to underscores
 	void fixupDBName();
@@ -143,12 +161,16 @@ private:
 	struct DBInfo fDBInfo;
 	QString fPathName,fDBName;
 	char*       fAppInfo;
-	int         fAppLen;
+	PI_SIZE_T   fAppLen;
 	int         fNumRecords;
 	int         fCurrentRecord;
 	PilotRecord* fRecords[10000]; // Current max records in DB.. hope it's enough
 	int         fPendingRec; // Temp index for the record about to get an ID.
 
+#ifdef SHADOW_LOCAL_DB
+	QValueList<PilotRecord *> fRecordList;
+	QValueList<PilotRecord *>::Iterator fRecordIndex;
+#endif
 
 	/**
 	* For databases opened by name only (constructor 2 -- which is the
