@@ -3,6 +3,7 @@
 /* syncAction.h			KPilot
 **
 ** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 **
 */
 
@@ -42,13 +43,16 @@ class QSocketNotifier;
 class KPilotUser;
 class SyncAction;
 
-class SyncAction : public QObject
+class KDE_EXPORT SyncAction : public QObject
 {
 Q_OBJECT
 
 public:
 	SyncAction(KPilotDeviceLink *p,
 		const char *name=0L);
+	SyncAction(KPilotDeviceLink *p,
+		QWidget *visibleparent,
+		const char *name=0L);
 	~SyncAction();
 
 	typedef enum { Error=-1 } Status;
@@ -151,9 +155,6 @@ public:
 	};
 
 protected:
-	QTimer *fTickleTimer;
-	unsigned fTickleCount,fTickleTimeout;
-
 	/**
 	* Call startTickle() some time before showing a dialog to the
 	* user (we're assuming a local event loop here) so that while
@@ -167,36 +168,11 @@ protected:
 	*/
 	void startTickle(unsigned count=0);
 	void stopTickle();
-protected slots:
-	/**
-	* Called whenever the tickle state is on, uses dlp_tickle()
-	* or something like that to prevent the pilot from timing out.
-	*/
-	void tickle();
 signals:
 	void timeout();
-};
 
 
 
-class InteractiveAction : public SyncAction
-{
-Q_OBJECT
-public:
-	// Note that this takes a QWidget as additional parameter,
-	// so that it can display windows relative to that if necessary.
-	//
-	//
-	InteractiveAction(KPilotDeviceLink *p,
-		QWidget *visibleparent=0L,
-		const char *name=0L);
-	virtual ~InteractiveAction();
-
-	// Reminder that the whole point of the class is to implement
-	// the pure virtual function exec().
-	//
-	// virtual void exec()=0;
-
 
 protected:
 	QWidget *fParent;
@@ -211,12 +187,21 @@ protected:
 	*
 	* @p caption Message Box caption, uses "Question" if null.
 	* @p key     Key for the "Don't ask again" code.
-	* @p timeout Timeout, in ms.
+	* @p timeout Timeout, in seconds.
 	*/
 	int questionYesNo(const QString &question ,
 		const QString &caption = QString::null,
 		const QString &key = QString::null,
-		unsigned timeout = 20000);
-} ;
+		unsigned timeout = 20,
+		const QString &yes = QString::null,
+		const QString &no = QString::null );
+	int questionYesNoCancel(const QString &question ,
+		const QString &caption = QString::null,
+		const QString &key = QString::null,
+		unsigned timeout = 20,
+		const QString &yes = QString::null,
+		const QString &no = QString::null ) ;
+};
+
 
 #endif
