@@ -21,7 +21,7 @@
 #include <qapplication.h>
 
 #include <kabc/addressee.h>
-#include <kaddressbook/kabprefs.h>
+#include <kabprefs.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <klocale.h>
@@ -31,11 +31,11 @@
 #include <libkcal/freebusyurlstore.h>
 #include <libkdepim/kpimprefs.h>
 
+#include "kabc_egroupwareprefs.h"
 #include "kabc_resourcexmlrpc.h"
 #include "kabc_resourcexmlrpcconfig.h"
 
-#include "uidmapper.h"
-
+#include "synchronizer.h"
 #include "xmlrpciface.h"
 
 using namespace KABC;
@@ -47,39 +47,46 @@ static const QString LoadCategoriesCommand = "addressbook.boaddressbook.categori
 static const QString LoadCustomFieldsCommand = "addressbook.boaddressbook.customfields";
 
 ResourceXMLRPC::ResourceXMLRPC( const KConfig *config )
-  : Resource( config ), mServer( 0 )
+  : ResourceCached( config ), mServer( 0 )
 {
-  if ( config ) {
-    init( KURL( config->readEntry( "XmlRpcUrl" ) ),
-          config->readEntry( "XmlRpcDomain", "default" ),
-          config->readEntry( "XmlRpcUser" ),
-          KStringHandler::obscure( config->readEntry( "XmlRpcPassword" ) ) );
-  } else {
-    init( KURL(), "default", "", "" );
-  }
+  init();
+
+  mPrefs->addGroupPrefix( identifier() );
+
+  if ( config )
+    mPrefs->readConfig();
+
+  initEGroupware();
 }
 
-ResourceXMLRPC::ResourceXMLRPC( const KURL &url, const QString &domain,
+ResourceXMLRPC::ResourceXMLRPC( const QString &url, const QString &domain,
                                 const QString &user, const QString &password )
-  : Resource( 0 ), mServer( 0 )
+  : ResourceCached( 0 ), mServer( 0 )
 {
-  init( url, domain, user, password );
+  init();
+
+  mPrefs->addGroupPrefix( identifier() );
+
+  mPrefs->setUrl( url );
+  mPrefs->setDomain( domain );
+  mPrefs->setUser( user );
+  mPrefs->setPassword( password );
+
+  initEGroupware();
 }
 
-void ResourceXMLRPC::init( const KURL &url, const QString &domain,
-                           const QString &user, const QString &password )
+void ResourceXMLRPC::init()
 {
   setType( "xmlrpc" );
 
-  mSyncComm = false;
+  mSynchronizer = new Synchronizer;
 
-  mUidMapper = new UIDMapper( locateLocal( "data", "kabc/egroupware_cache/" + url.host() + "/cache.dat" ) );
-  mUidMapper->load();
+  mPrefs = new EGroupwarePrefs;
+}
 
-  mURL = url;
-  mDomain = domain;
-  mUser = user;
-  mPassword = password;
+void ResourceXMLRPC::initEGroupware()
+{
+  KURL url( mPrefs->url() );
 
   mAddrTypes.insert( "dom", Address::Dom );
   mAddrTypes.insert( "intl", Address::Intl );
@@ -89,29 +96,29 @@ void ResourceXMLRPC::init( const KURL &url, const QString &domain,
 
 ResourceXMLRPC::~ResourceXMLRPC()
 {
-  mUidMapper->store();
-
-  delete mUidMapper;
-  mUidMapper = 0;
+  saveCache();
 
   delete mServer;
   mServer = 0;
+
+  delete mPrefs;
+  mPrefs = 0;
+
+  delete mSynchronizer;
+  mSynchronizer = 0;
 }
 
 void ResourceXMLRPC::writeConfig( KConfig *config )
 {
   Resource::writeConfig( config );
 
-  config->writeEntry( "XmlRpcUrl", mURL.url() );
-  config->writeEntry( "XmlRpcDomain", mDomain );
-  config->writeEntry( "XmlRpcUser", mUser );
-  config->writeEntry( "XmlRpcPassword", KStringHandler::obscure( mPassword ) );
+  mPrefs->writeConfig();
 }
 
 Ticket *ResourceXMLRPC::requestSaveTicket()
 {
   if ( !addressBook() ) {
-	  kdDebug(5700) << "no addressbook" << endl;
+    kdDebug(5700) << "no addressbook" << endl;
     return 0;
   }
 
@@ -129,19 +136,19 @@ bool ResourceXMLRPC::doOpen()
     delete mServer;
 
   mServer = new KXMLRPC::Server( KURL(), this );
-	mServer->setUrl( KURL( mURL ) );
+  mServer->setUrl( KURL( mPrefs->url() ) );
   mServer->setUserAgent( "KDE-AddressBook" );
 
   QMap<QString, QVariant> args;
-  args.insert( "domain", mDomain );
-  args.insert( "username", mUser );
-  args.insert( "password", mPassword );
+  args.insert( "domain", mPrefs->domain() );
+  args.insert( "username", mPrefs->user() );
+  args.insert( "password", mPrefs->password() );
 
   mServer->call( "system.login", QVariant( args ),
                  this, SLOT( loginFinished( const QValueList<QVariant>&, const QVariant& ) ),
                  this, SLOT( fault( int, const QString&, const QVariant& ) ) );
 
-  enter_loop();
+  mSynchronizer->start();
 
   return true;
 }
@@ -156,14 +163,13 @@ void ResourceXMLRPC::doClose()
                  this, SLOT( logoutFinished( const QValueList<QVariant>&, const QVariant& ) ),
                  this, SLOT( fault( int, const QString&, const QVariant& ) ) );
 
-  enter_loop();
-
-  delete mServer;
-  mServer = 0;
+  mSynchronizer->start();
 }
 
 bool ResourceXMLRPC::load()
 {
+  mAddrMap.clear();
+
   return true;
 }
 
@@ -172,6 +178,10 @@ bool ResourceXMLRPC::asyncLoad()
   if ( !mServer )
     return false;
 
+  mAddrMap.clear();
+
+  loadCache();
+
   QMap<QString, QVariant> args;
   args.insert( "start", "1" );
   args.insert( "limit", "1000" );
@@ -197,50 +207,62 @@ bool ResourceXMLRPC::asyncLoad()
 }
 
 
-bool ResourceXMLRPC::save( Ticket* )
+bool ResourceXMLRPC::save( Ticket *ticket )
 {
-  return false; // readonly
+  return asyncSave( ticket );
 }
 
-bool ResourceXMLRPC::asyncSave( Ticket* )
+bool ResourceXMLRPC::asyncSave( Ticket */*ticket*/ )
 {
-  return false; // readonly
+  KABC::Addressee::List::ConstIterator it;
+
+  const KABC::Addressee::List addedList = addedAddressees();
+  for ( it = addedList.begin(); it != addedList.end(); ++it ) {
+    addContact( *it );
+  }
+
+  const KABC::Addressee::List changedList = changedAddressees();
+  for ( it = changedList.begin(); it != changedList.end(); ++it ) {
+    updateContact( *it );
+  }
+
+  const KABC::Addressee::List deletedList = deletedAddressees();
+  for ( it = deletedList.begin(); it != deletedList.end(); ++it ) {
+    deleteContact( *it );
+  }
+
+  return true;
 }
 
-void ResourceXMLRPC::insertAddressee( const Addressee& addr )
+void ResourceXMLRPC::addContact( const Addressee& addr )
 {
   QMap<QString, QVariant> args;
   writeContact( addr, args );
 
-  if ( mAddrMap.find( addr.uid() ) == mAddrMap.end() ) { // new entry
-    mAddrMap.insert( addr.uid(), addr );
-
-    mServer->call( AddContactCommand, args,
-                   this, SLOT( addContactFinished( const QValueList<QVariant>&, const QVariant& ) ),
-                   this, SLOT( fault( int, const QString&, const QVariant& ) ),
-                   QVariant( addr.uid() ) );
-  } else {
-    mAddrMap[ addr.uid() ] = addr;
-    args.insert( "id", mUidMapper->remoteUid( addr.uid() ) );
-    mServer->call( AddContactCommand, args,
-                   this, SLOT( updateContactFinished( const QValueList<QVariant>&, const QVariant& ) ),
-                   this, SLOT( fault( int, const QString&, const QVariant& ) ) );
-  }
-
-  enter_loop();
+  mServer->call( AddContactCommand, args,
+                 this, SLOT( addContactFinished( const QValueList<QVariant>&, const QVariant& ) ),
+                 this, SLOT( addContactFault( int, const QString&, const QVariant& ) ),
+                 QVariant( addr.uid() ) );
 }
 
-void ResourceXMLRPC::removeAddressee( const Addressee& addr )
+void ResourceXMLRPC::updateContact( const Addressee& addr )
 {
-  QString id = mUidMapper->remoteUid( addr.uid() );
+  QMap<QString, QVariant> args;
+  writeContact( addr, args );
 
-  mServer->call( DeleteContactCommand, id,
-                 this, SLOT( deleteContactFinished( const QValueList<QVariant>&, const QVariant& ) ),
-                 this, SLOT( fault( int, const QString&, const QVariant& ) ),
+  args.insert( "id", idMapper().remoteId( addr.uid() ) );
+  mServer->call( AddContactCommand, args,
+                 this, SLOT( updateContactFinished( const QValueList<QVariant>&, const QVariant& ) ),
+                 this, SLOT( updateContactFault( int, const QString&, const QVariant& ) ),
                  QVariant( addr.uid() ) );
+}
 
-
-  enter_loop();
+void ResourceXMLRPC::deleteContact( const Addressee& addr )
+{
+  mServer->call( DeleteContactCommand, idMapper().remoteId( addr.uid() ),
+                 this, SLOT( deleteContactFinished( const QValueList<QVariant>&, const QVariant& ) ),
+                 this, SLOT( deleteContactFault( int, const QString&, const QVariant& ) ),
+                 QVariant( addr.uid() ) );
 }
 
 void ResourceXMLRPC::loginFinished( const QValueList<QVariant> &variant,
@@ -248,7 +270,7 @@ void ResourceXMLRPC::loginFinished( const QValueList<QVariant> &variant,
 {
   QMap<QString, QVariant> map = variant[0].toMap();
 
-  KURL url = mURL;
+  KURL url( mPrefs->url() );
   if ( map[ "GOAWAY" ].toString() == "XOXO" ) { // failed
     mSessionID = mKp3 = "";
     addressBook()->error( i18n( "Login failed, please check your username and password." ) );
@@ -261,7 +283,7 @@ void ResourceXMLRPC::loginFinished( const QValueList<QVariant> &variant,
   url.setPass( mKp3 );
   mServer->setUrl( url );
 
-  exit_loop();
+  mSynchronizer->stop();
 }
 
 void ResourceXMLRPC::logoutFinished( const QValueList<QVariant> &variant,
@@ -272,23 +294,24 @@ void ResourceXMLRPC::logoutFinished( const QValueList<QVariant> &variant,
   if ( map[ "GOODBYE" ].toString() != "XOXO" )
     addressBook()->error( i18n( "Logout failed, please check your username and password." ) );
 
-  KURL url = mURL;
+  KURL url( mPrefs->url() );
   mSessionID = mKp3 = "";
   url.setUser( mSessionID );
   url.setPass( mKp3 );
   mServer->setUrl( url );
 
-  exit_loop();
+  mSynchronizer->stop();
 }
 
 void ResourceXMLRPC::listContactsFinished( const QValueList<QVariant> &mapList,
                                            const QVariant& )
 {
-  QValueList<QVariant> contactList = mapList[ 0 ].toList();
-  QValueList<QVariant>::Iterator contactIt;
+  const QValueList<QVariant> contactList = mapList[ 0 ].toList();
+  QValueList<QVariant>::ConstIterator contactIt;
 
+  KABC::Addressee::List serverContacts;
   for ( contactIt = contactList.begin(); contactIt != contactList.end(); ++contactIt ) {
-    QMap<QString, QVariant> map = (*contactIt).toMap();
+    const QMap<QString, QVariant> map = (*contactIt).toMap();
 
     Addressee addr;
     QString uid;
@@ -299,43 +322,48 @@ void ResourceXMLRPC::listContactsFinished( const QValueList<QVariant> &mapList,
       addr.setResource( this );
       addr.setChanged( false );
 
-      QString localUid = mUidMapper->localUid( uid );
-      if ( localUid.isEmpty() ) {
-        mUidMapper->add( addr.uid(), uid );
+      QString local = idMapper().localId( uid );
+      if ( local.isEmpty() ) { // new entry
+        idMapper().setRemoteId( addr.uid(), uid );
       } else {
-        addr.setUid( localUid );
+        addr.setUid( local );
       }
 
       mAddrMap.insert( addr.uid(), addr );
+      serverContacts.append( addr );
     }
   }
 
-  exit_loop();
+  cleanUpCache( serverContacts );
+  saveCache();
 
   emit loadingFinished( this );
 }
 
-void ResourceXMLRPC::deleteContactFinished( const QValueList<QVariant>&,
-                                            const QVariant &id )
-{
-  mAddrMap.remove( id.toString() );
-  mUidMapper->removeByLocal( id.toString() );
-
-  exit_loop();
-}
-
-void ResourceXMLRPC::updateContactFinished( const QValueList<QVariant>&,
-                                            const QVariant& )
-{
-  exit_loop();
-}
-
 void ResourceXMLRPC::addContactFinished( const QValueList<QVariant> &list,
                                          const QVariant &id )
 {
-  mUidMapper->add( id.toString(), list[ 0 ].toString() );
+  clearChange( id.toString() );
+  idMapper().setRemoteId( id.toString(), list[ 0 ].toString() );
 
-  exit_loop();
+  saveCache();
+}
+
+void ResourceXMLRPC::updateContactFinished( const QValueList<QVariant>&,
+                                            const QVariant &id )
+{
+  clearChange( id.toString() );
+
+  saveCache();
+}
+
+void ResourceXMLRPC::deleteContactFinished( const QValueList<QVariant>&,
+                                            const QVariant &id )
+{
+  clearChange( id.toString() );
+  idMapper().removeRemoteId( idMapper().remoteId( id.toString() ) );
+
+  saveCache();
 }
 
 void ResourceXMLRPC::fault( int error, const QString &errorMsg,
@@ -344,54 +372,54 @@ void ResourceXMLRPC::fault( int error, const QString &errorMsg,
   QString msg = i18n( "<qt>Server sent error %1: <b>%2</b></qt>" ).arg( error ).arg( errorMsg );
   addressBook()->error( msg );
 
-  exit_loop();
+  mSynchronizer->stop();
 }
 
-
-void ResourceXMLRPC::setURL( const KURL &url )
+void ResourceXMLRPC::addContactFault( int, const QString &errorMsg,
+                                      const QVariant &id )
 {
-  mURL = url;
+  KABC::Addressee addr = mAddrMap[ id.toString() ];
+
+  mAddrMap.remove( addr.uid() );
+
+  QString msg = i18n( "Unable to add contact %1 to server. (%2)" );
+  addressBook()->error( msg.arg( addr.formattedName(), errorMsg ) );
 }
 
-KURL ResourceXMLRPC::url() const
+void ResourceXMLRPC::updateContactFault( int, const QString &errorMsg,
+                                         const QVariant &id )
 {
-  return mURL;
+  KABC::Addressee addr = mAddrMap[ id.toString() ];
+
+  QString msg = i18n( "Unable to update contact %1 on server. (%2)" );
+  addressBook()->error( msg.arg( addr.formattedName(), errorMsg ) );
 }
 
-void ResourceXMLRPC::setDomain( const QString &domain )
+void ResourceXMLRPC::deleteContactFault( int, const QString &errorMsg,
+                                         const QVariant &id )
 {
-  mDomain = domain;
-}
+  KABC::Addressee addr;
+  addr.setFormattedName( "coolo" );
 
-QString ResourceXMLRPC::domain() const
-{
-  return mDomain;
-}
+  const KABC::Addressee::List deletedList = deletedAddressees();
+  KABC::Addressee::List::ConstIterator it;
+  for ( it = deletedList.begin(); it != deletedList.end(); ++it ) {
+    if ( (*it).uid() == id.toString() ) {
+      addr = *it;
+      break;
+    }
+  }
 
-void ResourceXMLRPC::setUser( const QString &user )
-{
-  mUser = user;
-}
+  mAddrMap.insert( addr.uid(), addr );
 
-QString ResourceXMLRPC::user() const
-{
-  return mUser;
-}
-
-void ResourceXMLRPC::setPassword( const QString &password )
-{
-  mPassword = password;
-}
-
-QString ResourceXMLRPC::password() const
-{
-  return mPassword;
+  QString msg = i18n( "Unable to delete contact %1 from server. (%2)" );
+  addressBook()->error( msg.arg( addr.formattedName(), errorMsg ) );
 }
 
 QString ResourceXMLRPC::addrTypesToTypeStr( int typeMask )
 {
   QStringList types;
-  QMap<QString, int>::Iterator it;
+  QMap<QString, int>::ConstIterator it;
   for ( it = mAddrTypes.begin(); it != mAddrTypes.end(); ++it )
     if ( it.data() & typeMask )
       types.append( it.key() );
@@ -399,27 +427,6 @@ QString ResourceXMLRPC::addrTypesToTypeStr( int typeMask )
   return types.join( ";" );
 }
 
-void qt_enter_modal( QWidget *widget );
-void qt_leave_modal( QWidget *widget );
-
-void ResourceXMLRPC::enter_loop()
-{
-  QWidget dummy( 0, 0, WType_Dialog | WShowModal );
-  dummy.setFocusPolicy( QWidget::NoFocus );
-  qt_enter_modal( &dummy );
-  mSyncComm = true;
-  qApp->enter_loop();
-  qt_leave_modal( &dummy );
-}
-
-void ResourceXMLRPC::exit_loop()
-{
-  if ( mSyncComm ) {
-    mSyncComm = false;
-    qApp->exit_loop();
-  }
-}
-
 void ResourceXMLRPC::writeContact( const Addressee &addr, QMap<QString, QVariant> &args )
 {
   args.insert( "access", ( addr.secrecy().type() == Secrecy::Private ? "private" : "public" ) );
@@ -443,12 +450,12 @@ void ResourceXMLRPC::writeContact( const Addressee &addr, QMap<QString, QVariant
 
   // CATEGORIES
   QStringList::ConstIterator catIt;
-  QStringList categories = addr.categories();
+  const QStringList categories = addr.categories();
 
   QMap<QString, QVariant> catMap;
   int counter = 0;
   for ( catIt = categories.begin(); catIt != categories.end(); ++catIt ) {
-    QMap<QString, int>::Iterator it = mCategoryMap.find( *catIt );
+    QMap<QString, int>::ConstIterator it = mCategoryMap.find( *catIt );
     if ( it == mCategoryMap.end() ) // new category
       catMap.insert( QString::number( counter-- ), *catIt );
     else
@@ -541,8 +548,8 @@ void ResourceXMLRPC::writeContact( const Addressee &addr, QMap<QString, QVariant
   }
 
 
-  QStringList customFields = addr.customs();
-  QStringList::Iterator it;
+  const QStringList customFields = addr.customs();
+  QStringList::ConstIterator it;
   for ( it = customFields.begin(); it != customFields.end(); ++it ) {
     int colon = (*it).find( ":" );
     QString identifier = (*it).left( colon );
@@ -679,15 +686,15 @@ void ResourceXMLRPC::readContact( const QMap<QString, QVariant> &args, Addressee
       addr.insertEmail( it.data().toString(), false );
     } else if ( it.key() == "email_home_type" ) {
     } else if ( it.key() == "cat_id" ) {
-      QMap<QString, QVariant> categories = it.data().toMap();
-      QMap<QString, QVariant>::Iterator it;
+      const QMap<QString, QVariant> categories = it.data().toMap();
+      QMap<QString, QVariant>::ConstIterator it;
 
       for ( it = categories.begin(); it != categories.end(); ++it )
         addr.insertCategory( it.data().toString() );
     }
   }
 
-  QMap<QString, QString>::Iterator cfIt;
+  QMap<QString, QString>::ConstIterator cfIt;
   for ( cfIt = mCustomFieldsMap.begin(); cfIt != mCustomFieldsMap.end(); ++cfIt ) {
     if ( args[ cfIt.key() ].toString().isEmpty() )
       continue;
@@ -711,16 +718,18 @@ void ResourceXMLRPC::loadCategoriesFinished( const QValueList<QVariant> &mapList
 {
   mCategoryMap.clear();
 
-  QMap<QString, QVariant> map = mapList[ 0 ].toMap();
-  QMap<QString, QVariant>::Iterator it;
+  const QMap<QString, QVariant> map = mapList[ 0 ].toMap();
+  QMap<QString, QVariant>::ConstIterator it;
 
   KABPrefs *prefs = KABPrefs::instance();
   for ( it = map.begin(); it != map.end(); ++it ) {
     mCategoryMap.insert( it.data().toString(), it.key().toInt() );
 
-    if ( prefs->mCustomCategories.find( it.data().toString() ) == prefs->mCustomCategories.end() ) {
-      prefs->mCustomCategories.append( it.data().toString() );
-    }
+    QStringList categories = prefs->customCategories();
+    if ( categories.find( it.data().toString() ) == categories.end() )
+      categories.append( it.data().toString() );
+
+    prefs->mCustomCategories = categories;
   }
 }
 
@@ -729,8 +738,8 @@ void ResourceXMLRPC::loadCustomFieldsFinished( const QValueList<QVariant> &mapLi
 {
   mCustomFieldsMap.clear();
 
-  QMap<QString, QVariant> map = mapList[ 0 ].toMap();
-  QMap<QString, QVariant>::Iterator it;
+  const QMap<QString, QVariant> map = mapList[ 0 ].toMap();
+  QMap<QString, QVariant>::ConstIterator it;
 
   for ( it = map.begin(); it != map.end(); ++it )
     mCustomFieldsMap.insert( it.key(), it.data().toString() );
