@@ -3,6 +3,7 @@
 
     Copyright (c) 1998 Preston Brown
     Copyright (c) 2001,2003,2004 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -27,25 +28,71 @@
 #include <qdatetime.h>
 #include <qptrlist.h>
 #include <qdict.h>
+#include <kdepimmacros.h>
 
 #include "customproperties.h"
 #include "event.h"
 #include "todo.h"
 #include "journal.h"
 #include "kcalversion.h"
+#include "person.h"
 
 #define _TIME_ZONE "-0500" /* hardcoded, overridden in config file. */
 
 class KConfig;
 
+/** @file */
+
+/** 
+    Namespace KCal is for Calendaring-related definitions. 
+*/
 namespace KCal {
 
+/**
+   Sort direction.
+*/
+enum SortDirection {
+  SortDirectionAscending,
+  SortDirectionDescending
+};
+
+/**
+   How events are to be sorted.
+*/
+enum EventSortField {
+  EventSortUnsorted,
+  EventSortStartDate,
+  EventSortEndDate,
+  EventSortSummary
+};
+
+/**
+   How todos are to be sorted.
+*/
+enum TodoSortField {
+  TodoSortUnsorted,
+  TodoSortStartDate,
+  TodoSortDueDate,
+  TodoSortPriority,
+  TodoSortPercentComplete,
+  TodoSortSummary
+};
+
+/**
+   How journals are to be sorted.
+*/
+enum JournalSortField {
+  JournalSortUnsorted,
+  JournalSortDate,
+  JournalSortSummary
+};
+
 class CalFilter;
 
 /**
   This is the main "calendar" object class for KOrganizer.  It holds
   information like all appointments/events, user information, etc. etc.
-  one calendar is associated with each CalendarView (@see calendarview.h).
+  one calendar is associated with each CalendarView ( @see calendarview.h ).
   This is an abstract base class defining the interface to a calendar. It is
   implemented by subclasses like @see CalendarLocal, which use different
   methods to store and access the data.
@@ -58,7 +105,7 @@ class CalFilter;
   visible in the Calendar. You shouldn't delete any Event object you get from
   Calendar.
 */
-class Calendar : public QObject, public CustomProperties,
+class LIBKCAL_EXPORT Calendar : public QObject, public CustomProperties,
                  public IncidenceBase::Observer
 {
     Q_OBJECT
@@ -82,20 +129,12 @@ class Calendar : public QObject, public CustomProperties,
     /**
       Return the owner of the calendar's full name.
     */
-    const QString &getOwner() const;
+    const Person &getOwner() const;
     /**
       Set the owner of the calendar. Should be owner's full name.
     */
-    void setOwner( const QString &os );
-    /**
-      Return the email address of the calendar owner.
-    */
-    const QString &getEmail();
-    /**
-      Set the email address of the calendar owner.
-    */
-    void setEmail( const QString & );
-  
+    void setOwner( const Person &owner );
+
     /**
       Set time zone id (see /usr/share/zoneinfo/zone.tab for list of legal
       values).
@@ -113,19 +152,40 @@ class Calendar : public QObject, public CustomProperties,
       Return whether local time is being used.
     */
     bool isLocalTime() const;
-  
+
+    /**
+       Sort eventList according to sortField
+    */
+    static Event::List sortEvents( Event::List *eventList,
+                            EventSortField sortField,
+                            SortDirection sortDirection );
+
+    /**
+       Sort todoList according to sortField
+    */
+    static Todo::List sortTodos( Todo::List *todoList,
+                          TodoSortField sortField,
+                          SortDirection sortDirection );
+
+    /**
+       Sort journalList according to sortField
+    */
+    static Journal::List sortJournals( Journal::List *journalList,
+                                JournalSortField sortField,
+                                SortDirection sortDirection );
+
     /**
       Add an incidence to calendar.
-      
+
       @return true on success, false on error.
     */
-    virtual bool addIncidence( Incidence * );  
+    virtual bool addIncidence( Incidence * );
     /**
       Delete an incidence from calendar.
-      
+
       @return true on success, false on error.
     */
-    virtual bool deleteIncidence( Incidence * );  
+    virtual bool deleteIncidence( Incidence * );
     /**
       Return filtered list of all incidences of this calendar.
     */
@@ -136,19 +196,17 @@ class Calendar : public QObject, public CustomProperties,
       Return unfiltered list of all incidences of this calendar.
     */
     virtual Incidence::List rawIncidences();
-    
-    /** 
+
+    /**
       Return a list of all categories used in this calendar.
     */
     QStringList incidenceCategories();
 
     /**
       Adds a Event to this calendar object.
-      @param anEvent a pointer to the event to add
-      
       @return true on success, false on error.
     */
-    virtual bool addEvent( Event *anEvent ) = 0;
+    virtual bool addEvent( Event * ) = 0;
     /**
       Delete event from calendar.
     */
@@ -162,7 +220,9 @@ class Calendar : public QObject, public CustomProperties,
       date specified. useful for dayView, etc. etc.
       The calendar filter is applied.
     */
+    //TODO: Deprecate
     Event::List events( const QDate &date, bool sorted = false );
+    //Event::List events( const QDate &date, EventSorted sortField = EventSortUnsorted, SortDirection sortDirection = SortDirectionAscending );
     /**
       Get events, which occur on the given date.
       The calendar filter is applied.
@@ -176,17 +236,17 @@ class Calendar : public QObject, public CustomProperties,
     Event::List events( const QDate &start, const QDate &end,
                         bool inclusive = false);
     /**
-      Return filtered list of all events in calendar.
+       Return filtered list of all events sorted according to sortField.
     */
-    virtual Event::List events();
+    virtual Event::List events( EventSortField sortField = EventSortUnsorted, SortDirection sortDirection = SortDirectionAscending );
     /**
       Return unfiltered list of all events in calendar.
     */
-    virtual Event::List rawEvents() = 0;
+    virtual Event::List rawEvents( EventSortField sortField = EventSortUnsorted, SortDirection sortDirection = SortDirectionAscending ) = 0;
 
     /**
       Add a todo to the todolist.
-      
+
       @return true on success, false on error.
     */
     virtual bool addTodo( Todo *todo ) = 0;
@@ -195,9 +255,9 @@ class Calendar : public QObject, public CustomProperties,
     */
     virtual void deleteTodo( Todo * ) = 0;
     /**
-      Return filterd list of todos.
+      Return filtered list of todos.
     */
-    virtual Todo::List todos();
+    virtual Todo::List todos( TodoSortField = TodoSortUnsorted, SortDirection sortDirection = SortDirectionAscending );
     /**
       Searches todolist for an event with this unique string identifier,
       returns a pointer or null.
@@ -210,7 +270,7 @@ class Calendar : public QObject, public CustomProperties,
     /**
       Return unfiltered list of todos.
     */
-    virtual Todo::List rawTodos() = 0;
+    virtual Todo::List rawTodos( TodoSortField sortField = TodoSortUnsorted, SortDirection sortDirection = SortDirectionAscending ) = 0;
     /**
       Return unfiltered list of todos.
     */
@@ -218,7 +278,7 @@ class Calendar : public QObject, public CustomProperties,
 
     /**
       Add a Journal entry to calendar.
-      
+
       @return true on success, false on error.
     */
     virtual bool addJournal( Journal * ) = 0;
@@ -226,25 +286,38 @@ class Calendar : public QObject, public CustomProperties,
       Remove a journal entry from the calendar.
     */
     virtual void deleteJournal( Journal * ) = 0;
-    /**
-      Return Journal for given date.
-    */
-    virtual Journal *journal( const QDate & ) = 0;
     /**
       Return Journal with given UID.
     */
-    virtual Journal *journal( const QString &UID ) = 0;
+    virtual Journal *journal( const QString &uid ) = 0;
     /**
       Return list of all Journal entries.
     */
-    virtual Journal::List journals() = 0;
-    // TODO: Add rawJournals() and rawJournal( QDate )
+    virtual Journal::List journals( JournalSortField sortField = JournalSortUnsorted, SortDirection sortDirection = SortDirectionAscending );
+    /**
+      Returns list of journals for the specified date.
+    */
+    virtual Journal::List journals( const QDate &date );
+    /**
+       Return unfiltered list of all journals in calendar.
+    */
+    virtual Journal::List rawJournals( JournalSortField sortField = JournalSortUnsorted, SortDirection sortDirection = SortDirectionAscending ) = 0;
+    /**
+      Return unfiltered list of journals for a given date.
+    */
+    virtual Journal::List rawJournalsForDate( const QDate &date ) = 0;
 
     /**
       Searches all incidence types for an incidence with this unique
       string identifier, returns a pointer or null.
     */
-    Incidence *incidence( const QString &UID );
+    Incidence *incidence( const QString &uid );
+
+    /**
+      Searches all events and todos for an incidence with this
+      scheduling ID. Returns a pointer or null.
+    */
+    Incidence *incidenceFromSchedulingID( const QString &uid );
 
     /**
       Setup relations for an incidence.
@@ -254,7 +327,7 @@ class Calendar : public QObject, public CustomProperties,
       Remove all relations to an incidence
     */
     virtual void removeRelations( Incidence * );
-    
+
     /**
       Set calendar filter, which filters events for the events() functions.
       The Filter object is owned by the caller.
@@ -264,7 +337,7 @@ class Calendar : public QObject, public CustomProperties,
       Return calendar filter.
     */
     CalFilter *filter();
-      
+
     /**
       Return all alarms, which ocur in the given time interval.
     */
@@ -279,11 +352,15 @@ class Calendar : public QObject, public CustomProperties,
         virtual void calendarIncidenceChanged( Incidence * ) {}
         virtual void calendarIncidenceDeleted( Incidence * ) {}
     };
-  
+
     void registerObserver( Observer * );
     void unregisterObserver( Observer * );
 
     void setModified( bool );
+    /**
+      Return whether the calendar was modified since opening / saving
+     */
+    bool isModified() const { return mModified; }
 
     /**
       Set product id returned by loadedProductId(). This function is only
@@ -306,11 +383,11 @@ class Calendar : public QObject, public CustomProperties,
     virtual bool beginChange( Incidence * );
     virtual bool endChange( Incidence * );
 
-    /** 
+    /**
       Dissociate an incidence from a recurring incidence. By default, only one
       single event for the given date will be dissociated and returned.
       If single == false, the recurrence will be split at date, the
-      old incidence will have its recurrence ending at date and the 
+      old incidence will have its recurrence ending at date and the
       new incidence (return value) will have all recurrences past the date.
     */
     Incidence *dissociateOccurrence( Incidence *incidence, QDate date,
@@ -321,6 +398,11 @@ class Calendar : public QObject, public CustomProperties,
     void calendarSaved();
     void calendarLoaded();
 
+  protected:
+    /**
+      The observer interface. So far not implemented.
+    */
+    void incidenceUpdated( IncidenceBase * );
   public:
     /**
       Get unfiltered events, which occur on the given date.
@@ -329,14 +411,17 @@ class Calendar : public QObject, public CustomProperties,
     /**
       Get unfiltered events, which occur on the given date.
     */
+    //TODO: Deprecate
     virtual Event::List rawEventsForDate( const QDate &date,
-                                          bool sorted = false ) = 0;  
+                                          bool sorted = false ) = 0;
+    //virtual Event::List rawEventsForDate( const QDate &date, EventSortField sortField = EventSortUnsorted, SortDirection sortDirection = SortDirectionAscending ) = 0;
     /**
       Get events in a range of dates. If inclusive is set to true, only events
       are returned, which are completely included in the range.
     */
     virtual Event::List rawEvents( const QDate &start, const QDate &end,
                                    bool inclusive = false ) = 0;
+
   protected:
     /**
       let the subclasses of KCal::Calendar set the time zone
@@ -351,21 +436,21 @@ class Calendar : public QObject, public CustomProperties,
 
   private:
     void init();
-  
-    QString mOwner;        // who the calendar belongs to
-    QString mOwnerEmail;   // email address of the owner
+
+    Person mOwner;         // who the calendar belongs to
+
     int mTimeZone;         // timezone OFFSET from GMT (MINUTES)
     bool mLocalTime;       // use local time, not UTC or a time zone
 
     CalFilter *mFilter;
     CalFilter *mDefaultFilter;
-    
+
     QString mTimeZoneId;
 
     QPtrList<Observer> mObservers;
     bool mNewObserver;
     bool mObserversEnabled;
-    
+
     bool mModified;
 
     QString mLoadedProductId;
@@ -377,7 +462,7 @@ class Calendar : public QObject, public CustomProperties,
     class Private;
     Private *d;
 };
-  
+
 }
 
 #endif
