@@ -2,6 +2,7 @@
     This file is part of libkcal.
 
     Copyright (c) 2001-2003 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (C) 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,13 +23,14 @@
 #define KCAL_JOURNAL_H
 
 #include "incidence.h"
+#include <kdepimmacros.h>
 
 namespace KCal {
 
 /**
   This class provides a Journal in the sense of RFC2445.
 */
-class Journal : public Incidence
+class LIBKCAL_EXPORT Journal : public Incidence
 {
   public:
     typedef ListBase<Journal> List;
