@@ -3,8 +3,8 @@
 // Contents:   Definitions for DwString
 // Maintainer: Doug Sauder <dwsauder@fwb.gulf.net>
 // WWW:        http://www.fwb.gulf.net/~dwsauder/mimepp.html
-// $Revision: 1.8 $
-// $Date: 2004/01/14 14:35:20 $
+// $Revision: 1.11 $
+// $Date: 2005/02/06 18:06:57 $
 //
 // Copyright (c) 1996, 1997 Douglas W. Sauder
 // All rights reserved.
@@ -40,7 +40,50 @@
 #define DW_MIN(a,b) ((a) <= (b) ? (a) : (b))
 #define DW_MAX(a,b) ((a) >= (b) ? (a) : (b))
 
+/* In some locales (such as tr_TR.UTF-8, az_AZ) using tolower() can cause
+   unexpected results. Keywords must be compared in a
+   locale-independent manner */
+static char dw_asciitolower( const char c )
+{
+  if ( c >= 'A' && c <= 'Z' )
+    return c - 'A' + 'a';
+  else
+    return c;
+}
 
+static char dw_asciitoupper( const char c )
+{
+  if ( c >= 'a' && c <= 'z' )
+    return c - 'a' + 'A';
+  else
+    return c;
+}
+
+static int dw_strasciicasecmp(const char* s1, size_t len1, const char* s2,
+    size_t len2)
+{
+    assert(s1 != 0);
+    assert(s2 != 0);
+    size_t len = DW_MIN(len1, len2);
+    for (size_t i=0; i < len; ++i) {
+        int c1 = dw_asciitolower( s1[i] );
+        int c2 = dw_asciitolower( s2[i] );
+
+        if ( c1 < c2 )
+            return -1;
+        else if ( c1 > c2 )
+            return 1;
+    }
+    if (len1 < len2) {
+        return -1;
+    }
+    else if (len1 > len2) {
+        return 1;
+    }
+    return 0;
+}
+
+#if 0
 static int dw_strcasecmp(const char* s1, size_t len1, const char* s2,
     size_t len2)
 {
@@ -65,6 +108,7 @@ static int dw_strcasecmp(const char* s1, size_t len1, const char* s2,
     }
     return 0;
 }
+#endif
 
 
 static int dw_strcmp(const char* s1, size_t len1, const char* s2, size_t len2)
@@ -187,7 +231,7 @@ DwStringRep::DwStringRep(FILE* aFile, size_t aSize)
     mPageMod = tell % pagesize;
     mSize = aSize;
     mRefCount = 1;
-    
+
     mBuffer = (char *)mmap(0, aSize + mPageMod, PROT_READ, MAP_SHARED, fileno(aFile), tell - mPageMod) + mPageMod;
     ++mPageMod;
     if (mBuffer == MAP_FAILED) {
@@ -345,7 +389,6 @@ DwString::DwString(FILE* aFile , size_t aLen)
 
 DwString::DwString(const char* aCstr)
 {
-    assert(aCstr != 0);
     if (sEmptyRep == 0) {
         sEmptyBuffer[0] = 0;
         sEmptyRep = new DwStringRep(sEmptyBuffer, kEmptyBufferSize);
@@ -356,8 +399,10 @@ DwString::DwString(const char* aCstr)
     mRep = new_rep_reference(sEmptyRep);
     mStart = 0;
     mLength = 0;
-    size_t len = (aCstr) ? strlen(aCstr) : 0;
-    _replace(0, mLength, aCstr, len);
+    if ( aCstr ) {
+        size_t len = (aCstr) ? strlen(aCstr) : 0;
+        _replace(0, mLength, aCstr, len);
+    }
 }
 
 
@@ -1041,7 +1086,7 @@ void DwString::ConvertToLowerCase()
     }
     char* buf = mRep->mBuffer + mStart;
     for (size_t i=0; i < mLength; ++i) {
-        buf[i] = (char) tolower(buf[i]);
+        buf[i] = (char) dw_asciitolower(buf[i]);
     }
 }
 
@@ -1053,7 +1098,7 @@ void DwString::ConvertToUpperCase()
     }
     char* buf = mRep->mBuffer + mStart;
     for (size_t i=0; i < mLength; ++i) {
-        buf[i] = (char) toupper(buf[i]);
+        buf[i] = (char) dw_asciitoupper(buf[i]);
     }
 }
 
@@ -1717,7 +1762,7 @@ int DwStrcasecmp(const DwString& aStr1, const DwString& aStr2)
     size_t len1 = aStr1.length();
     const char* s2 = aStr2.data();
     size_t len2 = aStr2.length();
-    return dw_strcasecmp(s1, len1, s2, len2);
+    return dw_strasciicasecmp(s1, len1, s2, len2);
 }
 
 
@@ -1728,7 +1773,7 @@ int DwStrcasecmp(const DwString& aStr, const char* aCstr)
     size_t len1 = aStr.length();
     const char* s2 = aCstr;
     size_t len2 = (aCstr) ? strlen(aCstr) : 0;
-    return dw_strcasecmp(s1, len1, s2, len2);
+    return dw_strasciicasecmp(s1, len1, s2, len2);
 }
 
 
@@ -1739,7 +1784,7 @@ int DwStrcasecmp(const char* aCstr, const DwString& aStr)
     size_t len1 =  (aCstr) ? strlen(aCstr) : 0;
     const char* s2 = aStr.data();
     size_t len2 = aStr.length();
-    return dw_strcasecmp(s1, len1, s2, len2);
+    return dw_strasciicasecmp(s1, len1, s2, len2);
 }
 
 
@@ -1751,7 +1796,7 @@ int DwStrncasecmp(const DwString& aStr1, const DwString& aStr2, size_t n)
     const char* s2 = aStr2.data();
     size_t len2 = aStr2.length();
     len2 = DW_MIN(len2, n);
-    return dw_strcasecmp(s1, len1, s2, len2);
+    return dw_strasciicasecmp(s1, len1, s2, len2);
 }
 
 
@@ -1764,7 +1809,7 @@ int DwStrncasecmp(const DwString& aStr, const char* aCstr, size_t n)
     const char* s2 = aCstr;
     size_t len2 = (aCstr) ? strlen(aCstr) : 0;
     len2 = DW_MIN(len2, n);
-    return dw_strcasecmp(s1, len1, s2, len2);
+    return dw_strasciicasecmp(s1, len1, s2, len2);
 }
 
 
@@ -1777,7 +1822,7 @@ int DwStrncasecmp(const char* aCstr, const DwString& aStr, size_t n)
     const char* s2 = aStr.data();
     size_t len2 = aStr.length();
     len2 = DW_MIN(len2, n);
-    return dw_strcasecmp(s1, len1, s2, len2);
+    return dw_strasciicasecmp(s1, len1, s2, len2);
 }
 
 
