@@ -3,8 +3,8 @@
 // Contents:   Declarations for DwHeaders
 // Maintainer: Doug Sauder <dwsauder@fwb.gulf.net>
 // WWW:        http://www.fwb.gulf.net/~dwsauder/mimepp.html
-// $Revision: 1.6 $
-// $Date: 2003/11/02 00:20:09 $
+// $Revision: 1.9 $
+// $Date: 2005/01/08 10:59:28 $
 //
 // Copyright (c) 1996, 1997 Douglas W. Sauder
 // All rights reserved.
@@ -26,6 +26,7 @@
 #define DW_HEADERS_H
 
 #include <iostream>
+#include <vector>
 
 #ifndef DW_CONFIG_H
 #include <mimelib/config.h>
@@ -321,11 +322,13 @@ public:
     //. This member function allows access to extension fields or
     //. user-defined fields.
 
-    DwString AllFieldBodiesAsString(const DwString& aFieldName);
-    //. Returns a string which the comma separated header field bodies of all
-    //. header fields with field name {\tt aFieldName}.  Can for example be
-    //. used to get the contents of all Cc headers of a message which (in 
-    //. violation of RFC 2822) has multiple Cc headers.
+    std::vector<DwFieldBody*> AllFieldBodies(const DwString& aFieldName);
+    //. Returns a vector of pointers to the {\tt DwFieldBody} objects for
+    //. all header fields with field name {\tt aFieldName}.  If the header
+    //. field does not already exist, it is created.  Use {\tt HasField()}
+    //. to test if the header field already exists without creating it.
+    //. This member function allows access to extension fields or
+    //. user-defined fields.
 
     int NumFields() const;
     //. Returns the number of {\tt DwField} objects contained by this
@@ -410,6 +413,7 @@ protected:
     //. Add field but don't set the is-modified flag
 
     DwField* mFirstField;
+    DwField* mLastField;
 
 protected:
 
